define(['collections/concessionCollection'], function () {
    window.ConcessionsHelper = (function () {
        function _getProductIds(name) {
            var concessionCollection = window.ObjectCollections.ConcessionCollection;
            return concessionCollection.where({ name: name })[0].get('products');
        }

        function _getProductsByIds(productIds) {
            var productIdList = productIds.split(',');
            var result = new ProductCollection();

            for (var i = 0; i < productIdList.length; i++) {
                result.add(window.ObjectCollections.ConcessionCollection.products.where({ id: productIdList[i] }));
            }

            return result;
        }

        function _getModifiersByProductId(id) {
            var currentIndex = this.getCurrentModifier();
            var modifiers = new ConcessionCollection(window.ObjectCollections.ConcessionCollection.products.where({ id: id })[0].get('modifiers')[currentIndex].modifierItems);
            var result = new ProductCollection();

            for (var i = 0; i < modifiers.models.length; i++) {
                var productId = modifiers.models[i].get('productId');
                var product = window.ObjectCollections.ConcessionCollection.products.where({ id: productId })[0];
                product.set('price', modifiers.models[i].get('price'));
                product.set('includedQuantity', modifiers.models[i].get('includedQuantity'));
                result.add(product);
            }

            return result;
        }

        function _createSelectedItem(product, modifier) {
            var selectedConcession = { concessionItemId: this.getGuid(), productId: product.get('id'), name: product.get('name'), price: product.get('price'), quantity: 1, tax: product.get('tax'), image: product.get('image') };
            if (modifier != null) {
                selectedConcession.modifiers = [{ quantity: 1, productId: modifier.get('id'), name: modifier.get('name'), price: modifier.get('price'), tax: modifier.get('tax'), image: modifier.get('image') }];
            } else {
                selectedConcession.modifiers = [];
            }

            return selectedConcession;
        }

        function _isMultiple(id) {
            var selectedProduct = window.ObjectCollections.ConcessionCollection.products.where({ id: id })[0];

            var modifiersCount = selectedProduct.get('modifiers').length;
            this.setTotalModifiersCount(modifiersCount);

            if (modifiersCount == 0)
                return null;

            var currentIndex = modifiersCount == 1 ? 0 : (window.ConcessionData.CurrentModifier > 1 ? window.ConcessionData.CurrentModifier - 1 : window.ConcessionData.CurrentModifier);
            return (!selectedProduct.get('modifiers')[currentIndex].requiredNumberOfModifiers && !selectedProduct.get('modifiers')[currentIndex].maxNumberOfModifiers)
                || (selectedProduct.get('modifiers')[currentIndex].requiredNumberOfModifiers != 1 && selectedProduct.get('modifiers')[currentIndex].maxNumberOfModifiers != 1);
        }

        function _saveItem(selectedConcession) {
            if (window.ConcessionData.ComboItemProducts.length > 0) {
                window.TempComboConcessionObject.comboItems.push(selectedConcession);
            } else {
                window.TempConcessionsObjects.push(selectedConcession);
            }
        }

        function _routeNext(product, theaterCulture, parentName) {
            if (parentName)
                window.ConcessionData.ParentName = parentName;

            var id = product.get('id');
            var multiple = this.isMultiple(id);
            var orderModel = window.ObjectModels.OrderModel;

            if (this.getTotalModifiersCount() > this.getCurrentModifier()) {
                var route = multiple ? 'concessionModifiersMulti' : 'concessionModifiersSingleById';
                window.AppMethods.ChangePage(route + '/' + id + '/' + theaterCulture);
            } else if (window.TempConcessionsObjects.length > 0) {
                var concession = window.TempConcessionsObjects[0];
                window.TempConcessionsObjects.shift();

                _.each(window.TempConcessionsObjects, function (c) {
                    concession.modifiers = concession.modifiers.concat(c.modifiers);
                });

                orderModel.get('concessions').add(concession);

                window.AppMethods.ChangePage('concessions/' + orderModel.get('theater').get('id') + '/new');
            } else if (window.ConcessionData.ComboProductsCount > 0) {
                if (multiple === null) {
                    var selectedConcession = this.createSelectedItem(product, null);
                    this.saveItem(selectedConcession);
                }

                _handleComboItems.call(this, theaterCulture);
            } else {
                var selectedConcession = this.createSelectedItem(product, null);
                this.updateTax(selectedConcession.tax);
                orderModel.get('concessions').add(selectedConcession);
                window.AppMethods.ChangePage('concessions/' + orderModel.get('theater').get('id') + '/new');
            }
        }

        function _handleComboItems(theaterCulture) {
            if (window.ConcessionData.ComboProductsCount > window.ConcessionData.CurrentComboProduct) {
                this.resetTotalModifiersCount();
                this.resetCurrentModifier();

                var productIds = window.ConcessionData.ComboItemProducts[window.ConcessionData.CurrentComboProduct];
                window.ConcessionData.ParentName = window.TempComboConcessionObject.name;

                window.AppMethods.ChangePage('concessionItem/' + encodeURIComponent(window.ConcessionData.ParentName) + '/' + productIds + '/' + theaterCulture + '/' + new Date().getTime());
                window.ConcessionData.CurrentComboProduct += 1;
            } else {
                window.ObjectModels.OrderModel.get('concessions').add(window.TempComboConcessionObject);
                require(['helpers/concessionsHelper'], function () {
                    var helper = new ConcessionsHelper();
                    helper.updateTax(window.ConcessionData.CurrentComboTax);
                });
                window.AppMethods.ChangePage('concessions/' + window.ObjectModels.OrderModel.get('theater').get('id') + '/new');
            }
        }

        function _removeItem() {
            if (window.ConcessionData.CurrentModifier > 1 || window.ConcessionData.CurrentModifier < window.ConcessionData.TotalModifiersCount) {
                window.ConcessionData.CurrentModifier -= 2;
                window.TempConcessionsObjects.splice(window.ConcessionData.CurrentModifier, 1);
            }
        }

        function _updateCurrentModifier() {
            window.ConcessionData.CurrentModifier += 1;
        }

        function _resetCurrentModifier() {
            window.ConcessionData.CurrentModifier = 0;
        }

        function _getCurrentModifier() {
            return window.ConcessionData.CurrentModifier;
        }

        function _setTotalModifiersCount(count) {
            window.ConcessionData.TotalModifiersCount = count;
        }

        function _resetTotalModifiersCount() {
            window.ConcessionData.TotalModifiersCount = 0;
        }

        function _getTotalModifiersCount() {
            return window.ConcessionData.TotalModifiersCount;
        }

        function _getGuid() {
            var rNumber = function () {
                return (((1 + Math.random()) * 0x10000) | 0).toString(16).substring(1);
            };
            return (rNumber() + rNumber() + "-" + rNumber() + "-" + rNumber() + "-" + rNumber() + "-" + rNumber() + rNumber() + rNumber());
        }

        function _clearData() {
            window.ConcessionData.CurrentComboProduct = 0;
            window.ConcessionData.ComboProductsCount = 0;
            window.ConcessionData.ComboItemProducts = [];

            window.ConcessionData.CurrentModifier = 0;
            window.ConcessionData.TotalModifiersCount = 0;

            window.TempConcessionsObjects = [];
            window.TempComboConcessionObject = {};
        }

        function _updateTax(tax) {
            if (tax) {
                var currentTax = window.ObjectModels.OrderModel.get('tax') || 0;
                var sum = parseFloat(currentTax + tax);
                window.ObjectModels.OrderModel.set('tax', sum);
            }
        }

        return {
            getProductIds: _getProductIds,
            getProductsByIds: _getProductsByIds,
            getModifiersByProductId: _getModifiersByProductId,
            createSelectedItem: _createSelectedItem,
            updateCurrentModifier: _updateCurrentModifier,
            resetCurrentModifier: _resetCurrentModifier,
            setTotalModifiersCount: _setTotalModifiersCount,
            resetTotalModifiersCount: _resetTotalModifiersCount,
            isMultiple: _isMultiple,
            routeNext: _routeNext,
            getCurrentModifier: _getCurrentModifier,
            getTotalModifiersCount: _getTotalModifiersCount,
            removeItem: _removeItem,
            saveItem: _saveItem,
            getGuid: _getGuid,
            clearData: _clearData,
            updateTax: _updateTax
        };
    });
});