﻿
define(['models/chainInfoModel'], function (ChainInfoModel) {
    // Returns device's connection status
    window.deviceConnectionStatus = navigator.network === undefined ? 'ONLINE' : navigator.connection.type !== Connection.NONE ? 'ONLINE' : 'OFFLINE';

    window.ConfigHelper = {
        initialize: function (callback) {
            window.getChainInfo(callback);
        },
        isDeviceOnline: function () {
            return window.deviceConnectionStatus == 'ONLINE';
        },
        deviceOfflineEvent: function () {
            window.deviceConnectionStatus = 'OFFLINE';
        },
        deviceOnlineEvent: function () {
            window.deviceConnectionStatus = 'ONLINE';
            window.getChainInfo(function () {
                if (window.Views.HomeView !== null && (window.location.hash === '#home' || window.location.hash === '')) {
                    window.Views.HomeView.curtainsMenuView.reloadTheaters = true;
                    window.Views.HomeView.curtainsMenuView.loadFinished = false;
                }
            });
        },
        onDeviceBackButton: function (e) {
            switch (window.location.hash) {
                case '#home':
                case '':
                    e.preventDefault();
                    navigator.app.exitApp();
                    break;
                case '#myTicketView':
                case '#loyaltycardview':
                case '#menuSettings':
                case '#menuLoyaltycard':
                case '#menuMovies':
                case '#menuTheaters':
                case '#menuConcessions':
                case '#menuPurchaseHistory':
                case '#menuVirtualGiftCards':
                case '#comingSoon':
                    window.AppMethods.ChangePage('');
                    break;
                default:
                    navigator.app.backHistory();
                    break;
            }
            window.AppProperties.nativeBackButton = true;
        },
        clearCache: function () {
            //Global App Properties - values should be updated over the app flow
            window.AppProperties.SelectedTheater = null;
            window.AppProperties.SelectedTheaterByZip = null;
            window.AppProperties.SelectedPerformance = null;
            window.AppProperties.HistoryOrders = null;
            window.AppProperties.HistoryOrderIndex = null;
            window.AppProperties.TapEvent = null;

            window.ConcessionData.CurrentModifier = 0;
            window.ConcessionData.TotalModifiersCount = 0;
            window.ConcessionData.ComboProductsCount = 0;
            window.ConcessionData.CurrentComboProduct = 0;
            window.ConcessionData.ComboItemProducts = [];
            window.ConcessionData.ParentName = null;

            window.TempConcessionsObjects = [];
            window.TempComboConcessionObject = {};
        }
    };

    window.getChainInfo = function (callback) {
        var model = new ChainInfoModel();

        model.fetch({
            success: function (result) {
                window.ChainInfo = null;
                window.ChainInfo = result;

                window.AppProperties.SearchUnit = window.ChainInfo.defaultSearchRadiusIsInMiles() ? window.ObjectCollections.Localization.result.miles : window.ObjectCollections.Localization.result.kilometers;

                if (result) {
                    $.event.trigger('chainInfoUpdated');

                    require(['sharedhelpers/storageProvider'], function () {
                        var storage = new StorageProvider();
                        storage.store('ChainInfo', result.attributes);
                    });
                }

                if (window.Views.HomeView !== null && (window.location.hash === '#home' || window.location.hash === '')) {
                    window.Views.HomeView.render();
                    $(window.Views.HomeView.el).find('#NCR-listview').listview();
                }

                callback(true);
            },
            error: function () {
                if (window.ChainInfo === undefined) {
                    window.ChainInfo = model;
                }
                callback(false);
            },
            async: false
        });
    };
});