﻿define([], function () {
    window.DeepLinkingHelper = (function () {
        function _validateFacebookAccount(facebookId, fbValidationError, notLogedOnFbError, callback) {
            require(['facebook', 'sharedhelpers/utilities', 'sharedhelpers/storageProvider'], function () {
                var facebook = new FacebookHelper();
                var utilities = new Utilities();
                var storage = new StorageProvider();

                facebookId = utilities.decrypt(decodeURIComponent(facebookId));

                var continueFlow = function (fbValid) {
                    if (!fbValid) {
                        window.showAlert(fbValidationError);
                    }

                    window.AppProperties.FBValid = fbValid;

                    if (callback) {
                        callback();
                    }
                };

                storage.read("fbAccount", function (value) {
                    if (value) {
                        continueFlow(facebookId == value.id);
                        return;
                    }

                    facebook.connect(function (token) {
                        if (token) {
                            facebook.getData(token, { action: 'me?' }, function (result) {
                                continueFlow(facebookId == result.id);
                            });
                        } else {
                            window.showAlert(notLogedOnFbError);

                            if (callback) {
                                callback();
                            }
                        }
                    });
                });
            });
        }

        function _loadTheaters(args, callback) {
            require(['collections/theaterCollection', 'models/theaterModel'], function (theaterCollection, theaterModel) {
                var collectionHandler = function (result) {
                    window.ObjectCollections.TheaterCollection = result;
                    window.ObjectCollections.MovieCollection = result.getMovies();

                    window.AppProperties.SelectedTheater = window.ObjectCollections.TheaterCollection.get(args.theaterId);

                    if (callback) {
                        callback(args);
                    }
                };

                var singleTheaterModel = new theaterModel({ id: args.theaterId });

                singleTheaterModel.getTheater(function (result) {
                    if (!result) {
                        new ErrorHelper().showAlertByErrorCode();
                        window.hidePageLoadingMessage();
                        return;
                    }

                    var theaters = new theaterCollection();
                    theaters.push(result);
                    collectionHandler(theaters);
                });
            });
        }

        function _fetchSchedule(args) {
            var theMovie = window.ObjectCollections.MovieCollection.getMovie(args.movieId);

            if (!theMovie) {
                window.showAlert(window.ObjectCollections.Localization.result.performanceHasExpired, {
                    postBack: function () {
                        window.AppMethods.ChangePage('', false);
                    }
                });

                return;
            }

            require(['collections/movieCollection'], function (movieCollection) {
                var movies = new movieCollection();

                movies.getMovieSchedule(args.movieId, args.theaterId, function (result) {
                    var movie = result.models[0];

                    theMovie.performances = new PerformanceCollection(movie.schedule.models[0].get('performances'));
                    if (args.callback) {
                        args.callback();
                    }
                });
            });
        }

        function _getSelectTicketsData(args) {
            ConfigHelper.initialize(function () {
                _loadTheaters(args, _fetchSchedule);
            });
        }

        function _validatePerformance(theater, movie, pPerformance) {
            var perf = movie.performances.get(pPerformance);

            window.AppProperties.SelectedPerformance = perf;

            if (!perf) {
                window.showAlert(window.ObjectCollections.Localization.result.performanceHasExpired, {
                    postBack: function () {
                        window.AppMethods.ChangePage('', false);
                    }
                });
                return false;
            } else {
                var time = new Date(perf.get('showTime'));
                time.setMinutes(time.getMinutes() + theater.get("performanceExpirationInMinutes"));
                var now = new Date().toISOString();

                if (time.toISOString() < now) {
                    window.showAlert(window.ObjectCollections.Localization.result.performanceStarted, {
                        postBack: function () {
                            window.AppMethods.ChangePage('', false);
                        }
                    });
                    return false;
                }

                return true;
            }
        }

        function _handleDeepLink(url) {
            window.deepLinkUrl = null;

            if (window.AppProperties.IsIOS) {
                url = decodeURIComponent(url);
            }

            var route = url.split('#')[1];

            window.AppMethods.ChangePage("#" + route);
        }

        return {
            handleDeepLink: _handleDeepLink,
            getSelectTicketsData: _getSelectTicketsData,
            validateFacebookAccount: _validateFacebookAccount,
            validatePerformance: _validatePerformance
        };
    });
});