﻿define(['sharedhelpers/configurationProvider'], function (configurationProvider) {
    window.dualCardHelper = (function () {
        var localize = window.ObjectCollections.Localization.result;

        function _getTeaterId() {
            var theaterCollection = window.ObjectCollections.TheaterCollection;
            var theater = theaterCollection && theaterCollection.models && theaterCollection.models.length > 0 ? theaterCollection.models[0] : new BaseModel();

            return theater.get('id') || configurationProvider.theaterId();
        };

        function _handleGiftCard(loyaltyNumber, loyaltyStoreModel, callback) {
            var that = this;

            require(['models/cardDecodeModel', 'sharedhelpers/storageProvider'], function (cardDecodeModel) {
                var cardDecode = new cardDecodeModel({
                    theaterId: _getTeaterId(),
                    cardnum: loyaltyNumber
                });

                cardDecode.fetch({
                    success: function (result) {
                        loyaltyStoreModel.multitype = _.any(result, function (type) {
                            return type == 'SVC';
                        });

                        if (!loyaltyStoreModel.multitype) {
                            if (callback) {
                                callback();
                            }
                            return;
                        }

                        var storage = new StorageProvider();
                        storage.store('LoyaltyCard', loyaltyStoreModel);

                        var saveDualGiftCard = function () {
                            require(['helpers/paymentMethodHelper'], function () {
                                var paymentMethodHelper = new PaymentMethodsHelper();

                                var paymentMethod = {
                                    Name: 'GiftCard',
                                    CardNumber: loyaltyNumber,
                                    Type: 'GiftCard'
                                };

                                paymentMethodHelper.insert({ paymentMethod: paymentMethod }, function (response) {
                                    if (response.cardExists) {
                                        if (callback) {
                                            callback();
                                        }
                                        return;
                                    }
                                    if (response.success) {
                                        window.showAlert(localize.alertDualGiftCardAdded, {
                                            postBack: function () {
                                                if (callback) {
                                                    callback();
                                                }
                                            }
                                        });
                                        return;
                                    }
                                   
                                    if (response.passwordExists == false) {
                                        window.hidePageLoadingMessage();
                                        window.showAlert(localize.alertAddDualGiftCard);
                                        require(['views/createPassDialogView'], function (createPassView) {
                                            var dialog = new createPassView({
                                                selectedPaymentMethodType: 'GiftCard',
                                                dialogTopNotification: that.localize.createPasswordNotification1,
                                                dialogBottomNotification: that.localize.createPasswordNotification2,
                                                callback: function (value) {
                                                    $(that.el).find('#popupDialog').popup('close');
                                                    window.showPageLoadingMessage();

                                                    if (value) {
                                                        saveDualGiftCard();
                                                    }
                                                }
                                            });

                                            $(that.el).find('#popupDialog').html(dialog.render());
                                            $(that.el).find('#popupDialog').popup('open');
                                            $(that.el).find('#popupDialog').enhanceWithin();
                                        });
                                        return;
                                    }
                                   

                                });
                            });
                        };

                        saveDualGiftCard();
                    },
                    error: function () {
                        if (callback) {
                            callback();
                        }
                    }
                });
            });
        }

        function _handleLoyaltyCard(cardNumber, callback) {
            var that = this;
            window.showPageLoadingMessage();

            require(['models/cardDecodeModel', 'sharedhelpers/storageProvider'], function (cardDecodeModel) {
                var storage = new StorageProvider();

                storage.read('LoyaltyCard', function (loyalty) {
                    if (loyalty) {
                        if (callback) {
                            callback();
                        }
                        return;
                    }

                    var cardDecode = new cardDecodeModel({
                        theaterId: _getTeaterId(),
                        cardnum: cardNumber
                    });

                    cardDecode.fetch({
                        success: function (result) {
                            var isLoyalty = _.any(result, function (type) {
                                return type == 'Loyalty';
                            });

                            if (!isLoyalty) {
                                if (callback) {
                                    callback();
                                }
                                return;
                            }

                            window.hidePageLoadingMessage();
                            require(['views/inputDialogView'], function (inputDialogView) {
                                var dialog = new inputDialogView({
                                    callback: function (lastName) {
                                        if (lastName) {
                                            _addDualLoyalty.call(that, cardNumber, lastName, callback);
                                            return;
                                        }

                                        $(that.el).find('#popupDialog').popup('close');

                                        if (callback) {
                                            callback();
                                        }
                                    }
                                });

                                $(that.el).find('#popupDialog').html(dialog.render());
                                $(that.el).find('#popupDialog').popup('open');
                                $(that.el).find('#popupDialog').enhanceWithin();
                            });
                            return;
                        },
                        error: function () {
                            if (callback) {
                                callback();
                            }
                        }
                    });
                });
            });
        }

        function _addDualLoyalty(cardNumber, lastName, callback) {
            var that = this;
            window.showPageLoadingMessage();

            require(['models/loyaltyAccountModel', 'sharedhelpers/storageProvider'], function (loyaltyAccountModel) {
                var model = new loyaltyAccountModel();
                model.set({ id: cardNumber, securityKey: lastName });

                model.fetch({
                    cache: false,
                    success: function () {
                        window.hidePageLoadingMessage();
                        var modelCardNumber = model.get('cardNumber') !== undefined ? model.get('cardNumber').value : null;
                        var modelCardLastName = model.get('lastName') !== undefined ? model.get('lastName').value : null;
                        var modelCardFirstName = model.get('firstName') !== undefined ? model.get('firstName').value : null;
                        var offers = model.get('loyaltyBanners') || null;

                        if (modelCardNumber === null) {
                            window.showAlert(localize.errorLoyaltyCardNotExists);
                            return;
                        } else {
                            var loyaltyStoreModel = {
                                number: cardNumber,
                                firstName: modelCardFirstName,
                                lastName: modelCardLastName,
                                offers: offers,
                                multitype: true
                            };

                            var storage = new StorageProvider();
                            storage.store('LoyaltyCard', loyaltyStoreModel);

                            if (callback) {
                                $(that.el).find('#popupDialog').popup('close');

                                callback();
                            }
                        }
                    },
                    error: function () {
                        window.hidePageLoadingMessage();
                        window.showAlert(localize.errorGettingLoyaltyCardData);
                    }
                });
            });
        }

        function _deleteCard(cardId, callback) {
            var onConfirm = function (button) {
                if (button == 1) {
                    require(['helpers/paymentMethodHelper', 'sharedhelpers/storageProvider'], function () {
                        var pmh = new PaymentMethodsHelper();
                        var storage = new StorageProvider();

                        pmh.remove(cardId, function (response) {
                            if (response.success) {
                                storage.remove("LoyaltyCard");

                                callback();
                            }
                        });
                    });
                }
            };

            if (window.isMobileDevice()) {
                navigator.notification.confirm(localize.removeDualCardFromApplication, onConfirm, localize.alertTitle, localize.ok + ', ' + localize.cancel);
            } else if (confirm(localize.removeDualCardFromApplication)) {
                onConfirm(1);
            }
        }

        return {
            handleGiftCard: _handleGiftCard,
            handleLoyaltyCard: _handleLoyaltyCard,
            deleteCard: _deleteCard
        };
    })();
});