define([
    'jquery',
    'helpers/uiHelper',
    'sharedhelpers/storageProvider',
    'childbrowser'
], function ($) {
    window.FacebookHelper = (function () {
        var facebook = {
            url: { //https is required!
                basic: 'https://www.facebook.com',
                library: 'https://connect.facebook.net/en_US/all.js',
                redirect: 'https://m.facebook.com'
            }
        };

        function getFacebookUrl() {
            var redirectUrl = window.location.href;

            redirectUrl = redirectUrl.substr(0, redirectUrl.indexOf('/www/'));
            redirectUrl += '/www/facebooklanding.html';

            if (redirectUrl.indexOf('file') != -1) {
                redirectUrl = facebook.url.redirect;
            }

            return facebook.url.basic + "/dialog/oauth/?client_id=" + appConfig.FacebookAPIKey + "&&redirect_uri=" + redirectUrl + "&state=somestatevalue&response_type=token&display=touch&scope=publish_stream";
        }


        function connect(callback, options) {
            options = options || {};
            isTokenValid(function (isValid, token) {
                if (isValid === true) {
                    callback(token);
                    return;
                }

                if (options.preventLogin) {
                    callback(null);
                } else {
                    login(callback);
                }
            });
        }

        function login(callback) {
            var url = getFacebookUrl();
            window.plugins.childBrowser.showWebPage(url);

            window.plugins.childBrowser.onLocationChange = function (browserUrl) {

                if (browserUrl.indexOf('access_token') != -1) {
                    var token = browserUrl.match(/access_token=(.*?)&/i)[1];;
                    callback(token);

                    window.plugins.childBrowser.close();
                }
            };
        }

        function logout(callback) {
            require([facebook.url.library], function () {
                try {
                    FB.init({
                        appId: appConfig.FacebookAPIKey
                    });
                } catch (e) {
                    //any subsequent FB.init call will cause an error, however this does not impact further execution
                } finally {
                    FB.getLoginStatus(function () {
                        FB.logout(function () {
                            callback();
                        });
                    });
                }
            });
        }

        function saveAccount(callback) {
            connect(function (result) {
                callback(result);
            });
        }

        function getData(accessToken, option, callback) {
            require([facebook.url.library], function () {
                FB.api("/" + option.action + "access_token=" + accessToken, function (result) {
                    if (result.error !== undefined) {
                        callback(null);
                        return;
                    }

                    callback(result);
                });
            });
        }

        function isTokenValid(callback) {
            var storage = new StorageProvider();
            storage.read('fbAccount', function (value) {
                if (!value) {
                    callback(false);
                    return;
                }

                require([facebook.url.library], function () {
                    FB.init({
                        appId: appConfig.FacebookAPIKey
                    });

                    FB.getLoginStatus(function (response) {
                        if (response.status == 'connected') {
                            callback(true, response.authResponse.accessToken);
                        } else {
                            callback(false);
                        }
                    });
                });
            });
        }

        function postToWall(token, options, callback) {
            require([facebook.url.library], function () {
                FB.api('/me/feed?access_token=' + token, 'post', {
                    name: options.name,
                    description: options.description,
                    caption: options.caption,
                    link: options.link,
                    picture: options.picture,
                    actions: [{
                        name: window.ObjectCollections.Localization.result.actionsShareName,
                        link: options.link
                    }]
                },
                function (response) {
                    callback(response);
                });
            });
        }

        function postToFriend(options, callback) {
            var eventUrl;
            var url = getFacebookShareUrl(options.link, options.picture, options.name, options.caption, options.description, options.userList, options.actionType);
            var delay = options.preventDelay ? 50 : 400;
            var ref = null;

            function loadStop() {
                ref.executeScript({
                    code: "document.getElementById('publish').onclick=function() {" +
                                "document.getElementById('page').style.pointerEvents='none';" +
                                "document.getElementById('publish').style.pointerEvents='none';" +
                                "var div = document.createElement('div');" +
                                "div.style.backgroundColor = 'rgba(22,22,22,0.7)';" +
                                "div.style.height = '100%';" +
                                "div.style.width = '100%';" +
                                "div.style.position = 'fixed';" +
                                "div.style.top = '0';" +
                                "document.getElementById('page').appendChild(div);" +
                                "document.getElementById('cancel').style.pointerEvents='none';" +
                            "}"
                });
            }

            function loadStart(event) {
                eventUrl = event.url;
                if (eventUrl.indexOf('&post_id') !== -1) {
                    ref.close();
                } else if (eventUrl.indexOf('facebook') === -1) {
                    ref.close();
                }
            }

            function exit() {
                showPageLoadingMessage();

                _.delay(function () {
                    callback({ success: eventUrl.indexOf('&post_id') !== -1 });
                }, delay);

                ref.removeEventListener('loadstart');
                ref.removeEventListener('loadstop');
                ref.removeEventListener('exit');
                ref.close();
                ref = null;
            }

            ref = window.open(url, '_blank', 'location=no,toolbar=no');
            ref.addEventListener('loadstop', loadStop);
            ref.addEventListener('loadstart', loadStart);
            ref.addEventListener('exit', exit);
        }

        function getFacebookShareUrl(url, img, name, caption, description, userList, actionType) {
            var actionsName = "";
            if (actionType == "gift") {
                actionsName = window.ObjectCollections.Localization.result.actionsGiftName;
            } else if (actionType == "hold") {
                actionsName = window.ObjectCollections.Localization.result.actionsHoldName;
            }
            var actionsProperties = "[{'name': '" + actionsName + "','link': '" + url + "'}]";

            return facebook.url.basic + "/dialog/feed?" +
                            "app_id=" + appConfig.FacebookAPIKey +
                            (url != null ? "&link=" + encodeURIComponent(url) : "") +
                            (img != null ? "&picture=" + img : "") +
                            "&name=" + encodeURIComponent(name) +
                            "&caption=" + encodeURIComponent(caption) +
                            "&description=" + encodeURIComponent(description) +
                            "&to=" + userList +
                            "&actions=" + encodeURIComponent(actionsProperties) +
                            "&redirect_uri=" + facebook.url.redirect + "?posted";
        }

        return {
            connect: connect,
            saveAccount: saveAccount,
            logout: logout,
            getData: getData,
            share: postToWall,
            post: postToFriend
        };
    });
});