define([
    'jquery',
    'helpers/uiHelper',
    'sharedhelpers/storageProvider'
], function ($) {
    window.FacebookHelper = (function () {
        var btnLabel = window.ObjectCollections.Localization.result.facebookInAppLabel;
        var facebook = {
            url: { //https is required!
                basic: 'https://www.facebook.com/',
                library: 'https://connect.facebook.net/en_US/all.js',
                redirect: appConfig.FacebookLanding
            }
        };

        function _checkPlatformStatus(callback) {
            $.ajax({
                url: facebook.url.basic + 'feeds/api_status.php',
                dataType: 'json',
                complete: function (result) {
                    if (JSON.parse(result.responseText).current.health == 1) {
                        callback();
                    } else {
                        window.showAlert(window.ObjectCollections.Localization.result.facebookIsDown);
                    }
                }
            });
        }

        function _isTokenValid(callback) {
            var storage = new StorageProvider();
            storage.read('fbAccount', function (value) {
                if (!value) {
                    callback(false);
                    return;
                }

                window.facebookConnectPlugin.getLoginStatus(function (response) {
                    if (response.status == 'connected') {
                        callback(true, response.authResponse.accessToken);
                    } else {
                        callback(false);
                    }
                }, true);
            });
        }

        function _connect(callback, options) {
            options = options || {};
            _isTokenValid(function (isValid, token) {
                if (isValid === true) {
                    callback(token);
                    return;
                }

                if (options.preventLogin) {
                    callback(null);
                } else {
                    _login(callback);
                }
            });
        }

        function _login(callback) {
            var url = _getFacebookUrl();
            var ref = window.open(url, '_blank', 'location=yes,closebuttoncaption=' + btnLabel);

            ref.addEventListener('loadstop', function (event) {
                if (event.url.indexOf('access_token') != -1) {
                    window.AppProperties.tempFbAccessToken = event.url.match(/access_token=(.*?)&/i)[1];

                    ref.close();
                }
            });

            ref.addEventListener('exit', function () {
                _.delay(function () {
                    callback(window.AppProperties.tempFbAccessToken);
                }, 50);

                ref.removeEventListener('exit');
                ref.removeEventListener('loadstop');
                ref = null;
            });
        }

        function _logout(callback) {
            var storage = new StorageProvider();
            storage.read('fbAccount', function (value) {
                if (!value) {
                    callback();
                    return;
                }

                var token = value.accessToken;
                var url = _getLogoutUrl(token);
                var ref = window.open(url, '_blank', 'location=yes,closebuttoncaption=' + btnLabel);

                ref.addEventListener('loadstop', function () {
                    ref.close();
                    callback();
                });

                ref.addEventListener('exit', function () {
                    ref.removeEventListener('loadstop');
                    ref.removeEventListener('exit');
                    ref = null;
                });
            });
        }

        function _getFacebookUrl() {
            var redirectUrl = window.location.href;

            redirectUrl = redirectUrl.substr(0, redirectUrl.indexOf('/www/'));
            redirectUrl += '/www/facebooklanding.html';

            if (redirectUrl.indexOf('file') != -1) {
                redirectUrl = facebook.url.redirect;
            }

            return facebook.url.basic + "/dialog/oauth/?client_id=" + appConfig.FacebookAPIKey + "&&redirect_uri=" + redirectUrl + "&state=somestatevalue&response_type=token&display=touch&scope=publish_stream";
        }

        function _getLogoutUrl(token) {
            return facebook.url.basic + '/logout.php?access_token=' + token + '&confirm=1&next=' + facebook.url.redirect;
        }

        function _saveAccount(callback) {
            var extendToken = function (token) {
                $.get('https://graph.facebook.com/oauth/access_token?client_id=' + appConfig.FacebookAPIKey + '&client_secret=' + appConfig.FacebookAPISecret + '&grant_type=fb_exchange_token&fb_exchange_token=' + token, function (data) {
                    if (data.indexOf('access_token') >= 0) {
                        var accessToken = data.match(/access_token=(.*?)&/i)[1];

                        callback(accessToken);
                    } else {
                        callback();
                    }
                });
            };

            _connect(function (result) {
                extendToken(result);
            });
        }

        function _getData(accessToken, option, callback) {
            require([facebook.url.library], function () {
                FB.api("/" + option.action + "access_token=" + accessToken, function (response) {
                    if (response.error) {
                        callback();
                        return;
                    }

                    callback(response);
                });
            });
        }

        function _postToWall(token, options, callback) {
            _checkPlatformStatus(function () {
                var postOptions = _getPostOptions(options);

                require([facebook.url.library], function () {
                    FB.api('/me/feed?access_token=' + token, 'post', postOptions, function (response) {
                        callback(response);
                    });
                });
            });
        }

        function _postToFriend(token, options, callback) {
            _checkPlatformStatus(function () {
                var eventUrl;
                var url = _getFacebookShareUrl(token, options.link, options.picture, options.name, options.caption, options.description, options.userList, options.actionType);
                var delay = options.preventDelay ? 50 : 400;

                var ref = window.open(url, '_blank', 'location=no,toolbar=no');
                ref.addEventListener('loadstop', _loadStop);
                ref.addEventListener('loadstart', _loadStart);
                ref.addEventListener('exit', _exit);

                function _loadStop() {
                    ref.executeScript({
                        code: "document.getElementById('publish').onclick=function() {" +
                                    "document.getElementById('page').style.pointerEvents='none';" +
                                    "document.getElementById('publish').style.pointerEvents='none';" +
                                    "var div = document.createElement('div');" +
                                    "div.style.backgroundColor = 'rgba(22,22,22,0.7)';" +
                                    "div.style.height = '100%';" +
                                    "div.style.width = '100%';" +
                                    "div.style.position = 'fixed';" +
                                    "div.style.top = '0';" +
                                    "document.getElementById('page').appendChild(div);" +
                                    "document.getElementById('cancel').style.pointerEvents='none';" +
                                "}"
                    });
                }

                function _loadStart(event) {
                    eventUrl = event.url;
                    if (eventUrl.indexOf('&post_id') !== -1) {
                        ref.close();
                    } else if (eventUrl.indexOf('facebook') === -1) {
                        ref.close();
                    }
                }

                function _exit() {
                    window.showPageLoadingMessage();

                    _.delay(function () {
                        callback({ success: eventUrl.indexOf('&post_id') !== -1 });
                    }, delay);

                    ref.removeEventListener('loadstart');
                    ref.removeEventListener('loadstop');
                    ref.removeEventListener('exit');
                    ref.close();
                    ref = null;
                }
            });
        }

        function _getFacebookShareUrl(token, url, img, name, caption, description, userList, actionType) {
            var actionsName = "";
            if (actionType == "gift") {
                actionsName = window.ObjectCollections.Localization.result.actionsGiftName;
            } else if (actionType == "hold") {
                actionsName = window.ObjectCollections.Localization.result.actionsHoldName;
            }
            var actionsProperties = "[{'name': '" + actionsName + "','link': '" + url + "'}]";

            return facebook.url.basic + "/dialog/feed?" +
                            "access_token=" + token +
                            "&app_id=" + appConfig.FacebookAPIKey +
                            (url != null ? "&link=" + encodeURIComponent(url) : "") +
                            (img != null ? "&picture=" + img : "") +
                            "&name=" + encodeURIComponent(name) +
                            "&caption=" + encodeURIComponent(caption) +
                            "&description=" + encodeURIComponent(description) +
                            "&to=" + userList +
                            "&actions=" + encodeURIComponent(actionsProperties) +
                            "&redirect_uri=" + facebook.url.redirect + "?posted";
        }

        function _getPostOptions(options, redirect) {
            var actionsName = window.ObjectCollections.Localization.result.actionsShareName;
            if (options.actionType == 'gift') {
                actionsName = window.ObjectCollections.Localization.result.actionsGiftName;
            } else if (options.actionType == 'hold') {
                actionsName = window.ObjectCollections.Localization.result.actionsHoldName;
            }

            var postOptions = {
                name: options.name,
                description: options.description,
                caption: options.caption,
                link: options.link,
                picture: options.picture,
                actions: [{
                    name: actionsName,
                    link: options.link
                }]
            };

            if (options.userList) {
                postOptions.to = options.userList;
            }

            if (redirect) {
                postOptions.redirect_uri = facebook.url.redirect + '?posted';
            }

            return postOptions;
        }

        return {
            connect: _connect,
            saveAccount: _saveAccount,
            logout: _logout,
            getData: _getData,
            share: _postToWall,
            post: _postToFriend
        };
    });
});