﻿
define(['sharedhelpers/storageProvider'], function () {
    var helper = {
        initialize: function () {

        },

        geoOptions: { maximumAge: 60000, timeout: 10000, enableHighAccuracy: false },

        acquireCurrentPosition: function (callback, suppressErrors) {
            var that = this;
            if (window.AppProperties.CityOrZip) {
                window.AppProperties.ShowTheaterDistance = true;
                callback();
                return;
            }

            window.AppProperties.CurrentPosition = { Latitude: 0, Longitude: 0 };
            window.AppProperties.ShowTheaterDistance = false;

            //Set default location for browser mode
            if (navigator.network === undefined) {
                window.AppProperties.CurrentPosition = { Latitude: 34.036124, Longitude: -84.575485 };
                window.AppProperties.ShowTheaterDistance = true;
            }

            if (!navigator.geolocation && !suppressErrors) {
                window.showAlert(this.localize.errorUnableToAcquireYourLocation);
                return;
            };
            // Acquire current position  
            navigator.geolocation.getCurrentPosition(function (position) {
                window.AppProperties.ShowTheaterDistance = true;
                window.AppProperties.CurrentPosition = { Latitude: position.coords.latitude, Longitude: position.coords.longitude };
                if (callback)
                    callback();
            },
            function (error) {
                var storage = new StorageProvider();
                var geolocationAllowed = 'geolocationAllowed';
                storage.read(geolocationAllowed, function (value) {
                    if (value == null) {
                        if (!suppressErrors)
                            that.handleGeolocationErrors(error);
                        storage.store(geolocationAllowed, error.code != 1);
                    }
                    else if (error.code == 1)
                        storage.store(geolocationAllowed, false);
                    else {
                        if (!suppressErrors)
                            that.handleGeolocationErrors(error);
                        storage.store(geolocationAllowed, true);
                    }
                });

                if (callback)
                    callback();
            }, that.geoOptions);
        },

        handleGeolocationErrors: function (error) {
            window.hidePageLoadingMessage();
            var localize = window.ObjectCollections.Localization.result;
            switch (error.code) {
                case 1:
                    window.showAlert(localize.errorUnableToAcquireYourLocationEnableLocation);
                    break;
                case 2:
                    window.showAlert(localize.errorDetectingCurrentPosition);
                    break;
                case 3:
                    if (window.AppProperties.IsAndroid) {
                        window.showAlert(localize.errorUnableToAcquireYourLocationEnableLocation);
                    } else {
                        window.showAlert(localize.errorRetriveingPositionTimedOut);
                    } 
                    break;
                default:
                    window.showAlert(localize.errorDetectingPosition);
                    break;
            }
        },

        getLatLong: function (searchValue, callback) {
            var that = this;
            var localize = window.ObjectCollections.Localization.result;
            var storage = new StorageProvider();

            var url = "https://maps.google.com/maps/api/geocode/json?address=" + searchValue + "&sensor=false";

            $.ajax({
                url: that.signUrl(url),
                dataType: 'json',
                type: 'GET',
                success: function (result) {
                    try {
                        if (result.status == "OK") {
                            window.AppProperties.CityOrZipSearch = searchValue;
                            window.AppProperties.CityOrZip = that.getLocationName(result.results[0].address_components) || searchValue;
                            window.AppProperties.ShowTheaterDistance = true;
                            window.AppProperties.CurrentPosition = { Latitude: result.results[0].geometry.location.lat, Longitude: result.results[0].geometry.location.lng };
                            storage.store('CurrentSearchLocation', {
                                'name': window.AppProperties.CityOrZip,
                                'search': window.AppProperties.CityOrZipSearch,
                                'lat': window.AppProperties.CurrentPosition.Latitude,
                                'long': window.AppProperties.CurrentPosition.Longitude
                            });
                            callback(result.results[0].geometry.location.lat, result.results[0].geometry.location.lng);
                        } else {
                            window.hidePageLoadingMessage();
                            window.showAlert(localize.errorSearchLocationNotFound);
                        }
                    } catch (e) {
                        window.hidePageLoadingMessage();
                        window.showAlert(localize.errorGoogleApi);
                    }
                },
                error: function () {
                    window.hidePageLoadingMessage();
                    window.showAlert(localize.errorGoogleApi);
                }
            });
        },

        getLocationName: function (response) {
            var locationName = $.grep(response, function (l) { return l.types.indexOf('locality') > -1; });

            if (locationName.length == 0)
                locationName = $.grep(response, function (l) { return l.types.indexOf('sublocality') > -1; });
            if (locationName.length == 0)
                locationName = $.grep(response, function (l) { return l.types.indexOf('establishment') > -1; });
            if (locationName.length == 0)
                locationName = $.grep(response, function (l) { return l.types.indexOf('administrative_area_level_3') > -1; });

            if (locationName.length == 0)
                return null;

            return locationName[0].short_name;
        },

        signUrl: function (url) {
            if (window.appConfig.GoogleMapsAPIKey) {
                url += '&key=' + window.appConfig.GoogleMapsAPIKey;
            }

            return url;
        }
    };
    return helper;
});