define(['collections/theaterCollection',
        'collections/movieCollection',
        'models/theaterModel',
        'views/theaters',
        'views/theaterDetailsView',
        'helpers/geolocationHelper',
        'sharedhelpers/configurationProvider',
        'sharedhelpers/storageProvider',
        'helpers/theatersHelper'],
    function (theaterCollection, movieCollection, theaterModel, theatersView, theaterDetailsView, geolocationHelper, configurationProvider) {
        window.HomeScreenHelper = function () {

            var theatersLoaded = false;
            var storage = new StorageProvider();
            var theatersHelper = new TheatersHelper();
            var tc = new theaterCollection();
            var localize = window.ObjectCollections.Localization.result;

            function _loyaltyInfoStatus(callback) {
                if (!configurationProvider.useLoyalty()) {
                    callback('LoyaltyDisabled');
                } else {
                    storage.read('LoyaltyCard', function (loyaltyCard) {
                        if (loyaltyCard) {
                            callback('LoyaltyInfoPresent');
                        } else {
                            callback('SignInLoyalty');
                        }
                    });
                }
            }

            function _getAllTheaters() {
                require(['sharedhelpers/giftCardHelper'], function () {

                    var latitude = AppProperties.CurrentPosition.Latitude;
                    var longitude = AppProperties.CurrentPosition.Longitude;

                    function _collectionHandler(result) {
                        if (!result || result.models.length == 0) {
                            theatersLoaded = true;
                            $.event.trigger("theatersLoaded");

                            return;
                        }

                        window.ObjectCollections.TheaterCollection = result;
                        window.ObjectCollections.MovieCollection = result.getMovies();

                        window.ObjectCollections.ComingSoonCollection = new MovieCollection(_.filter(window.ObjectCollections.MovieCollection.models, function (movie) {
                            return movie.get('releaseDate') > DtHelper.getBussinessDate(new Date());
                        }));

                        if (window.giftCardHelper.clearTheaterId()) {
                            window.giftCardHelper.setDefaultOptions(window.ObjectCollections.TheaterCollection.models[0]);
                        }

                        if (window.appConfig.GroupMovies) {
                            window.ObjectCollections.MovieCollection.groupedMovies = window.ObjectCollections.MovieCollection.groupMultiTypeMovies();
                            window.ObjectCollections.ComingSoonCollection.groupedMovies = window.ObjectCollections.ComingSoonCollection.groupMultiTypeMovies();
                        }

                        window.AppProperties.SelectedTheater = null;
                        theatersLoaded = true;
                        $.event.trigger("theatersLoaded");
                    }

                    if (configurationProvider.chainType() == 'single') {
                        var singleTheaterModel = new theaterModel({ id: configurationProvider.theaterId() });
                        singleTheaterModel.getTheater(function (result) {
                            var theatersColl = new theaterCollection();
                            theatersColl.push(result);
                            _collectionHandler(theatersColl);
                        });
                    } else {
                        if (window.giftCardHelper.clearTheaterId() && (window.ObjectCollections.TheaterCollection.models && window.ObjectCollections.TheaterCollection.models.length > 0)) {
                            window.giftCardHelper.setDefaultOptions(window.ObjectCollections.TheaterCollection.models[0]);
                        }

                        window.AppProperties.SelectedTheater = null;
                        theatersLoaded = true;
                        $.event.trigger("theatersLoaded");

                        var abridgedTheatres = new TheaterCollection();
                        abridgedTheatres.abridgedMilesCall(latitude, longitude, function (response) {
                            if (response) {
                                var sortedByDistance = _.sortBy(response.models, function (model) {
                                    return model.get('distance');
                                });

                                window.ObjectCollections.AbridgedTheaterCollection = new theaterCollection(sortedByDistance);
                            }
                        });
                    }
                });
            }

            function _fetchLoyaltyInfo() {
                if (configurationProvider.useLoyalty()) {
                    storage.read('LoyaltyCard', function (loyaltyCard) {
                        if (loyaltyCard == null) {
                            return;
                        }

                        var securityKey;
                        var cardNumber = loyaltyCard.number;
                        var lastName = loyaltyCard.lastName;

                        if (configurationProvider.usePwdForLoyaltyLogin() && loyaltyCard.password) {
                            securityKey = loyaltyCard.password;
                        } else {
                            securityKey = lastName;
                        }

                        require(['models/loyaltyAccountModel'], function (loyaltyAccountModel) {
                            var model = new loyaltyAccountModel({ id: cardNumber, securityKey: securityKey });
                            model.fetch({
                                success: function (value) {
                                    var loyaltyInfo = _buildLoyaltyData(value);
                                    $.event.trigger("loyaltyDataReceived", loyaltyInfo);
                                },
                                error: function () {
                                    var loyaltyInfo = { error: true };
                                    $.event.trigger("loyaltyDataReceived", loyaltyInfo);
                                }
                            });
                        });
                    });
                }
            }

            function _buildLoyaltyData(loyalty) {
                var loyaltyInfo = {};
                if (!loyalty || loyalty.get('errorCode')) {
                    loyaltyInfo.error = true;
                    return loyaltyInfo;
                }

                var loyaltyData = {
                    firstName: loyalty.get('firstName').value,
                    lastName: loyalty.get('lastName').value,
                    loyaltyProgramName: configurationProvider.loyaltyProgramName()
                };

                var plan = _.find(loyalty.get('planStatus'), function (p) { return p['nextReward'] != 'N/A' && p['planType'] == 'points'; });
                if (plan) {
                    var targetAmount = plan['targetAmount'] !== undefined ? plan['targetAmount'].replace(',', '') : '';
                    var currentAmount = plan['currentAmount'] !== undefined ? plan['currentAmount'].replace(',', '') : '';
                    var remeaningAmount = targetAmount - currentAmount;

                    //Show decimal points only if number is not an integer
                    loyaltyData.message = (remeaningAmount % 1 == 0 ? remeaningAmount : remeaningAmount.toFixed(2)) + " " + (remeaningAmount > 1 ? localize.points : localize.point) + " " + localize.untilYour + " " + plan['nextReward'];
                }

                loyaltyInfo.data = loyaltyData;

                return loyaltyInfo;
            }

            function _getHomeScreenData(callback) {
                function _locationCallback() {
                    theatersHelper.getFavoriteTheaters(function (favoriteTheaters) {
                        var counter = 0;

                        tc.theaterIds = configurationProvider.chainType() == 'single' ? [configurationProvider.theaterId()] : favoriteTheaters;
                        tc.latitude = window.AppProperties.CurrentPosition ? window.AppProperties.CurrentPosition.Latitude : 0;
                        tc.longitude = window.AppProperties.CurrentPosition ? window.AppProperties.CurrentPosition.Longitude : 0;
                        var errorDisplayed = false;

                        tc.getHomeTheaters(function(r) {
                                counter++;

                                window.ObjectCollections.TheaterCollection = tc;
                                if (tc.theaterIds.length == 0 || tc.theaterIds.length > 1) {

                                    window.ObjectCollections.MovieCollection = r.getMovies(null, true);
                                    window.ObjectCollections.ComingSoonCollection = new MovieCollection(_.filter(window.ObjectCollections.MovieCollection.models, function(movie) {
                                        return movie.get('releaseDate') > DtHelper.getBussinessDate(new Date());
                                    }));

                                    groupMovies();
                                }

                                if (((tc.theaterIds.length == 0 || tc.theaterIds.length > 1) && counter == 1) || (tc.theaterIds.length == 1 && counter == 2)) {
                                    renderView();
                                }
                            },
                            function () {
                                errorDisplayed = true;
                                errorCallback();
                            });


                        if (tc.theaterIds.length == 1) {
                            var movies = new movieCollection();
                            movies.getMovies(tc.theaterIds[0], Globalize.format(DtHelper.getCurrentBusinessDate(), 'yyyyMMdd'), function (result) {
                                counter++;

                                window.ObjectCollections.MovieCollection = result;
                                window.ObjectCollections.ComingSoonCollection = new MovieCollection(_.filter(window.ObjectCollections.MovieCollection.models, function (movie) {
                                    return movie.get('releaseDate') > DtHelper.getBussinessDate(new Date());
                                }));

                                groupMovies();

                                if (counter == 2) {
                                    renderView();
                                }
                            },
                            function() {
                                if (!errorDisplayed) {
                                    errorCallback();
                                }
                            });
                        }

                        var groupMovies = function () {
                            if (window.appConfig.GroupMovies) {
                                window.ObjectCollections.MovieCollection.groupedMovies = window.ObjectCollections.MovieCollection.groupMultiTypeMovies();
                                window.ObjectCollections.ComingSoonCollection.groupedMovies = window.ObjectCollections.ComingSoonCollection.groupMultiTypeMovies();
                            }
                        };

                        var renderView = function () {
                            callback(_getViewToRender());

                            _fetchLoyaltyInfo();
                            _getAllTheaters();
                        };

                    });

                }

                require(['sharedhelpers/storageProvider'], function () {
                    var storage = new StorageProvider();

                    storage.read('CurrentSearchLocation', function (value) {
                        if (value) {
                            window.AppProperties.CityOrZip = value.name;
                            window.AppProperties.CityOrZipSearch = value.search;
                            window.AppProperties.CurrentPosition = { Latitude: value.lat, Longitude: value.long };
                        }

                        if (configurationProvider.chainType() == 'multi') {
                        geolocationHelper.acquireCurrentPosition(_locationCallback, true);
                        } else {
                            window.AppProperties.CurrentPosition = { Latitude: 0, Longitude: 0 };
                            _locationCallback();
                        }
                    });
                });

                ConfigHelper.initialize(function () {

                });
            }

            function errorCallback(errorCode) {
                if (errorCode) {
                    new ErrorHelper().showAlertByErrorCode(errorCode, function () {
                        window.AppMethods.ChangePage('#home');
                    });
                } else {
                    window.showAlert(window.ObjectCollections.Localization.result.errorUnableConnectToInternet, {
                        postBack: function () {
                            window.AppMethods.ChangePage('#home');
                        }
                    });
                }
            }

            function _getViewToRender() {
                if (tc.theaterIds.length > 1 || tc.theaterIds.length == 0) {
                    window.ObjectCollections.TheaterCollection.theaterViewType = 'list';

                    return new theatersView({
                        collection: window.ObjectCollections.TheaterCollection,
                        favorites: tc.theaterIds,
                        hideBackButton: true,
                        homeScreen: true
                    });
                }
                return new theaterDetailsView({
                    theater: tc.models[0],
                    movies: window.ObjectCollections.MovieCollection,
                    homeScreen: true
                });
            }

            function _theatersLoaded() {
                return theatersLoaded;
            }

            return {
                getHomeScreenData: _getHomeScreenData,
                loyaltyInfoStatus: _loyaltyInfoStatus,
                theatersLoaded: _theatersLoaded,
                fetchLoyaltyInfo: _fetchLoyaltyInfo
            };
        }();
    });
