﻿
define([
  'sharedhelpers/configurationProvider',
  'sharedhelpers/storageProvider'
], function(configurationProvider) {
  window.MenuHelper = function() {
    var localize = window.ObjectCollections.Localization.result;
    var menuIds = {
      home: 'home',
      theaters: 'menuTheaters',
      movies: 'menuMovies',
      comingSoon: 'menuComingSoon',
      concessions: 'menuConcessions',
      purchaseHistory: 'menuPurchaseHistory',
      virtualGiftCards: 'menuVirtualGiftCards',
      loyaltyCard: 'menuLoyaltycard',
      settings: 'menuSettings',
      contactUs: 'menuContactUs'
    };

    function _getMenuIds() {
      return menuIds;
    };

    function _getMenuItems() {
      var menuItems = [{
          'id': menuIds.home,
          'routes': ['home'],
          'class': 'home',
          'text': localize.menuHome,
          'isListed': true
        },
        {
          'id': menuIds.theaters,
          'routes': ['menuTheaters', 'theaters'],
          'class': 'theaters',
          'text': localize.menuTheaters,
          'isListed': true
        },
        {
          'id': menuIds.movies,
          'routes': ['menuMovies', 'movies'],
          'class': 'movies',
          'text': localize.menuMovies,
          'isListed': true
        },
        {
          'id': menuIds.comingSoon,
          'routes': ['menuComingSoon', 'comingSoon'],
          'class': 'comingsoon',
          'text': localize.menuComingSoon,
          'isListed': window.AppConfig.EnableAlternateFeatures
        },
        {
          'id': menuIds.concessions,
          'routes': ['menuConcessions', 'concessions'],
          'class': 'concessions',
          'text': localize.menuConcessions,
          'isListed': false
        },
        {
          'id': menuIds.purchaseHistory,
          'routes': ['menuPurchaseHistory', 'purchaseHistory'],
          'class': 'tickets',
          'text': localize.menuMyTickets,
          'isListed': false
        },
        {
          'id': menuIds.virtualGiftCards,
          'routes': ['menuVirtualGiftCards', 'virtualGiftCards'],
          'class': 'giftcard',
          'text': localize.menuGiftCards,
          'isListed': configurationProvider.useVirtualGiftCard()
        },
        {
          'id': menuIds.loyaltyCard,
          'routes': ['menuLoyaltycard', 'loyaltycard', 'loyaltycardview'],
          'class': 'loyaltycard',
          'text': localize.menuLoyalty,
          'isListed': configurationProvider.useLoyalty()
        },
        {
          'id': menuIds.settings,
          'routes': ['menuSettings', 'settings'],
          'class': 'settings',
          'text': localize.menuSettings,
          'isListed': true
        },
        {
          'id': menuIds.contactUs,
          'routes': ['menuContactUs', 'contactUs'],
          'class': 'contactUs',
          'text': localize.menuContactUs,
          'isListed': true
        }
      ];

      if (configurationProvider.chainType() === 'single') {
        var theaterMenuItem = _.find(menuItems, function(item) {
          return item.id === menuIds.theaters;
        });

        theaterMenuItem.routes.push('theater');
        theaterMenuItem.text = localize.menuTheater;
        // if chainType is "single", few clients didn't wish to have the "Theater" Menu item in the Menu.
        if (window.appConfig.DisplayTheaterInMenuForSingleChain) {
          theaterMenuItem.isListed = true;
        } else {
          theaterMenuItem.isListed = false;
        }
      }

      if (configurationProvider.chainType() === 'single' && configurationProvider.useConssesion()) {
        var concessionsMenuItem = _.find(menuItems, function(item) {
          return item.id === menuIds.concessions;
        });

        concessionsMenuItem.isListed = true;
      }

      var storage = new StorageProvider();
      storage.read('PurchaseHistory', function(orders) {
        if (orders && orders.length > 0) {
          var purchaseHistoryMenuItem = _.find(menuItems, function(item) {
            return item.id === menuIds.purchaseHistory;
          });

          purchaseHistoryMenuItem.isListed = true;
        }
      });

      return menuItems;
    };

    function _updateMenu() {
      // Sometimes the chain info doesn't render before the menu
      // so the menu needs to be updated

      if (configurationProvider.useVirtualGiftCard())
        _showMenuItem(menuIds.virtualGiftCards);
      else
        _hideMenuItem(menuIds.virtualGiftCards);

      if (configurationProvider.useLoyalty())
        _showMenuItem(menuIds.loyaltyCard);
      else
        _hideMenuItem(menuIds.loyaltyCard);
    };

    function _showMenuItem(menuItemId) {
      $('#menu').find('#' + menuItemId).show();
    };

    function _hideMenuItem(menuItemId) {
      $('#menu').find('#' + menuItemId).hide();
    };

    function _render() {
      var menuItems = _getMenuItems();
      var menuContainer = $('<div>').addClass('menu-container');
      var menu = $('<div>').attr('id', 'menu').addClass('ncr-menu');
      var content = $('<ul>');

      if (window.AppProperties.IsIOS && (window.AppProperties.iOSVersion.indexOf('7.') === 0 || window.AppProperties.iOSVersion.indexOf('8.') === 0) || window.AppProperties.iOSVersion.indexOf('9.') === 0 || window.AppProperties.iOSVersion.indexOf('10.') === 0) {
        menu.addClass('status-bar-fix');
      }

      _.each(menuItems, function(item) {
        var menuItem = $('<li>').attr('id', item.id).addClass(item.class);
        menuItem.append($('<div>').addClass('menu-item-icon'));
        menuItem.append($('<span>').addClass('menu-item-title').html(item.text));

        if (!item.isListed) {
          menuItem.hide();
        }

        content.append(menuItem);
      });

      menu.html(content);
      menuContainer.html(menu);
      $('body').prepend(menuContainer);

      _initialize();
    }

    function _initialize() {
      $(document).on('tap', '#btnMenu', function(e) {
        e.preventDefault();
        e.stopPropagation();

        _showMenu();
        return false;
      });

      // Menu behaviour
      $('#menu li').tap(function(e) {
        if (isJqmGhostClick(e)) return;

        e.preventDefault();
        e.stopPropagation();

        //We need to close the menu if we tap on the menu item that is already active
        if ($(e.currentTarget).hasClass('active')) {
          _hideMenu();
          return;
        };

        var route = '#' + e.currentTarget.id;

        _hideMenu(route);
      });

      var menuItems = _getMenuItems();
      var setSelectedMenuItem = function(menu) {
        var selectedItem = _.find(menuItems, function(item) {
          var activeRoute = Backbone.history.fragment.split('/')[0];

          if (item.routes && activeRoute) {
            return item.routes.indexOf(activeRoute) >= 0;
          }

          return false;
        });

        if (selectedItem) {
          menu.find('ul li#' + selectedItem.id).addClass('active');
          menu.find('ul li').not('#' + selectedItem.id).removeClass('active');
        } else {
          menu.find('ul li').removeClass('active');
        }
      };

      //Backup option in case we didn't manage to catch menu transition end
      $(document).on('pagebeforecreate.ncr', function() {
        $(document).off('touchmove.ncr touchstart.ncr');

        var menu = $('#menu');
        menu.removeClass('slide-in');

        setSelectedMenuItem(menu);

        $('body').pagecontainer('getActivePage').off('tap.ncr swiperight.ncr');
      });

      $(document).on('pagebeforehide.ncr', function() {
        $('body').pagecontainer('getActivePage').off('tap.ncr swiperight.ncr');
      });

      $('body').on('chainInfoUpdated', _updateMenu);
    }

    function _openSlideMenu(menu) {
      menu.addClass('slide-in');

      $('body').pagecontainer('getActivePage').addClass('slide-menu-visible');

      menu.one('transitionend webkitTransitionEnd', function() {

        //Prevent Scrolling when menu is active
        $(document).on('touchmove.ncr touchstart.ncr', function(event) {
          event.preventDefault();
        });

        //Register tap event on page to dismiss slide menu
        $('body').pagecontainer('getActivePage').one('tap.ncr swiperight.ncr', function(event) {
          event.preventDefault();

          _hideMenu();
        });
      });
    }

    function _showMenu() {
      var menu = $('#menu');

      if (menu.hasClass('slide-in')) {
        return false;
      }

      var activePage = $('body').pagecontainer('getActivePage');

      //Close the active popup in case it is activated
      activePage.find('.ui-popup').popup('close');

      //Select Seats fix if the key map was active and the menu was activated
      activePage.find('#selectSeatsButton').css({
        'pointer-events': ''
      });
      activePage.find('.pullDownStatusHolder').css({
        'pointer-events': ''
      });

      //Close the native keyboard in case it is activated
      activePage.find('input:focus, textarea:focus, select:focus').blur();

      //Header should be visible when the menu is active
      window.scrollTo(0, 0);

      _openSlideMenu(menu);

      return false;
    }

    function _closeSlideMenu(menu, route) {
      menu.removeClass('slide-in');

      $('body').pagecontainer('getActivePage').off('tap.ncr swiperight.ncr');

      menu.one('transitionend webkitTransitionEnd', function() {
        $(document).off('touchmove.ncr touchstart.ncr');
        $('body').pagecontainer('getActivePage').removeClass('slide-menu-visible');


        if (route) {
          _changeRoute(route);
        }
      });
    }

    function _hideMenu(route) {
      var menu = $('#menu');
      if (!menu.hasClass('slide-in')) {
        return;
      }

      _closeSlideMenu(menu, route);
    }

    function _changeRoute(route) {
      var intervalObject = window.ObjectModels.OrderModel != null ? window.ObjectModels.OrderModel.get('interval') : null;

      var clearOrderModel = function() {
        var orderModel = window.ObjectModels.OrderModel;
        if (orderModel) {
          orderModel.cancelSale(function() {
            window.ObjectModels.OrderModel = null;
          });
        }
      };

      var onConfirm = function(confirm) {
        //OK = 1; Cancel = 2
        if (confirm === 1) {
          window.ObjectModels.OrderModel.set('orderTimeout', null);
          window.showPageLoadingMessage();

          clearOrderModel();

          _.defer(function() {
            window.AppMethods.ChangePage(route, false, true);
          });

          return false;
        }

        return false;
      };

      if (intervalObject) {
        if (window.isMobileDevice()) {
          navigator.notification.confirm(localize.messagePurchaseWillBeCancel, onConfirm, localize.alertTitle, localize.ok + ', ' + localize.cancel);
        } else if (confirm(localize.messagePurchaseWillBeCancel)) {
          onConfirm(1);
        }

        return false;
      }

      _.defer(function() {
        window.isPickup = false;

        clearOrderModel();

        window.AppMethods.ChangePage(route, false, true);
      });

      window.showPageLoadingMessage();
      return false;
    }

    return {
      render: _render,
      initialize: _initialize,
      getMenuIds: _getMenuIds,
      showMenuItem: _showMenuItem,
      hideMenuItem: _hideMenuItem
    };
  }();
});
