﻿define([
'helpers/theatersHelper',
'sharedhelpers/utilities',
'sharedhelpers/storageProvider'
], function () {
    window.PassHelper = (function () {
        var storage = new StorageProvider();
        var storageKeys = {
            purchaseHistory: 'PurchaseHistory',
            loyaltyPassbook: 'loyaltyPassbook',
            paymentMethod: 'PaymentMethod'
        };

        function _createTicketPassObject(order) {
            //Format Showtime
            var sdate = DtHelper.convertDate(order.performance.showTime);
            var time = Globalize.format(sdate, 't', order.theaterCulture);
            var ddate = Globalize.format(sdate, 'M', order.theaterCulture);

            //Get seat names
            var allTickets = {};
            var seatNames = "";

            _.each(order.selectedTickets, function (ticket) {
                if (ticket.name != 'HOLD') {
                    if (allTickets[ticket.name]) {
                        allTickets[ticket.name]++;
                    } else {
                        allTickets[ticket.name] = 1;
                    }

                    seatNames += ticket.seatName != 'GA' ? ticket.seatName + ", " : "";
                }
            });

            allTickets = _.map(allTickets, function (lenght, ticketName) {
                return lenght + ' ' + ticketName;
            }).join(', ');

            //Format Theater Details
            var thDetails = order.theaterDetails;
            var zip = thDetails.addressZip;
            var theaterDetails = thDetails ?
                thDetails.addressLine1 + '\n' + (thDetails.addressLine2 ? thDetails.addressLine2 + '\n' : '') + thDetails.addressCity + ', ' + (thDetails.addressState ? thDetails.addressState + ' ' : '') + (zip[zip.length - 1] == '-' ? zip.substring(0, [zip.length - 1]) : zip) + '\n' + thDetails.phone
                : '';

            var passObject = {
                theaterName: order.theaterName,
                theaterDetails: theaterDetails,
                latitude: thDetails ? thDetails.latitude : '',
                longitude: thDetails ? thDetails.longitude : '',
                relevantText: order.theaterName + ' ' + window.ObjectCollections.Localization.result.theatersNearby,
                relevantDate: order.performance.showTime,
                featureName: order.movieName,
                runtime: order.movieLength || 0,
                showtime: time + ' ' + dateFormat(sdate, 'ddd') + ', ' + ddate,
                auditorium: order.performance.auditorium,
                seats: seatNames.slice(0, -2) || window.ObjectCollections.Localization.result.na.toUpperCase(),
                tickets: allTickets,
                confirmationNumber: order.confirmationNumber,
                qrConfirmationNumber: window.appConfig.UseSimpleQrCode ? order.confirmationNumber : order.confirmationNumber + '^' + order.patronsName,
                groupingIdentifier: order.theaterId + order.performance.number
            };

            return passObject;
        }

        function _generateTicketFormData(order, callback) {
            var data = new FormData();
            var passObject = _createTicketPassObject(order);

            //Append JSON values
            data.append('values', new Blob([JSON.stringify(passObject)], { type: "application/json" }));

            //Append image/s
            var img = order.movieImage ? (order.movieImage.small || order.movieImage.medium || order.movieImage.large || window.appConfig.DefaultMoviePosterImage) : window.appConfig.DefaultMoviePosterImage;
            var xhr = new XMLHttpRequest();
            xhr.open('GET', img, true);
            xhr.responseType = 'arraybuffer';
            xhr.onload = function () {
                if (this.status == 200) {
                    data.append('thumbnail', new Blob([this.response]), { type: "application/octet-stream" });
                }
                callback(data);
            };
            xhr.send();
        }

        function _createTicketPass(order, orders) {
            require(['models/passModel'], function () {

                var callback = function (formData) {
                    var passModel = new PassModel({ id: window.appConfig.TicketPassTemplateId, formData: formData });

                    passModel.generatePassWithImages(function (result) {
                        order.passURL = result.get('url');
                        order.passSerial = result.get('serialNumber');

                        storage.store(storageKeys.purchaseHistory, orders);

                        _openPassUrl(order.passURL);
                    }, function(err){
                        hidePageLoadingMessage();
                        window.showAlert(window.ObjectCollections.Localization.result.errorPassbook);
                    });
                };

                _generateTicketFormData(order, callback);
            });
        }

        function _getTicketPassUrl(orders, orderId) {
            var order = _.find(orders, function (o) { return o.confirmationNumber == orderId; });

            if (order.passURL) {
                _openPassUrl(order.passURL);
            } else {
                _createTicketPass(order, orders);
            }
        }
        function _createLoyaltyCardPassObject(loyaltyCard, callback) {
            var theatersHelper = new TheatersHelper();
            theatersHelper.getFavoriteTheaters(function (favorites) {
                var coords = [];
                var theater;
                var theaterDetails = '';
                var localize = window.ObjectCollections.Localization.result;
                var theaterCollection = window.ObjectCollections.TheaterCollection;

                if (favorites && favorites.length > 0) {
                    _.each(favorites, function (favoriteId) {
                        theater = _.find(theaterCollection.models, function (m) { return m.id == favoriteId; });
                        var tDetails = _getTheaterDetails(theater);
                        theaterDetails = theaterDetails.concat(tDetails + '\n\n');
                        coords.push(_getCoordinates(theater));
                    });
                } else {
                    if (theaterCollection && theaterCollection.models && theaterCollection.models.length > 0) {
                        theater = theaterCollection.models[0];
                        theaterDetails = _getTheaterDetails(theater);
                        coords.push(_getCoordinates(theater));
                    }
                }
                var passObject = {
                    ownerName: (loyaltyCard.get('firstName').value || loyaltyCard.get('firstName')) + ' ' + (loyaltyCard.get('lastName').value || loyaltyCard.get('lastName')),
                    theaterDetails: theaterDetails ? theaterDetails : localize.passbookNoTheaters,
                    loyaltyCardId: loyaltyCard.id || loyaltyCard.get('number'),
                    locations: coords
                };

                if (!loyaltyCard.get('multitype')) {
                    callback(passObject);
                }

                storage.read(storageKeys.paymentMethod, function (paymentMethods) {
                    var giftCard = _.find(paymentMethods, function (card) {
                        return new Utilities().decrypt(card.CardNumber) == passObject.loyaltyCardId;
                    });

                    passObject.balance = giftCard && giftCard.Balance ? giftCard.Balance : localize.na;

                    callback(passObject);
                });
            });
        }
        function _getTheaterDetails(theater) {
            var zip = theater.get('addressZip');
            return theater.get('name') + '\n' + theater.get('addressLine1') + '\n' + (theater.get('addressLine2') ? theater.get('addressLine2') + '\n' : '') + theater.get('addressCity') + ', ' + (theater.get('addressState') ? theater.get('addressState') + ' ' : '') + (zip[zip.length - 1] == '-' ? zip.substring(0, [zip.length - 1]) : zip) + '\n' + theater.get('phone');
        }
        function _getCoordinates(theater) {
            return { latitude: theater.get('latitude'), longitude: theater.get('longitude'), relevantText: theater.get('name') + ' ' + window.ObjectCollections.Localization.result.theatersNearby };
        }
        function _createLoyaltyCardPass(loyaltyCard) {
            require(['models/passModel'], function () {

                var callback = function (passObject) {
                    var passModel = new PassModel(
                    {
                        id: loyaltyCard.get('multitype') ? window.appConfig.DualCardPassTemplateId : window.appConfig.LoyaltyCardPassTemplateId,
                        passObject: passObject

                    });
                    passModel.generatePass(function (result) {
                        var pass = {
                            id: loyaltyCard.id,
                            url: result.get('url'),
                            serialNumber: result.get('serialNumber'),
                            typeIdentifier: result.get('passTypeIdentifier')
                        };
                        storage.read(storageKeys.loyaltyPassbook, function (passes) {
                            if (passes) {
                                passes.push(pass);
                            } else {
                                passes = [pass];
                            }
                            storage.store(storageKeys.loyaltyPassbook, passes);
                        });
                        _openPassUrl(pass.url);
                    });
                };
                _createLoyaltyCardPassObject(loyaltyCard, callback);
            });
        }
        function _getLoyaltyCardPassUrl(loyaltyCard, preventPassCreation) {
            require(['models/passModel'], function () {

                storage.read(storageKeys.loyaltyPassbook, function (savedPasses) {
                    var savedPass = _.find(savedPasses, function (val) {
                        return val.id == (loyaltyCard.id || loyaltyCard.get('number'));
                    });
                    if (!savedPass && preventPassCreation) {
                        return;
                    }
                    if (!savedPass) {
                        _createLoyaltyCardPass(loyaltyCard);
                    } else {
                        var callback = function (passObject) {
                            var passModel = new PassModel({
                                id: loyaltyCard.get('multitype') ? window.appConfig.DualCardPassTemplateId : window.appConfig.LoyaltyCardPassTemplateId,
                                passObject: passObject,
                                type: savedPass.typeIdentifier,
                                serial: savedPass.serialNumber
                            });

                            passModel.updatePass(function () {
                                if (!preventPassCreation) {
                                    _openPassUrl(savedPass.url);
                                }
                            });
                        };
                        _createLoyaltyCardPassObject(loyaltyCard, callback);
                    }
                });
            });
        }
        function _createVirtualGiftCardPass(virtualGiftCard) {
            require(['models/passModel'], function () {
                var callback = function (passObject) {
                    var passModel = new PassModel({ id: window.appConfig.VirtualGiftCardPassTemplateId, passObject: passObject });
                    passModel.generatePass(function (result) {

                        storage.read(storageKeys.paymentMethod, function (paymentMethods) {
                            _.each(paymentMethods, function (paymentMethod) {
                                if (paymentMethod.Id == virtualGiftCard.Id) {

                                    paymentMethod.url = result.get('url');
                                    paymentMethod.serialNumber = result.get('serialNumber');
                                    paymentMethod.passTypeIdentifier = result.get('passTypeIdentifier');
                                }
                            });
                            storage.store(storageKeys.paymentMethod, paymentMethods);
                        });
                        _openPassUrl(result.get('url'));
                    });
                };
                _createVirtualGiftCardPassObject(virtualGiftCard, callback);
            });
        }
        function _createVirtualGiftCardPassObject(virtualGiftCard, callback) {
            var theatersHelper = new TheatersHelper();
            theatersHelper.getFavoriteTheaters(function (favorites) {
                var theater;
                var theaterDetails = '';
                var theaterCollection = window.ObjectCollections.TheaterCollection;
                this.localize = window.ObjectCollections.Localization.result;

                if (favorites && favorites.length > 0) {
                    _.each(favorites, function (favoriteId) {
                        theater = _.find(theaterCollection.models, function (m) { return m.id == favoriteId; });
                        var tDetails = _getTheaterDetails(theater);
                        theaterDetails = theaterDetails.concat(tDetails + '\n\n');
                    });
                } else if (theaterCollection && theaterCollection.models && theaterCollection.models.length > 0 && window.AppConfig.ChainType == 'single') {
                    theater = window.ObjectCollections.TheaterCollection.models[0];
                    theaterDetails = _getTheaterDetails(theater);
                } else {
                    theaterDetails = this.localize.passbookNoTheaters;
                }

                var passObject = {
                    virtualGiftCardNumber: new Utilities().decrypt(virtualGiftCard.CardNumber) || virtualGiftCard.CardNumber,
                    theaterDetails: theaterDetails,
                    virtualGiftCardBalance: virtualGiftCard.Balance ? virtualGiftCard.Balance : this.localize.na,
                    virtualGiftCardName: virtualGiftCard.Name
                };

                callback(passObject);
            });
        }
        function _getVirtualGiftCardPassUrl(virtualGiftCard, preventPassCreation) {
            require(['models/passModel'], function () {
                if (!virtualGiftCard.url && preventPassCreation) {
                    return;
                }

                if (!virtualGiftCard.url) {
                    _createVirtualGiftCardPass(virtualGiftCard);
                } else {
                    var callback = function (passObject) {
                        var passModel = new PassModel({
                            id: window.appConfig.VirtualGiftCardPassTemplateId,
                            passObject: passObject,
                            type: virtualGiftCard.passTypeIdentifier,
                            serial: virtualGiftCard.serialNumber
                        });
                        passModel.updatePass(function () {
                            if (!preventPassCreation) {
                                _openPassUrl(virtualGiftCard.url);
                            }
                        });
                    };
                    _createVirtualGiftCardPassObject(virtualGiftCard, callback);
                }
            });
        }
        function _openPassUrl(url) {
            Passbook.downloadPass(url,
                function () {
                    window.hidePageLoadingMessage();
                },
                function () {
                    window.hidePageLoadingMessage();
                    new ErrorHelper().showAlertByErrorCode();
                });
        }
        return {
            getTicketPassURL: _getTicketPassUrl,
            getLoyaltyCardPassURL: _getLoyaltyCardPassUrl,
            getVirtualGiftCardPassURL: _getVirtualGiftCardPassUrl
        };
    });
})