﻿define([
    'sharedhelpers/cardValidationHelper',
	'sharedhelpers/storageProvider',
    'sharedhelpers/utilities',
    'sharedjs/RSAEncryption/aes'
], function () {
    window.PaymentMethodsHelper = (function () {
        var storage = new StorageProvider();
        var paymentMethodKey = 'PaymentMethod';
        var paymentMethodKeyPass = 'PaymentMethodPassword';
        var paymentSettings = 'paymentSettings';
        var utilities = new Utilities();
        var cardValidation = new CardValidationHelper();
        var localize = window.ObjectCollections.Localization.result;

        function _insert(options, callback) {
            var paymentMethods = [];

            //check if password exists in storage
            storage.read(paymentMethodKeyPass, function (value) {
                if (!value && (options.checkPassword === undefined || options.checkPassword !== false)) {
                    callback({ success: false, passwordExists: false });
                }
                else {
                    var paymentMethod;
                    if (options.paymentMethod.Type == 'GiftCard' || options.paymentMethod.Type == 'VirtualGiftCard') {
                        paymentMethod = {
                            Id: options.paymentMethod.Id || _getGuid(),
                            Name: options.paymentMethod.Name,
                            CardNumber: utilities.encrypt(options.paymentMethod.CardNumber),
                            Balance: options.paymentMethod.Balance,
                            LastDigits: options.paymentMethod.CardNumber.substring(options.paymentMethod.CardNumber.length - 4, options.paymentMethod.CardNumber),
                            Type: options.paymentMethod.Type
                        };

                        if (options.paymentMethod.Type == 'VirtualGiftCard') {
                            paymentMethod.BackgroundFront = options.paymentMethod.BackgroundFront;
                        }
                    } else {
                        paymentMethod = {
                            Id: options.paymentMethod.Id || _getGuid(),
                            Name: options.paymentMethod.Name,
                            FirstName: options.paymentMethod.FirstName,
                            LastName: options.paymentMethod.LastName,
                            CardNumber: utilities.encrypt(cardValidation.normalize(options.paymentMethod.CardNumber)),
                            ExpMonth: options.paymentMethod.ExpMonth,
                            ExpYear: options.paymentMethod.ExpYear,
                            ZipCode: options.paymentMethod.ZipCode,
                            cVV2: options.paymentMethod.cVV2,
                            LastDigits: options.paymentMethod.CardNumber.substring(options.paymentMethod.CardNumber.length - 4, options.paymentMethod.CardNumber.length),
                            Type: options.paymentMethod.Type
                        };
                    }

                    storage.read(paymentMethodKey, function (p) {
                        paymentMethods = p || [];
                        var update = false;
                        var card = _.find(paymentMethods, function (item) { return utilities.decrypt(item.CardNumber) === cardValidation.normalize(options.paymentMethod.CardNumber); });

                        if (card && !options.update) {
                            callback({ success: false, cardExists: true });
                            return false;
                        }

                        if (card && options.update) {
                            card.Name = paymentMethod.Name;
                            card.UpdateBalance = true;
                            update = true;
                        } else {
                            paymentMethods.push(paymentMethod);
                        }

                        storage.store(paymentMethodKey, paymentMethods);

                        callback({ success: true, passwordExists: true, update: update, id: card ? card.Id : paymentMethod.Id });
                    });
                }
            });
        }

        function _remove(id, callback) {
            storage.read(paymentMethodKey, function (value) {
                if (value && value.length == 1) {
                    storage.remove(paymentMethodKeyPass);
                }

                storage.store(paymentMethodKey, $.grep(value, function (e) {
                    return e.Id != id;
                }));

                storage.read(paymentSettings, function (val) {
                    if (val && val.selectedCard == id) {
                        delete val.selectedCard;

                        storage.store(paymentSettings, val);
                    }

                    callback({ success: true });
                });
            });
        }

        function _get(id, callback) {
            var paymentMethods = [];

            storage.read(paymentMethodKey, function (value) {
                if (value) {
                    $.each(value, function (i, payment) {
                        payment.CardNumber = utilities.decrypt(payment.CardNumber);
                        paymentMethods.push(payment);
                    });

                    callback(!id ? paymentMethods : $.grep(paymentMethods, function (e) {
                        return e.Id == id;
                    }));
                } else {
                    callback(paymentMethods);
                }
            });
        }

        function _savePassword(password, callback) {
            var passwordExists = false;

            storage.read(paymentMethodKeyPass, function (value) {
                if (value) {
                    var pass = utilities.decrypt(value);
                    passwordExists = pass.length !== 0;

                    if (passwordExists) {
                        callback({ success: false, errorMessage: localize.errorPasswordAlreadyExists });
                    }
                }
                else {
                    storage.store(paymentMethodKeyPass, utilities.encrypt(password));
                    callback({ success: true });
                }
            });
        }

        function _saveAppPassword(password, callback) {
            storage.store(paymentMethodKeyPass, utilities.encrypt(password));
            callback({ success: true });
        }

        function _validatePass(enteredPass, existingPass) {
            return utilities.decrypt(existingPass) == enteredPass;
        }

        function _getGuid() {
            return (_rNumber() + _rNumber() + "-" + _rNumber() + "-" + _rNumber() + "-" + _rNumber() + "-" + _rNumber() + _rNumber() + _rNumber());
        }

        function _rNumber() {
            return (((1 + Math.random()) * 0x10000) | 0).toString(16).substring(1);
        }

        return {
            insert: _insert,
            savePassword: _savePassword,
            remove: _remove,
            get: _get,
            saveAppPassword: _saveAppPassword,
            validatePass: _validatePass,
            getGuid: _getGuid
        };
    });
});
