﻿define([
    'sharedhelpers/errorHelper'
], function () {
    window.SaleHelper = function () {
        function _lockTickets(callback) {
            var orderModel = window.ObjectModels.OrderModel;
            var localize = window.ObjectCollections.Localization.result;

            orderModel.get('tickets').lockTickets(function (success, errorResponse) {
                if (success) {
                    callback(true);
                    return;
                }

                if (errorResponse && errorResponse.error_sub_code) {
                    if (errorResponse.error_sub_code != new ErrorHelper().errorCodes.NotAValidTicketType) {
                        new ErrorHelper().showAlertByErrorCode(errorResponse.error_sub_code);
                        return;
                    }

                    var theater = orderModel.get('theater').get('id');
                    var performance = orderModel.get('performance').get('number');
                    var movie = orderModel.get('movie').get('code');
                    window.AppProperties.PageRefresh = true;

                    window.AppMethods.ChangePage('#tickets/' + performance + '/' + movie + '/' + theater, true);
                    return;
                }

                //Handle Subscription Error responses
                var isArray = _.isArray(errorResponse);

                if (isArray) {
                    var cardNumbers = [];

                    _.each(errorResponse, function (error) {
                        if (error.card_num) {
                            cardNumbers.push({
                                cardNumber: error.card_num,
                                message: error.message
                            });
                        }
                    });

                    if (cardNumbers.length > 0) {

                        var showCtmAlert = function () {
                            orderModel.set('invalidCtmCards', cardNumbers);

                            window.showAlert(localize.ctmCodeError, {
                                postBack: function () {
                                    if (callback)
                                        callback(false);
                                }
                            });
                        }


                        var showSubscriptionAlert = function () {
                            orderModel.set('invalidSubscriptions', cardNumbers);

                            window.showAlert(localize.subscriptionCardError, {
                                postBack: function () {
                                    if (_hasCtmCodeError(cardNumbers)) {
                                        showCtmAlert();
                                    } else if (callback) {
                                        callback(false);
                                    }
                                }
                            });
                        }

                        if (_hasSubscriptionError()) {
                            showSubscriptionAlert();
                        } else if (_hasCtmCodeError()) {
                            showCtmAlert();
                        }
                    }
                }
            });
        }

        function _hasSubscriptionError() {
            var showSubscriptionError;
            var orderModel = window.ObjectModels.OrderModel;

            var subscriptionTickets = _.filter(orderModel.get('tickets').models, function (ticket) {
                return ticket.get('ticketValidationRuleId') != 99 && ticket.get('quantity') > 0 && (ticket.get('discountFlag') || ticket.get('validate'));
            });

            if (subscriptionTickets.length > 0) {
                _.each(subscriptionTickets, function (ticket) {
                    if (ticket.get('subscriptionCardNumbers')) {
                        showSubscriptionError = true;
                    }
                });
            }
            return showSubscriptionError;
        }

        function _hasCtmCodeError() {
            var showCtmError;
            var orderModel = window.ObjectModels.OrderModel;

            var ctmTickets = _.filter(orderModel.get('tickets').models, function (ticket) {
                return ticket.get('ticketValidationRuleId') != 99 && ticket.get('quantity') > 0 && ticket.get('pricedTicketRequired');
            });

            if (ctmTickets.length > 0) {
                _.each(ctmTickets, function (ctmTicket) {
                    if (ctmTicket.get('ctmCards')) {
                        showCtmError = true;
                    }
                });
            }
            return showCtmError;
        }

        return {
            lockTickets: _lockTickets,
            hasSubscriptionError: _hasSubscriptionError,
            hasCtmCodeError: _hasCtmCodeError
        };
    }();
});