﻿define(['sharedhelpers/storageProvider'], function () {
    window.storageHelper = (function () {
        function _isStorageCapacityFull() {
            var maxStorageLimit = /iPad/.test(navigator.platform) ? 0.25 : 1; //represents the limit in megabytes

            return (((JSON.stringify(window.localStorage).length * 2) / 1024) / 1024).toFixed(5) >= maxStorageLimit;
        }

        function _deletePurchaseHistory(callback, orderId) {
            var passedOrders, i = 0, that = this;

            passedOrders = _.filter(window.AppProperties.HistoryOrders, function (item) {
                var currentBusinessDate = Globalize.format(DtHelper.getCurrentBusinessDate(), 'yyyyMMdd');
                var orderBusinessDate = item.performance ? item.performance.businessDate : DtHelper.getBussinessDate(item.purchasedDateTime);
                return orderBusinessDate < currentBusinessDate;
            });

            if (passedOrders.length == 0) {
                if (callback) {
                    callback();
                }
                return false;
            }

            passedOrders = _.sortBy(passedOrders, function (order) {
                var date = order.performance ? order.performance.showTime : order.purchasedDateTime;
                return date;
            });

            var storage = new StorageProvider();

            (function _deleteEntry() {
                var id = orderId || passedOrders[i].confirmationNumber;

                window.AppProperties.HistoryOrders = _.reject(window.AppProperties.HistoryOrders, function (item) { return item.confirmationNumber == id; });
                storage.store('PurchaseHistory', window.AppProperties.HistoryOrders);

                if (that.isStorageCapacityFull() && !orderId) {
                    i += 1;
                    if (passedOrders.length > i) {
                        _deleteEntry();
                    }
                }
            })();

            if (callback) {
                callback();
            }
        }

        function _deletePurchaseHistoryDialog(callback) {
            var localize = window.ObjectCollections.Localization.result;
            var that = this;
            var storage = new StorageProvider();

            navigator.notification.confirm(localize.deletePurchaseHistoryMessage,
                   function (buttonIndex) {
                       var clear = buttonIndex == 1;
                       storage.store('HistoryCleanup', clear);
                       if (clear) {
                           that.deletePurchaseHistory(callback);
                       } else {
                           callback();
                       }
                   },
                   localize.deletePurchaseHistoryTitle);
        }

        function _updatePurchaseHistory(order) {
            var storage = new StorageProvider();
            storage.read('PurchaseHistory', function (orders) {
                var i = -1;
                _.each(orders, function(o, index) {
                    if (o.confirmationNumber == order.confirmationNumber) {
                        i = index;
                    }
                });
                if (i > -1) {
                    orders[i] = order;
                }
                storage.store('PurchaseHistory', orders);
            });
        }
        return {
            isStorageCapacityFull: _isStorageCapacityFull,
            deletePurchaseHistory: _deletePurchaseHistory,
            deletePurchaseHistoryDialog: _deletePurchaseHistoryDialog,
            updatePurchaseHistory: _updatePurchaseHistory
        };
    })();
});
