﻿define([
    'helpers/uiHelper',
    'sharedhelpers/storageProvider'
], function () {
    window.UIHelper = {
        initialize: function () {
            this.pullDownStatus = false;
        },

        iOSHeaderFix: function (that) {
            if (window.AppProperties.IsIOS) {
                if (window.AppProperties.iOSVersion.indexOf('6.') === 0) {
                    var content = that.$('div[role="main"]');
                    var header = that.$('.fixed-header');
                    var pullDown = that.$('.pullDownStatusHolder');
                    var orderContainer = that.$('.order-status-container');

                    that.$('input, select').bind('focus', function (e) {
                        e.preventDefault();
                        e.stopPropagation();
                        content.css({ 'margin-top': '0' });
                        header.removeClass('fixed-header');
                        if (pullDown.length > 0) {
                            var position = new WebKitCSSMatrix(window.getComputedStyle(pullDown[0], null).webkitTransform).m42 + 'px';
                            orderContainer.addClass('iOSOrderStatus').css({ '-webkit-transform': "translate3d(0, " + position + ", 0)", 'height': pullDown.height() + 'px' });
                            pullDown.removeClass('pullDownStatusHolder pullDown').addClass('iOSPullDownFix').css({ '-webkit-transform': '', 'height': '' });
                        }
                    });

                    that.$('input, select').bind('blur', function (e) {
                        e.preventDefault();
                        e.stopPropagation();

                        if (e.originalEvent) {
                            content.css({ 'margin-top': '40px' });
                            header.addClass('fixed-header');
                            if (pullDown.length > 0) {
                                var position = new WebKitCSSMatrix(window.getComputedStyle(orderContainer[0], null).webkitTransform).m42 + 'px';
                                pullDown.removeClass('iOSPullDownFix').addClass('pullDownStatusHolder pullDown').css({ "-webkit-transform": "translate3d(0, " + position + ", 0)", 'height': orderContainer.height() + 'px' });
                                orderContainer.removeClass('iOSOrderStatus').css({ '-webkit-transform': '', 'height': '' });
                            }
                        } else {
                            that.$('input, select').unbind('focus');
                            that.$('input, select').unbind('blur');
                        }
                    });
                } else {
                    $(document)
                    .on('focus', 'input, select, textarea', function () {
                        if (!that.headerRepositioned) {
                            $('#header').css('position', 'absolute');
                            if ($('.pullDownStatusHolder').length > 0) {
                                $('.pullDownStatusHolder').css('position', 'absolute');
                            }
                            that.headerRepositioned = true;
                        }
                    })
                    .on('blur', 'input, select, textarea', function () {
                        if (that.headerRepositioned) {
                            $('#header').css('position', 'fixed');
                            if ($('.pullDownStatusHolder').length > 0) {
                                $('.pullDownStatusHolder').css('position', 'fixed');
                            }
                            setTimeout(function () {
                                window.scrollTo($(window).scrollLeft(), $(window).scrollTop());
                            }, 20);
                            that.headerRepositioned = false;
                        }
                    });
                }
            }
        },

        notificationBar: function (htmlText, callback) {
            $('.notification-container').show();
            var notificationContainer = $('.notification-container').find('.notification-top-bar');

            _.defer(function () {
                notificationContainer.html(htmlText).css({
                    'opacity': '100',
                    'text-align': 'center'
                });

                if (this.noteTimeout) clearTimeout(this.noteTimeout);
                this.noteTimeout = setTimeout(function () {
                    notificationContainer.css({ 'opacity': '0' });
                    _.delay(function () { $('.notification-container').hide(); }, 100);

                    if (callback)
                        callback();
                }, 3000);
            });
        },
        setPullDownStyle: function () {
            if (!this.pullDownStatus) {
                var height = window.orgScreenHeight - 50;
                var originalPosition = window.orgScreenHeight - 80;
                var positionStyle = '-webkit-transform: translate3d(0, -' + originalPosition + 'px, 0);';

                var style = document.createElement('style');
                style.type = 'text/css';
                style.innerHTML = '.pullDownStatusHolder { height: ' + height + 'px;' + positionStyle + '}';

                document.getElementsByTagName('head')[0].appendChild(style);

                this.pullDownStatus = true;
            }
        },

        showLargePoster: function (obj, imgSrc, callback) {
            var iPad = navigator.platform === "iPad";
            var top = ($(window).height() - (iPad ? 600 : 375)) / 2;
            var left = ($(window).width() - (iPad ? 400 : 250)) / 2;

            obj.find('.contentContainer').css({ 'pointer-events': 'none', 'opacity': '0.5' });

            var htmlContent = '<img class="largePoster" src="' + imgSrc + '" style="top:' + top + 'px;  left:' + left + 'px;"/>';
            obj.append(htmlContent);

            if (iPad) {
                obj.find('.largePoster').css({
                    'width': '400px', 'height': '600px'
                });
            }

            obj.find('.largePoster').off().on('tap', function () {
                setTimeout(function () {
                    obj.find('.largePoster').remove();
                    obj.find('.contentContainer').css({ 'pointer-events': '', 'opacity': '1' });
                    callback();
                }, 400);
            });
        },
        setCheckbox: function (selector, value) {
            try {
                if (value) {
                    selector.prop('checked', true).checkboxradio('refresh');
                } else {
                    selector.removeAttr('checked').checkboxradio('refresh');
                }
            } catch (e) { }
        },

        dismissKeyboard: function (el) {
            el.find('.search-field').bind('keypress', function (e) {
                if (e.keyCode === 13) {
                    e.preventDefault();
                    el.find(':input:focus').blur();
                    return;
                }
            });
        },

        tipsScreen: {
            listMovies: 'listMovies'
        },

        handleListMoviesTip: function () {
            var screenHeight = window.innerHeight - 100;
            var localize = window.ObjectCollections.Localization.result;
            var message = window.AppConfig.EnableAlternateFeatures ? localize.notificationTipListMovies : localize.notificationTipLocation;

            var form = $('body').pagecontainer('getActivePage').find('#listMovies');
            var contentToDisable = $('body').pagecontainer('getActivePage').find('.ContentBlock, .ui-navbar');

            var callback = function () {
                form.append('<div id="tip-overlay-screen" style="height:' + screenHeight + 'px"> ' +
                    '<div class="dismiss">' + localize.tapToDismiss + '</div>' +
                    '<div><img src="css/images/overlay-arrow.png"></div>' +
                    '<div class="message-container"><span class="title">' + localize.notificationTipTitle + '</span><br />' +
                    '<span>' + message + '</span></div>');
            };

            UIHelper.showNotificationTip(contentToDisable, UIHelper.tipsScreen.listMovies, callback);
        },

        showNotificationTip: function (content, tip, callback) {
            var storage = new StorageProvider();
            var activePage = $('body').pagecontainer('getActivePage');

            storage.read('notificationTips', function (value) {
                if (!value || !value[tip]) {
                    value = value || {};
                    value[tip] = true;

                    content.css({
                        'pointer-events': 'none',
                        'opacity': 0.1
                    });

                    activePage.on('tap', function () {
                        activePage.find('#tip-overlay-screen').hide();
                        $(document).off('touchmove.ncr touchstart.ncr');
                        content.css('opacity', 1);

                        setTimeout(function () {
                            content.css('pointer-events', 'auto');
                        }, 500);
                    });

                    $(document).on('touchmove.ncr touchstart.ncr', function (event) {
                        event.preventDefault();
                    });

                    storage.store('notificationTips', value);

                    if (callback) {
                        callback();
                    }
                }
            });
        },

        iOSNotificationPolicyFix: function (view) {
            var ios7 = window.AppProperties.iOSVersion.indexOf('7.') === 0;
            var ios8 = window.AppProperties.iOSVersion.indexOf('8.') === 0;
            var ios9 = window.AppProperties.iOSVersion.indexOf('9.') === 0;
            var ios10 = window.AppProperties.iOSVersion.indexOf('10.') === 0;

            var loyaltyContainer = $(view.el).find('.homescreen-loyalty-container');
            var notificationBar = $(view.el).find('.note-information.policy');

            if (ios7 || ios8 || ios9 || ios10) {
                if (loyaltyContainer.length > 0) {
                    loyaltyContainer.css({ 'margin-top': '0px' });
                    notificationBar.css({ 'margin-top': '0px' });
                }
            }
        },

        positionWindowCssFix: function () {
            var popupContainer = $('.ui-popup-container');
            var hasBanners = window.BannerHelper.isBannerAvailable();
            var ios7 = window.AppProperties.iOSVersion.indexOf('7.') === 0;
            var ios8 = window.AppProperties.iOSVersion.indexOf('8.') === 0;
            var ios9 = window.AppProperties.iOSVersion.indexOf('9.') === 0;
            var ios10 = window.AppProperties.iOSVersion.indexOf('10.') === 0;

            if (ios7 || ios8 || ios9 || ios10) {
                popupContainer.css("cssText", hasBanners ? "top: 177px !important;" : "top: 105px !important;");
            } else {
                if (hasBanners) {
                    popupContainer.css("cssText", "top: 158px !important;");
                }
            }
        }

    };

    return window.UIHelper;
});