﻿define(['helpers/uiHelper'], function () {
    window.Utils = (function () {
        function _scanBarcode(callback) {
            try {
                var scanner = cordova.plugins.barcodeScanner;
                scanner.scan(
                    function (result) {
                        callback(result);
                    },
                    function (result) {
                        if (result == "unable to obtain video capture device") {
                            window.showAlert(window.ObjectCollections.Localization.result.noCameraError);
                        }
                        else {
                            window.showAlert(window.ObjectCollections.Localization.result.barcodeScannerError);
                        }
                    }
                );
            } catch (ex) {
                return;
            }
        }

        function _showDirections(theaterId) {
            if (ConfigHelper.isDeviceOnline() == false) {
                window.showAlert(window.ObjectCollections.Localization.result.errorUnableConnectToInternet);
                return false;
            }

            var theater = window.ObjectCollections.TheaterCollection.get(theaterId);
            var destination = theater.get('addressLine1') + '+' + theater.get('addressCity') + '+' + theater.get('addressZip') + '+,' + (theater.get('addressState') ? theater.get('addressState') : '');
            var startCoords = AppProperties.CurrentPosition.Latitude + ',' + AppProperties.CurrentPosition.Longitude;

            //If User disabled location services, default location is latitude 0, longitude 0 - when taping on navigation startCoords should be blank
            if (startCoords == '0,0') {
                startCoords = '';
            }

            var url = 'http://maps.google.com/maps?saddr=' + startCoords + '&daddr=' + destination;

            url = encodeURI(url);

            var inAppWin;

            if (window.AppProperties.IsAndroid) {
                inAppWin = window.open(url, '_system');
            } else {
                inAppWin = window.open(url, '_blank');
            }

            inAppWin.addEventListener('exit', function () {
                inAppWin.addEventListener('loadstop');
                inAppWin.addEventListener('exit');
                inAppWin = null;
            });
        }

        function _deletePurchaseHistoryEntry(purchasedItems) {
            var passedOrders = _.filter(purchasedItems, function (item) {
                var date = item.performance ? item.performance.showTime : item.purchasedDateTime;
                return new Date(date) < new Date();
            });

            if (!passedOrders) {
                return false;
            }

            passedOrders = _.sortBy(passedOrders, function (order) { return order.performance.showTime; });
            purchasedItems = _.reject(purchasedItems, function (item) { return item.confirmationNumber == passedOrders[0].confirmationNumber; });

            return {
                purchasedItems: purchasedItems,
                passedOrders: passedOrders
            };
        }

        return {
            scanBarcode: _scanBarcode,
            showDirections: _showDirections,
            deletePurchaseHistoryEntry: _deletePurchaseHistoryEntry
        };
    });
});