﻿window.ValidationHelper = (function () {
    var localize = window.ObjectCollections.Localization.result;

    function _initValidation() {
        require(['libs/jquery.validate.min'], function () {
            $.validator.setDefaults({
                onkeyup: false,
                focusCleanup: true,
                errorPlacement: function (error, element) {
                    error.insertAfter(element.parent());
                },
                highlight: function (element) {
                    $(element).parent().addClass('error');
                },
                unhighlight: function (element) {
                    $(element).parent().removeClass('error');
                }
            });
        });
    }

    function _setupRules(selector) {
        require(['libs/jquery.validate.min', 'sharedhelpers/cultureHelper'], function () {
            var cultureHelper = new CultureHelper();
            var zipValidation = cultureHelper.zipValidation();
            var isAndroidKitKat = window.AppProperties.IsAndroidKitKat;

            selector.find('#creditCardNumber').one('blur', function () {
                if (!$(this).valid()) {
                    return;
                }
            });

            //Sending empty string for the message to prevent adding duplicate messages
            //Adding message for this is handled by the validateExpDate function
            $.validator.addMethod('expDate', function () {
                return _validateExpDate();
            }, '');

            if (isAndroidKitKat) {
                var firstNameLength = selector.find('#firstName').attr('maxlength');
                var lastNameLength = selector.find('#lastName').attr('maxlength');

                var firstNameLengthError = $.sprintf(localize.errorNumberOfCharacters, firstNameLength);
                var lastNameLengthError = $.sprintf(localize.errorNumberOfCharacters, lastNameLength);

                selector.find('#firstName, #lastName').removeAttr('maxlength');

                $.validator.addMethod('firstName', function (value) {
                    return value.length <= firstNameLength;
                }, firstNameLengthError);

                $.validator.addMethod('lastName', function (value) {
                    return value.length <= lastNameLength;
                }, lastNameLengthError);
            }
            $.validator.addMethod('zipCode', function (value) {
                return value == '' || zipValidation.regex.test(value.toUpperCase());
            }, zipValidation.msg);
        });

        function _validateExpDate() {
            $('[for=expirationDate]').remove();

            var expMonth = $('#expDateMonth :selected').val();
            var expYear = $('#expDateYear :selected').val();

            if (expMonth !== '' && expYear !== '') {
                var todayDate = new Date();

                if (todayDate.getFullYear() < expYear || (todayDate.getFullYear() == expYear && todayDate.getMonth() + 1 <= expMonth)) {
                    return true;
                }
                else {
                    if ($('[for=expirationDate]').length == 0)
                        $('<label for="expirationDate" generated="true" class="error" style="">' + localize.errorExpirationDateInvalid + '</label>').insertAfter('#expirationDate');
                    return false;
                }

            }
            return true;
        }
    }

    function _validatePassword(cardPassword) {
        var minPasswordLenght = window.appConfig.PaswordCriteria ? (window.appConfig.PaswordCriteria.minLenght || 4) : 4;

        var passwordLengthError = $.sprintf(localize.errorPasswordLength, minPasswordLenght);

        $.validator.addMethod('password-length', function (value) {
            return value.length >= minPasswordLenght;
        }, passwordLengthError);

        if (window.appConfig.PaswordCriteria && window.appConfig.PaswordCriteria.uppercase) {
            $.validator.addMethod('password-uppercase', function (value) {
                return value == '' || /^(?=.*[A-Z])/.test(value);
            }, localize.errorPasswordUppercase);
        }

        if (window.appConfig.PaswordCriteria && window.appConfig.PaswordCriteria.lowercase) {
            $.validator.addMethod('password-lowercase', function (value) {
                return value == '' || /^(?=.*[a-z])/.test(value);
            }, localize.errorPasswordLowercase);
        }

        if (window.appConfig.PaswordCriteria && window.appConfig.PaswordCriteria.number) {
            $.validator.addMethod('password-number', function (value) {
                return value == '' || /^(?=.*\d)/.test(value);
            }, localize.errorPasswordNumber);
        }

        $.validator.addMethod('repeatPassword', function (value) {
            return value == cardPassword;
        }, localize.errorPasswordMismatch);
    }

    return {
        initValidation: _initValidation,
        setupRules: _setupRules,
        validatePassword: _validatePassword
    };
})();