﻿window.virtualGiftCardHelper = (function () {
    var that = this;

    function _lockCard(amount, cardNumber, cardName) {
        window.showPageLoadingMessage();

        require(['sharedhelpers/configurationProvider', 'models/orderModel', 'models/purchaseVirtualGiftCardModel'], function (configurationProvider) {
            if (window.ObjectCollections.TheaterCollection.models && window.ObjectCollections.TheaterCollection.models.length > 0) {

                if (configurationProvider.useVirtualGiftCardsDefaultSite()) {
                    var theaterId = configurationProvider.virtualGiftCardsDefaultSite();
                    that.theater = window.ObjectCollections.TheaterCollection.get(theaterId);
                }

                that.theater = that.theater || window.ObjectCollections.TheaterCollection.models[0];
            }

            var theater = that.theater || new BaseModel();
            amount = amount ? parseFloat(amount, 10) : amount;

            var model = new PurchaseVirtualGiftCardModel();
            model.theaterId = configurationProvider.useVirtualGiftCardsDefaultSite() ? configurationProvider.virtualGiftCardsDefaultSite() : theater.get('id');
            model.set('giftCardNumber', cardNumber || '');
            model.set('amount', amount);

            Backbone.sync('create', model, {
                error: function () {
                    new ErrorHelper().showAlertByErrorCode();
                },
                success: function (result) {
                    var response = result.result;

                    if (result.result_code == 200) {
                        window.ObjectModels.OrderModel = new OrderModel();
                        window.ObjectModels.TempOrder = { concessions: null };

                        var orderModel = window.ObjectModels.OrderModel;
                        var localize = window.ObjectCollections.Localization.result;

                        var vgcItem = [
                            {
                                "name": cardName || localize.virtualGiftCard,
                                "price": amount,
                                "tax": 0,
                                "quantity": 1,
                                "total": amount,
                                "sortOrder": 1,
                                "canBeRemoved": false
                            }
                        ];

                        orderModel.set('theater', theater);
                        orderModel.set('saleId', response.saleId);
                        orderModel.set('totalPrice', amount);
                        orderModel.set('apiTotalPrice', amount);
                        orderModel.set('virtualGiftCard', { number: response.giftCardNumber, name: cardName });
                        orderModel.set('otherItems', vgcItem);
                        orderModel.set('encryptionKey', response.encryptionKey);
                        orderModel.set('timeout', response.timeoutInMinutes * 60);

                        window.AppMethods.ChangePage('#paymentMethod');
                    } else {
                        new ErrorHelper().showAlertByErrorCode(response.error_sub_code);
                    }
                },
                complete: function () {
                    window.hidePageLoadingMessage();
                }
            });
        });
    }

    function _saveCard(card, update, callback) {
        require(['helpers/paymentMethodHelper'], function () {
            var localize = window.ObjectCollections.Localization.result;
            var paymentMethodHelper = new PaymentMethodsHelper();
            //TODO: Use balance from the lock gift card response if it is returned
            var paymentMethod = {
                Name: card.name || localize.virtualGiftCard,
                CardNumber: card.number,
                Type: "VirtualGiftCard",
                Balance: card.balance,
                BackgroundFront: 'css/images/vgcdesigns/vgc-background-front.jpg' //Default VGC background
            };

            paymentMethodHelper.insert({ paymentMethod: paymentMethod, checkPassword: false, update: update }, function (response) {
                if (response.success) {
                    var status = response.update ? 'update' : 'new';
                    if (card.existingCard) {
                        status = 'existingCard';
                    }
                    window.AppMethods.ChangePage('#virtualGiftCards/' + response.id + '/' + status);
                    return;
                }

                if (response.cardExists) {
                    window.showAlert($.sprintf(localize.errorCardAlreadySaved, card.number));

                    if (callback) {
                        callback(false);
                    }
                    return;
                }

                window.showAlert(localize.errorUnableToSaveNewGiftCard);
            });
        });
    }

    function _updateBalance(card) {
        require(['helpers/passHelper', 'sharedhelpers/giftCardHelper'], function () {
            window.giftCardHelper.updateBalance({ CardNumber: card.CardNumber, Balance: card.Balance });
            if (card.url && window.AppProperties.PassbookAvailable.virtualGiftCardPass) {
                var passHelper = new PassHelper();
                passHelper.getVirtualGiftCardPassURL(card, true);
            }
        });
    }

    function _setAsPrimary(id) {
        require(['sharedhelpers/storageProvider'], function () {
            var storage = new StorageProvider();
            storage.read('PaymentMethod', function (paymentMethods) {
                _.each(paymentMethods, function (paymentMethod) {
                    if (paymentMethod.Type == 'VirtualGiftCard') {
                        paymentMethod.Primary = paymentMethod.Id == id;
                    }
                });

                storage.store('PaymentMethod', paymentMethods);
            });
        });
    }

    function _unsetPrimary(id) {
        require(['sharedhelpers/storageProvider'], function () {
            var storage = new StorageProvider();
            storage.read('PaymentMethod', function (paymentMethods) {
                _.each(paymentMethods, function (paymentMethod) {
                    if ((!id || paymentMethod.Id == id) && paymentMethod.Type == 'VirtualGiftCard') {
                        paymentMethod.Primary = false;
                    }
                });

                storage.store('PaymentMethod', paymentMethods);
            });
        });
    }

    function _updateDesign(id, cardDesign) {
        require(['sharedhelpers/storageProvider'], function () {
            var storage = new StorageProvider();
            storage.read('PaymentMethod', function (paymentMethods) {
                var vgc = _.find(paymentMethods, function (paymentMethod) {
                    return paymentMethod.Id == id;
                });

                if (vgc) {
                    vgc.BackgroundFront = cardDesign;
                }

                storage.store('PaymentMethod', paymentMethods);
            });
        });
    }

    function _openBackgroundPicker(currentBackground, callback) {
        var localize = window.ObjectCollections.Localization.result;
        var content = $('<div>').addClass('vgc-backgrounds-carousel swiper-container');
        var pagination = $('<div>').addClass('dialog-pagination');
        var swiperWrapper = $('<div>').addClass('swiper-wrapper');
        var swiperSlide;

        require(['sharedhelpers/configurationProvider'], function (configurationProvider) {
            var backgrounds = configurationProvider.getGiftCardBackgrounds();

            _.each(backgrounds, function (b, i) {
                if (i == 0 || i % 6 == 0) {
                    swiperSlide = $('<div>').addClass('swiper-slide');
                    swiperWrapper.append(swiperSlide);
                }

                var img = $('<img>').attr({
                    'src': b,
                    'data-backgroundfront': b
                });

                if (b == currentBackground) {
                    img.addClass('selected');
                }

                swiperSlide.append(img);
            });

            var createPagination = backgrounds.length > 6;
            content.append(swiperWrapper);

            if (createPagination) {
                content.append(pagination);
            }

            require(['views/infoDialogView'], function (dialogView) {
                var dialog = new dialogView({
                    title: localize.chooseDesign,
                    content: content[0].outerHTML,
                    confirmButton: {
                        text: localize.set,
                        callback: function () {
                            var selectedDesign = popup.find('img.selected').attr('data-backgroundfront');
                            callback(selectedDesign);

                            $('#popupDialog').popup('close');
                        }
                    },
                    cancelButton: {
                        text: localize.cancel,
                        callback: function () {
                            $('#popupDialog').popup('close');
                        }
                    }
                });

                var popup = $('#popupDialog');
                popup.html(dialog.render());

                popup.find('.vgc-backgrounds-carousel').swiper({
                    mode: 'horizontal',
                    pagination: createPagination ? '.dialog-pagination' : false,
                });

                popup.find('img').on('tap', function (e) {
                    e.preventDefault();

                    popup.find('img').not(this).removeClass('selected');
                    $(e.currentTarget).addClass('selected');
                });

                popup.popup('open');
                popup.enhanceWithin();
            });
        });


    }

    return {
        lockCard: _lockCard,
        saveCard: _saveCard,
        setAsPrimary: _setAsPrimary,
        unsetPrimary: _unsetPrimary,
        updateBalance: _updateBalance,
        updateDesign: _updateDesign,
        openBackgroundPicker: _openBackgroundPicker
    };
})();