require.config({
    paths: {
        jquery: 'shared_top_level/js/libs/jquery/jquery.min',
        underscore: 'shared_top_level/js/libs/underscore/underscore.min',
        backbone: 'shared_top_level/js/libs/backbone/backbone.min',
        jquerymobile: 'js/libs/jquery.mobile-1.4.5.min',
        jquerySprintf: 'shared_top_level/js/libs/jquery/jquery.sprintf',
        childbrowser: '../js/childbrowser',
        facebook: 'js/helpers/facebookHelper', //DEBUG: 'js/helpers/facebookHelper.browser',
        text: 'shared_top_level/js/libs/require/text',
        json: 'shared_top_level/js/libs/require/json',
        async: 'shared_top_level/js/libs/require/async',
        globalize: 'shared_top_level/js/libs/jquery/globalize',
        globalizeCultures: 'shared_top_level/js/libs/jquery/globalize/globalize.cultures',
        libs: 'js/libs',
        helpers: 'js/helpers',
        sharedjs: 'shared_top_level/js',
        sharedlibs: 'shared_top_level/js/libs',
        sharedhelpers: 'shared_top_level/js/helpers',
        collections: 'shared_top_level/js/objects/collections',
        models: 'shared_top_level/js/objects/models',
        views: 'js/views',
        idangerousSwiper: 'js/libs/idangerous.swiper-2.7.0.min',
        braintree: 'js/libs/braintree'
    },
    shim: {
        'jquery': {
            exports: function () {
                return $;
            }
        },
        'underscore': {
            exports: '_'
        },
        'backbone': {
            deps: ['underscore', 'jquery'],
            exports: 'Backbone'
        },
        'jquerymobile': {
            deps: ['jquery', 'js/jqm-config']
        },
        'jquerySprintf': {
            deps: ['jquery']
        },
        'js/jqm-config': {
            deps: ['jquery', 'shared_top_level/js/encode']
        },
        'sharedhelpers/dateTimeHelper': {
            deps: ['jquery', 'sharedhelpers/date.format']
        },
        'globalizeCultures': {
            deps: ['jquery', 'globalize']
        },
        'sharedhelpers/utilities': {
            deps: ['sharedjs/RSAEncryption/aes']
        }
    },
    waitSeconds: 0
});

// Prevent duplicate initialization
var initialized = false;

function onDeviceReady() {
    require([
		'js/app',
        'js/config',
		'jquery',
		'underscore',
        'sharedhelpers/routeHelper',
		'backbone',
		'jquerymobile',
        'jquerySprintf',
		'js/jqm-config',
		'async',
        'helpers/configHelper',
        'helpers/bannerHelper',
        'sharedhelpers/seatPreviewHelper',
        'sharedhelpers/JSON.prune',
        'collections/baseCollection',
        'models/baseModel'
    ], function (App) {
        if (!initialized) {
            App.initialize();
            _.defer(function () {             
                window.BannerHelper.initializeSplashScreen();
            });
           
            initialized = true;
            
            if (!window.ObjectCollections.Localization) {
                window.ObjectCollections.Localization = new LocalizeCollection({ path: 'shared_top_level/js/language/' });
            }

            //Check if device OS is Android or iOS
            if (typeof window.device != 'undefined') {
                window.open = cordova.InAppBrowser.open;
                if (window.device.platform === "Android") {
                    window.AppProperties.IsAndroid = true;

                    if (window.device.version.indexOf('4.1.1') === 0 || window.device.version.indexOf('4.1.2') === 0) {
                        $('head').append('<style type="text/css">' +
                            ' .slide.out, .slide.in, .slide.out.reverse, .slide.in.reverse, .slidefade.out, .slidefade.in, ' +
                            '.slidefade.out.reverse, .slidefade.in.reverse' +
                            ' { -webkit-animation-fill-mode: both; } </style>');
                    }

                    if (window.device.version.indexOf('4.3') === 0) {
                        window.AppProperties.IsAndroid4x = true;
                    }
                    if (window.device.version.indexOf('4.4') === 0) {
                        window.AppProperties.IsAndroidKitKat = true;
                    }

                } else if (window.device.platform === "iOS") {
                    window.AppProperties.IsIOS = true;
                    window.AppProperties.iOSVersion = window.device.version;

                    //check for passbook
                    var onResult = function (isAvailable) {
                        if (!isAvailable || !window.appConfig.PassslotAPIKey)
                            return;

                        window.AppProperties.PassbookAvailable.loyaltyPass = !!window.appConfig.LoyaltyCardPassTemplateId;
                        window.AppProperties.PassbookAvailable.ticketPass = !!window.appConfig.TicketPassTemplateId;
                        window.AppProperties.PassbookAvailable.virtualGiftCardPass = !!window.appConfig.VirtualGiftCardPassTemplateId;
                        window.AppProperties.PassbookAvailable.dualCardPass = !!window.appConfig.DualCardPassTemplateId;
                    };

                    window.Passbook ? window.Passbook.available(onResult) : onResult(false);
                }
            }
            window.ObjectCollections.Localization.getLocalizationGlobal(window.AppProperties.Language, function (localize) {
                require(['sharedhelpers/storageProvider'], function () {
                    var storage = new StorageProvider();
                    window.ChainInfo = new ChainInfoModel();
                    storage.read('ChainInfo', function (ci) {
                        if (ci) {
                            window.ChainInfo.attributes = ci;
                        }

                        window.AppProperties.SearchUnit = (!window.ChainInfo || window.ChainInfo.defaultSearchRadiusIsInMiles()) ? localize.miles : localize.kilometers;
                    });
                });

                // Browser mode
                if (navigator.network === undefined)
                    return;

                // Handle connection state events
                document.addEventListener("online", ConfigHelper.deviceOnlineEvent, false);

                document.addEventListener("offline", ConfigHelper.deviceOfflineEvent, false);

                document.addEventListener("backbutton", ConfigHelper.onDeviceBackButton, false);

                // Connection could be unavailable at the start up, check the state anyway            
                if (navigator.connection.type == Connection.NONE) {
                    ConfigHelper.deviceOfflineEvent();
                }

                window.DtHelper.initialize({
                    today: window.ObjectCollections.Localization.result.today,
                    tomorrow: window.ObjectCollections.Localization.result.tomorrow
                });
            });

            require(['helpers/googleAnalyticsHelper'], function () {
                window.GoogleAnalyticsHelper.initialize();
            });

            $(document).ajaxError(function (event, request) {
                window.AppProperties.TapEvent = null;

                if (request.statusText === 'timeout')
                    window.hidePageLoadingMessage();
            });

            try {
                window.cordova.plugins.barcodeScanner;
            } catch (e) {
                $('head').append('<style type="text/css">' +
                    '.loyalty-scan' + ' { display: none; } </style>');
            }

            require(['helpers/menuHelper'], function () {
                window.MenuHelper.render();
            });

            require(['helpers/validationHelper'], function () {
                window.ValidationHelper.initValidation();
            });
        }
    });
}
