define([
  'jquery',
  'underscore',
  'backbone',
  'collections/localizeCollection',
  'collections/performanceCollection',
  'sharedhelpers/htmlHelper',
  'helpers/uiHelper',
  'sharedhelpers/storageProvider',
  'sharedhelpers/dateTimeHelper'
], function($, _, Backbone,PerformanceCollection) {
  var AppRouter = Backbone.Router.extend({
    routes: {
      "": "handleRoutes",
      "home": "home",
      "holdSeats/:pTheater/:pPerformance": "holdSeats",
      "selectSeats/:pTheater/:pPerformance": "selectSeats",
      "concessions/:theaterId/:status": "concessions",
      "concessionItem/:name/:productIds/:theaterCulture/:timestamp": "concessionItem",
      "concessionModifiersMulti/:id/:theaterCulture": "concessionModifiersMulti",
      "concessionModifiersSingleById/:id/:theaterCulture": "concessionModifiersSingleById",
      "checkoutSummary": "checkoutSummary",
      "paymentMethod": "paymentMethod",
      "myTicketView": "myTicketView",
      "loyaltycard": "loyaltyCardPage",
      "loyaltycardview/:status": "loyaltyCardPage",
      "loyaltycardprofile/:page": "loyaltyCardDetails",
      "loyaltycardsignup": "loyaltyCardSignup",
      "loyaltycardstep2": "loyaltyCardSignupStep2",
      "loyaltycardfinalstep": "loyaltyCardSignupFinalStep",
      "selectRecipient/:id/:confirmationNumber": "selectRecipient",
      "confirmHoldSeats": "confirmHoldSeats",
      "settings": "settings",
      "locationSettings": "locationSettings",
      "purchaseHistory": "purchaseHistory",
      "purchaseHistory/:confirmationNumber": "purchaseHistory",
      "ph-orderdetails/:orderId": "purchaseHistoryOrderDetails",
      "paymentMethodSettings/:isVgc": "paymentMethodList",
      "paymentMethodSettings": "paymentMethodList",
      "paymentMethodSettingsDetails/:name": "paymentMethodDetails",
      "promotions": "promotions",
      'pickup/:serialNumber/:facebookId/:friendsName/:performanceNumber/:featureCode/:theaterId': 'giftPickup',
      "virtualGiftCards/:id/:status": "virtualGiftCards",
      "virtualGiftCards/:id": "virtualGiftCards",
      "virtualGiftCards": "virtualGiftCards",
      "seatsPreview/:movieId/:theaterIdsString": "seatsPreview",

      //Menu routes to handle the routes when triggered from the menu
      "menuSettings": "menuSettings",
      "menuLoyaltycard": "menuLoyaltyCardPage",
      "menuMovies": "menuListMovies",
      "menuTheaters": "menuTheaters",
      "menuConcessions": "menuConcessions",
      "menuPurchaseHistory": "menuPurchaseHistory",
      "menuVirtualGiftCards": "menuVirtualGiftCards",
      "menuComingSoon": "menuComingSoon",
      "menuContactUs": "menuContactUs"
    },

    initialize: function() {
      var that = this;
      that.storage = new StorageProvider();
      that.storageKeys = {
        purchaseHistory: 'PurchaseHistory',
        loyaltyCard: 'LoyaltyCard',
        historyCleanup: 'HistoryCleanup',
        paymentMethod: 'PaymentMethod'
      };

      //Adding routes shared between mobile and web app
      var routesHelper = new RoutesHelper();
      routesHelper.addSharedRoutes.call(that, true);

      that.homePage = false;
      that.firstCall = true;

      //Prevent event propagation to the elements shown under the header
      $('.ui-page[data-role="page"] #header').live('tap', function(event) {
        event.preventDefault();
        event.stopPropagation();
      });

      $('.ui-page[data-role="dialog"] #header .ui-btn-left').live('tap', function(event) {
        event.preventDefault();

      //  window.history.back();
          window.goBack();

        return false;
      });

      $('.ui-page[data-role="page"] #header .ui-btn-left').live('tap', function(event) {
        event.preventDefault();
        window.showPageLoadingMessage();

        window.AppProperties.TapEvent = null;
        window.goBack();

        return false;
      });

      $('.ui-content a.ui-btn, .ui-link, .ui-link-inherit').live('tap', function(event) {
        if ($(this).attr('href') && $(this).attr('href') != "#") {
          event.preventDefault();

          var route = $(this).attr('href').replace('#', '');
          that.navigate(route, {
            trigger: true
          });

          return false;
        } else return true;
      });

      // Use this instead of window.history.back() in code
      window.goBack = function(count) {
        window.tempHistoryCount -= 2;
        that.back = true;
        count = -count || -1;
        if (window.device && window.device.platform === "iOS") {
    let deviceVersion = window.device.version;
    if (deviceVersion.slice(0, deviceVersion.indexOf('.')) === "11" || deviceVersion.slice(0, deviceVersion.indexOf('.')) === "10") {
      var length = window.myhistory.length;
      //popping latest history entry.
      if (length > 1) {
        window.myhistory.pop(length - 1);
      }

      if (length == 0 || length == 1) {
        Backbone.history.navigate('home', true);
      } else {
        var route = window.myhistory[length - 2];
        console.log("route fragment:" + route.fragment);
        Backbone.history.navigate(route.fragment, true);
      }
    } else {
      window.history.go(count);
    }
  }else {
          window.history.go(count);
        }
      };
    },

    changePage: function(page) {
      var transition = $.mobile.defaultPageTransition;

      // We don't want to slide the first page
      if (this.firstCall) {
        transition = 'none';
        this.firstCall = false;
      }

      // Override transition if called from same page
      if (window.AppProperties.PageRefresh)
        transition = 'none';

      //Set PageRefresh back to null
      window.AppProperties.PageRefresh = null;

      var pageId = null;

      if (window.location.hash) {
        pageId = window.location.hash.replace('#', '');

        var index = pageId.indexOf('/');

        if (index != -1) {
          pageId = pageId.substr(0, index);
        }
      }

      if (pageId && pageId.length && $('#' + pageId + '.ui-page').length && $('body').pagecontainer('getActivePage').attr('id') != pageId) {
        page = $('#' + pageId + '.ui-page');

        $('body').pagecontainer('change', page, {
          changeHash: false,
          transition: transition,
          reverse: this.back
        });
      } else {
        $(page.el).attr('id', pageId);
        $(page.el).attr('data-role', 'page');

        page.render();

        this.homePage = false;

        $(page.el).attr('data-url', window.location.pathname + window.location.hash);

        $('body').append($(page.el));
        $('body').pagecontainer('change', $(page.el), {
          changeHash: false,
          transition: transition,
          reverse: this.back
        });

        this.currentPage = page;
      }

      this.back = false;
      window.AppProperties.nativeBackButton = false;

    },

    // Routes begin here
    handleRoutes: function() {
      if (window.deepLinkUrl) {
        require(['helpers/deepLinkingHelper'], function() {
          var deepLinking = new DeepLinkingHelper();

          deepLinking.handleDeepLink(window.deepLinkUrl);
        });
        return;
      }

      Backbone.history.navigate('home', true);
    },

    home: function() {
      var that = this;
      window.showPageLoadingMessage();
      //clearing contents of myhistory when someone hits home page
      if (window.device && window.device.platform === "iOS") {
    let deviceVersion = window.device.version;
    if (deviceVersion.slice(0, deviceVersion.indexOf('.')) === "11" || deviceVersion.slice(0, deviceVersion.indexOf('.')) === "10") {
      while (window.myhistory.length > 0) {
        window.myhistory.pop();
      }
    }
  }
      if (window.AppConfig.EnableAlternateFeatures) {
        require(['helpers/homeScreenHelper'], function() {
          window.HomeScreenHelper.getHomeScreenData(function(view) {
            window.tdSelectedBusinessDate = DtHelper.getCurrentBusinessDate();
            that.changePage(view);
            that.homePage = true;
          });

        });

        return;
      }

      this.homePage = true;
      if (!this.firstCall) {
        // Clear cache
        ConfigHelper.clearCache();

        //Reinitialize orderModel.
        var orderModel = window.ObjectModels.OrderModel;
        if (orderModel) {
          orderModel.cancelSale(function() {
            window.ObjectModels.OrderModel = null;
          });
        }
      }

      var now = new Date().toISOString();

      that.storage.read(that.storageKeys.purchaseHistory, function(orders) {
        var tickets = 0;
        var numberOfOrders = 0;

        if (orders) {
          tickets = _.filter(orders, function(item) {
            if (item.performance) {
              return new Date(item.performance.showTime).toISOString() > now;
            }
            return false;
          }).length;
          numberOfOrders = orders.length;
        }

        require(['views/home'], function(homeView) {
          window.hidePageLoadingMessage();

          window.Views.HomeView = new homeView({
            tickets: tickets,
            numberOfOrders: numberOfOrders
          });
          that.changePage(window.Views.HomeView);
        });
      });
    },

    checkoutSummary: function() {
      var that = this;

      window.showPageLoadingMessage();

      require(['views/checkoutSummaryView'], function(checkoutSummaryView) {
        var ticketModel = window.ObjectModels.OrderModel.get("tickets");
        var concessionsModel = window.ObjectModels.OrderModel.get('concessions');

        var selectedTickets = !ticketModel || (ticketModel.length == 0 && ticketModel.models.length == 0) ? [] : ticketModel.getSelectedTickets();
        var movie = window.ObjectModels.OrderModel.get("movie");
        var performance = window.ObjectModels.OrderModel.get("performance") || [];
        var theater = window.ObjectModels.OrderModel.get("theater") || [];
        var conc = concessionsModel.models || concessionsModel;
        var concessions = (!conc || conc.length == 0) ? undefined : concessionsModel;

        that.changePage(new checkoutSummaryView({
          concessions: concessions,
          selectedTickets: selectedTickets,
          movie: movie,
          performance: performance,
          theater: theater
        }));
      });
    },

    paymentMethod: function() {
      var that = this;

      window.showPageLoadingMessage();

      var localize = window.ObjectCollections.Localization.result;

      that.storage.read(that.storageKeys.loyaltyCard, function(value) {
        var loyaltyHolder = null;

        if (value) {
          loyaltyHolder = {
            name: value.firstName + ' ' + value.lastName,
            number: value.number.substring(value.number.length - 4, value.number)
          };
        }

        require(['views/paymentMethodView'], function(paymentMethodView) {
          if (window.ObjectModels.OrderModel.get("tickets") && window.ObjectModels.OrderModel.get("tickets").models.length > 0) {
            var hasHoldSeats = false;

            if (window.ObjectModels.OrderModel.get("tickets").getOnHoldTicketQuantity()) {
              hasHoldSeats = true;
            }


            if (window.ObjectModels.OrderModel != null) {
              window.hidePageLoadingMessage();
              var performance = window.ObjectModels.OrderModel.get("performance");
              that.changePage(new paymentMethodView({
                performance: performance,
                hasHoldSeats: hasHoldSeats,
                loyaltyHolder: loyaltyHolder
              }));
            } else {
              window.showAlert(localize.errorGettingPaymentMethod);
            };
          } else {
            window.hidePageLoadingMessage();
            that.changePage(new paymentMethodView({
              hasHoldSeats: false,
              loyaltyHolder: loyaltyHolder
            }));
          };
        });
      });
    },

    myTicketView: function() {
      window.showPageLoadingMessage();
      var that = this;
      var localize = window.ObjectCollections.Localization.result;

      require(['views/myTicketView'], function(myTicketView) {
        that.storage.read(that.storageKeys.purchaseHistory, function(orders) {
          if (window.ObjectModels.OrderModel != null) {
            var confirmationNumber = window.ObjectModels.OrderModel.get('confirmationNumber');
            var order = _.find(orders, function(o) {
              return o.confirmationNumber === confirmationNumber;
            });

            window.hidePageLoadingMessage();

            if (order) {
              that.changePage(new myTicketView({
                orders: orders,
                order: order
              }));
            } else {
              window.showAlert(localize.errorGettingMyTicketsView);
            }
          } else {
            window.showAlert(localize.errorGettingMyTicketsView);
          }
        });
      });
    },

    giftPickup: function(serialNumber, facebookId, friendsName, performanceNumber, featureCode, theaterId) {
      var that = this;
      window.showPageLoadingMessage();

      require(['models/theaterModel', 'helpers/deepLinkingHelper', 'sharedhelpers/utilities'], function(theaterModel) {
        var theater = new theaterModel({
          id: theaterId
        });
        theater.getTheater(function(res) {
          that.theater = res;

          serialNumber = new Utilities().decrypt(decodeURIComponent(serialNumber));

          var deepLinking = new DeepLinkingHelper();

          var continueFlow = function() {
            that.movie = window.ObjectCollections.MovieCollection.getMovie(featureCode);
            var performanceValid = deepLinking.validatePerformance(that.theater, that.movie, performanceNumber);

            if (!performanceValid) {
              window.hidePageLoadingMessage();
              return;
            }

            var fbValidationError = window.ObjectCollections.Localization.result.giftPickupFbValidationError;
            var notLoggedOnFbError = window.ObjectCollections.Localization.result.giftPickupNotLoggedOnFbError;

            deepLinking.validateFacebookAccount(facebookId, fbValidationError, notLoggedOnFbError, function() {
              window.hidePageLoadingMessage();

              if (!window.AppProperties.FBValid) {
                window.AppMethods.ChangePage("#tickets/" + performanceNumber + "/" + featureCode + "/" + theaterId);
                return;
              }

              var order = {
                performance: that.movie.performances.get(performanceNumber).attributes,
                movieCode: that.movie.get('code'),
                movieName: that.movie.get('name'),
                movieRating: that.movie.get('rating'),
                movieImage: that.movie.get('image'),
                confirmationNumber: serialNumber,
                theaterName: that.theater.get('name'),
                theaterCulture: that.theater.get('culture'),
                selectedTickets: [],
                concessions: [],
                giftedTicket: true
              };

              that.storage.read(that.storageKeys.purchaseHistory, function(orders) {
                orders = orders || [];

                if (orders.length > 0) {
                  var previouslyStored = _.filter(orders, function(item) {
                    return item.confirmationNumber == order.confirmationNumber;
                  });

                  if (previouslyStored.length > 0) {
                    window.showAlert(window.ObjectCollections.Localization.result.giftPickedup);
                    window.AppMethods.ChangePage('', true);
                    return;
                  }
                }

                orders.push(order);
                that.storage.store(that.storageKeys.purchaseHistory, orders);
                window.isPickup = true;
                window.AppMethods.ChangePage('#purchaseHistory/' + serialNumber, false, true);
              });
            });
          };

          var args = {
            callback: continueFlow,
            theaterId: theaterId,
            movieId: featureCode
          };

          deepLinking.getSelectTicketsData(args);
        });
      });
    },

    confirmHoldSeats: function() {
      window.showPageLoadingMessage();

      var orderModel = window.ObjectModels.OrderModel;
      if (!orderModel) {
        window.showAlert(window.ObjectCollections.Localization.result.errorGettingConfirmHoldSeats);
      }
      var that = this;

      require(['views/confirmHoldSeatsView'], function(confirmHoldSeatsView) {
        var movie = orderModel.get("movie");
        var performance = orderModel.get("performance");
        var theater = orderModel.get("theater");
        var confirmationNumber = orderModel.get('confirmationNumber');
        var holdTicketQuantity = orderModel.get('tickets').getOnHoldTicketQuantity();

        //Get hold seats for friends
        var friends = [];
        if (orderModel.get("tickets").getOnHoldTicketQuantity()) {
          var holdTickets = orderModel.get("tickets").models;
          for (var t in holdTickets) {
            if (holdTickets[t].friends != undefined) {
              friends = holdTickets[t].friends;
            }
          }
        }

        that.changePage(new confirmHoldSeatsView({
          friends: friends,
          movie: movie,
          performance: performance,
          theater: theater,
          confirmationNumber: confirmationNumber,
          holdTicketQuantity: holdTicketQuantity
        }));
      });
    },

    menuTheaters: function() {
      if (window.device && window.device.platform === "iOS") {
    let deviceVersion = window.device.version;
    if (deviceVersion.slice(0, deviceVersion.indexOf('.')) === "11" || deviceVersion.slice(0, deviceVersion.indexOf('.')) === "10") {
      while (window.myhistory.length > 0) {
        window.myhistory.pop();
      }
    }
  }
      window.activeCustomFlags = [];
      this.theaters(null, true);
    },

    theaters: function(sort, hideBack) {
      var that = this;
      require(['views/theaters', 'sharedhelpers/configurationProvider', 'collections/theaterCollection', 'collections/movieCollection', 'helpers/theatersHelper', 'helpers/homeScreenHelper', 'sharedhelpers/giftCardHelper'], function(theaters, configurationProvider, theaterCollection) {
        function loadTheatersView() {
          window.tdSelectedBusinessDate = DtHelper.getCurrentBusinessDate();

          if (configurationProvider.chainType() == 'single') {
            that.theaterDetails(configurationProvider.theaterId(), hideBack);
          } else {
            var theatersHelper = new TheatersHelper();
            theatersHelper.getFavoriteTheaters(function(value) {
              var theatersCol = new theaterCollection();

              var latitude = AppProperties.CurrentPosition.Latitude;
              var longitude = AppProperties.CurrentPosition.Longitude;

              theatersCol.getTheaters(latitude, longitude, function(result) {

                window.ObjectCollections.TheaterCollection = result;
                window.ObjectCollections.MovieCollection = result.getMovies(null, true);

                window.ObjectCollections.ComingSoonCollection = new MovieCollection(_.filter(window.ObjectCollections.MovieCollection.models, function(movie) {
                  return movie.get('releaseDate') > DtHelper.getBussinessDate(new Date());
                }));

                if (window.giftCardHelper.clearTheaterId() && (window.ObjectCollections.TheaterCollection.models && window.ObjectCollections.TheaterCollection.models.length > 0)) {
                  window.giftCardHelper.setDefaultOptions(window.ObjectCollections.TheaterCollection.models[0]);
                }

                if (window.appConfig.GroupMovies) {
                  window.ObjectCollections.MovieCollection.groupedMovies = window.ObjectCollections.MovieCollection.groupMultiTypeMovies();
                  window.ObjectCollections.ComingSoonCollection.groupedMovies = window.ObjectCollections.ComingSoonCollection.groupMultiTypeMovies();
                }
                var view = new theaters({
                  collection: window.ObjectCollections.TheaterCollection,
                  sort: sort,
                  favorites: value,
                  hideBackButton: hideBack
                });

                if (value.length == 0) {
                  that.changePage(view);
                  return;
                }
                if (!window.AppProperties.nativeBackButton && !that.back) {
                  window.ObjectCollections.TheaterCollection.theaterViewType = 'favorites';
                }
                view.getFavoriteTheaters(function() {
                  that.changePage(view);
                });
              });
            });
          }
        }

        if (that.homePage && window.AppConfig.EnableAlternateFeatures && !window.HomeScreenHelper.theatersLoaded() && configurationProvider.chainType() == 'multi') {
          $('body').on('theatersLoaded', loadTheatersView);
          return;
        }
        loadTheatersView();
      });
    },

    promotions: function() {
      var that = this;
      require(['views/marqueeView', 'sharedhelpers/configurationProvider'], function(marqueeView, configurationProvider) {
        var marqueeData = configurationProvider.promotions();
        that.changePage(new marqueeView({
          marqueeData: marqueeData
        }));
      });
    },

    menuListMovies: function() {
      if (window.device && window.device.platform === "iOS") {
      let deviceVersion = window.device.version;
      if (deviceVersion.slice(0, deviceVersion.indexOf('.')) === "11" || deviceVersion.slice(0, deviceVersion.indexOf('.')) === "10") {
      while (window.myhistory.length > 0) {
        window.myhistory.pop();
      }
    }
  }
      window.activeCustomFlags = [];
      //TODO: Check if the sort parameter is ever being sent to list movies
      this.listMovies(null, true);
    },

    listMovies: function(sort, hideBack) {
      var that = this;
      require(['views/listMoviesView', 'collections/theaterCollection', 'sharedhelpers/configurationProvider'], function(listMoviesView, theaterCollection, configurationProvider) {
        if (configurationProvider.chainType() === 'single') {
          that.changePage(new listMoviesView({
            movies: window.ObjectCollections.MovieCollection,
            sort: sort,
            hideBackButton: hideBack
          }));
        } else {
          var theatersCol = new theaterCollection();

          var latitude = AppProperties.CurrentPosition.Latitude;
          var longitude = AppProperties.CurrentPosition.Longitude;

          theatersCol.getTheaters(latitude, longitude, function(result) {

            window.ObjectCollections.TheaterCollection = result;
            window.ObjectCollections.MovieCollection = result.getMovies(null, true);

            window.ObjectCollections.ComingSoonCollection = new MovieCollection(_.filter(window.ObjectCollections.MovieCollection.models, function(movie) {
              return movie.get('releaseDate') > DtHelper.getBussinessDate(new Date());
            }));

            if (window.giftCardHelper.clearTheaterId() && (window.ObjectCollections.TheaterCollection.models && window.ObjectCollections.TheaterCollection.models.length > 0)) {
              window.giftCardHelper.setDefaultOptions(window.ObjectCollections.TheaterCollection.models[0]);
            }

            if (window.appConfig.GroupMovies) {
              window.ObjectCollections.MovieCollection.groupedMovies = window.ObjectCollections.MovieCollection.groupMultiTypeMovies();
              window.ObjectCollections.ComingSoonCollection.groupedMovies = window.ObjectCollections.ComingSoonCollection.groupMultiTypeMovies();
            }
            that.changePage(new listMoviesView({
              movies: window.ObjectCollections.MovieCollection,
              sort: sort,
              hideBackButton: hideBack
            }));
          });
        }
      });
    },

    movieDetails: function(featureCode, theaterId) {
      var that = this;
      var localize = window.ObjectCollections.Localization.result;
      featureCode = featureCode.trim();

      window.showPageLoadingMessage();

      var continueFlow = function() {

        if (!that.back && that.currentPage) {
          that.previousPage = that.currentPage.el.id;
        }

        var partialCallback = function(result) {
          window.hidePageLoadingMessage();

          if (!result) {
            window.showAlert(localize.errorLoadingMovieSchedule);
            window.history.back();
            return;
          }
        };

        require(['views/movieDetailsView', 'collections/movieCollection'], function(movieDetailsView, movieCollection) {
          // Attempt loading movie from the cache
          var movieUuid = featureCode + theaterId;
          var theMovie = window.ObjectCollections.CachedMovies[movieUuid];

          if (theMovie != undefined && theMovie.performances.length > 0) {
            setTimeout(function() {
              window.hidePageLoadingMessage();
              that.changePage(new movieDetailsView({
                model: theMovie,
                previousPage: that.previousPage,
                theaterId: theaterId
              }));
            }, 10);
            return;
          }

          var movies = new movieCollection();

          if (window.appConfig.GroupMovies) {
            theMovie = window.ObjectCollections.MovieCollection.getMovie(featureCode);

            //determine the group for the selected movie
            var movieGroup = window.ObjectCollections.MovieCollection.getMovieGroup(featureCode);
            if (theMovie) {
              theMovie.group = movieGroup;
            }

            //determine all the theaters where the movies from the group are shown
            var defaultTheaters = [],
              theaterList = theaterId.split(',');
            _.each(theaterList, function(t) {
              var theater = window.ObjectCollections.TheaterCollection.get(t);
              defaultTheaters.push(theater);
            });

            function AddPerformancesToTheaters(movieScheduleResult) {
              _.each(defaultTheaters, function(theater) {
                var groupedMovies = window.ObjectCollections.MovieCollection.groupMultiTypeMovies(result);
                var movieGroupFromResult = window.ObjectCollections.MovieCollection.getMovieGroup(featureCode, theMovie, groupedMovies);
                var movieGroup = movieGroupFromResult;
                var result = movieScheduleResult; //saving a copy of result and movieGroup objects
                _.each(movieGroup, function(mov) {
                  _.each(result.models, function(mod) {
                    if (mov.get('code').trim() === mod.get('code').trim() && mov.theaterId === theater.get('id')) {
                      mov.schedule.models = _.filter(mod.schedule.models, function(m) {
                        return m.get('theaterId') == theater.get('id');
                      });

                      if (mov.schedule.models.length > 0) {
                        _.each(mov.schedule.models, function(schedule) {
                          if (schedule.get('theaterId') === theater.get('id'))
                            mov.performances = new PerformanceCollection(schedule.get('performances'));
                        });
                      }
                      if (theMovie && (theMovie.get('id') == mod.get('id') || theMovie.get('tMDBId') == mod.get('tMDBId'))) {
                        theMovie.set('description', mod.get('description'));
                        theMovie.set('starring', mod.get('starring'));
                      }
                    }

                  });
                });
                let group = _.filter(movieGroup, function(mov) {
                  return theater.get('id') == mov.theaterId;
                });
              theater.group = new MovieCollection(group).changeSort('byPopularity').models;
              });

            }

            movies.getMovieSchedule('', theaterList, function(result) {
              partialCallback(result);
              AddPerformancesToTheaters(result);
              theMovie.theaters = defaultTheaters;
              that.changePage(new movieDetailsView({
                model: theMovie,
                previousPage: that.previousPage,
                theaterId: theaterId
              }));
            });
          } else {
            movies.getMovieSchedule(featureCode, theaterId, function(result) {
              partialCallback(result);
              theMovie = result.models[0];

              var tmpList = [];
              // connect with theaters
              _.each(theMovie.schedule.models, function(mod) {
                var theater = window.ObjectCollections.TheaterCollection.get(mod.get('theaterId'));
                theater.performances = mod.get('performances');
                tmpList.push(theater);
              });

              // now copy this to theaters property
              theMovie.theaters = tmpList;

              // Save movie for re-use
              window.ObjectCollections.CachedMovies[movieUuid] = theMovie;
              that.changePage(new movieDetailsView({
                model: theMovie,
                previousPage: that.previousPage,
                theaterId: theaterId
              }));
            });
          }
        });
      };


      //If we are accessing this view within the app, continue the flow
      if (window.ObjectCollections.MovieCollection && window.ObjectCollections.MovieCollection.length > 0) {
        continueFlow();
        return;
      }

      //... or if we are deeplinking into the app get the theater and movie collections
      window.AppProperties.CurrentPosition = {
        Latitude: 0,
        Longitude: 0
      };

      require(['collections/theaterCollection'], function(theaterCollection) {
        var theaters = new theaterCollection();
        theaters.latitude = window.AppProperties.CurrentPosition.Latitude;
        theaters.longitude = window.AppProperties.CurrentPosition.Longitude;

        theaters.fetch({
          error: function() {
            window.showAlert(window.ObjectCollections.Localization.result.unableToConnect);
          },
          success: function(result) {
            window.ObjectCollections.TheaterCollection = result;
            window.ObjectCollections.MovieCollection = result.getMovies();
            if (window.appConfig.GroupMovies) {
              window.ObjectCollections.MovieCollection.groupedMovies = window.ObjectCollections.MovieCollection.groupMultiTypeMovies();
            }
            continueFlow();
          }
        });
      });

    },

    theaterDetails: function(theaterId, hideBack) {
      var that = this;
      window.showPageLoadingMessage();
      var localize = window.ObjectCollections.Localization.result;

      require(['views/theaterDetailsView', 'collections/movieCollection', 'sharedhelpers/configurationProvider', 'helpers/theatersHelper'], function(theaterDetailsView, movieCollection, configurationProvider) {
        var orderModel = window.ObjectModels.OrderModel;
        if (orderModel) {
          orderModel.cancelSale(function() {
            window.ObjectModels.OrderModel = null;
          });
        }

        window.tdSelectedBusinessDate = window.tdSelectedBusinessDate || DtHelper.getCurrentBusinessDate();

        // Get complete schedule for a given theater
        var movies = new movieCollection();
        movies.getMovies(theaterId, Globalize.format(window.tdSelectedBusinessDate, 'yyyyMMdd'), function(result) {
          window.hidePageLoadingMessage();

          if (!result) {
            window.showAlert(localize.errorLoadingTheater);
            window.history.back();
            return;
          }

          that.renderTheaterDetails(theaterId, result, hideBack);
        });
      });
    },

    renderTheaterDetails: function(theaterId, movieCollection, hideBack) {
      var that = this;

      //Check theater collection for single chain types
      if (!window.ObjectCollections.TheaterCollection.models) {
        window.showAlert(localize.errorTheaterNotExists);
        window.history.back();
        return;
      }

      var theater = window.ObjectCollections.TheaterCollection.get(theaterId);
      window.AppProperties.SelectedTheater = theater;

      var theatersHelper = new TheatersHelper();
      theatersHelper.saveTheaterDefaultId(theater.get('id'));

      require(['views/theaterDetailsView'], function(theaterDetailsView) {
        that.changePage(new theaterDetailsView({
          theater: theater,
          movies: movieCollection,
          hideBackButton: hideBack
        }));
      });
    },

    holdSeats: function(pTheater, pPerformance) {
      var that = this;

      window.showPageLoadingMessage();
      require(['views/holdSeatsView'], function(holdSeatsView) {
        window.hidePageLoadingMessage();
        that.changePage(new holdSeatsView({
          pTheater: pTheater,
          pPerformance: pPerformance
        }));
      });
    },

    menuConcessions: function() {
      window.activeCustomFlags = [];
      var that = this;

      require(['sharedhelpers/configurationProvider'], function(configurationProvider) {
        window.tempHistoryCount = 0;
        window.ObjectModels.TempOrder = {
          concessions: null
        };

        that.concessions(configurationProvider.theaterId(), null, true);
      });
    },

    concessions: function(theaterId, status, hideBack) {
      // Concessions cannot be accessed while device is offline
      if (ConfigHelper.isDeviceOnline() == false) return;

      var that = this;

      window.showPageLoadingMessage();

      require(['views/concessionsView', 'helpers/concessionsHelper'], function(cView) {
        var theaterCallback = function(t) {
          var tCulture = t.get('culture');
          var orderModel = window.ObjectModels.OrderModel;
          var helper = new ConcessionsHelper();
          helper.clearData();

          if (that.back) status = null;

          if (orderModel && orderModel.get('saleId') && orderModel.get('concessions') && orderModel.get('concessions').length > 0) {
            that.changePage(new cView({
              concessions: window.ObjectCollections.ConcessionCollection,
              theaterCulture: tCulture,
              status: status,
              hideBackButton: hideBack
            }));
          } else {
            var concessionCollection = new ConcessionCollection();
            concessionCollection.getConcessions(theaterId, function(res) {
              window.hidePageLoadingMessage();

              if (res == undefined) {
                window.AppProperties.PageRefresh = true;
                window.history.back();
                return;
              }

              window.ObjectCollections.ConcessionCollection = res;
              that.changePage(new cView({
                concessions: res,
                theaterCulture: tCulture,
                status: status,
                hideBackButton: hideBack
              }));
            });
          }
        };

        if (window.ObjectModels.OrderModel && window.ObjectModels.OrderModel.get('theater')) {
          var theater = window.ObjectModels.OrderModel.get('theater');
          theaterCallback(theater);
        } else {
          require(['models/theaterModel'], function() {
            var theaterModel = new TheaterModel({
              id: theaterId
            });
            theaterModel.getTheater(function(res) {
              if (window.ObjectModels.OrderModel == null) {
                require(['models/orderModel'], function() {
                  var orderModel = new OrderModel();
                  orderModel.set('theater', res);
                  window.ObjectModels.OrderModel = orderModel;
                });
              }
              theaterCallback(res);
            });
          });
        }
      });
    },

    concessionItem: function(name, productIds, theaterCulture) {
      var that = this;

      window.showPageLoadingMessage();

      require(['views/concessionItemView', 'helpers/concessionsHelper'], function(concessionItemView) {
        window.hidePageLoadingMessage();

        var concessionsHelper = new ConcessionsHelper();
        concessionsHelper.resetCurrentModifier();
        concessionsHelper.resetTotalModifiersCount();

        var result = concessionsHelper.getProductsByIds(productIds);

        that.changePage(new concessionItemView({
          concessionItem: result,
          itemName: decodeURIComponent(name),
          theaterCulture: theaterCulture
        }));
      });
    },

    concessionModifiersMulti: function(id, theaterCulture) {
      var that = this;

      window.showPageLoadingMessage();

      require(['views/concessionModifiersMultiView', 'helpers/concessionsHelper'], function(concessionModifiersMultiView) {
        window.hidePageLoadingMessage();

        var concessionsHelper = new ConcessionsHelper();
        var result = concessionsHelper.getModifiersByProductId(id);
        concessionsHelper.updateCurrentModifier();

        that.changePage(new concessionModifiersMultiView({
          multiModifiers: result,
          parentId: id,
          theaterCulture: theaterCulture
        }));
      });
    },

    concessionModifiersSingleById: function(id, theaterCulture) {
      var that = this;

      window.showPageLoadingMessage();

      require(['views/concessionModifiersSingleView', 'helpers/concessionsHelper'], function(concessionModifiersSingleView) {
        window.hidePageLoadingMessage();

        var concessionsHelper = new ConcessionsHelper();
        var result = concessionsHelper.getModifiersByProductId(id);
        concessionsHelper.updateCurrentModifier();

        that.changePage(new concessionModifiersSingleView({
          singleModifiers: result,
          parentId: id,
          theaterCulture: theaterCulture
        }));
      });
    },

    menuVirtualGiftCards: function() {
      if (window.device && window.device.platform === "iOS") {
      let deviceVersion = window.device.version;
      if (deviceVersion.slice(0, deviceVersion.indexOf('.')) === "11" || deviceVersion.slice(0, deviceVersion.indexOf('.')) === "10") {
      while (window.myhistory.length > 0) {
        window.myhistory.pop();
      }
    }
  }
      window.activeCustomFlags = [];
      this.virtualGiftCards(null, null, true);
    },

    virtualGiftCards: function(id, status, hideBack) {
      var that = this;

      require(['views/virtualGiftCardsView'], function(virtualGiftCardsView) {
        that.storage.read(that.storageKeys.paymentMethod, function(paymentMethods) {
          var virtualGiftCards = _.filter(paymentMethods, function(card) {
            return card.Type == 'VirtualGiftCard';
          });

          that.changePage(new virtualGiftCardsView({
            cards: virtualGiftCards,
            cardId: id,
            status: that.back || window.AppProperties.nativeBackButton ? null : status,
            hideBackButton: hideBack
          }));
        });
      });
    },

    menuLoyaltyCardPage: function() {
      if (window.device && window.device.platform === "iOS") {
      let deviceVersion = window.device.version;
      if (deviceVersion.slice(0, deviceVersion.indexOf('.')) === "11" || deviceVersion.slice(0, deviceVersion.indexOf('.')) === "10") {
      while (window.myhistory.length > 0) {
        window.myhistory.pop();
      }
    }
  }
      window.activeCustomFlags = [];
      this.loyaltyCardPage(null, true);
    },

    loyaltyCardPage: function(status, hideBack) {
      var self = this;
      self.storage.read(self.storageKeys.loyaltyCard, function(value) {
        var loyaltyCard = value;

        if (loyaltyCard === null) {
          self.loyaltyCard(hideBack);
        } else {
          self.loyaltyCardDetails(self.back ? null : status, hideBack);
        }
      });
    },

    loyaltyCard: function(hideBack) {
      var that = this;

      require(['views/loyaltyCardView'], function(loyaltyCardView) {
        window.hidePageLoadingMessage();
        that.changePage(new loyaltyCardView({
          hideBackButton: hideBack
        }));
      });
    },

    loyaltyCardSignup: function() {
      var that = this;

      window.showPageLoadingMessage();

      require(['views/loyaltyCardSignupView', 'text!templates/LoyaltyCardSignupView.html', 'models/loyaltyAccountModel'], function(loyaltySignupView, template, loyaltyAccountModel) {
        window.hidePageLoadingMessage();
        var view = new loyaltySignupView({
          model: new loyaltyAccountModel()
        });
        view.tmpl = template;
        that.changePage(view);
      });
    },

    loyaltyCardSignupStep2: function() {
      var that = this;

      window.showPageLoadingMessage();

      require(['views/loyaltyCardSignupView', 'text!templates/LoyaltyCardSignupStep2View.html'], function(loyaltySignupView, template) {
        window.hidePageLoadingMessage();
        var view = new loyaltySignupView();
        view.tmpl = template;
        that.changePage(view);
      });
    },

    loyaltyCardSignupFinalStep: function() {
      var that = this;
      require(['views/loyaltyCardSignupView', 'text!templates/LoyaltyCardSignupFinalStepView.html', 'models/loyaltyAccountModel'], function(loyaltySignupView, template, loyaltyAccountModel) {

        window.hidePageLoadingMessage();

        var view = new loyaltySignupView({
          model: new loyaltyAccountModel()
        });
        view.tmpl = template;
        that.changePage(view);
      });
    },

    loyaltyCardDetails: function(page, hideBack) {
      var self = this;

      require(['sharedhelpers/configurationProvider', 'views/loyaltyCardDetailsView', 'views/loyaltyCardAccountProfileView', 'models/loyaltyAccountModel', 'sharedhelpers/errorHelper'], function(configurationProvider, loyaltyCardDetailsView, loyaltyCardAccountProfileView, loyaltyAccountModel) {
        self.storage.read(self.storageKeys.loyaltyCard, function(value) {
          var loyaltyCard = value || {};
          var cardNumber = loyaltyCard.number;
          var lastName = loyaltyCard.lastName;
          var securityKey;

          if (!cardNumber || !lastName) {
            window.hidePageLoadingMessage();
            self.loyaltyCard();
            return;
          }

          if (configurationProvider.usePwdForLoyaltyLogin() && loyaltyCard.password) {
            securityKey = loyaltyCard.password;
          } else {
            securityKey = lastName;
          }

          var model = new loyaltyAccountModel({
            id: cardNumber,
            securityKey: securityKey
          });

          var loyaltyCardDetails = function(loyaltyError) {
            var hideNewOffer = _.all(model.get('loyaltyBanners'), function(banner) {
              return _.find(loyaltyCard.offers, function(offer) {
                return offer.imageUrl == banner.imageUrl;
              });
            });

            //Save new loyalty banners each time we do a loyalty call
            loyaltyCard.offers = model.get('loyaltyBanners');
            self.storage.store(self.storageKeys.loyaltyCard, loyaltyCard);

            if (!loyaltyCard.multitype) {
              self.changePage(new loyaltyCardDetailsView({
                model: model,
                status: page,
                hideNewOffer: hideNewOffer,
                loyaltyError: loyaltyError,
                hideBackButton: hideBack
              }));
              return;
            }

            require(['helpers/paymentMethodHelper'], function() {
              var paymentMethod = new PaymentMethodsHelper();
              paymentMethod.get(name, function(paymentMethods) {
                var giftCard = _.find(paymentMethods, function(card) {
                  return card.CardNumber == cardNumber;
                });

                model.set('multitype', loyaltyCard.multitype);
                self.changePage(new loyaltyCardDetailsView({
                  model: model,
                  giftCard: giftCard,
                  status: page,
                  hideNewOffer: hideNewOffer,
                  hideBackButton: hideBack
                }));
              });
            });
          };

          if (ConfigHelper.isDeviceOnline() == false) {
            _.defer(function() {
              model.set('cardNumber', {
                value: cardNumber
              });
              loyaltyCardDetails(true);
            });
            return;
          }

          model.fetch({
            success: function() {
              window.hidePageLoadingMessage();

              if (model.get('cardNumber') === undefined) {
                self.loyaltyCard();
                return;
              }

              if (page == 'edit') {
                self.changePage(new loyaltyCardAccountProfileView({
                  model: model
                }));

                return;
              }

              loyaltyCardDetails();
            },
            error: function() {

              _.defer(function() {
                model.set('cardNumber', {
                  value: cardNumber
                });
                loyaltyCardDetails(true);
              });
              return;
            }
          });
        });
      });
    },

    selectTickets: function(pPerformance, pMovie, pTheater, onHoldConfirmationNumber, facebookId) {
      window.showPageLoadingMessage();

      var that = this;

      var startDeepLinking = onHoldConfirmationNumber || !window.AppProperties.SelectedPerformance;

      var continueFlow = function() {
        window.showPageLoadingMessage();

        that.storage.read(that.storageKeys.loyaltyCard, function(value) {
          var loyaltyHolder = null;

          if (value) {
            loyaltyHolder = {
              name: value.firstName + ' ' + value.lastName,
              number: value.number.substring(value.number.length - 4, value.number)
            };
          }
          require(['views/ticketsView', 'collections/ticketCollection'], function(ticketsView, ticketCollection) {
            var theater;

            if (pTheater == 0 && window.AppProperties.SelectedTheater) {
              theater = window.AppProperties.SelectedTheater;
            } else {
              theater = window.ObjectCollections.TheaterCollection.get(pTheater);
            }

            var movie = theater.movies.getMovie(pMovie);
            if (!movie) {
              movie = window.ObjectCollections.MovieCollection.getMovie(pMovie);
            }

            var loadSelectTickets = function() {
              var changePageOptions = {
                performance: window.AppProperties.SelectedPerformance,
                movie: movie,
                theater: theater,
                loyaltyHolder: loyaltyHolder,
                router: that,
                back: that.back
              };

              if (facebookId) {
                changePageOptions["facebookId"] = facebookId;
              }

              if (onHoldConfirmationNumber) {
                changePageOptions["onHoldConfirmationNumber"] = onHoldConfirmationNumber;
              }

              if (window.AppProperties.SelectedPerformance.tickets) {
                window.hidePageLoadingMessage();
                that.changePage(new ticketsView(changePageOptions));
              } else {
                var tickets = new ticketCollection();
                tickets.getTickets(pPerformance, pTheater, function(result) {
                  window.hidePageLoadingMessage();

                  if (result != undefined) {
                    changePageOptions["pTheater"] = pTheater;
                    changePageOptions["pPerformance"] = pPerformance;
                    changePageOptions["tickets"] = result;

                    that.changePage(new ticketsView(changePageOptions));
                  }
                });
              }
            };

            if (startDeepLinking) {
              require(['helpers/deepLinkingHelper'], function() {
                var deepLinking = new DeepLinkingHelper();

                var performanceValid = deepLinking.validatePerformance(theater, movie, pPerformance);
                if (performanceValid) {
                  loadSelectTickets();
                }
              });
            } else {
              loadSelectTickets();
            }
          });
        });
      };

      //If not deeplinking continue normal flow
      if (!startDeepLinking) {
        continueFlow();
        return;
      }

      //Handle deeplinking
      require(['helpers/deepLinkingHelper'], function() {
        var deepLinking = new DeepLinkingHelper();
        var args = {
          callback: continueFlow,
          theaterId: pTheater,
          movieId: pMovie
        };
        deepLinking.getSelectTicketsData(args);
      });
    },

    selectRecipient: function(id, confirmationNumber) {
      var that = this;

      require(['views/selectRecipientView'], function(selectRecipientView) {
        window.hidePageLoadingMessage();

        that.changePage(new selectRecipientView({
          id: id,
          confirmationNumber: confirmationNumber
        }));
      });
    },

    selectSeats: function(pTheater, pPerformance) {
      var that = this;

      window.showPageLoadingMessage();

      require(['views/selectSeats', 'collections/seatCollection'], function(selectSeatsView, seatCollection) {
        if (!window.performanceLayout) {
          var seats = new seatCollection({
            theaterId: pTheater,
            performanceId: pPerformance
          });

          seats.fetch({
            success: function(result) {
              window.performanceLayout = result;

              window.hidePageLoadingMessage();

              that.changePage(new selectSeatsView({
                router: that
              }));
            }
          });
        } else {
          window.hidePageLoadingMessage();

          that.changePage(new selectSeatsView({
            router: that
          }));
        }
      });
    },

    menuComingSoon: function() {
      if (window.device && window.device.platform === "iOS") {
      let deviceVersion = window.device.version;
      if (deviceVersion.slice(0, deviceVersion.indexOf('.')) === "11" || deviceVersion.slice(0, deviceVersion.indexOf('.')) === "10") {
      while (window.myhistory.length > 0) {
        window.myhistory.pop();
      }
    }
  }
      window.activeCustomFlags = [];
      this.comingSoon(null, true);
    },

    comingSoon: function(sort, hideBack) {
      var that = this;
      require(['views/listMoviesView', 'collections/theaterCollection', 'sharedhelpers/configurationProvider'], function(listMoviesView, theaterCollection, configurationProvider) {
        if (configurationProvider.chainType() === 'single') {
          that.changePage(new listMoviesView({
            movies: window.ObjectCollections.ComingSoonCollection,
            sort: sort,
            hideBackButton: hideBack,
            comingSoon: true
          }));
        } else {
          var theatersCol = new theaterCollection();

          var latitude = AppProperties.CurrentPosition.Latitude;
          var longitude = AppProperties.CurrentPosition.Longitude;

          theatersCol.getTheaters(latitude, longitude, function(result) {

            window.ObjectCollections.TheaterCollection = result;
            window.ObjectCollections.MovieCollection = result.getMovies(null, true);

            window.ObjectCollections.ComingSoonCollection = new MovieCollection(_.filter(window.ObjectCollections.MovieCollection.models, function(movie) {
              return movie.get('releaseDate') > DtHelper.getBussinessDate(new Date());
            }));

            if (window.giftCardHelper.clearTheaterId() && (window.ObjectCollections.TheaterCollection.models && window.ObjectCollections.TheaterCollection.models.length > 0)) {
              window.giftCardHelper.setDefaultOptions(window.ObjectCollections.TheaterCollection.models[0]);
            }

            if (window.appConfig.GroupMovies) {
              window.ObjectCollections.MovieCollection.groupedMovies = window.ObjectCollections.MovieCollection.groupMultiTypeMovies();
              window.ObjectCollections.ComingSoonCollection.groupedMovies = window.ObjectCollections.ComingSoonCollection.groupMultiTypeMovies();
            }
            that.changePage(new listMoviesView({
              movies: window.ObjectCollections.ComingSoonCollection,
              sort: sort,
              hideBackButton: hideBack,
              comingSoon: true
            }));
          });
        }
      });
    },

    menuSettings: function() {
      window.activeCustomFlags = [];
      this.settings(true);
    },

    settings: function(hideBack) {
      var self = this;
      if (window.device && window.device.platform === "iOS") {
       let deviceVersion = window.device.version;
       if (deviceVersion.slice(0, deviceVersion.indexOf('.')) === "11" || deviceVersion.slice(0, deviceVersion.indexOf('.')) === "10") {
      while (window.myhistory.length > 0) {
        window.myhistory.pop();
      }
    }
  }
      require(['helpers/touchIdHelper'], function() {
        var touchIdHelper = new TouchIdHelper();
        touchIdHelper.isTouchIdAvailable(function(result) {
          window.AppProperties.isTouchIdAvailable = result;
        }, function() {
          window.AppProperties.isTouchIdAvailable = result;
        });
      });

      require(['views/settingsView'], function(settingsView) {
        var view = new settingsView({
          hideBackButton: hideBack
        });
        window.hidePageLoadingMessage();

        self.changePage(view);
      });
    },

    locationSettings: function() {
      var that = this;

      require(['views/locationSettingsView'], function(view) {
        that.changePage(new view());
      });
    },

    menuPurchaseHistory: function() {
      window.activeCustomFlags = [];
      this.purchaseHistory(null, true);
    },

    purchaseHistory: function(confirmationNumber, hideBack) {
      var that = this;

      var historyCallback = function() {
        require(['views/purchaseHistoryView'], function(purchaseHistoryView) {
          var orders = window.AppProperties.HistoryOrders;
          var now = new Date().toISOString();

          if (confirmationNumber && !window.AppProperties.HistoryOrderIndex) {
            for (var i = 0; i < orders.length; i++) {
              if (orders[i].confirmationNumber == confirmationNumber) {
                window.AppProperties.HistoryOrderIndex = i;
                break;
              }
            }
          }

          var selectedIndex = window.AppProperties.HistoryOrderIndex != null ? window.AppProperties.HistoryOrderIndex : _.filter(orders, function(item) {
            var date = item.performance ? new Date(item.performance.showTime) : new Date(item.purchasedDateTime);
            return date.toISOString() < now;
          }).length;

          window.AppProperties.HistoryOrderIndex = null;

          if (orders.length <= selectedIndex) {
            selectedIndex = selectedIndex - 1;
          }

          window.hidePageLoadingMessage();
          that.changePage(new purchaseHistoryView({
            orders: orders,
            selectedIndex: selectedIndex,
            hideBackButton: hideBack
          }));
        });
      };

      require(['js/helpers/storageHelper'], function() {
        that.storage.read(that.storageKeys.purchaseHistory, function(orders) {
          if (!orders) {
            return;
          }

          window.AppProperties.HistoryOrders = _.sortBy(orders, function(item) {
            var date = item.performance ? new Date(item.performance.showTime) : new Date(item.purchasedDateTime);
            return date.toISOString();
          });

          _historyCleanup();
        });

        function _historyCleanup() {
          if (!window.storageHelper.isStorageCapacityFull()) {
            historyCallback();
            return;
          }

          that.storage.read(that.storageKeys.historyCleanup, function(value) {
            if (value === true) {
              window.storageHelper.deletePurchaseHistory(historyCallback);
            } else if (value === false) {
              historyCallback();
            } else {
              window.storageHelper.deletePurchaseHistoryDialog(historyCallback);
            }
          });
        }
      });
    },

    purchaseHistoryOrderDetails: function(orderId) {
      var that = this;

      window.showPageLoadingMessage();

      require(['views/purchaseHistoryOrderDetailsView'], function(purchaseHistoryOrderDetailsView) {
        var order = _.find(window.AppProperties.HistoryOrders, function(o, index) {
          if (o.confirmationNumber == orderId) {
            window.AppProperties.HistoryOrderIndex = index;
            return true;
          }

          return false;
        });

        var culture = 'en-US'; //setting default culture
        if (order.theaterCulture)
          culture = order.theaterCulture;

        window.hidePageLoadingMessage();

        that.changePage(new purchaseHistoryOrderDetailsView({
          selectedTickets: order.selectedTickets,
          movieName: order.movieName,
          performance: order.performance,
          theaterName: order.theaterName,
          displayNetTicketPrice: order.theaterDetails.displayNetTicketPrice,
          totalPrice: order.totalPrice,
          tax: order.tax,
          subTotal: order.subTotal,
          otherItems: order.otherItems,
          theaterCulture: culture,
          purchasedDateTime: order.purchasedDateTime,
          concessions: order.concessions
        }));
      });
    },

    paymentMethodList: function(isVgc) {
      var self = this;
      window.showPageLoadingMessage();

      require(['views/paymentMethodSettingsView', 'helpers/paymentMethodHelper'], function(paymentMethodSettingsView) {
        var paymentMethod = new PaymentMethodsHelper();
        window.hidePageLoadingMessage();

        paymentMethod.get(name, function(paymentMethods) {
          //Marcus fix ... display gift cards on the payment method settings list

          //paymentMethods = _.filter(paymentMethods, function (card) {
          //    return isVgc ? card.Type == 'VirtualGiftCard' : card.Type != 'VirtualGiftCard';
          //});

          self.changePage(new paymentMethodSettingsView({
            paymentMethods: paymentMethods,
            isVgc: isVgc
          }));
        });
      });
    },

    paymentMethodDetails: function(name) {
      var self = this;
      window.showPageLoadingMessage();

      require(['views/paymentMethodSettingsDetailsView', 'helpers/paymentMethodHelper'], function(paymentMethodSettingsDetailsView) {
        if (name === 'new') {
          self.changePage(new paymentMethodSettingsDetailsView({
            paymentMethod: [{}]
          }));
        } else {
          var paymentMethodHelper = new PaymentMethodsHelper();
          window.hidePageLoadingMessage();

          paymentMethodHelper.get(name, function(paymentMethod) {
            self.changePage(new paymentMethodSettingsDetailsView({
              paymentMethod: paymentMethod
            }));
          });
        }
      });
    },

    menuContactUs: function() {
      window.activeCustomFlags = [];
      var that = this;
      if (window.device && window.device.platform === "iOS") {
      let deviceVersion = window.device.version;
      if (deviceVersion.slice(0, deviceVersion.indexOf('.')) === "11" || deviceVersion.slice(0, deviceVersion.indexOf('.')) === "10") {
      while (window.myhistory.length > 0) {
        window.myhistory.pop();
      }
    }
  }
      require(['views/contactUsView'], function(contactUs) {
        that.changePage(new contactUs());
      });
    },

    seatsPreview: function(movieId, theaterIdsString) {
      var that = this;
      var theaters = [];
      _.each(theaterIdsString.split(','), function(theaterId) {
        theaters.push(_.find(window.ObjectCollections.TheaterCollection.models, function(t) {
          return t.get('id') == theaterId;
        }));
      });
      window.showPageLoadingMessage();
      require(['views/seatsPreview'], function(seatsPreview) {
        var movieGroup;
        var movie;

        if (window.appConfig.GroupMovies) {
          window.ObjectCollections.MovieCollection.groupedMovies = window.ObjectCollections.MovieCollection.groupMultiTypeMovies();
          movieGroup = _.find(window.ObjectCollections.MovieCollection.groupedMovies, function(g) {
            var movieById = _.find(g, function(m) {
              return m.get('id') === movieId;
            });
            return movieById;
          });
        } else {
          movie = _.find(window.ObjectCollections.MovieCollection.models, function(m) {
            return m.get('id') === movieId;
          });
          window.AppProperties.SelectedMovie = movie;
        }

        var spv = new seatsPreview({
          movie: movie,
          movieGroup: movieGroup,
          theaters: theaters
        });

        if (spv.performances.length > 0) {
          window.SeatPreviewHelper.getPerformancesLayouts(spv.performances, spv.theaters[0], function(layouts) {
            spv.layouts = layouts;
            if (spv.overrides) {
              that.changePage(spv);
            }
          });

          window.SeatPreviewHelper.getSeatsPreview(spv.performances, spv.theaters[0], function(overrides) {
            spv.overrides = overrides;
            if (spv.layouts) {
              that.changePage(spv);
            }
          });
        } else {
          that.changePage(spv);
        }

      });
    },
  });

  var initialize = function() {

    var app_router = new AppRouter;

    $(document).ajaxError(function(e, jqxhr, settings, exception) {
      var error = 'Url: ' + settings.url + '\nResult: ' + e.result + '\nException: ' + exception;

      window.GoogleAnalyticsHelper.trackAjaxError(error);
      console.log(error);
    });

    window.onerror = function _unhandledException(msg, url, lineNumber) {
      var unhandledException = 'Unhandled exception.\nLine: ' + lineNumber + '\nUrl: ' + url + '\nMessage: ' + msg;

      window.GoogleAnalyticsHelper.trackJSError(unhandledException);
      console.log(unhandledException);
      return false;
    };

    Backbone.history.start();

    function checkValue(value, arr) {
      for (var i = 0; i < arr.length; i++) {
        var name = arr[i];
        if (name.fragment.toString() == value.toString()) {
          return true;
        }
      }
      return false;
    }

      if (window.device && window.device.platform === "iOS") {
    let deviceVersion = window.device.version;
    if (deviceVersion.slice(0, deviceVersion.indexOf('.')) === "11" || deviceVersion.slice(0, deviceVersion.indexOf('.')) === "10") {
      Backbone.history.on("route", function(router, route) {
        console.log("On route:" + route);
        if (checkValue(Backbone.history.fragment, window.myhistory)) {
          console.log(window.myhistory);
        } else {
          window.myhistory.push({
            fragment: Backbone.history.fragment
          });
        }

      });
    }
  }

    require(['helpers/googleAnalyticsHelper'], function() {
      Backbone.history.on("route", function(router, route) {
        window.GoogleAnalyticsHelper.trackPage(route);
      });
    });
  };
  return {
    initialize: initialize
  };
});
