define([
  'text!templates/CheckoutSummaryView.html',
  'views/pullDownStatusView',
  'sharedhelpers/configurationProvider',
  'collections/concessionCollection'
], function (template, pullDownStatusView, configurationProvider, concessionCollection) {
    var view = Backbone.View.extend({
        template: _.template(template),
        localize: window.ObjectCollections.Localization.result,

        initialize: function () {
            window.tempHistoryCount++;

            this.TicketCount = 1;
        },

        render: function () {
            $(this.el).html(this.template({
                cosModel: this.options.cosModel,
                selectedTickets: this.options.selectedTickets,
                movie: this.options.movie,
                performance: this.options.performance,
                theater: this.options.theater,
                concessions: this.options.concessions,
                localize: this.localize,
                configurationProvider: configurationProvider
            }));

            this.pullDownStatusView = new pullDownStatusView();

            return this;
        },

        events:
    {
        "tap #removeItem": "removeItem",
        "tap .item-none-tap": "itemTap",
        "tap #btnCheckout": "btnCheckout",
        "tap #btnConcessions": "btnConcessions"
    },

        btnCheckout: function (e) {
            e.preventDefault();
            if (isJqmGhostClick(e) || isChangePageFired(e)) return;

            window.showPageLoadingMessage();


            var orderModel = window.ObjectModels.OrderModel;
            var concessions = orderModel.get('concessions').models || orderModel.get('concessions');
            if (!orderModel.get('tickets') && concessions.length == 0) {
                window.showAlert(this.localize.errorMustHaveItemToCompletePurchase);
                window.hidePageLoadingMessage();
                return;
            }

            var callback = function (success) {
                if (success || orderModel.get('tickets')) {
                    window.ObjectModels.TempOrder.concessionsChanged = false;
                    window.AppMethods.ChangePage('#paymentMethod');
                } else {
                    window.hidePageLoadingMessage();
                }
            };

            if (window.ObjectModels.TempOrder.concessions) {
                var tempConcessions = window.ObjectModels.TempOrder.concessions;
                var clonedCollection = new concessionCollection();

                tempConcessions.each(function (model) {
                    clonedCollection.add(new Backbone.Model(JSON.parse(JSON.stringify(model))));
                });

                window.ObjectModels.OrderModel.set('concessions', clonedCollection);
            }

            new ConcessionCollection().lockConcessions(callback, window.ObjectModels.TempOrder.concessionsChanged);
        },

        btnConcessions: function (e) {
            e.preventDefault();
            if (isJqmGhostClick(e) || isChangePageFired(e)) return;

            if (ConfigHelper.isDeviceOnline() == false) {
                window.showAlert(this.localize.errorUnableConnectToInternet);
                return;
            }

            window.AppMethods.ChangePage('concessions/' + window.ObjectModels.OrderModel.get('theater').get('id') + '/null');
        },

        removeItem: function (e) {
            e.preventDefault();
            var orderModel = window.ObjectModels.OrderModel;
            var totalPrice = $('#totalPrice');
            var subTotal = $('#subtotal');
            var tax = $('#tax');
            var modifierTax = 0;
            var modifierPrice = 0;

            var taxPrice = parseFloat(tax.attr('data-tax'));

            var result = $.grep(orderModel.get('concessions').models, function (c) { return c.get('concessionItemId') == $(e.currentTarget).parents(".item-taped:first").children('.item-concession').val(); })[0];

            var price = result.get('price');
            var quantity = result.get('quantity');
            var newPrice = price;


            if (quantity > 0) {
                var concessions = orderModel.get('concessions');
                var tempConcessions = window.ObjectModels.TempOrder.concessions;

                var tempConcession = $.grep(tempConcessions.models, function (t) {
                    return result.get('concessionItemId').indexOf(t.get('concessionItemId')) != -1;
                });

                var removedConcession = tempConcessions.models.splice(tempConcessions.indexOf(tempConcession[0]), 1)[0];

                if (quantity == 1) {
                    concessions.models.splice(concessions.indexOf(result), 1);
                    $(e.currentTarget).parents(".item-taped:first").next(".breakLine").remove();
                    $(e.currentTarget).parents('.item-taped:first').remove();
                }

                var modifiers = result.get('modifiers');
                if (modifiers && modifiers.length > 0) {

                    _.each(removedConcession.get('modifiers'), function (r) {
                        var remove = $.grep(modifiers, function (m) { return m.productId == r.productId; })[0];

                        modifiers.splice(modifiers.indexOf(remove), 1);

                        newPrice -= r.price;
                        modifierPrice += r.price;
                        if (r.price != 0) {
                            taxPrice -= r.tax;
                            modifierTax += r.tax;
                        }                     
                    });
                    result.set('modifiers', modifiers);

                    var modifierNames = '';
                    var unique = {};
                    _.each(modifiers, function (modifier) {
                        if (modifier.name != undefined && !unique[modifier.name]) {
                            modifierNames += modifier.name + ', ';
                            unique[modifier.name] = modifier.name;
                        }
                    });
                }
                var comboItems = removedConcession.get('comboItems');
                _.each(comboItems, function(comboItem) {
                    _.each(comboItem.modifiers, function(item) {
                        modifierPrice += item.price;
                        if (item.price != 0) {
                            taxPrice -= item.tax;
                            modifierTax += item.tax;
                        }
                    });
                });

                newPrice -= removedConcession.get('price');
                taxPrice -= removedConcession.get('tax');

                quantity -= 1;
                result.set('price', newPrice);
                result.set('quantity', quantity);

                $(e.currentTarget).parents('.item-taped:first').find("#item-modifiers").text(modifierNames ? modifierNames.slice(0, -2) : '');

                var iName = $(e.currentTarget).parents(".item-taped:first").find('.item-name');
                var iPrice = $(e.currentTarget).parents(".item-taped:first").find('span.item-price');
                iName.html(quantity + ' ' + result.get('name'));
                iPrice.html(Globalize.format(newPrice - removedConcession.get('tax'), 'c', this.options.theater.get('culture')));

                if (!concessions.models.length)
                    $('[data-horbar=concession]').remove();
            }

            totalPrice.attr('data-totalprice', parseFloat(totalPrice.attr('data-totalprice')) - (removedConcession.get('price') + modifierPrice));
            totalPrice.html(this.localize.total + ' ' + Globalize.format(parseFloat(totalPrice.attr('data-totalprice')), 'c', this.options.theater.get('culture')));

            var subtotalValue= window.ObjectModels.OrderModel.get('subTotal') - (removedConcession.get('price') + modifierPrice - removedConcession.get('tax') - modifierTax);
            subTotal.html("Subtotal: " + Globalize.format(subtotalValue, 'c', this.options.theater.get('culture')));
            tax.html("Tax: " + Globalize.format(taxPrice, 'c', this.options.theater.get('culture')));
            tax.attr('data-tax', taxPrice);

            window.ObjectModels.OrderModel.set('subTotal', subtotalValue);

            window.ObjectModels.OrderModel.set('tax', taxPrice);
            
            window.ObjectModels.TempOrder.concessionsChanged = true;

            return false;
        },

        itemTap: function (e) {
            e.preventDefault();

            $('.item-none-tap').not(e.currentTarget).removeClass('item-taped').children('.removeItemHolder').hide();

            $(e.currentTarget).toggleClass('item-taped');
            if ($(e.currentTarget).attr('class') == 'item-none-tap item-taped') {
                $(e.currentTarget).children('.removeItemHolder').show();
            } else {
                $(e.currentTarget).children('.removeItemHolder').hide();
            }

            return false;
        }

    });

    return view;
});