define([
  'text!templates/ConcessionsView.html',
  'views/pullDownStatusView',
  'collections/concessionCollection'
], function (template, pullDownStatusView, concessionCollection) {
    var view = Backbone.View.extend({
        template: _.template(template),
        localize: window.ObjectCollections.Localization.result,

        initialize: function () {
            window.tempHistoryCount++;

            this.pullDownStatusView = null;
        },

        events: {
            "tap .concession": "onSelectConcessionItem",
            "tap #conCheckout": "checkout",
            'pageshow': 'onShow'
        },

        onShow: function () {
            var that = this;
            if (that.options.status == 'new') {
                require(['helpers/uiHelper'], function () {
                    var pullDownStatusHolder = $('.pullDownStatusHolder');

                    pullDownStatusHolder.css({ 'opacity': '0' });

                    var noteCallback = function () {
                        pullDownStatusHolder.css({ 'opacity': '100' });
                    };

                    UIHelper.notificationBar(that.localize.concessionItemAdded, noteCallback);
                });
            }
        },

        onSelectConcessionItem: function (event) {
            if (isJqmGhostClickPropagated(event) || isJqmGhostClick(event) || isChangePageFired(event)) return;

            event.preventDefault();

            var name = $(event.currentTarget).attr("data-item-name");
            var productIds = $(event.currentTarget).attr("data-item-id-list");

            window.AppMethods.ChangePage('concessionItem/' + encodeURIComponent(name) + '/' + productIds + '/' + this.options.theaterCulture + '/' + new Date().getTime());
        },

        checkout: function (event) {
            if (isJqmGhostClickPropagated(event) || isJqmGhostClick(event)) return;

            event.preventDefault();
            window.showPageLoadingMessage();

            var orderModel = window.ObjectModels.OrderModel;
            var concessions = orderModel.get('concessions');
            var clonedCollection = new concessionCollection();

            concessions.each(function (model) {
                clonedCollection.add(new Backbone.Model(JSON.parse(JSON.stringify(model))));
            });

            window.ObjectModels.TempOrder.concessions = clonedCollection;

            var callback = function (success, errorCode) {
                if (errorCode) {
                    window.hidePageLoadingMessage();

                    new ErrorHelper().showAlertByErrorCode(errorCode);
                    return;
                }

                if (!orderModel.get('tickets') && success == null) {
                    window.showAlert(window.ObjectCollections.Localization.result.errorAddItemToContinue);
                    window.ObjectModels.TempOrder.concessions = null;
                } else if (orderModel.get('tickets') || success) {
                    if (!success)
                        orderModel.set('concessions', new ConcessionCollection());

                    window.AppMethods.ChangePage('#checkoutSummary');
                }

                window.hidePageLoadingMessage();
            };

            new ConcessionCollection().lockConcessions(callback);
        },

        render: function () {
            var that = this;

            if (window.ObjectModels.TempOrder.concessions) {
                window.ObjectModels.OrderModel.set('concessions', window.ObjectModels.TempOrder.concessions);
            }

            that.$el.html(that.template({
                localize: that.localize,
                theaterCulture: that.options.theaterCulture,
                concessions: that.options.concessions,
                hideBackButton: that.options.hideBackButton
            }));

            that.pullDownStatusView = new pullDownStatusView();
            that.assign(that.pullDownStatusView, '.pullDownStatusHolder');

            return that;
        },

        assign: function (childView, selector) {
            this.$(selector).empty();
            childView.setElement(this.$(selector)).render();
            this.$(selector).enhanceWithin();
        }
    });

    return view;
});