define([
  'text!templates/ConfirmHoldSeatsView.html',
  'sharedhelpers/configurationProvider',
  'facebook',
  'sharedhelpers/utilities'
], function (template, configurationProvider) {
    var view = Backbone.View.extend({
        template: _.template(template),
        localize: window.ObjectCollections.Localization.result,

        initialize: function () {
            var that = this;

            that.facebook = new FacebookHelper();
            that.ticketShared = false;
            that.initialFriends = [];

            _.each(that.options.friends, function (f) {
                that.initialFriends.push(f);
            });
        },

        render: function () {
            $(this.el).html(this.template({
                friends: this.options.friends,
                ticketQuantity: this.options.holdTicketQuantity,
                movie: this.options.movie,
                holdMessage: this.getFbHoldMsg(),
                performance: this.options.performance,
                theater: this.options.theater,
                localize: this.localize
            }));

            return this;
        },

        events:
        {
            "tap #sendInvitation": "sendInvitation",
            "tap #cancel": "cancel"
        },

        cancel: function (event) {
            event.preventDefault();

            var onConfirm = function (button) {
                if (button == 1) {
                    window.AppMethods.ChangePage("#myTicketView");
                    return;
                }
            };

            if (window.isMobileDevice()) {
                navigator.notification.confirm(this.localize.giveUpHoldSeats, onConfirm, this.localize.alertTitle, this.localize.ok + ', ' + this.localize.cancel);
                this.removeHoldSeats();
            } else if (confirm(this.localize.giveUpHoldSeats)) {
                // TODO: Send request to the CloudAPI to give up seats!
                this.removeHoldSeats();
                window.AppMethods.ChangePage("#myTicketView");
                return false;
            }

            return false;
        },
        removeHoldSeats: function () {
            var that = this;
            require(['sharedhelpers/storageProvider'], function () {
                var provider = new StorageProvider();
                provider.read('PurchaseHistory', function (item) {
                    var temp = $.grep(item, function (sItem) { return sItem.confirmationNumber === that.options.confirmationNumber; })[0];
                    var selectedTicketsArray = [];
                    _.each(temp.selectedTickets, function (t) {
                        if (t.name.toUpperCase() != 'HOLD') {
                            selectedTicketsArray.push(t);
                        }
                    });
                    temp.selectedTickets = selectedTicketsArray;

                    provider.remove('PurchaseHistory');
                    provider.store('PurchaseHistory', item);
                });
            });
        },
        sendInvitation: function (e) {
            if (e && isJqmGhostClick(e)) return;
            var that = this;

            var nextFriend = that.options.friends[that.options.friends.length - 1];

            var utilities = new Utilities();

            var fbLink = appConfig.SiteURL +
                '#tickets/'
                + $.trim(that.options.performance.get('number')) + "/"
                + $.trim(that.options.movie.get('code')) + "/"
                + $.trim(that.options.theater.get('id')) + "/"
                + that.options.confirmationNumber + "/"
                + encodeURIComponent(utilities.encrypt(nextFriend.id));

            that.facebook.post(that.token,
            {
                name: that.options.movie.get('name'),
                caption: that.localize.holdSeatsFbCaption,
                description: that.getFbMessage(that.options.movie.get('name'), that.options.theater.get('name'), that.options.performance.get('showTime'), that.options.holdTicketQuantity, that.options.theater.get('culture')),
                link: fbLink,
                picture: that.options.movie.get('image') ? that.options.movie.get('image')['medium'] || window.appConfig.DefaultMoviePosterImage : window.appConfig.DefaultMoviePosterImage,
                userList: nextFriend.id,
                actionType: 'hold',
                preventDelay: that.options.friends.length < 2
            },
            function (response) {
                if (response.success === true) {
                    that.ticketShared = true;
                }

                that.options.friends.pop();
                setTimeout(function () {
                    if (that.options.friends.length > 0) {
                        that.sendInvitation();
                    } else if (that.ticketShared) {
                        window.AppMethods.ChangePage("#myTicketView");
                    } else {
                        _.each(that.initialFriends, function (f) {
                            that.options.friends.push(f);
                        });
                    }
                }, 100);

                window.hidePageLoadingMessage();
            });
        },
        getFbHoldMsg: function () {
            var quantity = this.options.holdTicketQuantity;
            var holdSeatsMsg = $.sprintf(this.localize.fbHoldMessage,
                    quantity,
                   (quantity > 1) ? 's' : '',
                   (quantity > 1) ? 'these' : 'this',
                   (quantity > 1) ? 's' : '');

            return holdSeatsMsg;
        },
        getFbMessage: function (movieTitle, theater, date, ticketQuantity, theatreCulture) {
            var that = this;
            var showDate = DtHelper.convertDate(date);

            var dateValue = Globalize.format(showDate, "M", theatreCulture);
            var timeValue = Globalize.format(showDate, "t", theatreCulture);

            var holdTime = showDate.getTime() - configurationProvider.holdTimeInMinutesBeforeShowTime(that.options.theater) * 60000;
            var maxHoldTime = new Date().getTime() + configurationProvider.holdTimeinMinutes(that.options.theater) * 60000;
            var holdDate = holdTime > maxHoldTime ? new Date(maxHoldTime) : new Date(holdTime);

            var holdDateValue = Globalize.format(holdDate, "M", theatreCulture);
            var holdTimeValue = Globalize.format(holdDate, "t", theatreCulture);

            var fbMsg = $.sprintf(this.localize.fbMessage,
                        movieTitle,
                        theater,
                        dateValue,
                        timeValue,
                        ticketQuantity,
                        holdDateValue,
                        holdTimeValue);

            return fbMsg;
        }
    });

    return view;
});