define([
  'text!templates/ContactUsView.html'
], function (template) {
    var view = Backbone.View.extend({
        template: _.template(template),
        localize: window.ObjectCollections.Localization.result,

        initialize: function () {
            this.contact = window.appConfig.ContactUsInformation;
        },

        events: {
            'tap .facebook-icon': 'openFb',
            'tap .twitter-icon': 'openTwitter',
            'tap .web-icon': 'openUrl',
            'tap #phone-url': 'openPhoneUrl',
            'tap #email-url': 'openEmailUrl'
        },

        render: function () {
            var that = this;
            that.phoneDevice = ((typeof window.device != 'undefined') && window.device.platform == 'Android') ? 'Android' : 'iOS';

            that.$el.html(that.template({
                localize: that.localize,
                description: that.contact.Description,
                phone: that.contact.Phone,
                email: that.contact.Email,
                facebook: that.contact.Facebook,
                twitter: that.contact.Twitter,
                webPage: that.contact.WebPage,
                phoneDevice: that.phoneDevice
            }));

            return that;
        },

        openFb: function () {
            var that = this;

            if (!window.appAvailability) {
                window.open('https://www.facebook.com/' + that.contact.Facebook, '_blank');
                return;
            }

            window.appAvailability.check(
                that.phoneDevice == 'Android' ? 'com.facebook.katana' : 'fb://',
                function () {
                    window.open('https://www.facebook.com/' + that.contact.Facebook, '_blank');
                },
                function () {
                    window.open('https://www.facebook.com/' + that.contact.Facebook, '_blank');
                });
        },

        openTwitter: function () {
            var that = this;

            if (!window.appAvailability) {
                window.open('https://twitter.com/' + that.contact.Twitter, '_blank');
                return;
            }

            window.appAvailability.check(
                that.phoneDevice == 'Android' ? 'com.twitter.android' : 'twitter://',
                function () {
                    window.open('twitter://user?screen_name=' + that.contact.Twitter, '_system');
                },
                function () {
                    window.open('https://twitter.com/' + that.contact.Twitter, '_blank');
                });
        },

        openUrl: function (e) {
            window.open(this.contact.WebPage, '_blank');
        },

        openPhoneUrl: function (e) {
            window.open("tel:" + this.contact.Phone, '_system');
        },

        openEmailUrl: function (e) {
            window.open("mailto:" + this.contact.Email, '_system');
        }
    });

    return view;
});