﻿define([
        'sharedhelpers/configurationProvider',
        'text!templates/CurtainsMenuView.html'
], function (configurationProvider, template) {
    var view = Backbone.View.extend({
        template: _.template(template),
        localize: window.ObjectCollections.Localization.result,

        initialize: function () {
            this.reloadTheaters = false;
            this.loadFinished = false;
            this.callback = null;
        },

        render: function () {
            this.$el.html(this.template({
                localize: this.localize,
                showConcessions: configurationProvider.chainType() == 'single' && configurationProvider.useConssesion()
            }));
            return this;
        },

        events: {
            "tap #theaters": "navigateTheaters",
            "tap #movies": "navigateMovies",
            "tap #buyConcessions": "navigateConcessions"
        },

        navigateTheaters: function (e) {
            e.preventDefault();
            this.navigate(e, 'theaters');
        },

        navigateMovies: function (e) {
            e.preventDefault();
            this.navigate(e, 'movies');
        },
        navigateConcessions: function (e) {
            e.preventDefault();
            if (isJqmGhostClick(e) || isChangePageFired(e)) return;

            _.defer(function () {
                window.tempHistoryCount = 0;
                window.ObjectModels.TempOrder = { concessions: null };

                window.AppMethods.ChangePage('concessions/' + configurationProvider.theaterId() + '/null');
            });
        },
        navigate: function (e, path) {
            if (isJqmGhostClick(e) || isChangePageFired(e)) return;

            var that = this;

            if (ConfigHelper.isDeviceOnline() == false) {
                window.showAlert(that.localize.errorUnableConnectToInternet);
                return;
            }
            window.showPageLoadingMessage();

            _.defer(function () {
                that.callback = function () { window.AppMethods.ChangePage(path); };

                //If unable to preload theaters, try now
                if (that.reloadTheaters) {
                    that.loadTheaters(false);
                } else if (that.loadFinished) {
                    that.callback();
                }
            });
        },

        loadTheaters: function (suppressErrors) {
            var that = this;

            var loadCallback = function () {
                require(['collections/theaterCollection', 'models/theaterModel', 'sharedhelpers/giftCardHelper'], function (theaterCollection, theaterModel) {
                    var theaters = new theaterCollection();

                    var latitude = AppProperties.CurrentPosition.Latitude;
                    var longitude = AppProperties.CurrentPosition.Longitude;

                    function _collectionHandler(result) {
                        that.loadFinished = true;
                        that.reloadTheaters = false;

                        window.ObjectCollections.TheaterCollection = result;
                        window.ObjectCollections.MovieCollection = result.getMovies();

                        if (window.giftCardHelper.clearTheaterId()) {
                            window.giftCardHelper.setDefaultOptions(window.ObjectCollections.TheaterCollection.models[0]);
                        }

                        if (window.appConfig.GroupMovies) {
                            window.ObjectCollections.MovieCollection.groupedMovies = window.ObjectCollections.MovieCollection.groupMultiTypeMovies();
                        }

                        window.AppProperties.SelectedTheater = null;

                        if (that.callback) {
                            that.callback();
                        }
                    }

                    function _handleErrors(result) {
                        if (!result) {
                            if (!suppressErrors) {
                                new ErrorHelper().showAlertByErrorCode();
                            } else {
                                that.reloadTheaters = true;
                            }

                            window.hidePageLoadingMessage();
                            return;
                        }
                    }

                    if (configurationProvider.chainType() == 'single') {
                        var singleTheaterModel = new theaterModel({ id: configurationProvider.theaterId() });
                        singleTheaterModel.getTheater(function (result) {
                            _handleErrors(result);

                            var theatersColl = new theaterCollection();
                            theatersColl.push(result);
                            _collectionHandler(theatersColl);
                        });
                    } else {
                        theaters.getTheaters(latitude, longitude, function (result) {
                            _handleErrors(result);
                            _collectionHandler(result);
                        });

                        theaters.abridgedMilesCall(latitude, longitude, function (response) {
                            if (response) {
                                var sortedByDistance = _.sortBy(response.models, function (model) {
                                    return model.get('distance');
                                });

                                window.ObjectCollections.AbridgedTheaterCollection = new theaterCollection(sortedByDistance);
                            }
                        });
                    }
                });
            };
            require(['sharedhelpers/storageProvider'], function () {
                var storage = new StorageProvider();
                storage.read('CurrentSearchLocation', function (value) {
                    if (value) {
                        window.AppProperties.CityOrZip = value.name;
                        window.AppProperties.CityOrZipSearch = value.search;
                        window.AppProperties.CurrentPosition = { Latitude: value.lat, Longitude: value.long };
                    }

                    require(['helpers/geolocationHelper'], function (geolocationHelper) {
                        if (configurationProvider.chainType() == 'multi') {
                            geolocationHelper.acquireCurrentPosition(loadCallback, true);
                        } else {
                            window.AppProperties.CurrentPosition = { Latitude: 0, Longitude: 0 };
                            loadCallback();
                        }
                    });
                });
            });
        }
    });

    return view;
});