define([
  'text!templates/EmailAddressDialogView.html',
  'libs/jquery.validate.min',
  'sharedhelpers/storageProvider',
  'sharedhelpers/cultureHelper'
], function (template) {
    var view = Backbone.View.extend({
        template: _.template(template),
        localize: window.ObjectCollections.Localization.result,

        initialize: function () {
            $("#frmEmailDialog").validate();
            this.redirect = false;
            this.emailAddressKey = 'emailAddress';

            this.setupValidation();
        },

        render: function () {
            this.$el.html(this.template({
                localize: this.localize,
                model: this.options.model
            }));

            return $(this.el);
        },

        events:
		{
		    "tap #ok": "saveEmail",
		    "tap #cancel": "cancel",
		    "keypress #txtEmail": "handleEmailKeypress"
		},

        setupValidation: function () {
            var cultureHelper = new CultureHelper();
            var emailValidation = cultureHelper.emailValidation();

            $.validator.addMethod('email', function (value) {
                return emailValidation.regex.test(value);
            }, emailValidation.msg);
        },

        cancel: function (event) {
            event.preventDefault();

            this.options.callback(false);
            return false;
        },

        handleEmailKeypress: function (e) {
            var that = this;

            if (e.keyCode == 13) {
                e.preventDefault();

                that.saveEmail(e);

                return;
            }
        },

        saveEmail: function (event) {
            event.preventDefault();

            if (this.$el.find('#frmEmailDialog').valid()) {
                //Remove email at first
                var storage = new StorageProvider();
                storage.remove(this.emailAddressKey);
                storage.store(this.emailAddressKey, this.$el.find('#txtEmail').val());
                this.options.callback(true);
            }

            return false;
        }
    });

    return view;
});