define([
	'text!templates/FacebookShareView.html',
    'facebook'
], function (template) {
    var facebookShareView = Backbone.View.extend({
        template: _.template(template),
        localize: window.ObjectCollections.Localization.result,

        initialize: function () {
            this.movieName = this.options.movieName;
            var theaterCulture = this.options.theaterCulture;
            var theatreName = this.options.theaterName === '' ? '' : 'at ' + this.options.theaterName;
            var showTime = this.options.showTime;
            this.shareText = "{text} {movieDetails} {theatreDetails} {timeDetails}";
            this.facebook = new FacebookHelper();
            this.postingToFacebook = false;
            this.caption = this.localize.confirmationMovie;

            if (showTime) {
                var date = new Date();
                var month = date.getMonth() + 1;
                var dateTime = date.getFullYear() + '-' + month + '-' + date.getDate() + 'T' + date.getHours() + ':' + date.getMinutes() + ':' + date.getSeconds();
                var showDate = DtHelper.convertDate(showTime);
                var diff = DtHelper.dateDiff(DtHelper.convertDate(dateTime), showDate);

                if (diff < 0) {
                    this.shareText = this.shareText.replace("{text}", "I'm going to see");
                    this.shareText = this.shareText.replace("{timeDetails}", 'on ' + Globalize.format(showDate, "M", theaterCulture) + ' at ' + Globalize.format(showDate, 't', theaterCulture));
                } else {
                    this.shareText = this.shareText.replace("{text}", "I just saw");
                    this.shareText = this.shareText.replace("{timeDetails}", '');
                }

                if (this.shareText.indexOf('{text}') > -1) {
                    this.shareText = this.shareText.replace("{text}", "I'm going to see");
                }
                if (this.shareText.indexOf('{timeDetails}') > -1) {
                    this.shareText = this.shareText.replace("{timeDetails}", '');
                }
            }
            else {
                this.shareText = this.shareText.replace("{text}", "I'm going to see");
                this.shareText = this.shareText.replace("{timeDetails}", '');
            }

            this.shareText = this.shareText.replace("{movieDetails}", this.movieName);
            this.shareText = this.shareText.replace("{theatreDetails}", theatreName);
        },
        render: function (callback) {
            var that = this;

            this.$el.html(this.template({ localize: that.localize }));
            this.callback = callback;
            $(this.el).find('#facebook-share-text').val(this.shareText);
            $("div[role='main']").css({ 'pointer-events': 'none' });

            return $(this.el);
        },
        events: {
            'tap #facebook-share-submit': 'facebookShareSubmit',
            'tap #facebook-share-close': 'closePanel'
        },
        facebookShareSubmit: function (e) {
            e.preventDefault();
            if (isJqmGhostClick(e)) return;

            if (ConfigHelper.isDeviceOnline() == false) {
                window.showAlert(this.localize.errorUnableConnectToInternet);
                return;
            }

            var self = this;
            if (self.postingToFacebook) return;

            self.postingToFacebook = true;

            if ($("#frmFacebookShare").valid()) {
                self.facebook.connect(function (token) {
                    if (!token) {
                        self.postingToFacebook = false;
                        return;
                    }

                    var options = {
                        name: self.movieName,
                        description: $("#facebook-share-text").val(),
                        caption: self.caption,
                        link: self.options.theaterUrl,
                        picture: self.options.img
                    };

                    self.facebook.share(token, options, function (response) {
                        if (!response || response.error) {
                            require(['sharedhelpers/errorHelper'], function () {
                                new ErrorHelper().showAlertByErrorCode();
                                self.postingToFacebook = false;
                            });
                            return;
                        }

                        self.callback(true);
                        setTimeout(function () {
                            $("div[role='main']").css({ 'pointer-events': '' });
                            self.postingToFacebook = false;
                        }, 500);

                        require(['helpers/uiHelper'], function () {
                            UIHelper.notificationBar(self.localize.facebookTimelineUpdated);
                        });
                    });
                });
            }
        },
        closePanel: function (e) {
            e.preventDefault();
            if (isJqmGhostClick(e)) return;
            var self = this;

            self.callback(true);
            setTimeout(function () {
                $("div[role='main']").css({ 'pointer-events': '' });
            }, 500);
        }
    });

    return facebookShareView;
});