define([
  'text!templates/HoldSeatsView.html',
  'text!templates/HoldSeatsViewFriendList.html',
  'views/pullDownStatusView',
  'sharedhelpers/configurationProvider',
  'models/orderModel',
  'facebook',
  'js/libs/jquery.sonar.min',
  'helpers/uiHelper',
  'helpers/saleHelper'
], function (template, templateFriendList, pullDownStatusView, configurationProvider) {
    var view = Backbone.View.extend({
        template: _.template(template),
        localize: window.ObjectCollections.Localization.result,

        initialize: function () {
            window.tempHistoryCount++;

            var that = this;
            var orderModel = window.ObjectModels.OrderModel;

            if (orderModel != null) {
                if (orderModel.get('saleId')) {
                    var savedOrderModel = window.ObjectModels.OrderModel;

                    savedOrderModel.cancelSale(function () {
                        window.performanceLayout = null;

                        orderModel = new OrderModel();
                        orderModel.set('tickets', savedOrderModel.get('tickets'));
                        orderModel.set('holdTickets', savedOrderModel.get('holdTickets'));
                        orderModel.set('performance', savedOrderModel.get('performance'));
                        orderModel.set('movie', savedOrderModel.get('movie'));
                        orderModel.set('theater', savedOrderModel.get('theater'));
                        orderModel.set('onHoldConfirmationNumber', savedOrderModel.get('onHoldConfirmationNumber'));
                        orderModel.set('performanceToday', savedOrderModel.get('performanceToday'));

                        window.ObjectModels.OrderModel = orderModel;
                    });
                }

                that.tickets = window.ObjectModels.OrderModel.get('tickets');

                that.onHoldTickets = that.tickets.where({
                    'isOnHoldTicketType': true
                })[0];

                if (!that.onHoldTickets) {
                    that.onHoldTickets = [];

                    _.each(window.ObjectModels.OrderModel.get('holdTickets').models, function (ht) {
                        window.ObjectModels.OrderModel.get('tickets').models.push(ht);
                        that.onHoldTickets.push(ht);
                    });

                    that.onHoldTickets = that.onHoldTickets[0];
                }


                if (!that.onHoldTickets.friends)
                    that.onHoldTickets.friends = [];

                that.pullDownStatusView = null;
            }
        },

        events: {
            'tap #continueToSelectSeats': 'onSkip',
            "tap #continueToCheckout": "continueToCheckout",
            'tap .connect-to-facebook-button': 'onLogin',
            'pagehide': 'onHide',
            "tap .tickets-add-ticket": "addTicket",
            "tap .tickets-remove-ticket": "removeTicket",
            "tap .search-field .ui-icon-delete": "resetFilterFriendsByName",
            "keypress div.ui-input-search": "filterFriendsByName",
            "pagebeforeshow": "onPageShow",
            "pagebeforehide ": "onPageHide"
        },

        showKeyboardHandler: function () {
            $('.hold-seats-footer-container').hide();
            $('form a').show();
            // IPAD fix for pulldown status 
            $(document).on('touchmove.ncr touchstart.ncr', function (event) {
                event.preventDefault();
            });
        },

        hideKeyboardHandler: function () {
            $(document).off("touchmove.ncr touchstart.ncr");
            $('.hold-seats-footer-container').show();
            $('form a').hide();

            var friendImages = $('.ui-listview').find('li:not(.ui-screen-hidden) .fb-friend-img');

            _.each(friendImages, function (user) {
                _.defer(function () {
                    user.src = user.dataset.src;
                });
            });
        },

        onPageHide: function () {
            $(document).off('hidekeyboard');
            $(document).off('showkeyboard');
        },

        onPageShow: function () {
            var that = this;

            that.facebook = new FacebookHelper();
            that.facebook.connect(function (result) {
                if (result) {
                    $(that.el).find('.hold-seats-content').hide();
                    that.getFBstuff(result);
                }
            }, { preventLogin: true });
        },

        resetFilterFriendsByName: function (e) {
            if (isJqmGhostClick(e)) return;

            $(this.el).find('input').val('');
            $(this.el).find('.ui-listview li').removeClass('ui-screen-hidden');
        },

        filterFriendsByName: function (e) {
            var that = this;

            if (e.keyCode == 13) {
                e.preventDefault();

                var input = $(e.currentTarget).find('input');
                var searchValue = $.trim(input.val()).toLowerCase();
                var friends = $(that.el).find('.ui-listview li:not(.ui-li-divider)');
                var dividers = $(that.el).find('.ui-listview li.ui-li-divider');

                if (!searchValue) {
                    friends.removeClass('ui-screen-hidden');
                    dividers.removeClass('ui-screen-hidden');
                    input.blur();
                    return;
                }

                var noFriendsInList = true;
                _.each(friends, function (friend) {
                    var name = $.trim($(friend).find('.friend-name').html()).toLowerCase();

                    if (name.indexOf(searchValue) >= 0) {
                        $(friend).removeClass('ui-screen-hidden');
                        noFriendsInList = false;
                    } else {
                        $(friend).addClass('ui-screen-hidden');
                    }
                });

                _.each(dividers, function (divider) {
                    var dividerLetter = $(divider).text();
                    var visibleItem = $(divider).nextAll(':not(.ui-screen-hidden)').eq(0);

                    var nextElement = $.trim(visibleItem.find('.friend-name').html());

                    if (dividerLetter == nextElement.slice(0, 1)) {
                        $(divider).removeClass('ui-screen-hidden');
                    } else {
                        $(divider).addClass('ui-screen-hidden');
                    }
                });

                if (noFriendsInList) {
                    $(this.el).find('#no-friends-msg').show();
                } else {
                    $(this.el).find('#no-friends-msg').hide();
                }

                input.blur();
                return;
            }
        },

        onSkip: function (event) {
            if (isJqmGhostClickPropagated(event) || isJqmGhostClick(event)) return false;

            event.preventDefault();

            if (ConfigHelper.isDeviceOnline() == false) {
                window.showAlert(this.localize.errorUnableConnectToInternet);

                return false;
            }

            var that = this;
            window.ObjectModels.TempOrder = { concessions: null };

            window.SaleHelper.lockTickets(function (success) {
                if (success) {
                    Backbone.history.navigate('selectSeats' + '/' + that.options.pTheater + '/' + that.options.pPerformance, true);
                } else {
                    window.goBack();
                }
            });

            return false;
        },

        continueToCheckout: function (e) {
            if (isJqmGhostClickPropagated(e) || isJqmGhostClick(e)) return false;

            e.preventDefault();

            if (ConfigHelper.isDeviceOnline() == false) {
                window.showAlert(this.localize.errorUnableConnectToInternet);

                return false;
            }

            var orderModel = window.ObjectModels.OrderModel;

            var continueTo = 'checkoutSummary';

            window.ObjectModels.TempOrder = { concessions: null };

            if (configurationProvider.useConssesion(orderModel.get('theater'))) {
                continueTo = 'concessions/' + orderModel.get('theater').get('id') + '/null';
            }

            window.SaleHelper.lockTickets(function (success) {
                if (success) {
                    Backbone.history.navigate(continueTo, true);
                } else {
                    window.goBack();
                }
            });

            return false;
        },

        onHide: function () {
            $(this.el).find('.ui-content').css('height', '');
        },

        onLogin: function () {
            var that = this;

            if (ConfigHelper.isDeviceOnline() == false) {
                window.hidePageLoadingMessage();
                window.showAlert(this.localize.errorUnableConnectToInternet);
                return;
            }
            that.facebook.connect(function (result) {
                if (result) {
                    window.showPageLoadingMessage();
                    that.getFBstuff(result);
                }
            });
        },

        connectSonar: function () {
            $(".fb-friend-img").bind("scrollin", function () {
                var myimg = this;
                var $myimg = $(myimg);
                $myimg.unbind("scrollin");
                myimg.src = $myimg.attr("data-src");
            });

            return false;
        },

        getFBstuff: function (token) {
            var that = this;

            that.facebook.getData(token, { action: 'me/friends?fields=id,name,first_name,last_name&' }, function (json) {
                $(that.el).find('.hold-seats-status-container').show();
                $(that.el).find('.hold-seats-content').hide();
                $(that.el).find('.hold-seats-friends-container').show();
                $(that.el).find('.hold-seats-header-message').css('margin-top', 70);

                var orderModel = window.ObjectModels.OrderModel;

                if (configurationProvider.useConssesion(orderModel.get('theater'))) {
                    $(that.el).find('#continueToCheckout').html(that.localize.continueToConcessions);
                } else {
                    $(that.el).find('#continueToCheckout').html(that.localize.continueToCheckout);
                }

                $('.ui-input-search .ui-input-clear').css({ 'top': '', 'margin-top': '' });

                var filter = $(that.el).find('#searchFriends');
                filter.css('margin', 0);

                filter.appendTo('.search-field');

                that.friends = json.data;

                var friendsList = $(that.el).find('.hold-seats-friends-container ul');
                friendsList.css('margin', 0);

                that.sortFriends(that.friends);

                friendsList.append(_.template(templateFriendList, { friends: that.friends }));

                var tempHoldTickets = that.onHoldTickets.friends;
                that.onHoldTickets.friends = [];

                _.each(tempHoldTickets, function (i) {
                    var friend = _.find(that.friends, function (f) { return f.id == i.id; });

                    friend.quantity = i.quantity;
                    that.onHoldTickets.friends.push(friend);

                    that.refreshView(friend.id, i.quantity);
                });

                if (window.AppProperties.IsIOS && window.AppProperties.iOSVersion.indexOf('7.') == 0) {
                    $('.hold-seats-friends-container').addClass('ios7-hold-seats-search-box');
                }

                that.connectSonar();
                window.hidePageLoadingMessage();
            });
        },

        sortFriends: function (friends) {
            friends.sort(function (a, b) {
                var nameA = a.name.toLowerCase();
                var nameB = b.name.toLowerCase();

                if (nameA < nameB) return -1;
                if (nameA > nameB) return 1;
                return 0;
            });
        },

        addTicket: function (e) {
            e.preventDefault();
            e.stopPropagation();

            var friendId = $(e.currentTarget).attr("data-friend-id");
            var friend = _.find(this.friends, function (f) { return f.id == friendId; });
            if (this.getTotalSelectedTicketCount() == this.tickets.maxOnHoldTickets) return false;

            friend.quantity++;

            if (friend.quantity == 1) {
                this.onHoldTickets.friends.push(friend);
            }

            this.refreshView(friendId, friend.quantity);
            return false;
        },

        removeTicket: function (e) {
            e.preventDefault();
            e.stopPropagation();

            var friendId = $(e.currentTarget).attr("data-friend-id");
            var friend = _.find(this.friends, function (f) { return f.id == friendId; });
            if (friend.quantity == 0) return false;

            friend.quantity--;

            if (friend.quantity == 0) {
                this.onHoldTickets.friends.splice(this.onHoldTickets.friends.indexOf(friend), 1);
            }

            this.refreshView(friendId, friend.quantity);
            return false;
        },

        refreshView: function (friendId, quantity) {
            var removeCount = $(this.el).find('#ticket-count-' + friendId);
            var btnMessage = this.onHoldTickets.friends.length ? this.localize.selectSeats : this.localize.skipThisStep;

            removeCount.html(quantity);

            var total = this.getTotalSelectedTicketCount();

            this.onHoldTickets.set('quantity', total, { silent: true });

            $(this.el).find('#continueToSelectSeats').html(btnMessage);
            $(this.el).find('#selected-friends-count').html(this.onHoldTickets.friends.length);
            $(this.el).find('#remaining-tickets-to-hold').html(total);

            if (quantity == 0) {
                removeCount.prev().hide();
            } else {
                removeCount.prev().show();
            }
        },

        getTotalSelectedTicketCount: function () {
            var totalCount = 0;

            for (var i = 0; i < this.onHoldTickets.friends.length; i++) {
                totalCount += this.onHoldTickets.friends[i].quantity;
            }

            return totalCount;
        },

        render: function () {
            var that = this;
            that.changePageTrigger = false;

            var orderModel = window.ObjectModels.OrderModel;

            var theater = orderModel.get('theater');
            var performance = orderModel.get('performance');

            var showDate = DtHelper.convertDate(performance.get('showTime'));

            var holdTime = showDate.getTime() - configurationProvider.holdTimeInMinutesBeforeShowTime(theater) * 60000;
            var maxHoldTime = new Date().getTime() + configurationProvider.holdTimeinMinutes(theater) * 60000;
            var holdDate = holdTime > maxHoldTime ? new Date(maxHoldTime) : new Date(holdTime);

            var holdDateValue = Globalize.format(holdDate, "M", theater.get('culture'));
            var holdTimeValue = Globalize.format(holdDate, "t", theater.get('culture'));

            this.$el.html(this.template({
                maxOnHoldTickets: this.tickets.maxOnHoldTickets,
                holdDateValue: holdDateValue,
                holdTimeValue: holdTimeValue,
                localize: that.localize
            }));

            var searchInput = $('#searchFriends input');
            searchInput.live('focus', that.showKeyboardHandler);
            searchInput.live('blur', that.hideKeyboardHandler);

            $(document).on("showkeyboard", that.showKeyboardHandler);
            $(document).on("hidekeyboard", function () {
                searchInput.blur();
                that.hideKeyboardHandler();
            });

            var fixHeader = function () {
                setTimeout(function () {
                    $("#header").css("top", "0px!important").css("left", "0px!important");
                }, 100);
            };

            window.onkeyup = fixHeader;
            window.onscroll = fixHeader;

            if (orderModel.get('tickets').getReservedTicketQuantity() > 0) {
                $(this.el).find('#continueToSelectSeats').show();
                $(this.el).find('#continueToCheckout').hide();
            } else {
                $(this.el).find('#continueToSelectSeats').hide();
                $(this.el).find('#continueToCheckout').show();
            }

            that.pullDownStatusView = new pullDownStatusView();
            that.assign(that.pullDownStatusView, '.pullDownStatusHolder');

            $(this.el).find('#remaining-tickets-to-hold').html(this.getTotalSelectedTicketCount());

            UIHelper.dismissKeyboard($(this.el));
            return this;
        },

        assign: function (childView, selector) {
            this.$(selector).empty();
            childView.setElement(this.$(selector)).render();
            this.$(selector).enhanceWithin();
        }
    });

    return view;
});