define([
    'text!templates/HomeView.html',
    'sharedhelpers/configurationProvider',
    'views/curtainsMenuView',
    'helpers/uiHelper'],
    function (template, configurationProvider, curtainsMenuView) {
        window.HomeView = Backbone.View.extend({
            template: _.template(template),
            localize: window.ObjectCollections.Localization.result,

            initialize: function () {
                this.curtainsMenu = null;
                this.headerRepositioned = false;
            },

            render: function () {
                var that = this;
                var showPromotions = configurationProvider.promotions() && configurationProvider.promotions().length > 0;

                if (window.ObjectCollections.TheaterCollection) {
                    window.ObjectCollections.TheaterCollection.theaterViewType = 'list';
                }

                $(that.el).html(that.template({
                    tickets: that.options.tickets,
                    numberOfOrders: that.options.numberOfOrders,
                    localize: that.localize,
                    showPromotions: showPromotions
                }));

                that.curtainsMenu = new curtainsMenuView();
                that.assign(that.curtainsMenu, '.inFrontOfCurtainsHolder');
                var retView = configurationProvider.arangeFlow(that);

                UIHelper.iOSHeaderFix(that);

                //enable Passbook Dialog on #myTicketView
                window.passbookDialog = true;

                return retView;
            },

            events: {
                "tap #virtualgiftcard": "navigateVirtualGiftCard",
                "tap #loyaltycard": "navigateLoyaltyCard",
                "tap #purchase-history": "navigatePurchaseHistory",
                'tap #settings': 'navigateSettings',
                "tap #promotions": "navigatePromotions",
                "pagebeforeshow": "onPageShow"
            },

            onPageShow: function () {
                var that = this;

                ConfigHelper.initialize(function () {
                    that.curtainsMenu.loadTheaters(true);
                });
            },

            assign: function (view, selector) {
                this.$(selector).empty();
                view.setElement(this.$(selector)).render();
                this.$(selector).enhanceWithin();
            },

            navigateVirtualGiftCard: function (e) {
                e.preventDefault();
                if (isJqmGhostClick(e) || isChangePageFired(e)) return;
                window.showPageLoadingMessage();

                window.AppMethods.ChangePage('#virtualGiftCards', false, true);
            },

            navigateLoyaltyCard: function (e) {
                e.preventDefault();
                if (isJqmGhostClick(e) || isChangePageFired(e)) return;
                window.showPageLoadingMessage();

                _.defer(function () {
                    window.AppMethods.ChangePage('#loyaltycard', false, true);
                });
            },

            navigatePurchaseHistory: function (e) {
                e.preventDefault();
                if (isJqmGhostClick(e) || isChangePageFired(e)) return;
                window.showPageLoadingMessage();

                _.defer(function () {
                    window.AppMethods.ChangePage('#purchaseHistory', false, true);
                });
            },

            navigateSettings: function (e) {
                e.preventDefault();
                if (isJqmGhostClick(e) || isChangePageFired(e)) return;
                window.showPageLoadingMessage();

                _.defer(function () {
                    window.AppMethods.ChangePage('#settings', false, true);
                });
            },

            navigatePromotions: function (e) {
                e.preventDefault();
                if (isJqmGhostClick(e) || isChangePageFired(e)) return;
                window.showPageLoadingMessage();

                _.defer(function () {
                    window.AppMethods.ChangePage('#promotions', false, true);
                });
            }
        });

        return window.HomeView;
    });