define([
  'jquery',
  'underscore',
  'backbone',
  'text!templates/ListMoviesView.html',
  'collections/movieCollection',
  'models/performanceModel',
  'helpers/geolocationHelper',
  'sharedhelpers/configurationProvider',
  'helpers/uiHelper',
  'sharedhelpers/storageProvider',
  'sharedhelpers/htmlHelper',
  'sharedhelpers/dateTimeHelper'
], function ($, _, Backbone,template,MovieCollection,PerformanceModel,geolocationHelper, configurationProvider,UIHelper) {
    var view = Backbone.View.extend({
        sort: window.AppConfig.DefaultMovieSort || 'byName',
        template: _.template(template),
        localize: window.ObjectCollections.Localization.result,

        initialize: function () {
            this.searchUnit = window.AppProperties.SearchUnit;
        },

        render: function () {
            var self = this;
            var searchValue = window.AppProperties.CityOrZip || this.localize.yourLocation;
            var moviesCollection = self.options.comingSoon ? window.ObjectCollections.ComingSoonCollection : window.ObjectCollections.MovieCollection;

            self.searchRadius = window.AppProperties.searchRadius ? window.AppProperties.searchRadius : window.ObjectCollections.TheaterCollection.distance;

            if (this.options.sort == undefined)
                this.options.sort = this.sort;
            if (window.ObjectCollections.MovieCollection.sortComparator !== undefined) {
                this.options.sort = window.ObjectCollections.MovieCollection.sortComparator;
            }

            var showSearchBox = configurationProvider.chainType() != 'single' && configurationProvider.chainType() != 'small';
            var showDistance = showSearchBox && window.AppProperties.ShowTheaterDistance;
            var moviesToShow = new MovieCollection();


            var customFlags = this.getAvailableCustomFlags();
            if (window.appConfig.GroupMovies) {
                _.each(moviesCollection.groupedMovies, function (group) {
                    var g = new MovieCollection(group).changeSort(self.sort);

                    var model = _.find(g.models, function (m) {
                        return !$.isEmptyObject(m.get('nextPerformance'));
                    });

                    if (!model) {
                        model = g.models[0];
                    }
                    var mov = _.find(moviesCollection.models, function(m) {
                        return m.get('id') === model.get('id');
                    });

                    if (!mov) {
                        mov = moviesCollection.getMovie(model.get('code'));
                    }

                    var moviePoster = _.find(g.models, function (m) {
                        return m.get('image').medium != undefined;
                    });

                    if (moviePoster) mov.set('image', moviePoster.get('image'));

                    var popularity = 0;

                    _.each(g.models, function (m) {
                        popularity += m.get('totalPerformanceCount');
                    });

                    if (mov.popularityCalculated != 'movies') {
                        mov.set('totalPerformanceCount', popularity);
                        mov.popularityCalculated = 'movies';
                    }

                    mov.group = group;
                    moviesToShow.add(mov);
                });
                moviesToShow = moviesToShow.changeSort(this.options.sort);
            } else {
                moviesToShow = this.options.movies;
            }

            $(this.el).html(this.template({
                movies: moviesToShow.models,
                sort: this.options.sort,
                localize: this.localize,
                showDistance: showDistance,
                searchValue: searchValue,
                searchUnit: self.searchUnit,
                searchRadius: self.searchRadius,
                showSearchBox: showSearchBox,
                isSingleChainType: configurationProvider.chainType() == 'single',
                hideBackButton: this.options.hideBackButton,
                comingSoon: this.options.comingSoon,
                customFlags: customFlags
            }));

            if (!this.options.comingSoon) {
                _.delay(function () {
                    $(self.el).find('#searchMovies')[0].setAttribute('type', 'search');
                    $(self.el).find('input').blur();
                }, 5);
            }

            window.BannerHelper.addBanner();
            return this;
        },

        events:
		{
		    "pageshow": "pageShow",
		    "tap #ulSortOptions li": "sortByDropdownChanged",
		    "tap .movie-list-item": "showMovieDetails",
		    "tap #distance": "drivingDirections",
		    "tap #showTime": "selectMovieTickets",
		    "tap .filter-btn": "openExperienceFilter",
		    "tap #changeLocation": "navigateLocationSettings",
		    "tap .search-field .ui-icon-delete": "resetFilterMoviesByName",
		    "keypress div.ui-input-search": "filterMoviesByName",
		    "popupafterclose #positionWindow": "toggleMovieFiltering",
		    'pagebeforeshow': 'beforeShow',
		    "pagebeforehide": "beforeHide",
		    "change .custom-flags-filter": "customFlagsHandler"
		},

        getAvailableCustomFlags: function () {
            var movies = this.options.movies.models;
            var availableFlags = [];
            var allCustomFlags = configurationProvider.getCustomFlags();
            _.each(movies, function (m) {
                if (m.get('customFlags') && m.get('customFlags').length > 0) {
                    _.each(m.get('customFlags'), function (mcf) {
                        var customFlag = _.find(allCustomFlags, function (cf) {
                            return cf.id == mcf && cf.searchable;
                        });
                        if (customFlag && !_.contains(availableFlags, customFlag)) {
                            availableFlags.push(customFlag);
                        }
                    });
                }
            });
            return availableFlags;
        },
        customFlagsHandler: function (e) {
            var id = $(e.currentTarget).attr('id');
            if (_.contains(window.activeCustomFlags, id)) {
                window.activeCustomFlags = _.without(window.activeCustomFlags, id);
            } else {
                window.activeCustomFlags.push(id);
            }

            this.handleAttributeFilters();
        },

        handleAttributeFilters: function () {
            $(this.el).find('#searchMovies').val('').blur();
            if (window.activeCustomFlags.length == 0) {
                $('.movie-list-item:hidden').show();
                $(this.el).find('#no-movies-msg').hide();

                $('.custom-flags-divider').show();
                $('.custom-flags-header').show();
                return;
            }
            $('.custom-flags-divider').hide();

            $('.custom-flags-header').each(function () {
                if (_.contains(window.activeCustomFlags, $(this).data('custom-flag'))) {
                    $(this).show();
                } else {
                    $(this).hide();
                }
            });

            var moviesToHide = [];
            var groups = window.ObjectCollections.MovieCollection.groupedMovies;
            if (window.appConfig.GroupMovies) {
                _.each(groups, function (g) {
                    var counter = 0;
                    _.each(g, function (m) {
                        _.each(window.activeCustomFlags, function (flag) {
                            if (m.get('customFlags') && _.contains(m.get('customFlags'), flag)) {
                                counter++;
                            }
                        });
                    });
                    if (counter == 0) {
                        _.each(g, function(s) {
                            moviesToHide.push(s.get('code').trim());
                        });
                    }
                });
            } else {
                var grouped = _.groupBy(this.options.movies.models, function (item) {
                    return item.get('code');
                });
                _.each(grouped, function (movieGroup) {
                    var counter = 0;
                    _.each(movieGroup, function (m) {
                        _.each(window.activeCustomFlags, function (flag) {
                            if (m.get('customFlags') && _.contains(m.get('customFlags'), flag)) {
                                counter++;
                            }
                        });
                    });
                    if (counter == 0) {
                        moviesToHide.push(movieGroup[0].get('code').trim());
                    }
                });
            }

            $('.movie-list-item').each(function () {
                var featureCode = $(this).data('featurecode').toString();
                if (_.contains(moviesToHide, featureCode)) {
                    $(this).hide();
                } else if (!$(this).is(':visible')) {
                    $(this).show();
                }
            });

            if ((window.appConfig.GroupMovies ? groups.length : this.options.movies.models.length) == moviesToHide.length) {
                $(this.el).find('#no-movies-msg').show();
            } else {
                $(this.el).find('#no-movies-msg').hide();
            }
        },

        toggleMovieFiltering: function () {
            $('.ui-page-active').css({
                'overflow-x': 'hidden'
            });

            if ($(this.el).find('#positionWindow input:checked').length > 0) {
                $(this.el).find('.filter-btn').addClass('active-filter-movies');
            } else {
                $(this.el).find('.filter-btn').removeClass('active-filter-movies');
            }
        },

        resetFilterMoviesByName: function (e) {
            if (window.isJqmGhostClick(e)) return;

            $(this.el).find('[data-role="listview"] li').show();
            $('.custom-flags-divider').show();
            $('.custom-flags-header').show();
        },

        openExperienceFilter: function (e) {
            $('.ui-page-active').css({
                'overflow-x': 'visible'
            });

            e.preventDefault();
            if (window.isJqmGhostClick(e)) return;

            $(this.el).find('#positionWindow').popup('open');
            UIHelper.positionWindowCssFix();
        },

        filterMoviesByName: function (e) {
            var that = this;

            if (e.keyCode == 13) {
                e.preventDefault();

                var input = $(e.currentTarget).find('input');
                var searchValue = $.trim($(e.currentTarget).find('#searchMovies').val()).toLowerCase();
                var movies = $(that.el).find('[data-role="listview"] li');

                if (!searchValue) {
                    movies.show();
                    input.blur();

                    $('.custom-flags-divider').show();
                    $('.custom-flags-header').show();

                    return;
                }
                var activeCustomFlags = [];

                var noMoviesInList = true;
                _.each(movies, function (movie) {
                    var name = $(movie).attr('data-moviefilter').toLowerCase();

                    if (name.indexOf(searchValue) >= 0) {
                        $(movie).show();
                        noMoviesInList = false;
                        if (!_.contains(activeCustomFlags, $(movie).data('custom-flag'))) {
                            activeCustomFlags.push($(movie).data('custom-flag'));
                        }
                    } else {
                        $(movie).hide();
                    }
                });

                $('.custom-flags-header').each(function () {
                    if (!_.contains(activeCustomFlags, $(this).data('custom-flag'))) {
                        $(this).hide();
                    } else {
                        $(this).show();
                    }
                });

                if ((activeCustomFlags.length > 1 && !_.contains(activeCustomFlags, 'noGroup')) || activeCustomFlags.length == 1) {
                    $('.custom-flags-divider').hide();
                }

                if (noMoviesInList) {
                    $(this.el).find('#no-movies-msg').show();
                    $('.custom-flags-divider').hide();
                    $('.custom-flags-header').hide();
                } else {
                    $(this.el).find('#no-movies-msg').hide();
                }

                input.blur();
                return;
            }
        },

        reset: function () {
            this.render();

            $(this.el).enhanceWithin();

            window.activeCustomFlags = [];
        },

        beforeShow: function () {
            this.toggleMovieFiltering();
            if (!this.options.comingSoon) {
                UIHelper.handleListMoviesTip();
            }

            if (window.BannerHelper.isBannerAvailable()) {
                $('#tip-overlay-screen').css('top', '130px');
                $('.dismiss').css('top', '85%');
            }
        },

        pageShow: function () {
            if (window.ObjectCollections.MovieCollection.length < 1) {
                var msg = $.sprintf(this.localize.noTheatersWithinRadius, this.searchRadius, this.searchUnit);
                window.showAlert(msg);
            } else if (this.options.comingSoon && this.options.movies.length < 1) {
                $(this.el).find('#no-movies-msg span').html(this.localize.noComingSoonMovies);
                $(this.el).find('#no-movies-msg').show();
            }
        },

        beforeHide: function () {
            $(this.el).find('#NCR-listview li:visible:gt(9)').remove();
        },

        sortByDropdownChanged: function (e) {
            e.preventDefault();
            if (window.isJqmGhostClick(e)) return;

            var that = this;
            var sort = $(e.currentTarget).attr('sort');
            window.showPageLoadingMessage();

            window.ObjectCollections.MovieCollection.sortComparator = sort;
            that.options.sort = sort;
            that.options.movies.changeSort(that.options.sort);
            setTimeout(function () {
                that.reset();
                window.hidePageLoadingMessage();
                that.toggleMovieFiltering();
            }, 100);
        },

        drivingDirections: function (e) {
            e.preventDefault();
            e.stopPropagation();

            var theaterId = $(e.currentTarget).attr('data-theaterid');

            require(['helpers/utilities'], function () {
                var utils = new window.Utils();
                utils.showDirections(theaterId);
            });
        },

        selectMovieTickets: function (e) {
            e.stopPropagation();

            var that = this;

            var theater = $(e.currentTarget).attr('data-theaterid');
            var featureCode = $(e.currentTarget).attr('data-featurecode');
            var performance = $(e.currentTarget).attr('data-performance');

            var perfObj = null;

            _.each(this.options.movies.models, function (movie) {
                if (movie.get('nextPerformance') && movie.get('nextPerformance').number == performance && perfObj == null) {
                    perfObj = movie.get('nextPerformance');
                }
            });

            if (!perfObj) {
                window.showAlert(that.localize.theaterUpdatedSchedule);

                window.showPageLoadingMessage();

                var latitude = window.AppProperties.CurrentPosition.Latitude;
                var longitude = window.AppProperties.CurrentPosition.Longitude;

                that.refreshMoviesList(latitude, longitude, true);
                return;
            }

            // clone it to add ticket types
            var pNew = new PerformanceModel(perfObj);
            window.AppProperties.SelectedPerformance = pNew;
            window.AppMethods.ChangePage('#tickets/' + performance + '/' + featureCode + '/' + theater);
        },

        showMovieDetails: function (e) {
            e.preventDefault();
            e.stopPropagation();
            var that = this;

            // Tapped on the theater directions symbol
            if ($(e.target).attr('class') == 'location-marker') {
                return;
            }

            // Prevent date collisions
            window.mdSelectedBusinessDate = undefined;

            // Tapped on the movie name
            var featureCode = $(e.currentTarget).attr('data-featurecode');
            var theaterId = $(e.currentTarget).attr('data-theater');

            var theaterList = [theaterId];

            if (window.appConfig.GroupMovies) {
                var theaterObjectList = [];
                var movieGroup = window.ObjectCollections.MovieCollection.getMovieGroup(featureCode);

                _.each(movieGroup, function (mov) {
                    for (var i = 0; i < that.options.movies.models.length; i++) {
                        var item = that.options.movies.models[i];

                        if (item.get('code').trim() == mov.get('code').trim()) {
                            var theaters = _.pluck(item.group, 'theaterId');

                            _.each(theaters, function (t) {
                                if (theaterList.indexOf(t) < 0 && mov.theaterId != t) {
                                    theaterList.push(t);
                                    theaterObjectList.push(window.ObjectCollections.TheaterCollection.get(t));
                                }
                            });
                        }
                    }
                });

                window.ObjectCollections.MovieCollection.groupedMovies.theaterList = theaterObjectList;
            } else {
                for (var i = 0; i < that.options.movies.models.length; i++) {
                    if (that.options.movies.models[i].get('code').trim() == featureCode) {
                        if (that.options.movies.models[i].theaterId != theaterId) {
                            theaterList.push(that.options.movies.models[i].theaterId);
                        }
                    }
                }
            }

            window.AppMethods.ChangePage('#movie/' + featureCode + '/' + theaterList.join(','));
        },

        refreshMoviesList: function (lat, lng, update) {
            var that = this;

            window.AppProperties.searchRadius = $('#searchRadius :selected').val();
            if (update) {
                window.AppProperties.CurrentPosition = { Latitude: lat, Longitude: lng };
            }

            var sort = $('#ulSortOptions li a[class*="ui-btn-active"]').parent().attr('sort');
            that.options.sort = sort;

            that.options.movies = that.options.movies.refresh(sort, lat, lng, function (movies) {
                that.options.movies = movies;
                if (window.appConfig.GroupMovies) {
                    window.ObjectCollections.MovieCollection.groupedMovies = window.ObjectCollections.MovieCollection.groupMultiTypeMovies();
                }

                that.reset();

                window.hidePageLoadingMessage();

                if (that.options.movies.length < 1) {
                    var noTheatersMsg = $.sprintf(that.localize.noTheatersWithinRadius, that.searchRadius, that.searchUnit);
                    window.showAlert(noTheatersMsg);
                }
            });
        },

        navigateLocationSettings: function (e) {
            e.preventDefault();
            if (window.isJqmGhostClick(e) || window.isChangePageFired(e)) return;

            window.showPageLoadingMessage();
            window.AppMethods.ChangePage('#locationSettings', false, true);
        }
    });
    return view;
});
