﻿define([
  'text!templates/LocationSettingsView.html',
  'helpers/geolocationHelper',
  'helpers/uiHelper'
], function (template, geolocationHelper) {
    var view = Backbone.View.extend({
        template: _.template(template),
        localize: window.ObjectCollections.Localization.result,
        distances: ["5", "10", "25", "50", "75", "100", "200"],

        initialize: function () {
            this.searchUnit = window.AppProperties.SearchUnit;
        },

        events: {
            'tap #buttonAcquireGeolocation': 'buttonAcquireGeolocationClick',
            'change #searchRadius': 'searchRadiusChanged'
        },

        render: function () {
            var that = this;

            var searchValue = window.AppProperties.CityOrZip || that.localize.yourLocation;
            var searchRadius = window.AppProperties.searchRadius || window.ObjectCollections.TheaterCollection.distance;

            that.$el.html(this.template({
                showDistance: window.AppProperties.ShowTheaterDistance,
                searchUnit: that.searchUnit,
                searchValue: searchValue,
                localize: that.localize,
                theaterDistance: window.ObjectCollections.TheaterCollection.distance,
                distanceExists: !_.contains(that.distances, window.ObjectCollections.TheaterCollection.distance)
            }));

            var radiusSelector = $(that.el).find('#searchRadius');

            if ($(that.el).find('#searchRadius option[value="' + searchRadius + '"]').length < 1) {
                radiusSelector.append('<option value=' + searchRadius + '> ' + searchRadius + ' ' + that.searchUnit + ' </option>');
            }

            radiusSelector.val(searchRadius).attr('selected', true).siblings('option').removeAttr('selected');

            var searchSelector = $(that.el).find('div.ui-input-search');
            searchSelector.die('keypress');
            searchSelector.live('keypress', function (e) {
                if (e.keyCode === 13) {
                    e.preventDefault();
                    that.searchByCityOrZip();
                    return;
                }
            });

            setTimeout(function () {
                that.$('#theater-zip')[0].setAttribute('type', 'search');
                that.$('input').blur();
            }, 5);

            return that;
        },

        reset: function () {
            this.render();

            $(this.el).enhanceWithin();
        },

        searchByCityOrZip: function () {
            var that = this;

            if (ConfigHelper.isDeviceOnline() == false) {
                window.showAlert(this.localize.errorUnableConnectToInternet);
                return;
            }

            window.showPageLoadingMessage();
            var cityOrZip = $('#theater-zip').val() || window.AppProperties.CityOrZipSearch;

            if (cityOrZip && cityOrZip.length > 0) {
                geolocationHelper.getLatLong(cityOrZip, function (lat, lng) {
                    that.geolocationChanged(lat, lng, false);
                });
                return;
            }

            if (!cityOrZip) {
                window.hidePageLoadingMessage();
            }
        },

        searchRadiusChanged: function (e) {
            e.preventDefault();

            var that = this;
            window.showPageLoadingMessage();

            window.AppProperties.searchRadius = $('#searchRadius :selected').val();

            if (window.AppProperties.CityOrZip) {
                that.searchByCityOrZip();
            } else {
                that.buttonAcquireGeolocationClick();
            }
        },

        buttonAcquireGeolocationClick: function (e) {
            if (e) {
                e.preventDefault();
                if (isJqmGhostClick(e)) return;
            }

            if (ConfigHelper.isDeviceOnline() == false) {
                window.showAlert(this.localize.errorUnableConnectToInternet);
                return;
            }

            window.showPageLoadingMessage();
            var that = this;

            // Acquire current position            
            navigator.geolocation.getCurrentPosition(function (position) {
                window.AppProperties.CityOrZipSearch = null;
                window.AppProperties.CityOrZip = null;
                window.AppProperties.ShowTheaterDistance = true;
                that.geolocationChanged(position.coords.latitude, position.coords.longitude, true);
            }, geolocationHelper.handleGeolocationErrors, geolocationHelper.geoOptions);

            var storage = new StorageProvider();

            storage.read('CurrentSearchLocation', function (value) {
                if (value) {
                    storage.remove('CurrentSearchLocation');
                }
            });
        },

        geolocationChanged: function (lat, lng, update) {
            var that = this;

            require(['collections/theaterCollection'], function (theaterCollection) {
                window.AppProperties.searchRadius = $('#searchRadius :selected').val();

                if (update)
                    window.AppProperties.CurrentPosition = { Latitude: lat, Longitude: lng };

                var theaters = new theaterCollection();
                theaters.getTheaters(lat, lng, function (result) {
                    window.hidePageLoadingMessage();

                    if (result === undefined) {
                        window.showAlert(this.localize.errorLoadingTheaters);
                        return;
                    }

                    window.ObjectCollections.TheaterCollection = result;
                    window.ObjectCollections.MovieCollection = result.getMovies();

                    if (window.appConfig.GroupMovies) {
                        window.ObjectCollections.MovieCollection.groupedMovies = window.ObjectCollections.MovieCollection.groupMultiTypeMovies();
                    }

                    that.reset();
                });
            });
        }
    });

    return view;
});