define([
  'jquery',
  'underscore',
  'backbone',
  'text!templates/LoyaltyCardDetailsView.html',
  'sharedlibs/jquery/jquery.qrcode.min'
], function ($, _, Backbone,template) {
    var view = Backbone.View.extend({
        template: _.template(template),
        localize: window.ObjectCollections.Localization.result,

        initialize: function () {
            this.onRotate();
            this.loyaltyCardNumber = this.model.get('cardNumber').value;
        },

        events: {
            'pagecreate': 'onLoad',
            'pageshow': 'onShow',
            'tap li[for="tab-value"]': 'switchTabs',
            'tap #manageAccountProfile': 'manageAccountProfile',
            'tap #loyalty-card-front-page': 'flipLoyaltyCard',
            'tap #loyalty-card-back-page': 'flipLoyaltyCard',
            'tap #add-to-passbook': 'addToPassbook',
            'tap .loyalty-banner': 'handleBannerUrl'
        },

        render: function () {
            var that = this;
            var selector = $(that.el);

            var passbookIcon = typeof window.device != 'undefined' ? window.device.version.slice(0, window.device.version.indexOf('.')) : 11;
            var showPassbookIcon = window.AppProperties.PassbookAvailable.loyaltyPass || (this.model.get('multitype') && window.AppProperties.PassbookAvailable.dualCardPass);
            var showOffersTab = this.model.get('loyaltyBanners') && this.model.get('loyaltyBanners').length > 0;
            var giftCard = this.options.giftCard;
            if(giftCard){
            let balanceTobeHidden= window.giftCardHelper.checkIfBalanceNeedsToBeHidden(giftCard.CardNumber);
            console.log("In Loyalty Screen: HideBalance Property: "+balanceTobeHidden);
            if(balanceTobeHidden){
                  giftCard.hideBalance=true;
            }
          }
            this.$el.html(this.template({
                localize: this.localize,
                model: this.model,
                giftCard: giftCard,
                showOffersTab: showOffersTab,
                hideNewOffer: this.options.hideNewOffer,
                passbookIcon: passbookIcon,
                showPassbookIcon: showPassbookIcon,
                hideBackButton: this.options.hideBackButton
            }));

            this.generateQrCode(selector);

            if (this.options.status == 'new' || selector.find('#btnMenu:visible').length == 0) {
                selector.find("#header a[data-rel='back']").on('tap', function () {
                    window.AppMethods.ChangePage('');
                    return false;
                });
            }

            return this;
        },

        onLoad: function () {
            if (this.options.giftCard && !this.options.giftCard.hideBalance) {
                this.getGiftCardBalance();
            }
        },

        onShow: function () {
            var that = this;

            if (that.options.status == 'new') {
                require(['helpers/uiHelper'], function () {
                    window.UIHelper.notificationBar(that.localize.loyaltyRegistrationComplete);
                });

                that.passbookPrompt();
            }

            if (this.options.loyaltyError) {
                window.showAlert(that.localize.errorLoyaltyAccess);
            }
        },

        switchTabs: function (e) {
            e.preventDefault();
            if (window.isJqmGhostClick(e)) return;

            var currentTab = e.currentTarget.attributes['value'].value;

            $(this.el).find('#' + currentTab).show();
            $(this.el).find('.loyalty-tab').not('#' + currentTab).hide();
        },

        getGiftCardBalance: function () {
            var that = this;
            var giftCard = that.options.giftCard;
            require(['sharedhelpers/giftCardHelper'], function () {

                var getBalance = function () {
                    window.giftCardHelper.showBalanceLoader.call(that, giftCard.CardNumber);

                    var handlers = {
                        onSuccess: function (balance) {
                            $(that.el).find('.gift-balance').html(balance);
                            window.giftCardHelper.updateBalance({ CardNumber: giftCard.CardNumber, Balance: balance });

                            if (window.AppProperties.PassbookAvailable.dualCardPass) {
                                require(['helpers/passHelper'], function () {
                                    var passHelper = new window.PassHelper();
                                    passHelper.getLoyaltyCardPassURL(that.model, true);
                                });
                            }
                        },
                        onError: function () {
                            $(that.el).find('.gift-balance').html(that.localize.na);
                        },
                        onComplete: function () {
                            window.giftCardHelper.hideBalanceLoader.call(that);
                        }
                    };

                    window.giftCardHelper.getBalance(giftCard.CardNumber, handlers);
                };

                if (!window.giftCardBalanceCleared) {
                    window.giftCardHelper.invalidateBalance(getBalance);
                    window.giftCardBalanceCleared = true;
                } else if ((!giftCard.Balance || giftCard.UpdateBalance) && !giftCard.hideBalance) {
                    getBalance();
                }
            });
        },

        generateQrCode: function (selector) {
            selector.find("#bar-code-container").qrcode({ text: this.loyaltyCardNumber.toString(), width: 150, height: 150 });
        },

        passbookPrompt: function () {
            var that = this;
            var onConfirm = function (button) {
                //OK = 1; Cancel = 2
                if (button == 1) {
                    that.addToPassbook();
                    return false;
                }
            };

            if (window.AppProperties.PassbookAvailable.loyaltyPass || (that.model.get('multitype') && window.AppProperties.PassbookAvailable.dualCardPass)) {
                if (window.isMobileDevice()) {
                    navigator.notification.confirm(that.localize.loyaltyPassbookPrompt, onConfirm, that.localize.alertTitle, that.localize.ok + ', ' + that.localize.cancel);
                } else if (confirm(that.localize.loyaltyPassbookPrompt)) {
                    onConfirm(1);
                }
            }
        },

        manageAccountProfile: function (event) {
            window.showPageLoadingMessage();
            event.preventDefault();
            window.AppMethods.ChangePage("#loyaltycardprofile/edit");
        },

        flipLoyaltyCard: function (e) {
            var that = this;
            if (e) {
                e.preventDefault();
            }

            $(that.el).find('.ticket-card').toggleClass('rotate');
            $(that.el).find('.tap-loyalty').html($('.ticket-card').hasClass('rotate') ? that.localize.tapToFlipLoyalty : that.localize.tapToScanLoyalty);
        },

        onRotate: function () {
            $(window).bind('orientationchange', function () {
                var loyaltyCardContent = $('.loyalty-card-info, .manage-account-btn');

                if ($.event.special.orientationchange.orientation() == "portrait") {
                    loyaltyCardContent.show();
                    $('.ticket-card').removeClass('rotate');
                } else {
                    loyaltyCardContent.hide();
                    $('.ticket-card').addClass('rotate');
                }
            });
        },

        addToPassbook: function (e) {
            window.showPageLoadingMessage();
            if (e) {
                e.preventDefault();

                if (window.isJqmGhostClick(e)) {
                    return false;
                }
            }
            var that = this;
            var loyaltyCard = that.model;

            require(['helpers/passHelper', 'sharedhelpers/errorHelper'], function () {
                var passHelper = new window.PassHelper();
                passHelper.getLoyaltyCardPassURL(loyaltyCard);
            });

            return false;
        },

        handleBannerUrl: function (e) {
            e.preventDefault();

            if (window.isJqmGhostClick(e)) {
                return false;
            }

            var bannerUrl = $(e.currentTarget).attr('data-hyperlink');

            if (!bannerUrl) {
                return false;
            }

            var inAppWin = window.open(bannerUrl, '_blank');
            inAppWin.addEventListener('exit', function () {
                inAppWin.addEventListener('exit');
                inAppWin = null;
            });

            return false;
        }
    });

    return view;
});
