﻿define([
  'text!templates/LoyaltyCardDialogView.html',
  'sharedhelpers/configurationProvider',
   'sharedhelpers/utilities',
  'libs/jquery.validate.min',
  'sharedhelpers/encryptionHelper'
], function (template, configurationProvider) {
    var view = Backbone.View.extend({
        template: _.template(template),
        localize: window.ObjectCollections.Localization.result,

        initialize: function () {
            this.setupValidation();
        },

        render: function () {
            this.$el.html(this.template({
                localize: this.localize,
                showLastNameInput: this.options.showLastNameInput,
                showPasswordInput: this.options.showPasswordInput,
                hideSignupBtn: this.options.hideSignupBtn,
                loyaltyCardNumber: this.options.loyaltyCardNumber || ""
            }));

            var retView = configurationProvider.arangeFlow(this);
            return $(retView.el);
        },

        events:
		{
		    'tap #ok': 'ok',
		    'tap #cancel': 'cancel',
		    'tap #btnLoyaltySignUp': 'loyaltySignUp',
		    'tap .loyalty-scan': 'scanLoyaltyCard',
		    'change #noPassword': 'toogleLastNameInput',
            'tap #resetpass': 'resetPass'
		},

        setupValidation: function () {
            $.validator.addMethod('loyaltynumber', function (value) {
                return /^[0-9]+([\s-]*[0-9]+)*$/.test(value);
            }, this.localize.errorLoyaltyCard);
        },

        cancel: function (event) {
            event.preventDefault();

            this.options.callback(false);
            return false;
        },

        resetPass: function (e) {
            e.preventDefault();
            this.options.callback(false);
            require(['views/LoyaltyCardResetPasswordDialog'], function (loyaltyCardResetPasswordDialog) {
                var dialog = new loyaltyCardResetPasswordDialog();
                $('#popupDialog').html(dialog.render());
                $('#popupDialog').popup('open');
                $('#popupDialog').enhanceWithin();
            });
        },

        ok: function (event) {
            event.preventDefault();
            var that = this;
            var form = $("#frmLoyaltyCard");
            var utilities = new Utilities();
            var encryptionHelper = new EncryptionHelper();
            var noPassword = form.find('#noPassword').is(":checked");

            if (form.valid()) {
                var loyaltyNumber = $('#loyaltyCardNumber').val().replace(/-/g, '').replace(/ /g, '');
                var securityKey = '';

                if (configurationProvider.usePwdForLoyaltyLogin()) {
                    if (window.appConfig.LoyaltyPasswordEncryptionDisabled) {
                        securityKey = noPassword ? utilities.capitalize($.trim($('#cardLastName').val())) : form.find('#pass').val();
                    } else {
                        securityKey = noPassword ? utilities.capitalize($.trim($('#cardLastName').val())) : encryptionHelper.encrypt(form.find('#pass').val());
                    }
                } else {
                    if (that.options.showLastNameInput) {
                        securityKey = utilities.capitalize($.trim($('#loyaltyCardLastName').val()));
                    }
                }
                that.options.callback(loyaltyNumber, encodeURIComponent(securityKey));
            }

            return false;
        },

        loyaltySignUp: function (event) {
            event.preventDefault();
            if (isJqmGhostClick(event)) return;

            $('#btnLoyaltySignUp').blur();

            _.defer(function () {
                window.AppMethods.ChangePage('#loyaltycardsignup');
            });
        },

        scanLoyaltyCard: function (e) {
            e.preventDefault();
            var obj = $('#popupDialog-popup');
            var popupTop = obj.css('top'),
                popupLeft = obj.css('left');

            var callback = function (result) {
                if (result.text) {
                    obj.find('#loyaltyCardNumber').val(result.text);

                    _.defer(function () {
                        obj.css('top', popupTop);
                        obj.css('left', popupLeft);
                    });
                }       
            };

            require(['js/helpers/utilities'], function () {
                var utils = new Utils();
                utils.scanBarcode(callback);
            });
        },

        toogleLastNameInput: function (e) {
            if ($(e.currentTarget).is(':checked')) {
                $(this.el).find('.last-name-wrapper').show();
                $(this.el).find('#pass').textinput('disable');
                $(this.el).find('#resetpass').hide();
            } else {
                $(this.el).find('#resetpass').show();
                $(this.el).find('.last-name-wrapper').hide();
                $(this.el).find('#pass').textinput('enable');
            }
        }
    });

    return view;
});