define([
  'text!templates/LoyaltyCardSignupView.html',
  'sharedhelpers/configurationProvider',
  'libs/jquery.validate.min',
  'helpers/uiHelper',
  'sharedhelpers/utilities',
  'sharedhelpers/cultureHelper',
  'sharedhelpers/storageProvider',
  'shared_top_level/js/encode',
  'sharedhelpers/encryptionHelper',
  'helpers/theatersHelper'
], function (template, configurationProvider) {
    var view = Backbone.View.extend({
        template: _.template(template),
        localize: window.ObjectCollections.Localization.result,

        initialize: function () {
            this.cultureHelper = new CultureHelper();
            this.phoneValidation = this.cultureHelper.phoneValidation();
            this.zipValidation = this.cultureHelper.zipValidation();
            this.dateValidation = this.cultureHelper.dateValidation();
            this.emailValidation = this.cultureHelper.emailValidation();

            this.theaters = window.ObjectCollections.AbridgedTheaterCollection.models || window.ObjectCollections.TheaterCollection.models;
        },

        render: function () {
            var self = this;
            this.settingsEmailKey = 'emailAddress';
            this.settingsPhoneKey = 'phoneNumber';

            if (this.tmpl == undefined)
                this.$el.html(this.template({
                    localize: self.localize,
                    model: this.options.model,
                    zipValidation: this.zipValidation,
                    theaters: this.theaters
                }));
            else {
                this.template = _.template(this.tmpl);
                this.$el.html(this.template({
                    model: this.model,
                    localize: self.localize,
                    zipValidation: this.zipValidation,
                    theaters: this.theaters
                }));
            }

            var address1 = $(this.el).find('#address1');
            var zipcode = $(this.el).find('#zipcode');
            var state = $(this.el).find('#state');
            var city = $(this.el).find('#city');

            var addOrRemoveRequiredAttribute = function (element) {
                switch (element.selector) {
                    case '#address1': if (window.appConfig.EnableAddress1AsRequiredField) address1.attr('required', true);
                    case '#zipcode' : if (window.appConfig.EnableZipCodeAsRequiredField) zipcode.attr('required', true);
                    case '#state'   : if (window.appConfig.EnableStateAsRequiredField) state.attr('required', true);
                    case '#city'    : if (window.appConfig.EnableCityAsRequiredField) city.attr('required', true);                                       
                }
                
            };
            addOrRemoveRequiredAttribute(address1);
            addOrRemoveRequiredAttribute(zipcode);
            addOrRemoveRequiredAttribute(state);
            addOrRemoveRequiredAttribute(city);

            $(this.el).find('#birthday').bind('keypress', function (e) {
                var selector = $('#termsAndConditions');
                if (e.keyCode == 13) {
                    e.preventDefault();
                    if (selector.is(":checked")) {
                        self.signUp(e);
                    }
                }
            });
            // If user goes back, prepopulate the form
            if (window.loyaltyCard) {
                this.prepopulate($(this.el));
            } else {
                window.loyaltyCard = {
                    profile: [],
                    step1: [],
                    step2: []
                };
            }

            this.prepopulateEmailAndPhone($(this.el));

            $.validator.addMethod('phone', function (value) {
                return self.phoneValidation.regex.test(value);
            }, self.phoneValidation.msg);
            $.validator.addMethod('zipcode', function (value) {
                return value == '' || (self.zipValidation.regex.test(value.toUpperCase()));
            }, self.zipValidation.msg);
            $.validator.addMethod('email', function (value) {
                return self.emailValidation.regex.test(value);
            }, self.emailValidation.msg);

            $.validator.addMethod('date', function (value) {
                return self.dateValidation.regex.test(value.replace(/\s/g, '/'));
            }, self.dateValidation.msg);

            $.validator.addMethod('birthday', function (value) {
                var dateOfBirth = value.replace(/\s/g, '/');
                return DtHelper.dateDiffInYears(dateOfBirth) > window.appConfig.LoyaltySignUpRequiredAge;
            }, $.sprintf(this.localize.errorLoyaltySignUpRequiredAge, window.appConfig.LoyaltySignUpRequiredAge));

            $.validator.addClassRules({ phone: { phone: true }, email: { email: true } });

            //Header fix for iOS when soft keyboard is active
            UIHelper.iOSHeaderFix(self);

            return this;
        },

        events: {
            "tap #continueStep1": "continueStep1",
            "tap #continueStep2": "continueStep2",
            "tap #signUp": "signUp",
            "tap #viewTerms": "viewTerms",
            "change #zipcode": "zipcodeChanged",
            "change #termsAndConditions": "termsAndConditions",
            "pagebeforeshow": "pageBeforeShow"
        },

        pageBeforeShow: function () {
            var that = this;
            var theatersHelper = new TheatersHelper();

            theatersHelper.getFavoriteTheaters(function (favoriteTheaters) {
                if (favoriteTheaters.length > 0) {
                    var favoriteTheater = _.find(that.theaters, function (model) {
                        return model.id == favoriteTheaters[0];
                    });
                    $('#favoriteTheater').val(favoriteTheater.get('name')).attr('selected', true).siblings('option').removeAttr('selected');
                    $('#favoriteTheater option:selected').val(favoriteTheater.get('name')).change();
                }
            });
        },

        signUp: function (e) {
            e.preventDefault();
            if (isJqmGhostClick(e)) return;

            var self = this;
            loyaltyCard.profile = [];

            var continueFlow = function (savedPassword) {
                window.showPageLoadingMessage();
                $('[data-role=content]').css({ 'pointer-events': 'none' });
                $('#signUp').css({ 'pointer-events': 'none' });

                var smsOpt;
                if ($(self.el).find('#smsOptIn').is(":checked")) {
                    smsOpt = '2';
                } else {
                    smsOpt = '0';
                }
                loyaltyCard.profile.push({ "name": "dateOfBirth", "value": $(self.el).find('#birthday').val().replace(/\s/g, '/') });
                loyaltyCard.profile.push({ "name": "favoriteTheater", "value": $(self.el).find('#favoriteTheater :not(:first-child):selected').text() });
                loyaltyCard.profile.push({ "name": "password", "value": savedPassword });
                loyaltyCard.profile.push({ "name": "smsOptIn", "value": smsOpt });

                if (window.appConfig.EnableGenderForLoyalty) {
                    loyaltyCard.profile.push({ "name": "gender", "value": $(self.el).find('#gender :not(:first-child):selected').val() });
                }

                loyaltyCard.profile.push.apply(loyaltyCard.profile, loyaltyCard.step1);
                loyaltyCard.profile.push.apply(loyaltyCard.profile, loyaltyCard.step2);

                $.each(loyaltyCard.profile, function (i, item) {
                    self.model.set(item.name, { "value": item.value, "name": item.name });
                });

                var callback = function (response) {
                    var accountInfo = response.attributes.result.accountInfo;
                    //TODO: Check the loyaltyBanners property when the Loyalty API gets updated
                    var loyaltyBanners = response.attributes.result.loyaltyBanners;
                    self.model.set('loyaltyBanners', loyaltyBanners);

                    _.each(accountInfo, function (item) {
                        self.model.set(item.name, {
                            "value": item.value,
                            "name": item.name
                        });
                    });

                    var loyaltyStoreModel =
                    {
                        number: self.model.get('cardNumber').value,
                        firstName: $.trim(self.model.get('firstName').value),
                        lastName: $.trim(self.model.get('lastName').value),
                        emailAddress: $.trim(self.model.get('emailAddress').value),
                        offers: self.model.get('loyaltyBanners') || null,
                        favoriteTheater: $.trim(self.model.get('favoriteTheater').value),
                        password: window.appConfig.LoyaltyPasswordEncryptionDisabled? savedPassword:$.trim(self.model.get('password').value),
                        smsOptIn: self.model.get('smsOptIn').value
                    };

                    if (window.appConfig.EnableGenderForLoyalty) {
                        loyaltyStoreModel.gender = $.trim(self.model.get('gender').value);
                    }

                    var storage = new StorageProvider();
                    storage.store('LoyaltyCard', loyaltyStoreModel);

                    var theater = _.find(window.ObjectCollections.TheaterCollection.models, function (model) {
                        return model.get('name') == self.model.get('favoriteTheater').value;
                    });

                    if (theater) {
                        var theatersHelper = new TheatersHelper();
                        theatersHelper.markTheaterAsFavorite(theater.id, true);
                    }

                    if (configurationProvider.useDualCards()) {
                        require(['helpers/dualCardHelper'], function () {
                            var callback = function () {
                               // Backbone.history.loadUrl(Backbone.history.fragment);
                                window.AppMethods.ChangePage("#loyaltycardview/new");
                            };

                            window.dualCardHelper.handleGiftCard.call(self, self.model.get('cardNumber').value, loyaltyStoreModel, callback);
                        });

                        return;
                    }

                    window.hidePageLoadingMessage();
                    window.AppMethods.ChangePage("#loyaltycardview/new");
                };

                var errorCallback = function (errorResponse) {
                    $('[data-role=content]').css({ 'pointer-events': '' });
                    $('#signUp').css({ 'pointer-events': '' });
                    if (errorResponse.result.error_sub_code === 52) { //EmainInUse
                        showConfirmation("Email already in use.  Would you like to use the existing account to login?", {
                            callback: function(buttonIndex) {
                                if (buttonIndex === 1) { // User pressed yes

                                    require(['views/loyaltyCardDialogView'], function(loyaltyCardDialogView) {
                                        var dialog = new loyaltyCardDialogView({
                                            showLastNameInput: false,
                                            showPasswordInput: true,
                                            hideSignupBtn: true,
                                            loyaltyCardNumber: "",
                                            callback: function(loyaltyNumber, securityKey) {
                                                self.fetchLoyaltyDetails(loyaltyNumber, securityKey);

                                                $('#popupDialog').popup('close');
                                            }
                                        });

                                        $('#popupDialog').html(dialog.render());
                                        $('#popupDialog').popup('open');
                                        $('#popupDialog').enhanceWithin();
                                    });
                                }
                            },
                            buttons: [self.localize.yes, self.localize.no]
                        });
                    }
                };

                window.showPageLoadingMessage();
                self.model.saveCustom({ callback: callback, errorCallback: errorCallback });              
            };

            if ($("#frmLoyaltyCardFinalStep").valid()) {
                if (configurationProvider.usePwdForLoyaltyLogin()) {
                    self.createPasswordDialog(continueFlow);
                } else {
                    continueFlow();
                }
            }
        },

        fetchLoyaltyDetails: function (loyaltyNumber, securityKey) {
            if (!loyaltyNumber || !securityKey) {
                return;
            }

            var that = this;

            require(['models/loyaltyAccountModel', 'sharedhelpers/storageProvider', 'sharedhelpers/errorHelper'], function (loyaltyAccountModel) {
                window.showPageLoadingMessage();

                var model = new loyaltyAccountModel();
                model.set({ id: loyaltyNumber, securityKey: securityKey });

                model.fetch({
                    cache: false,
                    success: function () {
                        var modelCardNumber = model.get('cardNumber') !== undefined ? model.get('cardNumber').value : null;
                        var modelCardLastName = model.get('lastName') !== undefined ? model.get('lastName').value : null;
                        var modelCardFirstName = model.get('firstName') !== undefined ? model.get('firstName').value : null;
                        var modelPassword = model.get('password') !== undefined ? model.get('password').value : null;
                        var modelFavoriteTheater = model.get('favoriteTheater') !== undefined ? model.get('favoriteTheater').value : null;
                        var modelCardEmail = model.get('emailAddress') !== undefined ? model.get('emailAddress').value : null;

                        var offers = model.get('loyaltyBanners') || null;

                        if (modelCardNumber === null) {
                            window.hidePageLoadingMessage();
                            new ErrorHelper().showAlertByErrorCode(model.get('errorCode'));
                            return;
                        } else {
                            var continueFlow = function () {
                                var loyaltyStoreModel = {
                                    number: modelCardNumber,
                                    firstName: modelCardFirstName,
                                    lastName: modelCardLastName,
                                    offers: offers,
                                    password: window.appConfig.LoyaltyPasswordEncryptionDisabled ? securityKey : modelPassword,
                                    favoriteTheater: modelFavoriteTheater,
                                    emailAddress: modelCardEmail
                                };

                                var storage = new StorageProvider();
                                storage.store('LoyaltyCard', loyaltyStoreModel);

                                var favoriteTheater = _.find(window.ObjectCollections.TheaterCollection.models, function (model) {
                                    return model.get('name') == modelFavoriteTheater;
                                });

                                if (favoriteTheater) {
                                    var theatersHelper = new TheatersHelper();
                                    theatersHelper.markTheaterAsFavorite(favoriteTheater.id, true);
                                }

                                if (configurationProvider.useDualCards()) {
                                    require(['helpers/dualCardHelper'], function () {
                                        var callback = function () {
                                            Backbone.history.loadUrl(Backbone.history.fragment);
                                        };

                                        window.dualCardHelper.handleGiftCard.call(that, modelCardNumber, loyaltyStoreModel, callback);
                                    });

                                    return;
                                }

                                Backbone.history.loadUrl(Backbone.history.fragment);
                            };
                        }

                        if (configurationProvider.usePwdForLoyaltyLogin() && !modelPassword) {
                            window.showAlert(that.localize.loyaltyCardSecurityNotification);
                            window.hidePageLoadingMessage();
                            that.createLoyaltyPassword(model);
                        } else {
                            continueFlow();
                        }
                    },

                    error: function () {

                        window.hidePageLoadingMessage();
                        window.showAlert(that.localize.errorGettingLoyaltyCardData);
                    }
                });
            });
        },
        continueStep1: function (event) {

            event.preventDefault();
            if (isJqmGhostClick(event)) return;

            var utilities = new Utilities();

            if ($("#frmLoyaltyCardStep1").valid()) {
                loyaltyCard.step1 = [].concat(
                { name: "firstName", value: utilities.capitalizeAfterHyphen($.trim($('#firstName').val())) },
                { name: "lastName", value: utilities.capitalizeAfterHyphen($.trim($('#lastName').val())) },
                { name: "emailAddress", value: $('#emailAddress').val() },
                { name: "phoneNumber", value: $('#phoneNumber').val().replace(/\D/g, '') });

                window.AppMethods.ChangePage("loyaltycardstep2");
            }
        },
        continueStep2: function () {
            if ($("#frmLoyaltyCardStep2").valid()) {
                loyaltyCard.step2 = [].concat(
                    { name: "address1", value: $('#address1').val() },
                    { name: "address2", value: $('#address2').val() },
                    { name: "postalCode", value: $('#zipcode').val() },
                    { name: "stateProvince", value: $('#state').val() },
                    { name: "city", value: $('#city').val() });

                window.AppMethods.ChangePage("loyaltycardfinalstep");
            }
        },

        viewTerms: function (e) {
            e.preventDefault();
            if (isJqmGhostClick(e)) return false;

            var termsUrl = $(e.currentTarget).attr('data-termsurl');

            var inAppWin = window.open(termsUrl, '_blank');

            inAppWin.addEventListener('exit', function () {
                inAppWin.addEventListener('loadstop');
                inAppWin.addEventListener('exit');
                inAppWin = null;
            });
        },

        zipcodeChanged: function () {
            if (!this.zipValidation.searchZip)
                return;

            var args = {
                zipCode: $('#zipcode').val(),
                successCallback: function (data) {
                    $("#state option[value=" + data.result[0].State + "]").attr("selected", "selected");
                    $('#city').val(data.result[0].City);
                    $('#state').selectmenu("refresh");
                },
                errorCallback: function () {
                    $("#state option[value='']").attr("selected", "selected");
                    $('#city').val('');
                    $('#state').selectmenu("refresh");
                }
            };

            this.zipValidation.searchZip(args);
        },

        termsAndConditions: function (e) {
            var selector = $(e.currentTarget);

            if (selector.is(":checked")) {
                $('#signUp').removeClass('ui-disabled');
            } else
                $('#signUp').addClass('ui-disabled');
        },

        prepopulate: function (selector) {
            var that = this;
            if (loyaltyCard.step1 !== undefined) {
                var steps = loyaltyCard.step1.slice();
                steps.push.apply(steps, loyaltyCard.step2);

                $(selector).find("input").each(function () {
                    var result = that.lookup(steps, $(this).attr('name'));
                    if (result.length > 0)
                        $(this).val(result[0].value);
                });
            }
        },

        prepopulateEmailAndPhone: function (selector) {
            var storage = new StorageProvider();
            var email = selector.find('#emailAddress').val();
            var phone = selector.find('#phoneNumber').val();

            if (email === '') {
                storage.read(this.settingsEmailKey, function (value) {
                    if (value !== null) {
                        selector.find('#emailAddress').val(value);
                    }
                });
            }

            if (phone === '') {
                storage.read(this.settingsPhoneKey, function (value) {
                    if (value !== null) {
                        selector.find('#phoneNumber').val(value);
                    }
                });
            }
        },

        lookup: function (steps, key) {
            return steps.filter(function (element) {
                return (element.name == key);
            });
        },
        createPasswordDialog: function (callback) {
            var that = this;
            var encryptionHelper = new EncryptionHelper();
            require(['views/createPassDialogView'], function (createPassView) {
                var dialog = new createPassView({
                    dialogTopNotification: that.localize.loyaltyDialogTopNotification,
                    dialogBottomNotification: '',
                    callback: function (value) {
                        $('#popupDialog').popup('close');
                        if (value) {
                            var loyaltyPass = window.appConfig.LoyaltyPasswordEncryptionDisabled ? $('#pass').val(): encryptionHelper.encrypt($('#pass').val());
                            callback(loyaltyPass);
                        }
                    }
                });

                $('#popupDialog').html(dialog.render());
                $('#popupDialog').popup('open');
                $('#popupDialog').enhanceWithin();
            });
        }
    });

    return view;
});