define([
  'text!templates/LoyaltyCardView.html',
  'sharedhelpers/configurationProvider',
  'sharedhelpers/utilities',
  'libs/jquery.validate.min',
  'sharedhelpers/encryptionHelper',
    'helpers/theatersHelper'
], function (template, configurationProvider) {
    var view = Backbone.View.extend({
        template: _.template(template),
        localize: window.ObjectCollections.Localization.result,

        events: {
            'tap #loyaltyCardSignin': 'openLoyaltySigninPopup',
            'tap #loyaltyCardSignup': 'loyaltyCardSignup'
        },

        render: function () {
            this.$el.html(this.template({
                localize: this.localize,
                hideBackButton: this.options.hideBackButton
            }));

            window.loyaltyCard = {
                profile: [],
                step1: [],
                step2: []
            };

            var retView = configurationProvider.arangeFlow(this);
            return retView.el;
        },
        openLoyaltySigninPopup: function (event) {
            event.preventDefault();

            if (ConfigHelper.isDeviceOnline() == false) {
                window.showAlert(this.localize.errorUnableConnectToInternet);
                return;
            }

            var that = this;
            var usePwdForLoyaltyLogin = configurationProvider.usePwdForLoyaltyLogin();

            require(['views/loyaltyCardDialogView'], function (loyaltyCardDialogView) {
                var dialog = new loyaltyCardDialogView({
                    showLastNameInput: !usePwdForLoyaltyLogin,
                    showPasswordInput: usePwdForLoyaltyLogin,
                    hideSignupBtn: true,
                    callback: function (loyaltyNumber, securityKey) {
                        that.fetchLoyaltyDetails(loyaltyNumber, securityKey);

                        $('#popupDialog').popup('close');
                    }
                });

                $('#popupDialog').html(dialog.render());
                $('#popupDialog').popup('open');
                $('#popupDialog').enhanceWithin();
            });
        },

        fetchLoyaltyDetails: function (loyaltyNumber, securityKey) {
            if (!loyaltyNumber || !securityKey) {
                return;
            }

            var that = this;

            require(['models/loyaltyAccountModel', 'sharedhelpers/storageProvider', 'sharedhelpers/errorHelper'], function (loyaltyAccountModel) {
                window.showPageLoadingMessage();

                var model = new loyaltyAccountModel();
                model.set({ id: loyaltyNumber, securityKey: securityKey });

                model.fetch({
                    cache: false,
                    success: function () {
                        var modelCardNumber = model.get('cardNumber') !== undefined ? model.get('cardNumber').value : null;
                        var modelCardLastName = model.get('lastName') !== undefined ? model.get('lastName').value : null;
                        var modelCardFirstName = model.get('firstName') !== undefined ? model.get('firstName').value : null;
                        var modelPassword = model.get('password') !== undefined ? model.get('password').value : null;
                        var modelFavoriteTheater = model.get('favoriteTheater') !== undefined ? model.get('favoriteTheater').value : null;
                        var modelCardEmail = model.get('emailAddress') !== undefined ? model.get('emailAddress').value : null;

                        var offers = model.get('loyaltyBanners') || null;

                        if (modelCardNumber === null) {
                            window.hidePageLoadingMessage();
                            new ErrorHelper().showAlertByErrorCode(model.get('errorCode'));
                            return;
                        } else {
                            var continueFlow = function () {
                                var loyaltyStoreModel = {
                                    number: modelCardNumber,
                                    firstName: modelCardFirstName,
                                    lastName: modelCardLastName,
                                    offers: offers,
                                    password: window.appConfig.LoyaltyPasswordEncryptionDisabled ? securityKey : modelPassword,
                                    favoriteTheater: modelFavoriteTheater,
                                    emailAddress: modelCardEmail
                                };

                                var storage = new StorageProvider();
                                storage.store('LoyaltyCard', loyaltyStoreModel);

                                var favoriteTheater = _.find(window.ObjectCollections.TheaterCollection.models, function (model) {
                                    return model.get('name') == modelFavoriteTheater;
                                });

                                if (favoriteTheater) {
                                    var theatersHelper = new TheatersHelper();
                                    theatersHelper.markTheaterAsFavorite(favoriteTheater.id, true);
                                }

                                if (configurationProvider.useDualCards()) {
                                    require(['helpers/dualCardHelper'], function () {
                                        var callback = function () {
                                            Backbone.history.loadUrl(Backbone.history.fragment);
                                        };

                                        window.dualCardHelper.handleGiftCard.call(that, modelCardNumber, loyaltyStoreModel, callback);
                                    });

                                    return;
                                }

                                Backbone.history.loadUrl(Backbone.history.fragment);
                            };
                        }

                        if (configurationProvider.usePwdForLoyaltyLogin() && !modelPassword) {
                            window.showAlert(that.localize.loyaltyCardSecurityNotification);
                            window.hidePageLoadingMessage();
                            that.createLoyaltyPassword(model);
                        } else {
                            continueFlow();
                        }
                    },

                    error: function () {

                        window.hidePageLoadingMessage();
                        window.showAlert(that.localize.errorGettingLoyaltyCardData);
                    }
                });
            });
        },

        loyaltyCardSignup: function (e) {
            e.preventDefault();
            window.AppMethods.ChangePage('#loyaltycardsignup');
        },

        createLoyaltyPassword: function (model) {
            var that = this;
            var encryptionHelper = new EncryptionHelper();
            require(['views/createPassDialogView'], function (createPassView) {
                var dialog = new createPassView({
                    dialogTopNotification: that.localize.loyaltyDialogTopNotification,
                    dialogBottomNotification: '',
                    callback: function (value) {
                        $('#popupDialog').popup('close');
                        if (value) {
                            var password = window.appConfig.LoyaltyPasswordEncryptionDisabled ? $('#pass').val(): encryptionHelper.encrypt($('#pass').val());
                            that.updateLoyaltyCard(model, password);
                        }
                    }
                });

                $('#popupDialog').html(dialog.render());
                $('#popupDialog').popup('open');
                $('#popupDialog').enhanceWithin();
            });
        },

        updateLoyaltyCard: function (model, password) {
            var that = this;
            window.showPageLoadingMessage();

            var callback = function () {
                that.fetchLoyaltyDetails(model.get('id'), password);
            };

            model.set('password', { name: 'password', value: password, isUpdated: true });
            model.saveCustom({ isUpdate: true, callback: callback, isPassUpdate: true });
        }
    });
    return view;
});