define([
  'sharedhelpers/configurationProvider',
  'models/performanceModel',
  'text!templates/MovieDetailsView.html',
  'helpers/uiHelper',
  'libs/jquery.validate.min',
  'sharedhelpers/dateTimeHelper',
  'sharedhelpers/htmlHelper'
], function (configurationProvider, performanceModel, template) {
    var view = Backbone.View.extend({
        template: _.template(template),
        localize: window.ObjectCollections.Localization.result,
        showTimes: [],
        dateIndex: 0,

        initialize: function () {
            if (this.options.model.schedule.models) {
                this.today = DtHelper.getCurrentBusinessDate();
                this.today.setHours(0, 0, 0, 0);

                this.previousPage = this.options.previousPage;
            }
            this.theaterId = this.options.theaterId;
        },

        events:
		{
		    'tap #seats-preview': 'showSeatsPreview',
		    "tap .selected-theater-details": "theaterDetails",
		    "tap .location-marker": "drivingDirections",
		    "tap .movie-details-movie-ticket": "selectMovieTickets",
		    'tap #previousDay': 'previousDateClick',
		    'tap #nextDay': 'nextDateClick',
		    'tap #facebook-share-show': 'facebookSharePanel',
		    'tap .trailer-btn': 'trailerButtonClick',
		    'tap #currentDay': 'currentDay',
		    "change #performances-calendar-select": "selectDate",
		    "tap .movie-poster-ncr": "showLargePoster"
		},

        showSeatsPreview: function () {
            var that = this;
            var theaterIds = _.pluck(that.options.model.theaters, 'id');

            var theaterIdsString = '';
            _.each(theaterIds, function(theaterId, i) {
                theaterIdsString += theaterId;
                if (i + 1 < theaterIds.length) {
                    theaterIdsString += ',';
                }
            });
            window.AppMethods.ChangePage('#seatsPreview/' + that.options.model.get('id') + '/' + theaterIds);
        },

        showLargePoster: function (e) {
            e.preventDefault();
            var obj = $(this.el);
            var imgSrc = this.model.get('image').large;
            if (!imgSrc) return;

            var callback = function () {
                obj.find('#NCR-calendar').css({ 'pointer-events': '' });
            };
            obj.find('#NCR-calendar').css({ 'pointer-events': 'none' });

            UIHelper.showLargePoster(obj, imgSrc, callback);
        },
        theaterDetails: function (e) {
            e.preventDefault();
            if (isJqmGhostClick(e) || isChangePageFired(e)) return;

            var theaterId = $(e.currentTarget).attr('data-theaterid');

            window.AppMethods.ChangePage("#theater/" + theaterId);
        },

        drivingDirections: function (e) {
            e.preventDefault();
            e.stopPropagation();

            var theaterId = $(e.currentTarget).attr('data-theaterid');

            require(['helpers/utilities'], function () {
                var utils = new Utils();
                utils.showDirections(theaterId);
            });
        },

        trailerButtonClick: function () {
            if (ConfigHelper.isDeviceOnline() == false) {
                window.showAlert(this.localize.errorUnableConnectToInternet);
                return false;
            }

            var that = this;
            var url = encodeURI(that.options.model.get('trailerUrl'));
            var ios10 = window.AppProperties.iOSVersion.indexOf('10.') == 0;
            var youtubeIndex = url.indexOf('youtube');
            var inAppWin;

            if (window.AppProperties.IsAndroid) {
                if (youtubeIndex < 0) {
                    inAppWin = window.open(url, '_system');
                } else {
                    inAppWin = window.open(url, '_blank');
                }
            }
            else {
                if (window.AppProperties.IsIOS && ios10 && youtubeIndex >= 0){
                    inAppWin = window.open(that.addPlaysinlineTrailerURL(url), '_blank', 'allowInlineMediaPlayback=yes');
                }
                else{
                    inAppWin = window.open(url, '_blank', 'location=no');
                }
            }

            var emptyUrl = function () {
                inAppWin.executeScript({
                    code: "if(document.getElementById('player1')) {document.getElementById('player1').pauseVideo()}"
                });
            };

            var inAppWinClose = function () {
                if (window.AppProperties.IsAndroid) {
                    emptyUrl();
                }

                inAppWin.removeEventListener('exit', inAppWinClose);
                inAppWin.removeEventListener('loadstop');
                inAppWin.close();
                inAppWin = null;
            };

            inAppWin.addEventListener('exit', inAppWinClose);
        },

        selectMovieTickets: function (e) {
            e.stopPropagation();
            if (e.currentTarget.className.indexOf('ui-disabled') !== -1 || isJqmGhostClick(e) || isChangePageFired(e)) {
                return;
            }

            var theater = $(e.currentTarget).attr("data-theater");
            var performance = $(e.currentTarget).attr("data-performance");
            var featureCode = $(e.currentTarget).attr("data-featurecode");

            var perfObj = null;

            // find selected performance
            _.each(this.options.model.theaters, function (th) {

                if (th.id == theater && perfObj == null) {
                    var performances = [];
                    if (appConfig.GroupMovies) {
                        _.each(th.group, function (group) {
                            performances.push(group.performances.models);
                        });
                        performances = _.flatten(performances);
                    } else {
                        performances = th.performances;
                    }
                    _.each(performances, function (p) {
                        var performanceNumber = p.number;
                        if (appConfig.GroupMovies) {
                            performanceNumber = p.get('number');
                        }
                        if (performanceNumber == performance && perfObj == null) {
                            perfObj = p;
                        }
                    });
                }
            });

            if (!perfObj) {
                window.showAlert(this.localize.theaterUpdatedSchedule);

                window.ObjectCollections.CachedMovies = {};

                var theaterList = new Array();

                for (var i = 0; i < this.options.model.theaters.length; i++) {
                    theaterList.push(this.options.model.theaters[i].theaterId);
                }

                window.AppProperties.PageRefresh = true;
                window.AppMethods.ChangePage("#movie/" + featureCode + "/" + theaterList.join(','), true);
                return;
            } else {
                // clone it to add ticket types
                var pNew = new performanceModel(perfObj);
                window.AppProperties.SelectedPerformance = pNew;
                window.AppMethods.ChangePage("#tickets/" + performance + "/" + featureCode + "/" + theater);
            }
        },

        previousDateClick: function (e) {
            e.preventDefault();
            // Check for ghost click
            if (isJqmGhostClick(e)) return;
            window.AppProperties.SelectedPerformance = null;

            var self = this;
            var pager = this.calendarPager({ action: 'previous' });

            if (isNaN(pager.date.getTime())) {
                window.showAlert(this.localize.alertNoShowtimesMovie);
                return;
            }

            if (pager.date.getTime() == pager.selectedDate) {
                window.showAlert(this.localize.alertPastShowtimes);
                return;
            }

            window.showPageLoadingMessage();
            setTimeout(function () {
                window.hidePageLoadingMessage();
                self.render(pager.date);

                window.mdSelectedBusinessDate = pager.date;

                $(self.el).enhanceWithin();

            }, 10);
        },
        nextDateClick: function (e) {
            e.preventDefault();
            if (isJqmGhostClick(e)) return;
            window.AppProperties.SelectedPerformance = null;

            var self = this;
            var pager = this.calendarPager({ action: 'next' });

            if (isNaN(pager.date.getTime())) {
                window.showAlert(this.localize.alertNoShowtimesMovie);
                return;
            }

            if (pager.date.getTime() == pager.selectedDate) {
                window.showAlert(this.localize.alertNoMoreShowtimesMovie);
                return;
            }

            window.showPageLoadingMessage();
            setTimeout(function () {
                window.hidePageLoadingMessage();
                window.mdSelectedBusinessDate = pager.date;
                self.render(pager.date);
                $(self.el).enhanceWithin();
            }, 10);
        },

        currentDay: function (e) {
            if (isJqmGhostClick(e)) return;
        },

        selectDate: function (e) {
            e.preventDefault();
            var self = this;

            window.AppProperties.SelectedPerformance = null;
            $("#NCR-calendar").focus();

            var pager = this.calendarPager({ action: 'select', value: $(e.currentTarget).val() });

            window.showPageLoadingMessage();
            setTimeout(function () {
                window.hidePageLoadingMessage();
                window.mdSelectedBusinessDate = pager.date;
                self.render(pager.date);
                $(self.el).enhanceWithin();
            }, 300);
        },

        calendarPager: function (option) {
            var that = this;

            var selectedDateTime = new Date(mdCurrentDate).getTime();
            var showDateTimes = [];

            var previousDate = DtHelper.getCurrentBusinessDate();

            var isSameDay = mdCurrentDate.format('yyyymmdd') == that.today.format('yyyymmdd');
            var allowPreviousDay = previousDate.format('yyyymmdd') != that.today.format('yyyymmdd');

            //Add currenty day.
            if (option.action == 'previous') {
                showDateTimes.push(that.today);
            }

            if (option.action == 'select') {
                return { date: new Date(option.value), selectedDate: selectedDateTime };
            }

            _.each(this.performancesDays, function (showDt) {
                var showDateTime = showDt.date.getTime();
                if (showDateTime >= that.today.getTime()) {
                    showDateTimes.push(showDateTime);
                }
            });

            var selectedDateIndex = jQuery.inArray(selectedDateTime, showDateTimes);
            var index = option.action == 'previous' ?
                            selectedDateIndex > 0 ? selectedDateIndex - 1 : selectedDateIndex :
                        selectedDateIndex < showDateTimes.length - 1 ?
                            selectedDateIndex + 1 : selectedDateIndex;


            var date = new Date(showDateTimes[index]);

            if (isSameDay && allowPreviousDay && option.action == "previous") {
                date = previousDate;
                showDateTimes.push(previousDate);
            }

            return { date: date, selectedDate: selectedDateTime };
        },
        facebookSharePanel: function (e) {
            var that = this;
            e.preventDefault();

            var movie = that.options.model;
            var ticketsOnSaleNow = movie.get('nextPerformance') && !jQuery.isEmptyObject(movie.get('nextPerformance'));
            var showTheaterName = ticketsOnSaleNow && movie.theaters.length == 1;

            //TODO: Get SiteURL from ChainInfo;
            var theaterUrl = appConfig.SiteURL + '#movie/' + movie.get('code').trim();

            var facebookParameters = {
                movieName: window.appConfig.GroupMovies ? movie.get('tMDBName') || movie.get('name') : movie.get('name'),
                theaterCulture: '',
                theaterName: '',
                theaterUrl: theaterUrl,
                img: movie.get('image') ? movie.get('image').medium || window.appConfig.DefaultMoviePosterImage : window.appConfig.DefaultMoviePosterImage
            };

            if (that.previousPage != 'movies') {
                if (that.options.model.theaters.length == 1) {
                    var theater = that.options.model.theaters[0];
                    facebookParameters.theaterUrl += '/' + theater.get('id');
                    facebookParameters.theaterName = showTheaterName ? theater.get('name') : '';
                    facebookParameters.theaterCulture = theater.get('culture');
                }
            } else {
                facebookParameters.theaterUrl += "/" + _.pluck(movie.theaters, 'id').join(',');
            }

            var shareText = "{text} {movieDetails} {theatreDetails} {timeDetails}";
            var theatreName = facebookParameters.theaterName === '' ? '' : 'at ' + facebookParameters.theaterName;

            shareText = shareText.replace("{text}", "I'm going to see");
            shareText = shareText.replace("{timeDetails}", '');

            shareText = shareText.replace("{movieDetails}", facebookParameters.movieName);
            shareText = shareText.replace("{theatreDetails}", theatreName);

            var options = {
                method: "share",
                href: facebookParameters.theaterUrl,
                caption: facebookParameters.movieName,
                description: shareText,
                picture: facebookParameters.img,
                share_feedBrowser: true // iOS only

            };
            window.facebookConnectPlugin.showDialog(options, function (response) {
                if (response && response.postId) {
                    window.showAlert('Posted successfully');
                } else if (window.AppProperties.IsAndroid) {
                    window.appAvailability.check(
                        'com.facebook.katana', null, function () {
                            window.showAlert('Posted successfully');
                        });
                }
            });
        },

        render: function (date) {
            var that = this;

            if (window.ObjectModels.OrderModel != null) {
                window.ObjectModels.OrderModel.attributes.performance = null;
            }

            window.mdSelectedBusinessDate = window.mdSelectedBusinessDate || DtHelper.getCurrentBusinessDate();
            window.mdCurrentDate = date || window.mdSelectedBusinessDate;

            that.performancesDays = [];
            var performancesDaysCollection = [];

            var defaultCulture;
            var showSeatPreview;

            if (window.appConfig.GroupMovies) {
                that.resetFilters = true;
                that.experienceFilters = window.HtmlHelper.getStandards();

            }

            _.each(that.options.model.theaters, function (model) {
                //setting the first theater culture as default so we don't get different cultures in the select list
                if (!defaultCulture)
                    defaultCulture = model.get('culture');
                var performances = [];

                if (window.appConfig.GroupMovies) {
                    _.each(model.group, function (mov) {
                        if (_.filter(that.options.model.theaters, function (theater) {
                             return theater.id == mov.theaterId;
                        }).length > 0) {
                            if (that.resetFilters) {
                                for (var i = 0; i < that.experienceFilters.length; i++) {
                                    that.options.model.group[that.experienceFilters[i].key] = false;
                                }
                                that.resetFilters = false;
                            }

                            for (var i = 0; i < that.experienceFilters.length; i++) {
                                if (mov.get(that.experienceFilters[i].key) && !that.options.model.group[that.experienceFilters[i].key]) {
                                    that.options.model.group[that.experienceFilters[i].key] = mov.get(that.experienceFilters[i].key);
                                }
                            }

                        }
                        _.each(mov.performances.models, function (perf) {
                            performances.push(perf);
                        });
                    });
                } else {
                    performances = model.performances;
                }

                var reservedPerf = [];
                var currentBusinessDay = DtHelper.getBussinessDate(window.mdSelectedBusinessDate);

                _.each(performances, function (perf) {
                    if (perf instanceof Backbone.Model) {
                        if (perf.get('businessDate') === currentBusinessDay && perf.get('isReservedSeating') && perf.get('status') === 'O') {
                            reservedPerf.push(perf);
                        }
                    } else {
                        if (perf.businessDate === currentBusinessDay && perf.isReservedSeating && perf.status === 'O') {
                            reservedPerf.push(perf);
                        }
                    }
                });
                if (reservedPerf.length > 0) {
                    showSeatPreview = true;
                }
                _.each(performances, function (performance) {
                    var d = DtHelper.convertDate(performance.showTime);
                    if (window.appConfig.GroupMovies) {
                        d = DtHelper.convertDate(performance.get('showTime'));
                    }
                    d.setHours(0, 0, 0, 0);

                    var ddate = Globalize.format(d, 'd', defaultCulture);
                    ddate = ddate.substring(0, ddate.length - 5);
                    var tmpList = _.pluck(that.performancesDays, 'displayDate');
                    if (!_.contains(tmpList, dateFormat(d, 'ddd') + ' (' + ddate + ')')) {
                        that.performancesDays.push({ displayDate: dateFormat(d, 'ddd') + ' (' + ddate + ')', date: d });
                    }
                });
            });

            //Current day will be always shown.
            performancesDaysCollection.push({ displayDate: dateFormat(that.today.getTime(), 'ddd') + ' ' + that.localize.todayInCalendar, date: that.today });

            that.performancesDays = _.sortBy(that.performancesDays, function (d) {
                return d.date;
            });

            _.each(that.performancesDays, function (item) {
                if (item.date.getTime() > that.today.getTime()) {
                    performancesDaysCollection.push(item);
                }
            });

            that.setInitialMovieTheatersSort();

            var starring = that.formatMovieCast(that.options.model.get('starring'));
            $(that.el).html(that.template({
                movie: that.options.model,
                starring: starring,
                localize: that.localize,
                showDay: window.mdCurrentDate,
                performancesDays: performancesDaysCollection,
                showDistance: window.AppProperties.ShowTheaterDistance,
                showSeatPreview: showSeatPreview
            }));

            var currentDate = window.mdCurrentDate;
            currentDate.setHours(0, 0, 0, 0);
            $(that.el).find("#performances-calendar-select").val(currentDate.toString()).attr('selected', true).siblings('option').removeAttr('selected');

            if (performancesDaysCollection.length > 0 && performancesDaysCollection[0].date.getTime() == currentDate.getTime())
                $(that.el).find('#previousDay').addClass('ui-disabled');
            if (performancesDaysCollection.length > 0 && performancesDaysCollection[performancesDaysCollection.length - 1].date.getTime() == currentDate.getTime())
                $(that.el).find('#nextDay').addClass('ui-disabled');

            var retView = configurationProvider.arangeFlow(that);

            window.BannerHelper.addBanner();

            return retView;
        },

        formatMovieCast: function (cast) {
            if (!cast || cast.length == 0) return false;

            var castArr = cast.split(',');

            cast = '';
            for (var i = 0 ; i < castArr.length - 1; i++) {
                cast += $.trim(castArr[i]) + ", ";
            }
            return cast.substring(0, cast.length - 2);
        },
        setInitialMovieTheatersSort: function () {
            var that = this;

            //sort only in case of multiple theaters
            if (that.options.model.theaters.length > 1) {
                var sortedByDistance = _.sortBy(that.options.model.theaters, function (theater) {
                    return theater.get("distance");
                });

                that.options.model.theaters = _.sortBy(sortedByDistance, function (theater) {
                    var showDay = window.mdCurrentDate;
                    showDay.setHours(0, 0, 0, 0);

                    var tomorrow = new Date(showDay);
                    tomorrow.setDate(showDay.getDate() + 1);
                    tomorrow.setHours(6); //Performances close at 6AM next day

                    if (showDay === that.today) {
                        showDay = new Date();
                        showDay.setMinutes(showDay.getMinutes() - theater.get("performanceExpirationInMinutes"));
                    }

                    // Filter only today's shows
                    var moviePerformances = _.filter(theater.get('performances'), function (perf) {
                        return (new Date(perf.showTime).toISOString() >= showDay.toISOString() && new Date(perf.showTime).toISOString() < tomorrow.toISOString());
                    });

                    return -(moviePerformances.length > 0);
                });
            }
        },

        addPlaysinlineTrailerURL: function (url) {
            if (url)
            {
                if (url.indexOf("?") > 0)
                    url = url.concat("&");
                else
                    url = url.concat("?");

                url = url.concat("playsinline=1");

                return url;
            }
        }
    });

    return view;
});