define([
  'sharedhelpers/configurationProvider',
  'text!templates/MyTicketView.html',
  'sharedlibs/jquery/jquery.qrcode.min',
  'helpers/nativeCalendarHelper',
  'helpers/uiHelper'
], function (configurationProvider, template) {
    var view = Backbone.View.extend({
        template: _.template(template),
        localize: window.ObjectCollections.Localization.result,

        initialize: function () {
            this.orderModel = window.ObjectModels.OrderModel;
            this.nativeCalendarHelper = new NativeCalendarHelper();
        },

        render: function () {
            var that = this;

            var selector = $(that.el);
            that.order = that.options.order;

            var passbookIcon = typeof window.device != 'undefined' ? window.device.version.slice(0, window.device.version.indexOf('.')) : 11;
            var showCalendarButton = this.nativeCalendarHelper.useCalendarPlugin() && that.order.selectedTickets.length > 0;
            
            selector.html(that.template({
                localize: that.localize,
                order: that.order,
                passbookIcon: passbookIcon,
                showCalendarButton: showCalendarButton,
                disableCalendarButton: that.order.addedToCalendar
            }));

            var confirmationNumber = that.order.confirmationNumber;

            selector.find('[data-serialnumber=' + confirmationNumber + ']:not(.qrcode-mock)').qrcode({
                text: configurationProvider.useSimpleQrCode() ? confirmationNumber : confirmationNumber + '^' + that.order.patronsName,
                width: 100, height: 100
            });

            _.each(that.order.selectedTickets, function (item) {
                selector.find('[data-serialnumber=' + item.serialNumber + ']').qrcode({ text: item.serialNumber, width: 100, height: 100 });

                if (item.isGifted) {
                    selector.find('[data-giftedtickets=gifted]').show();
                }
            });
            var retView = configurationProvider.arangeFlow(this);
            return retView;
        },

        events:
        {
            "tap .ticket-card": "flipTicket",
            "tap .giftButton": "giftTicket",
            "tap #facebook-share-show": "facebookSharePanel",
            "tap .ticket-pass,.ios11-ticket-pass": "addToPassbook",
            "pageshow": "onPageShow",
            "tap .calendar-btn": "addToCalendar"
        },
        addTicketPass: function () {
            var that = this;

            window.hidePageLoadingMessage();

            var onConfirm = function (button) {
                //OK = 1; Cancel = 2
                if (button == 1) {
                    that.addToPassbook();
                    return false;
                }
            };

            if (window.AppProperties.PassbookAvailable.ticketPass && that.options.order.selectedTickets.length > 0) {
                if (window.passbookDialog) {
                    if (window.isMobileDevice()) {
                        navigator.notification.confirm(that.localize.ticketPassbookPrompt, onConfirm, that.localize.alertTitle, that.localize.ok + ', ' + that.localize.cancel);
                    } else if (confirm(that.localize.ticketPassbookPrompt)) {
                        onConfirm(1);
                    }
                    window.passbookDialog = false;
                }
            }
        },

        onPageShow: function () {
            var that = this;
            var cardId = that.orderModel.get('checkDualCardId');

            if (!cardId) {
                that.addTicketPass();
                return;
            }

            that.orderModel.unset('checkDualCardId');

            require(['helpers/paymentMethodHelper', 'helpers/dualCardHelper'], function () {
                var pmh = new PaymentMethodsHelper();

                pmh.get(cardId, function (paymentMethods) {
                    var paymentMethod = paymentMethods[0];

                    if (!paymentMethod) {
                        that.addTicketPass();
                        return;
                    }

                    window.dualCardHelper.handleLoyaltyCard.call(that, paymentMethod.CardNumber, function () { that.addTicketPass(); });
                });
            });
        },

        addToPassbook: function (e) {
            window.showPageLoadingMessage();
            if (e) {
                e.preventDefault();

                if (isJqmGhostClick(e)) {
                    return false;
                }
            }
            var that = this;

            require(['helpers/passHelper', 'sharedhelpers/errorHelper'], function () {
                var passHelper = new PassHelper();
                passHelper.getTicketPassURL(that.options.orders, that.order.confirmationNumber);
            });
            return false;
        },

        flipTicket: function (e) {
            if (e) {
                e.preventDefault();
            }

            $(e.currentTarget).toggleClass('rotate');
        },

        giftTicket: function (e) {
            e.preventDefault();

            var selector = $(e.currentTarget);
            window.AppMethods.ChangePage('#selectRecipient/' + selector.attr('data-ticketid') + '/' + selector.attr('data-confirmationnumber'));

            return false;
        },

        facebookSharePanel: function () {
            var self = this;
            var performance = self.order.performance;
            var theaterUrl = appConfig.SiteURL + '#tickets/' + performance.number + '/' + self.order.movieCode.trim() + '/' + self.order.theaterId + '/share';
            var movieName = self.order.movieName;
            var theaterCulture = self.order.theaterCulture;
            var theaterName = self.order.theaterName;
            var showTime = performance.showTime;
            var img = self.order.movieImage ? self.order.movieImage.medium || window.appConfig.DefaultMoviePosterImage : window.appConfig.DefaultMoviePosterImage;
            var shareText = "{text} {movieDetails} {theatreDetails} {timeDetails}";


            if (showTime) {
                var date = new Date();
                var month = date.getMonth() + 1;
                var dateTime = date.getFullYear() + '-' + month + '-' + date.getDate() + 'T' + date.getHours() + ':' + date.getMinutes() + ':' + date.getSeconds();
                var showDate = DtHelper.convertDate(showTime);
                var diff = DtHelper.dateDiff(DtHelper.convertDate(dateTime), showDate);

                if (diff < 0) {
                    shareText = shareText.replace("{text}", "I'm going to see");
                    shareText = shareText.replace("{timeDetails}", 'on ' + Globalize.format(showDate, "M", theaterCulture) + ' at ' + Globalize.format(showDate, 't', theaterCulture));
                } else {
                    shareText = shareText.replace("{text}", "I just saw");
                    shareText = shareText.replace("{timeDetails}", '');
                }

                if (shareText.indexOf('{text}') > -1) {
                    shareText = shareText.replace("{text}", "I'm going to see");
                }
                if (shareText.indexOf('{timeDetails}') > -1) {
                    shareText = shareText.replace("{timeDetails}", '');
                }
            }
            else {
                shareText = shareText.replace("{text}", "I'm going to see");
                shareText = shareText.replace("{timeDetails}", '');
            }

            shareText = shareText.replace("{movieDetails}", movieName);
            shareText = shareText.replace("{theatreDetails}", theaterName);


            var options = {
                method: "share",
                href: theaterUrl,
                caption: movieName,
                description: shareText,
                picture: img,
                share_feedBrowser: true // iOS only

            };
            window.facebookConnectPlugin.showDialog(options, function (response) {
                if (response && response.postId) {
                    window.showAlert('Posted successfully');
                } else if (window.AppProperties.IsAndroid) {
                    window.appAvailability.check(
                        'com.facebook.katana', null, function () {
                            window.showAlert('Posted successfully');
                        });
                }
            });
        },
        addToCalendar: function (e) {
            var that = this;

            var successCallback = function() {
                UIHelper.notificationBar(that.localize.calendarSuccess);
                $(e.currentTarget).html(that.localize.addedToCalendar).addClass('ui-disabled');
          
                require(['js/helpers/storageHelper'], function () {
                    that.order.addedToCalendar = true;
                    window.storageHelper.updatePurchaseHistory(that.order);
                });
               
            };

            var movie = this.orderModel.get('movie');
            var startDate = new Date(this.orderModel.get('performance').get('showTime'));
            var endDate = new Date(this.orderModel.get('performance').get('showTime'));
            endDate.setMinutes(endDate.getMinutes() + movie.get('length'));

            var calendarInfo = {
                'startDate': startDate,
                'endDate': endDate,
                'title': movie.get('name'),
                'location': that.order.theaterName,
                'notes': that.localize.calendarNotes
            };

            this.nativeCalendarHelper.addToCalendar(calendarInfo, successCallback);
        }
    });

    return view;
});