define([
  'text!templates/NewCreditCardView.html',
  'libs/jquery.validate.min',
  'sharedhelpers/storageProvider',
  'sharedhelpers/utilities',
  'sharedhelpers/cultureHelper',
  'sharedhelpers/cardValidationHelper',
  'helpers/validationHelper',
  'helpers/paymentMethodHelper'
], function (template) {
    var view = Backbone.View.extend({
        template: _.template(template),
        localize: window.ObjectCollections.Localization.result,

        initialize: function () {
            this.cardId = null;
            this.orderModel = window.ObjectModels.OrderModel;

            this.theater = this.orderModel.get('theater');

            this.useMasterCard = this.theater.get('acceptsMasterCard');
            this.useAmericanExpress = this.theater.get('acceptsAmericanExpress');
            this.useDiscover = this.theater.get('acceptsDiscover');
            this.useVisa = this.theater.get('acceptsVisa');
            this.useAVS = this.theater.get('aVSRequiredForCredit');
            this.useCVV2 = this.theater.get('cVV2RequiredForCredit');

            this.cardValidation = new CardValidationHelper();
            this.cultureHelper = new CultureHelper();
            this.zipValidation = this.cultureHelper.zipValidation();
            this.isAndroidKitKat = window.AppProperties.IsAndroidKitKat;
        },

        render: function () {
            this.$el.html(this.template({
                localize: this.localize,
                model: this.options.model,
                theater: this.theater,
                useMasterCard: this.useMasterCard,
                useAmericanExpress: this.useAmericanExpress,
                useDiscover: this.useDiscover,
                useVisa: this.useVisa,
                zipValidation: this.zipValidation
            }));

            this.initializeExpirationDateYears($(this.el).find("#expDateYear"));

            this.setUpValidationRules($(this.el));

            return this;
        },
        setUpValidationRules: function (selector) {
            var self = this;

            selector.find('#frmPaymentMethodsCreditCard').bind('onfocus', function (e) {
                if (e.keyCode == 13) {
                    e.preventDefault();
                    $(this).valid();

                    return;
                }
            });
            $.validator.addMethod('creditcard', function (value) {
                var creditCard = self.cardValidation.isCreditCardValid({
                    cardNumber: value,
                    useMasterCard: self.useMasterCard,
                    useAmericanExpress: self.useAmericanExpress,
                    useDiscover: self.useDiscover,
                    useVisa: self.useVisa,
                    checkIsSupported: true
                });

                if (creditCard.isSupported !== undefined && !creditCard.isSupported) {
                    $.validator.messages.creditcard = self.localize.errorUnsupportedCreditCardType;
                }
                else {
                    $.validator.messages.creditcard = self.localize.errorEnterValidCreditCardNumber;
                }
                return creditCard.isValid;
            }, this.localize.errorEnterValidCreditCardNumber);

            if (this.useCVV2) {
                $.validator.addMethod('cvv2Format', function (value) {
                    var creditCardNumber = $('#creditCardNumber');

                    if (creditCardNumber.val() == "") {
                        creditCardNumber.addClass('error');
                    }

                    var creditCard = {
                        cardNumber: creditCardNumber.val(),
                        useMasterCard: self.useMasterCard,
                        useAmericanExpress: self.useAmericanExpress,
                        useDiscover: self.useDiscover,
                        useVisa: self.useVisa,
                        checkIsSupported: true
                    };

                    if (self.cardValidation.isCreditCardValid(creditCard)) {
                        var cvv2DigitType = self.cardValidation.getCVV2Digits(creditCard);

                        var regExp = new RegExp('[0-9]{' + cvv2DigitType + '}');
                        var isValid = value.length == cvv2DigitType && regExp.test(value);

                        if (isValid) {
                            $('.error[for="cvv2"]').remove();
                            $('input#cvv2').removeClass('error');
                        }

                        return isValid;
                    }

                    return false;
                }, this.localize.cvv2FormatMessage);
            }
            window.ValidationHelper.setupRules(selector);
        },

        initializeExpirationDateYears: function (selector) {
            var currentYear = (new Date).getFullYear();
            var maxYears = 15;

            selector.html('').append($('<option>', { value: '' }).text(this.localize.year));

            for (var i = 0; i <= maxYears; i++) {
                var value = currentYear + i;
                selector.append($('<option>', { value: value }).text(value));
            }
        },

        savePaymentMethod: function (callback) {
            var self = this;

            var form = $("#frmPaymentMethodsCreditCard");

            if (form.valid()) {
                var paymentMethod = new PaymentMethodsHelper();
                paymentMethod.insert({ paymentMethod: self.getPaymentMethodObject(), checkPassword: false }, function (response) {
                    if (response.cardExists !== undefined && response.cardExists === true) {
                        var errorMessage = $.sprintf(self.localize.errorCardAlreadySaved, $('#creditCardNumber').val());
                        window.showAlert(errorMessage);
                        return;
                    }

                    if (response.success) {
                        callback(true, self.cardId);
                    } else {
                        window.showAlert(self.localize.errorUnableToSaveNewCreditCard);
                        callback(false);
                    }
                });
            }
            else {
                return; //Invalid form, stay on page.
            }
        },

        validateForm: function (callback) {
            var self = this;

            var form = $("#frmPaymentMethodsCreditCard");

            if (form.valid()) {
                callback(true, self.getPaymentMethodObject());
            } else {
                //Invalid form, stay on page.
                callback(false);
                return;
            }
        },
        getPaymentMethodObject: function () {
            var selectedPaymentMethodType = 'CreditCard';
            var that = this;
            var utilities = new Utilities();
            var paymentMethod = new PaymentMethodsHelper();

            that.cardId = paymentMethod.getGuid();

            var paymentMethodObject = {
                Id: that.cardId,
                Name: that.cardValidation.getCardType($('#creditCardNumber').val().trim()).name,
                FirstName: utilities.capitalize($('#firstName').val()),
                LastName: utilities.capitalize($('#lastName').val()),
                CardNumber: $('#creditCardNumber').val(),
                ZipCode: that.useAVS ? $('#zipCode').val().toUpperCase() : null,
                ExpMonth: $('#expDateMonth :selected').val(),
                ExpYear: $('#expDateYear :selected').val(),
                Type: selectedPaymentMethodType
            };

            return paymentMethodObject;
        }
    });

    return view;
});