define([
  'text!templates/NewGiftCardView.html',
  'libs/jquery.validate.min',
  'helpers/paymentMethodHelper'
], function (template) {
    var view = Backbone.View.extend({
        template: _.template(template),
        localize: window.ObjectCollections.Localization.result,

        initialize: function () {
            this.cardId = null;
        },

        events: {
            'blur #giftCardNumber': 'getGiftCardBalance'
        },

        getGiftCardBalance: function () {
            var cardNumber = $('#giftCardNumber').val();

            if (this.lastGiftCard && cardNumber === this.lastGiftCard) {
                return;
            }

            this.lastGiftCard = cardNumber;

            $('.giftCardBalance').html('');

            if (cardNumber != "") {
                this.parent.getGiftCardBalance({ CardNumber: cardNumber });
            }
        },

        render: function (parent) {
            this.parent = parent;

            this.$el.html(this.template({
                localize: this.localize,
                model: this.options.model
            }));

            $('#saveCreditCard').closest('.ui-field-contain').addClass('saveGiftCardCheckbox');
            $('.giftCardBalanceContainer').addClass('newGiftCardBalance');

            $(this.el).find('#frmPaymentMethodsGiftCard').bind('keypress', function (e) {
                if (e.keyCode == 13) {
                    e.preventDefault();
                    $(this).valid();

                    return;
                }
            });

            return this;
        },

        savePaymentMethod: function (callback) {
            var self = this;
            var form = $("#frmPaymentMethodsGiftCard");

            if (form.valid()) {
                var paymentMethod = new PaymentMethodsHelper();
                paymentMethod.insert({ paymentMethod: self.getPaymentMethodObject(), checkPassword: false }, function (response) {
                    if (response.cardExists !== undefined && response.cardExists === true) {
                        var errorMessage = $.sprintf(self.localize.errorCardAlreadySaved, $('#giftCardNumber').val());
                        window.showAlert(errorMessage);
                        return;
                    }

                    if (response.success) {
                        callback(true, self.cardId);
                    } else {
                        window.showAlert(self.localize.errorUnableToSaveNewGiftCard);
                        callback(false);
                    }
                });
            } else {
                //Invalid form, stay on page.
                return;
            }
        },

        validateForm: function (callback) {
            var self = this;

            var form = $("#frmPaymentMethodsGiftCard");

            if (form.valid()) {
                callback(true, self.getPaymentMethodObject());
            } else {
                //Invalid form, stay on page.
                callback(false);
            }
        },

        getPaymentMethodObject: function () {
            var that = this;
            var paymentMethod = new PaymentMethodsHelper();
            that.cardId = paymentMethod.getGuid();

            var balanceValue = $(".giftCardBalance").html();
            if (balanceValue == that.localize.balanceUnavailable || balanceValue == that.localize.na) {
                balanceValue = '';
            }

            return {
                Id: that.cardId,
                Name: "GiftCard",
                CardNumber: $('#giftCardNumber').val(),
                Type: "GiftCard",
                Balance: balanceValue
            };
        }
    });

    return view;
});