define([
  'jquery',
  'underscore',
  'backbone',
  'text!templates/PaymentMethodSettingsDetailsView.html',
  'sharedhelpers/configurationProvider',
  'libs/jquery.validate.min',
  'sharedhelpers/storageProvider',
  'sharedhelpers/cultureHelper',
  'helpers/uiHelper',
  'helpers/paymentMethodHelper',
  'sharedhelpers/giftCardHelper',
  'sharedhelpers/cardValidationHelper',
  'helpers/validationHelper'
], function ($, _, Backbone,template, configurationProvider) {
    var view = Backbone.View.extend({
        template: _.template(template),
        localize: window.ObjectCollections.Localization.result,

        initialize: function () {
            this.cardValidation = new window.CardValidationHelper();
            this.cultureHelper = new window.CultureHelper();
            this.zipValidation = this.cultureHelper.zipValidation();
            this.isAndroidKitKat = window.AppProperties.IsAndroidKitKat;
        },

        events: {
            'tap #creditCardTab': 'onTabChange',
            'tap #giftCardTab': 'onTabChange',
            'tap #btnSaveDeletePaymentMethod': 'saveDeletePaymentMethod',
            'pagecreate': 'onLoad',
            'blur #giftCardNumber': 'giftCardNumberBlurHandler'
        },

        giftCardNumberBlurHandler: function () {
            var that = this;
            var cardNumber = $(that.el).find('#giftCardNumber').val();

            if (that.lastGiftCard && cardNumber === that.lastGiftCard) {
                return;
            }

            that.lastGiftCard = cardNumber;
            $(that.el).find('.giftCardBalance').html('');

            if (cardNumber != "") {
                window.giftCardHelper.abortBalanceCall();
                that.getGiftCardBalance(cardNumber);
            }
        },
        getGiftCardBalance: function (cardNumber) {
            var that = this;
            let balanceTobeHidden= window.giftCardHelper.checkIfBalanceNeedsToBeHidden(cardNumber);
            console.log("In Settings- New Payment card screen: HideBalance Property: "+ balanceTobeHidden);
               if(balanceTobeHidden)
                      $(that.el).find('.giftCardBalanceSpace').hide();
              else
                      $(that.el).find('.giftCardBalanceSpace').show();
            window.giftCardHelper.showBalanceLoader.call(that, cardNumber);

            var handlers = {
                onSuccess: function (balance) {
                    that.balanceCallFinished = true;
                    $(that.el).find('.giftCardBalance').html(balance);
                },
                onError: function () {
                    that.balanceCallFinished = false;
                    $(that.el).find('.giftCardBalance').html(that.localize.balanceUnavailable);
                },
                onComplete: function () {
                    window.giftCardHelper.hideBalanceLoader.call(that);
                }
            };

            window.giftCardHelper.getBalance(cardNumber, handlers);
        },

        onLoad: function () {
            this.setUpInitialValues($(this.el));
            this.setUpValidationRules($(this.el));
            this.loadGiftCardBalance();
            this.cancelSale();
        },

        cancelSale: function () {
            if (window.ObjectModels.OrderModel && window.ObjectModels.OrderModel.get('saleId')) {
                window.ObjectModels.OrderModel.cancelSale();
            }
        },

        loadStoredGiftCardBalance: function (cardNumber, callback) {
            var storage = new window.StorageProvider();
            storage.read('PaymentMethod', function (value) {
                if (value.length > 0) {
                    var cards = value;
                    _.each(cards, function (card) {
                        if (card.Type === "GiftCard" && new window.Utilities().decrypt(card.CardNumber) == cardNumber) {
                            callback(card);
                        }
                    });
                }
            });
        },
        loadGiftCardBalance: function () {
            var self = this;
            var selectedCard = self.options.paymentMethod[0];
            if (selectedCard.Type && selectedCard.Type == 'GiftCard') {
                self.loadStoredGiftCardBalance(selectedCard.CardNumber, function (card) {
                  var cardNumber = new window.Utilities().decrypt(card.CardNumber) || card.CardNumber;
                  let balanceTobeHidden= window.giftCardHelper.checkIfBalanceNeedsToBeHidden(cardNumber);
                    console.log("In Settings- Stored Payment card screen: HideBalance Property: "+ balanceTobeHidden);
                     if(balanceTobeHidden){
                            $(self.el).find('.giftCardBalanceSpace').hide();
                          }
                     else{
                    if (!card || !card.Balance || card.UpdateBalance) {
                        self.getGiftCardBalance(selectedCard.CardNumber);
                    }

                    $(self.el).find('.giftCardBalance').html(card.Balance);
                }
              });
            }
        },
        render: function () {
            var self = this;
            this.$el.html(this.template({
                localize: self.localize,
                paymentMethod: this.options.paymentMethod,
                zipValidation: this.zipValidation,
                configurationProvider: configurationProvider
            }));

            //Header fix for iOS when soft keyboard is active
            window.UIHelper.iOSHeaderFix(self);
            return this;
        },

        setUpInitialValues: function () {
            var self = this;

            self.initializeExpirationDateYears($(self.el).find("#expDateYear"));
            $(self.el).find("#expDateMonth").val(self.options.paymentMethod[0]['ExpMonth']).attr('selected', true).siblings('option').removeAttr('selected');
            $(self.el).find("#expDateYear").val(self.options.paymentMethod[0]['ExpYear']).attr('selected', true).siblings('option').removeAttr('selected');
            $(self.el).find('#expDateMonth, #expDateYear').selectmenu('refresh');
            $(self.el).find("#btnSaveDeletePaymentMethod").html(self.options.paymentMethod[0].Id === undefined ? self.localize.saveButton : self.localize.deleteButton).addClass(self.options.paymentMethod[0].Id === undefined ? 'ncr-btn' : 'red-btn');

            if (self.options.paymentMethod.length > 1 && !this.options.paymentMethod[0].ZipCode) $(self.el).find('#zipCode').closest('li').hide();

            if (self.options.paymentMethod[0].Id) {
                $(self.el).find("#locationDetails").hide();
                $(self.el).find(".notification-bar, .breakLine").hide();
                self.disableFields($(self.el));
            }

            if (self.options.paymentMethod[0].Type == 'GiftCard') {
                $(self.el).find('#gift-card-container').show();
                $(self.el).find('#credit-card-container').hide();
            }
        },

        setUpValidationRules: function (selector) {
            var self = this;
            var cardPassword = $("#cardPassword").val();

            $.validator.addMethod('creditcard', function (value) {
                var creditCard = self.cardValidation.isCreditCardValid({
                    cardNumber: value,
                    checkIsSupported: false
                });

                return creditCard.isValid;
            }, self.localize.errorEnterValidCreditCardNumber);

            $.validator.addMethod('giftCard', function (value) {
                var regExp = new RegExp('^[0-9]+$');
                return regExp.test(value);
            }, self.localize.errorGiftCardNumber);

            if (cardPassword) {
                window.ValidationHelper.validatePassword(cardPassword);
            }

            //custom validation rules
            selector.find('#cardPassword, #repeatPassword, #frmPaymentMethodsGiftCard, #frmPaymentMethodsCreditCard').bind('keypress', function (e) {
                if (e.keyCode == 13) {
                    e.preventDefault();
                    self.setExpDateErrorLabelPosition($(self.el));
                    switch (e.currentTarget.id) {
                        case 'cardPassword':
                        case 'repeatPassword':
                            self.savePassword(e);
                            break;
                        case 'frmPaymentMethodsCreditCard':
                        case 'frmPaymentMethodsGiftCard':
                            self.saveDeletePaymentMethod(e);
                            break;
                    }
                    return;
                }
            });

            selector.find('#createPassword').on({
                popupafterclose: function () {
                    $(self.el).find('input').blur();
                    $(self.el).find('select').blur();
                    $(self.el).find('#btnSaveDeletePaymentMethod').blur();
                },
                popupbeforeposition: function () {
                    $(self.el).find('.ui-popup-screen').off();
                }
            });

            window.ValidationHelper.setupRules(selector);
        },

        setExpDateErrorLabelPosition: function (selector) {
            if (selector !== undefined) {
                $('<label class="error" generated="true">' + this.localize.errorExpirationDateInvalid + '</label>').insertAfter($("#expirationDate"));
                return;
            }
            $('<label class="error" generated="true">' + this.localize.errorExpirationDateInvalid + '</label>').insertAfter($("#expirationDate"));
        },

        initializeExpirationDateYears: function (selector) {
            var currentYear = (new Date).getFullYear();
            var maxYears = 15;
            selector.html('').append($('<option>', { value: '' }).text(this.localize.year));

            for (var i = 0; i <= maxYears; i++) {
                var value = currentYear + i;
                selector.append($('<option>', { value: value }).text(value));
            }
        },

        onTabChange: function (e) {
            e.preventDefault();
            this.clearScreen();

            if (e.currentTarget.id === 'giftCardTab') {
                this.renderGiftCardTab();
            } else {
                this.renderCreditCardTab();
            }
        },

        clearScreen: function () {
            $(this.el).find('form').find('input').val("");
            $(this.el).find('label.error').remove();
            $(this.el).find('.error').removeClass('error');
            $(this.el).find(':selected').parent().removeAttr('selected');
        },

        renderCreditCardTab: function () {
            $(this.el).find('#gift-card-container').hide();
            $(this.el).find('#credit-card-container').show();

            $(this.el).find('.card-container:visible').attr('param', 'CreditCard');

            $(this.el).find('label[for=creditCardNumber]').html(this.localize.creditCard);
            $(this.el).find('#btnSaveDeletePaymentMethod').removeClass('gift-cards-btn');
        },

        renderGiftCardTab: function () {
            $(this.el).find('#credit-card-container').hide();
            $(this.el).find('#gift-card-container').show();
            $(this.el).find('.giftCardBalanceSpace').hide();

            var balance = this.options.paymentMethod[0].balance || window.Globalize.format(0, 'c', window.AppConfig.AppCulture);
            $(this.el).find('.giftCardBalance').html(balance);
            $(this.el).find('.giftCardBalance').addClass('paymentMethodsDetailsBalance');
            $(this.el).find('.loyalty-card-signup').css('margin-bottom', '15px');

            $(this.el).find('#btnSaveDeletePaymentMethod').addClass('gift-cards-btn');
        },

        disableFields: function (selector) {
            selector.find(":input").attr('disabled', 'disabled').parent().addClass('ui-disabled');
        },

        saveDeletePaymentMethod: function (event) {
            event.preventDefault();
            var btnText = $(this.el).find('#btnSaveDeletePaymentMethod').text();

            if (btnText == this.localize.saveButton) {
                this.savePaymentMethod();
            } else {
                this.deletePaymentMethod();
            }
        },

        savePaymentMethod: function () {
            var self = this;
            var selectedPaymentMethodType = $(self.el).find('.card-container:visible').attr('param');

            var isFormValid = selectedPaymentMethodType === 'CreditCard' ? $(self.el).find("#frmPaymentMethodsCreditCard").valid() :
                            selectedPaymentMethodType === 'GiftCard' ? $(self.el).find("#frmPaymentMethodsGiftCard").valid() : true;

            $(self.el).find('select').parent().find('label.error').remove();

            if (isFormValid) {
                window.showPageLoadingMessage();
                var paymentMethodHelper = new window.PaymentMethodsHelper();
                var paymentMethodObject = self.getPaymentMethodObject();

                paymentMethodHelper.insert({ paymentMethod: paymentMethodObject }, function (response) {
                    if (response.passwordExists === false) {
                        window.hidePageLoadingMessage();
                        require(['views/createPassDialogView'], function (createPassView) {
                            var dialog = new createPassView({
                                selectedPaymentMethodType: selectedPaymentMethodType,
                                dialogTopNotification: self.localize.createPasswordNotification1,
                                dialogBottomNotification: self.localize.createPasswordNotification2,
                                callback: function (value) {
                                    $(self.el).find('#popupDialog').popup('close');

                                    if (value) {
                                        self.savePaymentMethod();
                                    }
                                }
                            });

                            $(self.el).find('#popupDialog').html(dialog.render());
                            $(self.el).find('#popupDialog').popup('open');
                            $(self.el).find('#popupDialog').enhanceWithin();
                        });
                        return;
                    }

                    if (response.cardExists) {
                        window.hidePageLoadingMessage();
                        var errorMessage = $.sprintf(self.localize.errorCardAlreadySaved,
                            $(self.el).find('.card-container:visible').attr('param') === 'GiftCard' ? $(self.el).find('#giftCardNumber').val() : $(self.el).find('#creditCardNumber').val()
                        );

                        window.showAlert(errorMessage);
                        return;
                    }

                    if (response.success) {
                        if (configurationProvider.useDualCards() && paymentMethodObject.Type == 'GiftCard') {
                            require(['helpers/dualCardHelper'], function () {
                                var callback = function () {
                                  window.goBack();
                                };

                                window.dualCardHelper.handleLoyaltyCard.call(self, paymentMethodObject.CardNumber, callback);
                            });

                            return;
                        }
                        window.goBack();
                    }
                });
            }
        },

        deletePaymentMethod: function () {
            var self = this;

            var deletePaymentCard = function () {
                var onConfirm = function (button) {
                    if (button == 1) {

                        var pmId = self.options.paymentMethod[0].Id;
                        var pmh = new window.PaymentMethodsHelper();
                        window.showPageLoadingMessage();

                        pmh.remove(pmId, function (response) {
                            if (response.success) {

                                if (!configurationProvider.useVirtualGiftCard()) {
                                    //Update Menu to hide Gift Cards
                                    var menuIds = window.MenuHelper.getMenuIds();
                                    window.MenuHelper.hideMenuItem(menuIds.virtualGiftCards);
                                }
                                window.hidePageLoadingMessage();
                                window.goBack();
                            }
                        });
                    }
                };

                if (window.isMobileDevice()) {
                    navigator.notification.confirm(self.localize.removePaymentCardFromApplication, onConfirm, self.localize.alertTitle, self.localize.ok + ', ' + self.localize.cancel);
                } else if (confirm(self.localize.removePaymentCardFromApplication)) {
                    onConfirm(1);
                }
            };

            if (!configurationProvider.useDualCards()) {
                deletePaymentCard();
                return;
            }

            var storage = new window.StorageProvider();

            storage.read('LoyaltyCard', function (loyaltyCard) {
                if (loyaltyCard && loyaltyCard.number == self.options.paymentMethod[0].CardNumber) {
                    require(['helpers/dualCardHelper'], function () {
                        var callback = function () {                          
                             window.goBack();
                        };

                        window.dualCardHelper.deleteCard(self.options.paymentMethod[0].Id, callback);
                    });

                    return;
                }

                deletePaymentCard();
            });
        },

        getPaymentMethodObject: function () {
            var selectedPaymentMethodType = $(this.el).find('.card-container:visible').attr('param');
            var paymentMethod;

            if (selectedPaymentMethodType === 'GiftCard') {
                paymentMethod = {
                    Name: 'GiftCard',
                    CardNumber: $(this.el).find('#giftCardNumber').val(),
                    Type: selectedPaymentMethodType
                };

                if (this.balanceCallFinished && configurationProvider.useGiftCardBalance() && $(this.el).find('.giftCardBalance').html().indexOf('error') == -1) {
                    paymentMethod['Balance'] = $(this.el).find('.giftCardBalance').html();
                }
            } else {
                paymentMethod = {
                    FirstName: $(this.el).find('#firstName').val(),
                    LastName: $(this.el).find('#lastName').val(),
                    Name: this.cardValidation.getCardType($(this.el).find('#creditCardNumber').val()).name,
                    CardNumber: $(this.el).find('#creditCardNumber').val(),
                    ExpMonth: $(this.el).find('#expDateMonth :selected').val(),
                    ExpYear: $(this.el).find('#expDateYear :selected').val(),
                    Type: selectedPaymentMethodType
                };

                if (window.AppConfig.AppCulture == "en-GB") {
                    paymentMethod['ZipCode'] = $(this.el).find('#zipCode').val().match(/\d/g).join("");
                } else {
                    paymentMethod['ZipCode'] = $(this.el).find('#zipCode').val();
                }
            }

            return paymentMethod;
        }
    });

    return view;
});
