define([
  'jquery',
  'underscore',
  'backbone',
  'text!templates/PaymentMethodSettingsView.html',
  'sharedhelpers/configurationProvider',
  'sharedhelpers/giftCardHelper',
  'sharedhelpers/storageProvider',
  'helpers/virtualGiftCardHelper'
], function ($, _, Backbone,template, configurationProvider) {
    var view = Backbone.View.extend({
        template: _.template(template),
        localize: window.ObjectCollections.Localization.result,

        events: {
            'pagebeforeshow': 'onPageShow',
            'pagebeforehide': 'onPageHide',
            'tap #newPayment': 'navigateNewPayment',
            'tap #paymentDetails': 'navigatePaymentDetails'
        },

        render: function () {
            this.$el.html(this.template({
                localize: this.localize,
                paymentMethods: this.options.paymentMethods,
                configurationProvider: configurationProvider,
                isVgc: this.options.isVgc
            }));
            return this;
        },

        onPageHide: function () {
            window.giftCardHelper.abortBalanceCall(true);
        },

        navigateNewPayment: function (e) {
            e.preventDefault();
            if (window.isJqmGhostClick(e) || window.isChangePageFired(e)) return false;

            window.showPageLoadingMessage();

            var that = this;

            _.defer(function () {
                window.AppMethods.ChangePage(that.options.isVgc ? '#virtualGiftCard' : '#paymentMethodSettingsDetails/new', false, true);
            });
        },

        navigatePaymentDetails: function (e) {
            e.preventDefault();
            if (window.isJqmGhostClick(e) || window.isChangePageFired(e)) return false;

            window.showPageLoadingMessage();

            var that = this;
            var id = $(e.target).closest('#paymentDetails').attr('data-id');
            var navigateTo = that.options.isVgc ? '#virtualGiftCard/' + id : '#paymentMethodSettingsDetails/' + id;

            _.defer(function () {
                window.AppMethods.ChangePage(navigateTo, false, true);
            });
        },

        loadStoredGiftCardBalances: function (callback) {
            var that = this;

            _.each(that.options.paymentMethods, function (card) {
                if (card.Type === "GiftCard" || card.Type == 'VirtualGiftCard') {
                    let balanceTobeHidden= window.giftCardHelper.checkIfBalanceNeedsToBeHidden(card.CardNumber);
                      console.log("In Settings-All stored payment cards listed page, HideBalance Property: "+balanceTobeHidden);
                    if(!balanceTobeHidden){
                    var balance = card.Balance || window.Globalize.format(0, 'c', window.AppConfig.AppCulture);
                    $(that.el).find('.ps-giftCardBalance[data-card="' + card.CardNumber + '"]').html("(" + balance + ")");
                       }
                      else {
                  $(that.el).find('.ps-giftCardBalance[data-card="' + card.CardNumber + '"]').hide();
                  }
              }
            });

            if (callback) {
                callback();
            }
        },

        onPageShow: function () {
            var that = this;

            that.loadStoredGiftCardBalances(function () {
                if (!window.giftCardBalanceCleared) {
                    window.giftCardHelper.invalidateBalance(function () {
                        that.getGiftCardBalance();
                    });
                    window.giftCardBalanceCleared = true;
                } else {
                    var fetchBalance = false;
                    _.each(that.options.paymentMethods, function (card) {
                        if ((card.Type === "GiftCard" || card.Type === "VirtualGiftCard") && (!card.Balance || card.UpdateBalance)) {
                            fetchBalance = true;
                        }
                    });

                    if (fetchBalance) {
                        that.getGiftCardBalance();
                    }
                }
            });
        },

        getGiftCardBalance: function () {
            var that = this,
                paymentCards,
                giftCards = [],
                storage = new window.StorageProvider(),
                util = new window.Utilities();

            storage.read('PaymentMethod', function (value) { paymentCards = value; });

            var loyaltyCard;
            if (window.AppProperties.PassbookAvailable.dualCardPass) {
                storage.read('LoyaltyCard', function (value) { loyaltyCard = value; });
            }

            _.each(paymentCards, function (paymentObject) {
                var condition = (that.options.isVgc ? paymentObject.Type === "VirtualGiftCard" : paymentObject.Type === "GiftCard") && (!paymentObject.Balance || paymentObject.UpdateBalance);

                if (condition) {
                    paymentObject.CardNumber = util.decrypt(paymentObject.CardNumber);
                    giftCards.push(paymentObject);
                }
            });

            _.each(giftCards, function (paymentObject) {
                var paymentCardNumber = paymentObject.CardNumber;

                var handlers = {
                    onSuccess: function (balance) {
                        var updateCard = _.find(giftCards, function (card) {
                            if (paymentCardNumber == card.CardNumber) {
                                card.Balance = balance;
                                $(that.el).find('.ps-giftCardBalance[data-card="' + card.CardNumber + '"]').html("(" + card.Balance + ")");
                                return true;
                            }
                        });

                        if (updateCard.Type == 'VirtualGiftCard') {
                            window.virtualGiftCardHelper.updateBalance(updateCard);
                        } else {
                            window.giftCardHelper.updateBalance(updateCard);

                            if (loyaltyCard && loyaltyCard.number == updateCard.CardNumber) {
                                require(['models/loyaltyAccountModel', 'helpers/passHelper'], function (loyaltyAccountModel) {
                                    loyaltyCard = new loyaltyAccountModel(loyaltyCard);

                                    var passHelper = new window.PassHelper();
                                    passHelper.getLoyaltyCardPassURL(loyaltyCard, true);
                                });
                            }
                        }
                    },
                    onError: function () {
                        $(that.el).find('.ps-giftCardBalance[data-card="' + paymentCardNumber + '"]').html("(" +that.localize.na + ")");
                    },
                    onComplete: function () {
                        window.giftCardHelper.hideBalanceLoader.call(that, paymentCardNumber);
                    },
                    onProgress: function () {
                        window.giftCardHelper.showBalanceLoader.call(that, paymentCardNumber);
                    }
                };
                let balanceTobeHidden= window.giftCardHelper.checkIfBalanceNeedsToBeHidden(paymentCardNumber);
                if(!balanceTobeHidden){
              window.giftCardHelper.getBalance(paymentCardNumber, handlers);
            }
            else {
              console.log("In Settings-All stored payment cards list screen, HideBalance Property: "+balanceTobeHidden+ " Aborted Balance call");
            }
            });
        }
    });

    return view;
});
