define([
  'jquery',
  'underscore',
  'backbone',
  'sharedhelpers/configurationProvider',
  'text!templates/PaymentMethodView.html',
  'views/pullDownStatusView',
  'views/newCreditCardView',
  'views/newGiftCardView',
  'views/creditCardsHolderView',
  'sharedhelpers/storageProvider',
  'helpers/uiHelper',
  'helpers/configHelper',
  'sharedhelpers/utilities',
  'sharedjs/RSAEncryption/BigInt',
  'sharedjs/RSAEncryption/Barrett',
  'sharedjs/RSAEncryption/RSA',
  'sharedhelpers/errorHelper',
  'sharedhelpers/giftCardHelper',
  'helpers/virtualGiftCardHelper',
  'sharedhelpers/cardValidationHelper',
  'helpers/touchIdHelper'
], function ($, _, Backbone,configurationProvider, template, pullDownStatusView, newCreditCardView, newGiftCardView, creditCardsHolderView) {
    var view = Backbone.View.extend({
        template: _.template(template),
        localize: window.ObjectCollections.Localization.result,

        initialize: function () {
            window.tempHistoryCount++;

            var cvv2ElementWrapper = $('<div>').addClass('cvv-wrapper').css({ 'width': '50%', 'margin-left': 10 });
            var cvv2Input = '<input type="tel" name="cvv2" id="cvv2" placeholder="' + this.localize.cvv2Placeholder + '" value="" />';
            this.cvv2Element = cvv2ElementWrapper.append(cvv2Input);

            this.pullDownStatusView = null;
            this.newCreditCardView = null;
            this.newGiftCardView = null;

            this.storageKeys = {
                phoneNumber: 'phoneNumber',
                emailAddress: 'emailAddress',
                password: 'PaymentMethodPassword',
                paymentMethod: 'PaymentMethod',
                fb: 'fbAccount',
                paymentSettings: 'paymentSettings',
                loyaltyCards: 'LoyaltyCard',
                purchaseHistory: 'PurchaseHistory'
            };

            this.phone = null;
            this.email = null;
            this.paymentCardId = null;
            this.usedPaymentCards = [];
            this.partialAmounts = [];

            this.unsavedPaymentCard = null;
            this.preventSave = false;

            this.phoneChecked = true;
            this.emailChecked = true;
            this.enterPass = true;

            this.saveInfo = false;
            this.sendEmail = true;

            this.theater = window.ObjectModels.OrderModel.get('theater');
            this.useEmailConfirmation = configurationProvider.useEmailConfirmation();
            this.useSMSConfirmation = configurationProvider.useSMSConfirmation();
            this.useLoyalty = configurationProvider.useLoyalty(this.theater);
            this.useMasterCard = this.theater.get('acceptsMasterCard');
            this.useAmericanExpress = this.theater.get('acceptsAmericanExpress');
            this.useDiscover = this.theater.get('acceptsDiscover');
            this.useVisa = this.theater.get('acceptsVisa');
            this.useCvv2 = this.theater.get('cVV2RequiredForCredit');
        },

        getSubscriptionCards: function () {
            var cards = {};
            var cardNumbers = [];
            var orderModel = window.ObjectModels.OrderModel;
            var tickets = orderModel.get('tickets') ? orderModel.get('tickets').models : [];

            _.each(tickets, function (ticket) {
                if (ticket.get('subscriptionCardNumbers')) {
                    _.each(ticket.get('subscriptionCardNumbers'), function (card) {
                        cardNumbers.push(card.number);
                    });

                    cards[ticket.get('name')] = cardNumbers.join(', ');
                    cardNumbers = [];
                }
            });

            return cards;
        },

        render: function () {
            var that = this;
            $(that.el).html(that.template({
                localize: that.localize,
                loyalty: that.options.loyaltyHolder,
                loyaltyNumber: window.ObjectModels.OrderModel.get('loyaltyCardNumber'),
                subscriptionCards: that.getSubscriptionCards(),
                acceptsLoyalty: that.useLoyalty,
                performance: that.options.performance,
                hasHoldSeats: that.options.hasHoldSeats,
                isVgc: !!window.ObjectModels.OrderModel.get('virtualGiftCard'),
                useEmailConfirmation: that.useEmailConfirmation,
                useSMSConfirmation: that.useSMSConfirmation
            }));

            that.pullDownStatusView = new pullDownStatusView();
            that.assign(that.pullDownStatusView, '.pullDownStatusHolder');

            that.newCardHolder = $(that.el).find('#newCardHolder');
            that.cardHolder = $(that.el).find('#cardHolder');

            $(that.el).find('#passDialog').on({
                popupafterclose: function () {
                    $(that.el).find('#passDialog').empty();
                },
                popupbeforeposition: function () {
                    $(that.el).find('.ui-popup-screen').off();
                }
            });

            //Header fix for iOS when soft keyboard is active
            window.UIHelper.iOSHeaderFix(that);

            var retView = configurationProvider.arangeFlow(that);

            that.getStoredValue(that.storageKeys.paymentSettings, function (val) {
                val = val || {};
                if (val.saveInfo !== undefined) {
                    that.saveInfo = val.saveInfo;
                }
                if (val.sendEmail !== undefined) {
                    that.sendEmail = val.sendEmail;
                }

                that.initCards(val.selectedCard, function () {
                    var isVgc = !!window.ObjectModels.OrderModel.get('virtualGiftCard');
                    var saveInfoCheckbox = $(that.el).find('#saveCreditCard');
                    var sendEmailCheckbox = $(that.el).find('#email');

                    window.UIHelper.setCheckbox(saveInfoCheckbox, that.saveInfo);
                    window.UIHelper.setCheckbox(sendEmailCheckbox, isVgc || that.sendEmail);

                    that.displayGiftIcon();
                });
            });
            return retView;
        },
        displayGiftIcon: function () {
            var that = this;
            var selectedVgc = $(that.el).find('#selectCard option:contains("' + that.localize.virtualGiftCard + '")');
            if (selectedVgc) {
                selectedVgc.addClass('vgc-icon');
            }
        },
        initCards: function (selected, callback) {
            var that = this;

            var showCardList = function (list) {
                var holderView = new creditCardsHolderView({ cards: list, acceptsSVC: that.theater.get('acceptsSVC') });
                that.assign(holderView, that.cardHolder);

                _.each(that.usedPaymentCards, function (cardId) {
                    $(that.el).find('#selectCard option[value="' + cardId + '"]').attr('disabled', 'disabled');
                });
            };

            that.getStoredValue(that.storageKeys.paymentMethod, function (cards) {
                cards = cards || [];

                //During Virgual Gift Card creation/update flow same VGC shouldn't be used as a payment method and shouldn't be marked as selected
                var vgcCard = window.ObjectModels.OrderModel.get('virtualGiftCard');

                if (vgcCard) {
                    cards = _.filter(cards, function (card) {
                        if (card.Type == 'VirtualGiftCard' && card.Id == selected) {
                            selected = false;
                        }

                        return new window.Utilities().decrypt(card.CardNumber) != vgcCard.number;
                    });
                }

                if (cards.length > 0) {
                    _.each(cards, function (item) {
                        if (!selected || selected == 'newCreditCard' || selected == 'newGiftCard') {
                            cards[cards.length - 1].selected = true;
                        }

                        if (item.Id == selected) {
                            item.selected = true;
                        }
                    });

                    var card = _.find(cards, function (item) {
                        return item.selected;
                    });

                    that.paymentCardId = card.Id;

                    showCardList(cards);

                    if (card.Type == 'CreditCard' && that.useCvv2) {
                        $(that.el).find('.giftCardBalanceContainer').hide();
                        that.cardHolder.append(that.cvv2Element.enhanceWithin());
                    }
                    if ((card.Type == 'GiftCard' || card.Type == 'VirtualGiftCard') && configurationProvider.useGiftCardBalance()) {
                        $(that.el).find('.giftCardBalanceContainer').removeClass('newGiftCardBalance');
                        that.handleGiftCardBalance(card);
                    }
                } else {
                    showCardList(cards);

                    that.newCreditCardView = new newCreditCardView();
                    that.assign(that.newCreditCardView, that.newCardHolder);
                }

                if (callback) {
                    callback();
                }
            });
        },
        reloadCards: function () {
            var that = this;
            that.getStoredValue(that.storageKeys.paymentMethod, function (value) {
                if (value) {
                    that.initCards(value.selectedCard);
                    that.newCreditCardView = null;
                    that.newGiftCardView = null;
                    that.assign(that.newCreditCardView, that.newCardHolder);
                }
            });
        },
        events:
        {
            "pagehide": "pageHide",
            "change #selectCard": "selectCardChanged",
            "tap #myTicketsView": "myTicketView",
            "tap .loyalty-scan": "scanLoyaltyCard",
            "pagebeforehide": "beforeHide"
        },

        handleGiftCardBalance: function (card) {
            var that = this;
            $(that.el).find('.giftCardBalanceContainer').show();
            var balance = card.Balance;
            if (!balance || card.UpdateBalance) {
                  _.defer(function () {
                      that.getGiftCardBalance(card);
                  });
              }

            var cardNumber = new window.Utilities().decrypt(card.CardNumber) || card.CardNumber;
            let balanceTobeHidden= window.giftCardHelper.checkIfBalanceNeedsToBeHidden(cardNumber);
            console.log("In Checkout screen- Payment Card: HideBalance Property : "+balanceTobeHidden);
               if(!balanceTobeHidden)
                      {
                            $(that.el).find('.giftCardBalance').html(balance || window.Globalize.format(0, 'c', window.AppConfig.AppCulture));
                      }
                      else {
                             $(that.el).find('.giftCardBalanceContainer').hide();
                      }

            if (!window.giftCardBalanceCleared) {
                window.giftCardHelper.invalidateBalance();
                window.giftCardBalanceCleared = true;
            }
        },

        getGiftCardBalance: function (card) {
            var that = this;
            var cardNumber = new window.Utilities().decrypt(card.CardNumber) || card.CardNumber;

            if (this.lastGiftCard && cardNumber == this.lastGiftCard) {
                return;
            }

            this.lastGiftCard = cardNumber;
          let balanceTobeHidden= window.giftCardHelper.checkIfBalanceNeedsToBeHidden(cardNumber);
          console.log("In Checkout screen- New Payment Card: HideBalance Property : "+balanceTobeHidden);
             if(!balanceTobeHidden && configurationProvider.useGiftCardBalance())
                    {
                        $(that.el).find('.giftCardBalanceContainer').show();
                        $(that.el).find('.saveGiftCardCheckbox').css({ 'top': '50px' });
                    }
                    else {
                      $(that.el).find('.giftCardBalanceContainer').hide();
                      $(that.el).find('.saveGiftCardCheckbox').css({ 'top': '3px' });
                    }
            window.giftCardHelper.abortBalanceCall();

            var handlers = {
                onSuccess: function (balance) {
                    if (card.Type == 'VirtualGiftCard') {
                        card.Balance = balance;
                        card.CardNumber = cardNumber;
                        window.virtualGiftCardHelper.updateBalance(card);
                    } else {
                        window.giftCardHelper.updateBalance({ CardNumber: cardNumber, Balance: balance });

                        if (window.AppProperties.PassbookAvailable.dualCardPass) {
                            that.storage.read(that.storageKeys.loyaltyCards, function (loyaltyCard) {
                                if (loyaltyCard && loyaltyCard.number == cardNumber) {
                                    require(['models/loyaltyAccountModel', 'helpers/passHelper'], function (loyaltyAccountModel) {
                                        loyaltyCard = new loyaltyAccountModel(loyaltyCard);

                                        var passHelper = new window.PassHelper();
                                        passHelper.getLoyaltyCardPassURL(loyaltyCard, true);
                                    });
                                }
                            });
                        }
                    }

                    $(that.el).find('.giftCardBalance').html(balance);
                },
                onError: function () {
                    $(that.el).find('.giftCardBalance').html(that.localize.balanceUnavailable);
                },
                onComplete: function () {
                    window.giftCardHelper.hideBalanceLoader.call(that);
                },
                onProgress: function () {
                    window.giftCardHelper.showBalanceLoader.call(that, cardNumber);
                }
            };

            window.giftCardHelper.getBalance(cardNumber, handlers);
        },
        beforeHide: function () {
            $(this.el).find('#myTicketsView').css({ 'pointer-events': 'none' });
        },
        pageHide: function () {
            $(this.el).find('.ui-content').hide();
        },
        scanLoyaltyCard: function (e) {
            e.preventDefault();
            var that = this;

            var callback = function (result) {
                if (result.text) {
                    $(that.el).find('#loyaltyCardNumber').val(result.text);
                }
            };

            require(['js/helpers/utilities'], function () {
                var utils = new window.Utils();
                utils.scanBarcode(callback);
            });
        },
        getStoredValue: function (key, callback) {
            this.storage = this.storage || new window.StorageProvider();
            this.storage.read(key, function (value) {
                callback(value);
            });
        },
        setStoredValue: function (key, value, overwrite) {
            this.storage = this.storage || new window.StorageProvider();
            try {
                if (overwrite) {
                    this.storage.remove(key);
                }
                this.storage.store(key, value);
            } catch (e) {
                console.log('error', e);
            }
        },
        selectCardChanged: function (e) {
            e.preventDefault();
            var that = this;
            var selector = $(e.currentTarget);
            that.lastGiftCard = null;

            that.newCreditCardView = new newCreditCardView();
            that.newGiftCardView = new newGiftCardView();
            window.giftCardHelper.abortBalanceCall();

            $(that.el).find('.giftCardBalanceContainer').removeClass('newGiftCardBalance');
            $(that.el).find('#saveCreditCard').closest('.ui-field-contain').removeClass('saveGiftCardCheckbox');
            $(that.el).find('.giftCardBalanceContainer').hide();

            if (selector.val() == 'newCreditCard') {
                that.assign(that.newCreditCardView, that.newCardHolder);
                that.newGiftCardView = null;
                that.paymentCardId = null;
            } else if (selector.val() == 'newGiftCard') {
                that.assign(that.newGiftCardView, that.newCardHolder);
                that.newCreditCardView = null;
                that.paymentCardId = null;
                $(that.el).find('.giftCardBalanceContainer').hide();
                $(that.el).find('.saveGiftCardCheckbox').css({ 'top': '0' });
            } else {
                that.paymentCardId = selector.val();

                that.getStoredValue(that.storageKeys.paymentMethod, function (creditCards) {
                    var item = _.find(creditCards, function (c) { return c.Id == that.paymentCardId; });
                    if ((item.Type == 'GiftCard' || item.Type == 'VirtualGiftCard') && configurationProvider.useGiftCardBalance()) {
                        that.handleGiftCardBalance(item);
                    } else {
                        $(that.el).find('.giftCardBalanceContainer').hide();
                    }
                });

                that.clear(that.newCardHolder);
                that.newCreditCardView = null;
                that.newGiftCardView = null;
            }

            that.hideValidationError(selector);
            that.hideValidationError(that.cardHolder.find('#cvv2'));

            var cvvWrapper = that.cardHolder.find('.cvv-wrapper');
            cvvWrapper.remove();

            if (that.isCreditCard() && selector.val() != 'newCreditCard' && that.useCvv2) {
                that.cardHolder.append(that.cvv2Element.enhanceWithin());
            }
            var isVgc = !!window.ObjectModels.OrderModel.get('virtualGiftCard');
            var saveInfoCheckbox = $(that.el).find('#saveCreditCard');
            var sendEmailCheckbox = $(that.el).find('#email');

            window.UIHelper.setCheckbox(saveInfoCheckbox, that.saveInfo);
            window.UIHelper.setCheckbox(sendEmailCheckbox, isVgc || that.sendEmail);

            that.displayGiftIcon();
        },

        assign: function (inview, selector) {
            this.$(selector).empty();
            inview.setElement(this.$(selector)).render(this);
            this.$(selector).enhanceWithin();
        },

        clear: function (selector) {
            this.$(selector).empty();
        },

        showValidationError: function (obj, message, afterObj) {
            obj.parent().addClass('error');

            var id = obj.attr('id') || obj.attr('name');
            $('<label class="error" for="' + id + '" generated="true">' + message + '</label>').insertAfter(afterObj || obj.parent());
        },

        hideValidationError: function (obj) {
            obj.parent().removeClass('error');

            var id = obj.attr('id') || obj.attr('name');
            $(this.el).find('.error[for="' + id + '"]').remove();
        },

        cvv2Validation: function (obj, callback) {
            this.hideValidationError(obj);

            if (obj.val().trim() == '') {
                this.showValidationError(obj, this.localize.fieldIsRequired);
                window.hidePageLoadingMessage();
                return;
            }

            var that = this;

            that.getStoredValue(that.storageKeys.paymentMethod, function (creditCards) {
                var item = _.find(creditCards, function (c) { return c.Id == that.paymentCardId; }) || that.unsavedPaymentCard;

                if (!item) {
                    callback(true);
                }

                var cvv2Digits = new window.CardValidationHelper().getCVV2Digits({ cardName: item.Name });

                var regExp = new RegExp('[0-9]{' + cvv2Digits + '}');
                var isValid = (obj.val().length == cvv2Digits && regExp.test(obj.val()));

                if (!isValid) {
                    that.showValidationError(obj, that.localize.cvv2FormatMessage);
                }

                callback(isValid);
            });
        },
        isChecked: function (selector) {
            if (selector.length == 0) return null;

            return selector.hasClass('ui-checkbox-on') || selector.is(':checked');
        },
        isCreditCard: function () {
            var selectedCardType = $(this.el).find('#selectCard option:selected').data('card-type');

            return this.paymentCardId && $(this.el).find('#selectCard').val() !== 'newGiftCard' && selectedCardType != 'GiftCard' && selectedCardType != 'VirtualGiftCard';
        },
        myTicketView: function (event) {
            $('input').blur();
            if (event) {
                event.preventDefault();
                if ((window.isJqmGhostClickPropagated(event) || window.isJqmGhostClick(event))) return;
            }
            window.showPageLoadingMessage();

            var that = this;
            var selectedCardValue = $(that.el).find('#selectCard').val();
            var selectedView = that[selectedCardValue + 'View'];
            var saveCardSelector = $(that.el).find('#saveCreditCard');
            var emailSelector = $(that.el).find('#email');
            var smsSelector = $(that.el).find('#sms');

            var touchIdHelper = new window.TouchIdHelper();

            that.isFormValid(selectedView, function (response, save) {
                window.hidePageLoadingMessage();
                if (!response) return;

                var saveCard = !that.preventSave && that.isChecked(saveCardSelector) && selectedView != null;
                var smsChecked = that.useSMSConfirmation && that.phoneChecked && that.isChecked(smsSelector);
                var emailChecked = that.useEmailConfirmation && that.emailChecked && that.isChecked(emailSelector);

                var saveCallback = function (success, id) {
                    if (success) {
                        that.paymentCardId = id;
                        that.preventSave = true;
                        that.enterPass = false;

                        if (selectedView instanceof newGiftCardView && configurationProvider.useDualCards()) {
                            that.checkDualCardId = id;
                        }

                        that.myTicketView(); //success.
                    }
                };

                if (saveCard) {
                    var savePaymentMethod = function () {
                        if (selectedView && selectedView.savePaymentMethod) {
                            selectedView.savePaymentMethod(saveCallback);
                        } else {
                            saveCallback(true, null);
                        }
                    };

                    var passwordDialog = function () {
                        that.openPasswordDialog(function (res) {
                            if (res) {
                                savePaymentMethod();
                            }
                        });
                    };

                    touchIdHelper.verifyFingerprint(savePaymentMethod, passwordDialog);
                    return;

                } else if (that.isChecked(saveCardSelector) === false && selectedView != null) {
                    that.enterPass = false;
                }

                var selectedVgc = $(that.el).find('#selectCard option:selected').attr('data-card-type') == 'VirtualGiftCard';
                if (that.enterPass && !selectedVgc) {
                    var continueFlow = function () {
                        that.enterPass = false;
                        that.myTicketView(); //success.
                    };

                    var confirmPasswordDialog = function () {
                        that.openPasswordDialog(function (res) {
                            if (res) {
                                continueFlow();
                            }
                        });
                    };

                    touchIdHelper.verifyFingerprint(continueFlow, confirmPasswordDialog);
                    return;
                }

                //Handle email checkbox
                if (emailChecked) {
                    that.openDataDialog(that.storageKeys.emailAddress, function (value) {
                        that.emailChecked = false;
                        that.email = value;
                    });
                    return; //Stay on page.
                }

                //Handle phone checkbox
                if (smsChecked) {
                    that.openDataDialog(that.storageKeys.phoneNumber, function (value) {
                        that.phoneChecked = false;
                        that.phone = value;
                    });
                    return; //Stay on page.
                }

                that.confirmSale(save);
                that.enterPass = true;
            });

            that.saveInfo = that.isChecked(saveCardSelector);
            that.sendEmail = that.isChecked(emailSelector);

            that.setStoredValue(that.storageKeys.paymentSettings, {
                saveInfo: that.saveInfo,
                sendEmail: that.sendEmail,
                selectedCard: $(that.el).find('#selectCard').val()
            });
        },

        isFormValid: function (view, callback) {
            var that = this;
            var saveCardSelector = $(that.el).find('#saveCreditCard');
            var selectedCardSelector = $(that.el).find('#selectCard');

            that.hideValidationError(selectedCardSelector);

            var selectedCardValue = selectedCardSelector.val();
            var state = { Save: 0, NoSave: 1, CreditCard: 2, GiftCard: 3 };
            var selectedCardType = $(that.el).find('#selectCard option:selected').data('card-type');

            var selectedState = (selectedCardValue == 'newGiftCard' || selectedCardValue == 'newCreditCard') ? (that.isChecked(saveCardSelector) ? state.Save : state.NoSave) :
                                !that.newCreditCardView && selectedCardType != 'GiftCard' && selectedCardType != 'VirtualGiftCard' ? state.CreditCard : state.GiftCard;

            var cardCallback = function (isValid, card) {
                var save = selectedState == state.Save;

                if (isValid) {
                    if (card && (selectedState == state.NoSave || selectedState == state.Save)) {
                        that.unsavedPaymentCard = card;
                    } else {
                        that.unsavedPaymentCard = null;
                    }

                    if ((that.isCreditCard() || selectedCardValue == 'newCreditCard') && that.useCvv2) {
                        that.cvv2Validation($(that.el).find('#cvv2'), function (valid) {
                            callback(valid, save);
                        });
                        return;
                    }
                }

                callback(isValid, save);
            };

            switch (selectedState) {
                case state.Save:
                case state.NoSave:
                    if (view && view.validateForm) {
                        view.validateForm(cardCallback);
                    } else {
                        cardCallback(true, null);
                    }
                    break;
                case state.CreditCard:
                    that.getStoredValue(that.storageKeys.paymentMethod, function (storageCards) {
                        if (storageCards && storageCards.length > 0) {
                            var selected = $(that.el).find('#selectCard option:selected').val();
                            var storageCard = _.find(storageCards, function (item) { return item.Id === selected; });

                            var creditCard = new window.CardValidationHelper().isCreditCardValid({
                                cardNumber: new window.Utilities().decrypt(storageCard.CardNumber),
                                useMasterCard: that.useMasterCard,
                                useAmericanExpress: that.useAmericanExpress,
                                useDiscover: that.useDiscover,
                                useVisa: that.useVisa,
                                checkIsSupported: true
                            });

                            if (!creditCard.isSupported) {
                                that.showValidationError(selectedCardSelector, that.localize.errorUnsupportedCreditCardType, selectedCardSelector.parent());
                            }

                            cardCallback(creditCard.isValid, creditCard);
                        }
                    });
                    break;
                case state.GiftCard:
                    cardCallback(true, null);
                    break;
            }
        },
        openPasswordDialog: function (callback) {
            var that = this;

            require(['views/enterPassDialogView', 'views/createPassDialogView'], function (enterPassDialogView, createPassDialogView) {
                that.getStoredValue(that.storageKeys.password, function (value) {
                    var dialogCallback = function (response) {
                        $(that.el).find('#pass').blur();
                        $(that.el).find('#passDialog').popup('close');
                        callback(response);
                    };
                    that.openPopupDialog(value ? new enterPassDialogView({ callback: dialogCallback, cinemaPass: value }) : new createPassDialogView({ callback: dialogCallback, dialogTopNotification: that.localize.createPasswordNotification1, dialogBottomNotification: that.localize.createPasswordNotification2 }));
                });
            });
        },
        openDataDialog: function (data, callback) {
            var that = this;
            that.getStoredValue(data, function (value) {
                if (value) {
                    if (callback) {
                        callback(value); //load saved data
                    }
                    that.myTicketView();
                    return;
                }

                var viewName = 'views/' + data + 'DialogView';
                require([viewName], function (view) {
                    var close = function (redirect) {
                        $(that.el).find('#passDialog').popup('close');

                        if (redirect) {
                            that.myTicketView(); //remove dialog
                        }
                    };

                    that.openPopupDialog(new view({ callback: close }));
                });
            });
        },

        openPopupDialog: function (dialogView) {
            var dialog = $(this.el).find('#passDialog');
            dialog.html(dialogView.render());
            dialog.popup('open');
            dialog.enhanceWithin();
        },

        confirmSale: function (savePayment) {
            var that = this;

            if (window.ConfigHelper.isDeviceOnline() == false) {
                window.showAlert(that.localize.errorUnableConnectToInternet);

                if (savePayment) {
                    that.reloadCards();
                }
                return;
            }

            window.showPageLoadingMessage();

            clearInterval(window.ObjectModels.OrderModel.get('interval'));
            window.ObjectModels.OrderModel.set('interval', null);

            require(['models/confirmSaleModel', 'collections/ticketCollection', 'collections/concessionCollection'], function (confirmSaleModel, ticketCollection, concessionCollection) {
                that.getStoredValue(that.storageKeys.paymentMethod, function (creditCards) {
                    creditCards = creditCards || [];

                    if (savePayment && creditCards.length == 0) {
                        window.showAlert(that.localize.errorMessageDeleteCreditCard);
                    }

                    var orderModel = window.ObjectModels.OrderModel;

                    var model = new confirmSaleModel();
                    model.theaterId = orderModel.get('virtualGiftCard') && configurationProvider.useVirtualGiftCardsDefaultSite()
                                    ? configurationProvider.virtualGiftCardsDefaultSite() : that.theater.get('id');
                    model.saleId = orderModel.get('saleId');

                    that.getStoredValue(that.storageKeys.loyaltyCards, function (value) {
                        if (that.useLoyalty) {
                            var loyaltyCardNumber = orderModel.get('loyaltyCardNumber') || $(that.el).find('#loyaltyCardNumber').val() || '';
                            loyaltyCardNumber = loyaltyCardNumber.trim();

                            if (value || loyaltyCardNumber) {
                                var loyaltyCards = new Array();
                                loyaltyCards.push(value ? value.number : loyaltyCardNumber);

                                model.set('loyaltyCardNumbers', loyaltyCards);
                            }
                        }

                        //Get all Subscription cards used in the purchase
                        var subscriptionCardNumbers = [];
                        var tickets = orderModel.get('tickets') ? orderModel.get('tickets').models : [];

                        _.each(tickets, function (ticket) {
                            if (ticket.get('subscriptionCardNumbers')) {
                                subscriptionCardNumbers = subscriptionCardNumbers.concat(ticket.get('subscriptionCardNumbers'));
                            }
                        });

                        if (subscriptionCardNumbers.length > 0) {
                            model.set('subscriptionCardNumbers', subscriptionCardNumbers);
                        }

                        var paymentCard = _.find(creditCards, function (c) { return c.Id === that.paymentCardId; });

                        if (!paymentCard) {
                            if (!that.unsavedPaymentCard) {
                                //Unable to find any payment card.
                                window.showAlert(that.localize.errorCreditCardNotFound);
                                window.hidePageLoadingMessage();
                                return;
                            }

                            paymentCard = that.unsavedPaymentCard;
                        }

                        that.getPatronDetails(function (result) {
                            var isCreditCard = paymentCard.Type == 'CreditCard';
                            var utilities = new window.Utilities();

                            var patronInfo = {
                                firstName: isCreditCard ? utilities.capitalize(paymentCard.FirstName) : "",
                                lastName: isCreditCard ? utilities.capitalize(paymentCard.LastName) : "",
                                email: that.email || result.emailAddress,
                                phone: that.phone || result.phoneNumber,
                                facebookId: result.facebookId,
                                deviceId: window.device && window.device.uuid ? window.device.uuid : utilities.randomUUID()
                            };

                            var cardNumber = utilities.decrypt(paymentCard.CardNumber) || paymentCard.CardNumber;
                            var amount = orderModel.get('remainingAmount') || orderModel.get('apiTotalPrice').toFixed(2);
                            var paymentCardModel = that.setupPaymentCard(paymentCard, cardNumber, amount, orderModel.get('encryptionKey'));
                            window.ObjectModels.OrderModel.set('paymentType', isCreditCard ? 'credit' : 'gift');

                            model.set({
                                allowPartialPayment: true,
                                authorizePayment: true,
                                authorizationCode: null,
                                sendSmsReceipt: (that.useSMSConfirmation && that.isChecked($(that.el).find('#sms'))) || false,
                                sendEmailReceipt: (that.useEmailConfirmation && that.isChecked($(that.el).find('#email'))) || false,
                                patronInfo: patronInfo,
                                paymentCards: [paymentCardModel],
                                reportAsNode: window.AppProperties.IsAndroid ? window.appConfig.NodeId.Android : window.appConfig.NodeId.IOS
                            });
                            if (window.AppConfig.BraintreePaymentEnabled) {
                                require(['braintree', 'models/braintreeModel'], function (braintree) {

                                    var braintreeModel = new window.BraintreeModel();

                                    var successCallback = function (token) {
                                        var client = new braintree.api.Client({
                                            clientToken: token
                                        });

                                        client.tokenizeCard({
                                            number: paymentCard.CardNumber,
                                            expirationDate: paymentCard.ExpMonth + '/' + paymentCard.ExpYear.substring(2)
                                        }, function (err, nonce) {
                                            if (err) {
                                                if (savePayment) {
                                                    that.reloadCards();
                                                }
                                                window.hidePageLoadingMessage();
                                                new window.ErrorHelper().showAlertByErrorCode();
                                            } else {
                                                model.set({
                                                    authorizePayment: false,
                                                    authorizationCode: 'not required',
                                                    paymentMethodNonce: nonce,
                                                    btTransactionAmount: paymentCardModel.amount
                                                });
                                                that.fulfillSaleConfirmation.call(that, model, amount, patronInfo, savePayment, paymentCard, ticketCollection, concessionCollection);
                                            }
                                        });
                                    };

                                    var errorCallback = function () {
                                        if (savePayment) {
                                            that.reloadCards();
                                        }

                                        window.hidePageLoadingMessage();
                                        new window.ErrorHelper().showAlertByErrorCode();
                                    };
                                    braintreeModel.getToken(successCallback, errorCallback);
                                });
                            } else {
                                that.fulfillSaleConfirmation.call(that, model, amount, patronInfo, savePayment, paymentCard, ticketCollection, concessionCollection);
                            }
                        }); // end getPatronDetails
                    }); // end getLoyaltyDetails
                }); //end storage provider
            }); //end required
        },

        fulfillSaleConfirmation: function (model, amount, patronInfo, savePayment, paymentCard, ticketCollection, concessionCollection) {
            var that = this;
            var orderModel = window.ObjectModels.OrderModel;
            Backbone.sync('update', model, {

                error: function (error) {
                    if (savePayment) {
                        that.reloadCards();
                    }
                    console.log("Error: "+error);
                    window.hidePageLoadingMessage();
                    new window.ErrorHelper().showAlertByErrorCode();
                },
                success: function (response) {
                    var data = response.result;
                    that.resetFormTempValues();

                    if (response.result_code == 200) {
                        paymentCard.amount = amount;
                        that.storage.read('PaymentMethod', function (cards) {
                            that.usedPaymentCards.push(that.paymentCardId);

                            _.each(cards, function (c) {
                                if (that.usedPaymentCards.indexOf(c.Id) > -1) {
                                    c.UpdateBalance = true;
                                }
                            });
                            that.storage.store('PaymentMethod', cards);
                        });

                        if (orderModel.get('virtualGiftCard')) {
                            require(['helpers/virtualGiftCardHelper'], function () {
                                window.virtualGiftCardHelper.saveCard(orderModel.get('virtualGiftCard'), true);
                            });

                            that.savepaymentCards(paymentCard);
                            return;
                        }

                        var saleResult = [];
                        var holdSeats = _.filter(orderModel.get('sentSeats'), function (s) { return s.get('status').toUpperCase() == 'HOLD'; });
                        _.each(data.tickets, function (item) {
                            if (holdSeats.length > 0 && item.name.toUpperCase() == 'HOLD') {
                                item.seatName = holdSeats[0].get('seat').get('name');
                                holdSeats.shift();
                            }
                            var ticketItem = _.find(data.receipt.items, function (i) {
                                return i.name.toUpperCase() === item.name.toUpperCase();
                            });
                            var ticketTax = ticketItem ? ticketItem.tax : 0;
                            saleResult.push({
                                name: item.name,
                                serialNumber: item.serialNumber,
                                seatName: item.seatName,
                                price: item.price,
                                tax: ticketTax,
                                displayPrice: item.displayPrice,
                                friendId: '',
                                confirmationNumber: data.receipt.confirmationNumber,
                                isGifted: false
                            });
                        });

                        var tickets = new ticketCollection(data.tickets);
                        tickets.filterReceipt(data.receipt.items, tickets.models);

                        var concessions = new concessionCollection();
                        concessions.filterReceipt(data.receipt.items, orderModel.get('concessions'));

                        var otherItems = _.filter(data.receipt.items, function (it) {
                            return it.quantity > 0;
                        });

                        orderModel.set('confirmSaleResult', saleResult);
                        orderModel.set('confirmationNumber', data.receipt.confirmationNumber);
                        orderModel.set('purchasedDateTime', data.receipt.purchasedDateTime);
                        orderModel.set('otherItems', _.sortBy(otherItems, function (i) { return i.sortOrder; }));
                        orderModel.set('totalPrice', data.receipt.total);
                        orderModel.set('apiTotalPrice', data.receipt.total);
                        orderModel.set('subTotal', data.receipt.subTotal);
                        orderModel.set('tax', data.receipt.tax);

                        orderModel.set('checkDualCardId', that.checkDualCardId);

                        that.addPurchaseHistoryEntry(saleResult, patronInfo);

                        if (that.options.hasHoldSeats) {
                            window.AppMethods.ChangePage('#confirmHoldSeats');
                        } else {
                            window.AppMethods.ChangePage('#myTicketView');
                        }
                    } else {
                        that.pullDownStatusView = new pullDownStatusView();
                        if (savePayment) {
                            that.reloadCards();
                        } else {
                            that.populateFields(paymentCard);
                            that.unsavedPaymentCard = paymentCard;
                        }
                        window.hidePageLoadingMessage();

                        if (data.error_sub_code == 99) {
                            that.handlePartialPayment(paymentCard, parseFloat(data.message || 0));
                            that.savepaymentCards(paymentCard);
                        } else {
                            new window.ErrorHelper().showAlertByErrorCode(data.error_sub_code);
                        }
                    }
                }
            });
        },

        savepaymentCards: function (paymentCard) {
            var orderModel = window.ObjectModels.OrderModel;
            var paymentCards = orderModel.get('paymentCards') || [];

            paymentCards.push(paymentCard);
            orderModel.set('paymentCards', paymentCards);
        },
        handlePartialPayment: function (paymentCard, payedAmount) {
            var that = this;
            var remainingAmount = that.setPartialAmount(paymentCard, payedAmount);
            var insufficientFundsMessage = $.sprintf(that.localize.insufficientFundsError, window.Globalize.format(remainingAmount, 'c', that.theater.get('culture')));
            paymentCard.amount = payedAmount;

            var onConfirm = function (buttonIndex) {

                if (buttonIndex == 1) {
                    //hide back button for partial payments, because the problem of calculating the total price when going back/forward through the application
                    $(that.el).find('#header [data-rel="back"]').hide();

                    that.usedPaymentCards.push(paymentCard.Id);

                    var cardInfo = paymentCard.Name + '  **** **** **** ' + (paymentCard.LastDigits || paymentCard.CardNumber.slice(-4));
                    payedAmount = window.Globalize.format(payedAmount, 'c', window.ObjectModels.OrderModel.get('theater').get('culture'));
                    remainingAmount = window.Globalize.format(remainingAmount, 'c', window.ObjectModels.OrderModel.get('theater').get('culture'));

                    var form = $(that.el).find('#usedPaymentCards');
                    var pullDownStatus = $(that.el).find('.pullDownScroll');
                    var amountDue = $(that.el).find('.order-total .order-currency-time');

                    if (that.usedPaymentCards.length == 1) {
                        form.append('<div class="horBar">' + that.localize.usedPaymentCards + '</div>');
                        pullDownStatus.append('<br/><h2 style="margin-bottom: 2px">' + that.localize.usedPaymentCards + '</h2>');
                    }

                    form.append('<div class="usedPaymentCards"> ' +
                                '<label>' + cardInfo + '</label>' +
                                '<span>' + payedAmount + '</span>' +
                                '<div class="clear"></div>' +
                                '</div>');

                    $(that.el).find('#selectCard option[value="' + paymentCard.Id + '"]').attr('disabled', 'disabled');

                    pullDownStatus.append('<span style="float: left; padding-left: 6px">' + cardInfo + '</span>' +
                                          '<span style="float: right; padding-right: 2px">- ' + payedAmount + '</span><br /><br />');

                    amountDue.html(remainingAmount);

                    //return to NewGiftCard View
                    $(that.el).find('#selectCard option[value="newGiftCard"]').attr('selected', true);
                    $(that.el).find("#selectCard").change();

                    return;
                } else {
                    if (window.ObjectModels.OrderModel && window.ObjectModels.OrderModel.get('saleId')) {
                        window.ObjectModels.OrderModel.cancelSale();
                        window.AppMethods.ChangePage('');
                    }
                }
            };
            if (window.isMobileDevice()) {
                navigator.notification.confirm(insufficientFundsMessage, onConfirm, that.localize.alertTitle, that.localize.ok + ', ' + that.localize.cancel);
            } else if (confirm(insufficientFundsMessage)) {
                onConfirm(1);
            }
        },

        setPartialAmount: function (paymentCard, payedAmount) {
            var that = this;
            payedAmount = parseFloat(payedAmount || 0);
            if (payedAmount <= 0) {
                return [];
            }

            var existing = _.find(that.partialAmounts, function (p) { return p.number == paymentCard.CardNumber; });

            if (existing) {
                existing.amount = value + parseFloat(existing.amount);
            } else {
                that.partialAmounts.push({ number: paymentCard.CardNumber, amount: payedAmount });
            }

            var rest = window.ObjectModels.OrderModel.get('apiTotalPrice');
            _.each(that.partialAmounts, function (item) {
                rest = rest - item.amount;
            });

            window.ObjectModels.OrderModel.set('remainingAmount', rest);

            return rest;
        },

        setupPaymentCard: function (creditCard, cardNumber, price, encryptionKey) {
            var isCreditCard = creditCard.Type == 'CreditCard';
            var useAvs = this.theater.get('aVSRequiredForCredit');
            var cvv2Value = !this.newCreditCardView ? this.cardHolder.find('#cvv2').val() : this.newCardHolder.find('#cvv2').val();

            setMaxDigits(260);
            var rsaKey = new RSAKeyPair(encryptionKey.exponentHexString, "", encryptionKey.modulusHexString);

            var paymentCard = {
                cardNumber: cardNumber ? encryptedString(rsaKey, cardNumber.replace(/ /g, "")) : null,
                amount: price,
                expirationMonth_MM: isCreditCard && creditCard.ExpMonth ? encryptedString(rsaKey, creditCard.ExpMonth) : null,
                expirationYear_YY: isCreditCard && creditCard.ExpYear ? encryptedString(rsaKey, creditCard.ExpYear) : null,
                cVV2: this.useCvv2 && isCreditCard ? encryptedString(rsaKey, cvv2Value) : null
            };

            if (useAvs && isCreditCard && creditCard.ZipCode) {
                if (window.AppConfig.AppCulture == 'en-GB') {
                    paymentCard['aVS'] = encryptedString(rsaKey, creditCard.ZipCode.match(/\d/g).join(""));
                } else {
                    paymentCard['aVS'] = encryptedString(rsaKey, creditCard.ZipCode);
                }
            } else {
                paymentCard['aVS'] = null;
            }

            return paymentCard;
        },
        populateFields: function (cardInfo) {
            $(this.el).find('#firstName').val(cardInfo.FirstName);
            $(this.el).find('#lastName').val(cardInfo.LastName);
            $(this.el).find('#creditCardNumber').val(cardInfo.CardNumber);
            $(this.el).find('#zipCode').val(cardInfo.ZipCode);
        },
        getPatronDetails: function (callback) {
            var that = this;
            var storageValue = { phoneNumber: null, facebookId: null };

            that.getStoredValue(that.storageKeys.phoneNumber, function (phoneNumber) {
                if (phoneNumber) {
                    storageValue.phoneNumber = phoneNumber;
                }
                that.getStoredValue(that.storageKeys.emailAddress, function (emailAddress) {
                    if (emailAddress) {
                        storageValue.emailAddress = emailAddress;
                    }

                    that.getStoredValue(that.storageKeys.fb, function (value) {
                        if (value == null) {
                            callback(storageValue);
                            return;
                        }

                        storageValue.facebookId = value.id;
                        callback(storageValue);
                    });
                });
            });
        },

        resetFormTempValues: function () {
            this.preventSave = false;
            this.unsavedPaymentCard = null;
            this.phone = null;
        },

        addPurchaseHistoryEntry: function (selectedTickets, patronInfo) {
            var that = this;
            require(['sharedhelpers/storageProvider', 'helpers/menuHelper'], function () {
                that.getStoredValue(that.storageKeys.purchaseHistory, function (orders) {
                    orders = orders || [];

                    var removeExisting = orders.length > 0;

                    var orderModel = window.ObjectModels.OrderModel;
                    var concessions = orderModel.get('concessions') ? orderModel.get('concessions').models : [];

                    var groupedConcessions = concessions.length > 0 ? _.groupBy(concessions, function (model) {
                        if (model.get('modifiers') && model.get('modifiers').length == 0)
                            return model.get('productId');

                        return model.cid;
                    }) : null;

                    var movie = orderModel.get('movie');
                    var theater = orderModel.get('theater');
                    var theaterDetails = {
                        addressLine1: theater.get('addressLine1'),
                        addressLine2: theater.get('addressLine2'),
                        addressCity: theater.get('addressCity'),
                        addressState: theater.get('addressState'),
                        addressZip: theater.get('addressZip'),
                        latitude: theater.get('latitude'),
                        longitude: theater.get('longitude'),
                        phone: theater.get('phone'),
                        displayNetTicketPrice: theater.get('displayNetTicketPrice')
                    };

                    var order = {
                        confirmationNumber: orderModel.get('confirmationNumber'),
                        movieCode: movie ? movie.get('code') : null,
                        movieName: movie ? movie.get('name') : null,
                        movieRating: movie ? movie.get('rating') : null,
                        movieImage: movie ? movie.get('image') : null,
                        movieLength: movie ? window.DtHelper.getRuntime(movie.get('length')) : null,
                        movieLenghtInMinutes: movie ? movie.get('length') : null,
                        theaterId: theater.get('id'),
                        theaterName: theater.get('name'),
                        theaterDetails: theaterDetails,
                        theaterUrl: theater.get('facebookURL'),
                        theaterCulture: theater.get('culture'),
                        performance: orderModel.get('performance') ? orderModel.get('performance').toJSON() : null,
                        otherItems: orderModel.get('otherItems'),
                        totalPrice: orderModel.get('totalPrice'),
                        apiTotalPrice: orderModel.get('apiTotalPrice'),
                        tax: orderModel.get('tax'),
                        subTotal: orderModel.get('subTotal'),
                        concessions: groupedConcessions,
                        selectedTickets: selectedTickets,
                        purchasedDateTime: orderModel.get('purchasedDateTime'),
                        patronsName: patronInfo ? patronInfo.firstName + ' ' + patronInfo.lastName : null
                    };

                    orders.push(order);

                    that.setStoredValue(that.storageKeys.purchaseHistory, orders, removeExisting);

                    //Update Menu to show My Tickets link
                    var menuIds = window.MenuHelper.getMenuIds();
                    window.MenuHelper.showMenuItem(menuIds.purchaseHistory);
                });
            });
        }
    });

    return view;
});
