define([
  'text!templates/PhoneNumberDialogView.html',
  'sharedhelpers/storageProvider',
  'sharedhelpers/cultureHelper',
  'libs/jquery.validate.min'
], function (template) {
    var view = Backbone.View.extend({
        template: _.template(template),
        localize: window.ObjectCollections.Localization.result,

        initialize: function () {
            this.storage = new StorageProvider();
            this.phoneNumberKey = 'phoneNumber';

            this.setupValidation();
        },

        render: function () {
            this.$el.html(this.template({
                localize: this.localize,
                model: this.options.model,
                hasHoldSeats: this.options.hasHoldSeats
            }));
            this.redirect = false;

            return $(this.el);
        },

        events:
		{
		    "tap #ok": "savePhoneNumber",
		    "tap #cancel": "cancel",
		    "keypress #number": "handlePhoneKeypress"
		},

        setupValidation: function () {
            var cultureHelper = new CultureHelper();
            var phoneValidation = cultureHelper.phoneValidation();

            $.validator.addMethod('phone', function (value) {
                var number = value.replace(/ /g, "");
                return phoneValidation.regex.test(number);
            }, phoneValidation.msg);
        },

        cancel: function (event) {
            event.preventDefault();

            this.options.callback(false);
            return false;
        },

        savePhoneNumber: function (event) {
            event.preventDefault();
            
            if (this.$el.find('#frmPhoneDialog').valid()) {
                //Remove phone at first
                this.storage.remove(this.phoneNumberKey);
                this.storage.store(this.phoneNumberKey, this.$el.find('#number').val());

                this.options.callback(true);
            }

            return false;
        },

        handlePhoneKeypress: function (e) {
            var that = this;

            if (e.keyCode == 13) {
                e.preventDefault();

                that.savePhoneNumber(e);

                return;
            }
        }
    });

    return view;
});