﻿define([
  'text!templates/PullDownStatusView.html',
  'sharedhelpers/configurationProvider',
  'helpers/uiHelper'
], function (template, configurationProvider) {
    var view = Backbone.View.extend({
        template: _.template(template),
        localize: window.ObjectCollections.Localization.result,

        initialize: function () {
            var that = this;

            var orderModel = window.ObjectModels.OrderModel;

            if (orderModel && orderModel.get('timeout') && !orderModel.get('interval'))
                orderModel.set('interval', setInterval(function () { that.countdown(orderModel.get('timeout')); }, 1000));

            UIHelper.setPullDownStyle();
        },

        events: {
            "tap .gripIcon": "disableJQMClick",
            "touchstart .gripIcon": "statusBarStart",
            "touchmove .gripIcon": "statusBarMove",
            "touchend .gripIcon": "statusBarEnd"
        },

        render: function () {
            var that = this;
            that.originalPosition = window.orgScreenHeight - 80;

            var orderModel = window.ObjectModels.OrderModel;
            var tickets = orderModel.get('tickets') && orderModel.get('tickets').models ? orderModel.get('tickets').getSelectedTickets() : null;
            var concessions;

            if (window.ObjectModels.TempOrder && window.ObjectModels.TempOrder.concessions) {
                concessions = window.ObjectModels.TempOrder.concessions.getGroupedConcessions();
            } else {
                concessions = orderModel.get('concessions') || orderModel.get('concessions').models ? orderModel.get('concessions').getGroupedConcessions() : null;
            }

            var otherItems = orderModel.get('otherItems');
            var totalPrice = parseFloat(orderModel.get('totalPrice') || 0);
            var theater = orderModel.get('theater');
            var serviceCharge = tickets && !otherItems ? orderModel.get('tickets').getTicketServiceChargeDisplay(orderModel.get('theater').get('culture')) : null;
            var theaterName = orderModel.get('virtualGiftCard') && configurationProvider.useVirtualGiftCardsDefaultSite() &&
                              (theater.get('id') != configurationProvider.virtualGiftCardsDefaultSite()) ?
                              configurationProvider.virtualGiftCardsDefaultSiteName() : theater ? theater.get('name') : null;

            that.$el.html(that.template({
                localize: that.localize,
                tickets: tickets,
                concessions: concessions,
                otherItems: otherItems,
                totalPrice: totalPrice,
                serviceCharge: serviceCharge,
                theaterName: theaterName
            }));
            return this;
        },

        disableJQMClick: function (e) {
            if (isJqmGhostClick(e)) return;
        },

        statusBarStart: function (e) {
            this.orderStatusContainer = $('.pullDownStatusHolder');
            this.statusBarContent = $('#statusBarContent');

            if (this.orderStatusContainer.length < 1)
                return;

            this.touchPosition = 0;
            this.newStatusPosition = 0;

            this.positionFix = (((window.orgScreenHeight / 100) * 93) / 100 * 94);

            var touches = e.originalEvent.changedTouches;
            for (var i = 0; i < touches.length; i++) {
                this.touchPosition = touches[i].pageY;
            }

            this.statusBarContent.toggleClass('down');

            if (this.statusBarContent.hasClass('down')) {
                $('[data-role=content]').css({ 'pointer-events': 'none' });
            }
        },

        statusBarMove: function (e) {
            var that = this;

            if (that.orderStatusContainer.length < 1)
                return;

            var touches = e.originalEvent.changedTouches;
            for (var i = 0; i < touches.length; i++) {

                that.newStatusPosition = (that.touchPosition - touches[i].pageY) + 1;

                if (that.newStatusPosition > 2 || that.newStatusPosition < -2) {
                    e.preventDefault();
                    if (that.statusBarContent.hasClass('down') != true) {
                        that.updatePosition(that.newStatusPosition + 'px');
                    }
                    else {
                        var statusDown = that.positionFix + that.newStatusPosition;
                        that.updatePosition(statusDown + 'px');
                    }
                }
            }
        },

        statusBarEnd: function () {
            var that = this;

            if (that.orderStatusContainer.length < 1)
                return;

            that.orderStatusContainer.css({
                '-webkit-transition': 'all 250ms ease'
            });

            if (that.statusBarContent.hasClass('down') == true) {
                that.orderStatusContainer.addClass('stop');
                that.updatePosition('0');
                $('[data-role=content]').css({ 'pointer-events': 'none' });
            } else {
                that.orderStatusContainer.removeClass('stop');
                that.updatePosition(that.originalPosition + 'px');

                //enabling touch events on the content behind the pullDownStatus after 500ms to prevent propagation
                setTimeout(function () {
                    $('[data-role=content]').css({ 'pointer-events': '' });
                }, 500);
            }
            setTimeout(function () {
                that.orderStatusContainer.css({
                    '-webkit-transition': 'none'
                });
            }, 300);
        },
        updatePosition: function (position) {
            this.orderStatusContainer.css({ '-webkit-transform': "translate3d(0, -" + position + ", 0)" });
        },
        countdown: function (time) {
            var orderModel = window.ObjectModels.OrderModel;
            if (!orderModel) return;

            var now = new Date();
            var orderTimeout = orderModel.get('orderTimeout');

            if (!orderTimeout) {
                orderTimeout = new Date();
                orderTimeout = orderTimeout.setSeconds(orderTimeout.getSeconds() + time);
            }

            var diff = orderTimeout - now.getTime();
            var minutes = Math.floor(diff / (1000 * 60));
            var seconds = Math.floor(diff / 1000);
            seconds = seconds - minutes * 60;

            if (minutes < 1)
                $('.order-time').addClass('alert');

            orderModel.set('orderTimeout', orderTimeout);

            if (minutes < 0) {
                $('.order-time').html(this.localize.errorOrderHasTimedOutNotification);

                orderModel.set('orderTimeout', null);
                orderModel.cancelSale();

                window.showAlert(this.localize.errorOrderHasTimedOut);

                window.AppMethods.ChangePage('#home');
                return;
            }

            if (seconds < 10)
                seconds = '0' + seconds;

            $('.order-time').html((minutes < 1 ? '' : minutes) + ':' + seconds + ' ' + this.localize.timeLeftToBuy);
        }

    });

    return view;
});