define([
    'sharedhelpers/configurationProvider',
    'text!templates/PurchaseHistoryView.html',
    'views/PurchaseHistoryTicketView',
    'helpers/nativeCalendarHelper',
    'libs/jquery.validate.min',
    'sharedlibs/jquery/jquery.qrcode.min',
    'idangerousSwiper'
], function (configurationProvider, template, purchaseHistoryTicketView) {
    var view = Backbone.View.extend({
        template: _.template(template),
        localize: window.ObjectCollections.Localization.result,

        initialize: function () {
            this.nativeCalendarHelper = new NativeCalendarHelper();

            this.purchaseHistoryTicketView = new purchaseHistoryTicketView({ nativeCalendarHelper: this.nativeCalendarHelper });
            this.currentSlide = null;
            this.patronNames = {};
        },

        render: function () {
            var that = this;
            var selector = $(that.el);

            selector.html(that.template({
                localize: that.localize,
                orders: that.options.orders,
                hideBackButton: that.options.hideBackButton
            }));

            that.homeHeaderButton(selector);

            that.selectedIndex = that.options.selectedIndex || 0;
            that.beforeSelected = that.selectedIndex - 1;
            that.afterSelected = that.selectedIndex + 1;

            var setScreenWidth = window.innerWidth;
            var carouselItem = selector.find('.carousel-item');

            carouselItem.css({ 'width': setScreenWidth + 'px' });
            var startPosition = setScreenWidth * that.selectedIndex;
            $(that.el).find('.swiper-wrapper').css({ '-webkit-transform': "translate3d(-" + startPosition + "px, 0, 0)" });

            _.defer(function () {
                var renderOrders = selector.find('.carousel-item[data-selectedindex="' + that.selectedIndex + '"],.carousel-item[data-selectedindex="' + that.beforeSelected + '"],.carousel-item[data-selectedindex="' + that.afterSelected + '"]');
                _.each(that.options.orders, function (item) {
                    if (item.confirmationNumber == $(renderOrders[0]).attr('data-confirmationnumber') || item.confirmationNumber == $(renderOrders[1]).attr('data-confirmationnumber') || item.confirmationNumber == $(renderOrders[2]).attr('data-confirmationnumber')) {
                        that.assign(that.purchaseHistoryTicketView, '.swiper-container', item);
                        that.patronNames[item.confirmationNumber] = item.patronsName;
                    }
                });

                that.swiper = selector.find('.swiper-dynamic').swiper({
                    autoResize: false,
                    roundLengths: true,
                    watchActiveIndex: true,
                    simulateTouch: false,
                    resizeReinit: true,
                    onFirstInit: function () {
                        that.adjustSwiperHeight(that.selectedIndex);
                    },
                    onSlideChangeStart: function (e) {
                        var currentSlideIndex = e.activeIndex;

                        that.swiper.reInit();//reinitialize swiper instance to get the new number of slides

                        //Prevent from sliding past the last item after some items have been removed
                        if (currentSlideIndex > that.swiper.slides.length - 1) {
                            that.swiper.swipeTo(that.swiper.slides.length - 1);
                            return true;
                        }
                        if (that.currentSlide !== currentSlideIndex) {
                            that.updateVisibleOrders('.swiper-wrapper', e.positions.current < e.positions.start);
                            that.currentSlide = currentSlideIndex;
                        }

                        window.scrollTo(0, 0);
                        that.adjustSwiperHeight(currentSlideIndex);
                    }
                });
            });

            window.showPageLoadingMessage();

            return that;
        },
        adjustSwiperHeight: function (index) {
            var swiperContainer = $('.swiper-container');
            var currentItem = swiperContainer.find('.carousel-item[data-selectedindex="' + index + '"]');

            swiperContainer.css({ 'height': currentItem.height() });
        },
        assign: function (inview, selector, order) {
            inview.setElement($(selector).find('[data-confirmationnumber=' + order.confirmationNumber + ']')).render(order);

            this.$(selector).enhanceWithin();
        },

        homeHeaderButton: function (sender) {
            //if ($.mobile.urlHistory.stack.length > 0) {
                if (window.isPickup) {
                    sender.find('#header [data-rel="back"]').hide();
                    return false;
                }
            //}
        },
        events: {
            "tap .ticket-card": "flipTicket",
            "tap .giftButton": "giftTicket",
            "tap .orderDetails": "viewDetails",
            "tap #facebook-share-show": 'facebookSharePanel',
            "tap .calendar-btn": "addToCalendar",
            "tap .remove-ticket-btn": 'deleteOrder',
            "tap .ticket-pass,.ios11-ticket-pass": "addToPassbook"
        },
        addToCalendar: function (e) {
            var that = this;
            var orderId = $(e.currentTarget).data('orderid');
            var order = _.find(window.AppProperties.HistoryOrders, function (o) { return o.confirmationNumber == orderId; });

            if (!order || order.addedToCalendar) {
                return;
            }

            var successCallback = function () {
                order.addedToCalendar = true;
                UIHelper.notificationBar(that.localize.calendarSuccess);
                $(e.currentTarget).html(that.localize.addedToCalendar).addClass('ui-disabled');
                require(['js/helpers/storageHelper'], function () {
                    window.storageHelper.updatePurchaseHistory(order);
                });
            };
            var startDate = new Date(order.performance.showTime);
            var endDate = new Date(order.performance.showTime);
            endDate.setMinutes(endDate.getMinutes() + (order.movieLenghtInMinutes ? order.movieLenghtInMinutes : 180));

            var calendarInfo = {
                'startDate': startDate,
                'endDate': endDate,
                'title': order.movieName,
                'location': order.theaterName,
                'notes': that.localize.calendarNotes
            };

            this.nativeCalendarHelper.addToCalendar(calendarInfo, successCallback);
        },
        addToPassbook: function (e) {
            window.showPageLoadingMessage();
            e.preventDefault();

            if (isJqmGhostClick(e)) {
                return false;
            }

            var that = this;
            var orderId = $(e.currentTarget).attr('data-orderid');

            require(['helpers/passHelper', 'sharedhelpers/errorHelper'], function () {
                var passHelper = new PassHelper();
                passHelper.getTicketPassURL(that.options.orders, orderId);
            });
            return false;
        },

        deleteOrder: function (e) {
            e.preventDefault();

            var orderId = $(e.currentTarget).attr('data-orderid');
            var that = this;

            var purchasedItems = that.options.orders;

            require(['js/helpers/storageHelper', 'helpers/menuHelper'], function () {
                window.storageHelper.deletePurchaseHistory(function () {
                    purchasedItems = window.AppProperties.HistoryOrders;

                    //Fix selectedIndex attribute of remaining slides
                    var selectedSlideIndex = $(that.el).find('.carousel-item[data-confirmationnumber="' + orderId + '"]').attr('data-selectedindex');
                    selectedSlideIndex = parseInt(selectedSlideIndex);

                    var swiper = that.swiper;
                    swiper.removeSlide(selectedSlideIndex);

                    _.each($('.carousel-item'), function (item) {
                        if ($(that.el).find(item).attr('data-selectedindex') > selectedSlideIndex) {
                            var index = $(that.el).find(item).attr('data-selectedindex');
                            index -= 1;
                            $(that.el).find(item).attr('data-selectedindex', index);
                        }
                    });

                    that.options.orders = purchasedItems;

                    swiper.reInit();

                    if (swiper.slides.length == 0) {
                        if ($(that.el).find('.noTickets').length == 0) {
                            $('<div class="noTickets"><p>' + that.localize.noOrdersLeft + '</p></div>').insertBefore('.swiper-container');
                        }
                    } else {
                        if (selectedSlideIndex == swiper.slides.length) {
                            swiper.swipeTo(selectedSlideIndex - 1);
                        }
                        that.updateVisibleOrders('.swiper-wrapper', selectedSlideIndex == swiper.slides.length ? false : undefined);
                    }

                    //Update Menu to hide My Tickets link if there are no tickets left
                    if (that.options.orders == 0) {
                        var menuIds = window.MenuHelper.getMenuIds();
                        window.MenuHelper.hideMenuItem(menuIds.purchaseHistory);
                    }
                }, orderId);
            });
        },

        viewDetails: function (e) {
            e.preventDefault();
            if (isJqmGhostClick(e)) {
                return false;
            }

            var selector = $(e.currentTarget);

            window.AppMethods.ChangePage('#ph-orderdetails/' + selector.attr('data-confirmationnumber'));

            return false;
        },

        flipTicket: function (e) {
            var that = this;

            var qrSelector = $(e.currentTarget).find('.full-size-qr');
            var serialNumber = qrSelector.attr('data-serialnumber');
            var patronsName = that.patronNames[serialNumber];
            var qrText = (patronsName && !configurationProvider.useSimpleQrCode()) ? serialNumber + '^' + patronsName : serialNumber;

            var canvasSelector = $(e.currentTarget).find('canvas');

            if (e) {
                e.preventDefault();
            }

            if (canvasSelector.length == 0) {
                qrSelector.qrcode({ text: qrText, width: 100, height: 100 });
            }

            $(e.currentTarget).toggleClass('rotate');
        },

        giftTicket: function (e) {
            var that = this;
            e.preventDefault();

            window.AppProperties.HistoryOrderIndex = that.selectedIndex;

            var selector = $(e.currentTarget);
            window.AppMethods.ChangePage('#selectRecipient/' + selector.attr('data-ticketid') + '/' + selector.attr('data-confirmationnumber'));

            return false;
        },

        facebookSharePanel: function (e) {
            var that = this;

            var orderId = $(e.currentTarget).attr('data-orderid');
            var order = $.grep(that.options.orders, function (o) { return o.confirmationNumber == orderId; });

            if (order.length == 1) {
                var showTime = order[0].performance.showTime;

                //TODO: Get SiteURL from ChainInfo;
                var theaterUrl = appConfig.SiteURL + '#tickets/' + order[0].performance.number + '/' + order[0].movieCode.trim() + '/' + order[0].theaterId + '/share';
                var movieName = order[0].movieName;
                var theaterCulture = order[0].theaterCulture;
                var theaterName = order[0].theaterName;
                var img = order[0].movieImage ? order[0].movieImage.medium || window.appConfig.DefaultMoviePosterImage : window.appConfig.DefaultMoviePosterImage;
                var shareText = "{text} {movieDetails} {theatreDetails} {timeDetails}";

                if (showTime) {
                    var date = new Date();
                    var month = date.getMonth() + 1;
                    var dateTime = date.getFullYear() + '-' + month + '-' + date.getDate() + 'T' + date.getHours() + ':' + date.getMinutes() + ':' + date.getSeconds();
                    var showDate = DtHelper.convertDate(showTime);
                    var diff = DtHelper.dateDiff(DtHelper.convertDate(dateTime), showDate);                   

                    if (diff < 0) {
                        shareText = shareText.replace("{text}", "I'm going to see");
                        shareText = shareText.replace("{timeDetails}", 'on ' + Globalize.format(showDate, "M", theaterCulture) + ' at ' + Globalize.format(showDate, 't', theaterCulture));
                    } else {
                        shareText = shareText.replace("{text}", "I just saw");
                        shareText = shareText.replace("{timeDetails}", '');
                    }

                    if (shareText.indexOf('{text}') > -1) {
                        shareText = shareText.replace("{text}", "I'm going to see");
                    }
                    if (shareText.indexOf('{timeDetails}') > -1) {
                        shareText = shareText.replace("{timeDetails}", '');
                    }
                }
                else {
                    shareText = shareText.replace("{text}", "I'm going to see");
                    shareText = shareText.replace("{timeDetails}", '');
                }
                shareText = shareText.replace("{movieDetails}", movieName);
                shareText = shareText.replace("{theatreDetails}", theaterName);

                var options = {
                    method: "share",
                    href: theaterUrl,
                    caption: movieName,
                    description: shareText,
                    picture: img,
                    share_feedBrowser: true // iOS only

                };
                window.facebookConnectPlugin.showDialog(options, function (response) {
                    if (response && response.postId) {
                        window.showAlert('Posted successfully');
                    } else if (window.AppProperties.IsAndroid) {
                        window.appAvailability.check(
                        'com.facebook.katana', null, function () {
                            window.showAlert('Posted successfully');
                        });
                    }
                });

            } else if (order.length > 1) {
                window.showAlert(that.localize.errorDuplicatedOrderFound);
                return;
            } else {
                window.showAlert(that.localize.errorOrderNotFound);
                return;
            }
        },

        updateVisibleOrders: function (selector, increment) {
            var that = this;
            var addOrder;

            if (increment === true) {
                $(selector).find('.carousel-item[data-selectedindex="' + that.beforeSelected + '"]').empty();
                addOrder = $(selector).find('.carousel-item[data-selectedindex="' + that.afterSelected + '"]');
                that.selectedIndex++;
                that.afterSelected++;
                that.beforeSelected++;
            } else if (increment === false) {
                $(selector).find('.carousel-item[data-selectedindex="' + that.afterSelected + '"]').empty();
                that.selectedIndex--;
                that.afterSelected--;
                that.beforeSelected--;
                addOrder = $(selector).find('.carousel-item[data-selectedindex="' + that.beforeSelected + '"]');
            } else {
                addOrder = $(selector).find('.carousel-item[data-selectedindex="' + that.afterSelected + '"]');
            }
            _.each(that.options.orders, function (item) {
                if (item.confirmationNumber == addOrder.attr('data-confirmationnumber')) {
                    that.assign(that.purchaseHistoryTicketView, '.swiper-container', item);
                }
            });
        }
    });

    return view;
});