define([
  'sharedhelpers/configurationProvider',
  'models/performanceModel',
  'text!templates/SeatsPreview.html',
  'sharedhelpers/selectSeatsHelper',
  'helpers/uiHelper',
  'libs/jquery.validate.min',
  'sharedhelpers/dateTimeHelper',
  'sharedhelpers/htmlHelper'
], function (configurationProvider, performanceModel, template, selectSeatsHelper) {
    var view = Backbone.View.extend({
        template: _.template(template),
        localize: window.ObjectCollections.Localization.result,
        dateIndex: 0,

        initialize: function () {
            var that = this;

            that.theaters = that.options.theaters;
            that.theater = that.theaters[0];
            that.movie = that.options.movie;
            that.movieGroup = that.options.movieGroup;
            that.performances = that.getSelectedDatePerformances();
            that.selectedPerformance = _.filter(that.performances, function (p) { return p.get('isReservedSeating'); })[0]; 
        },
        getSelectedDatePerformances: function () {
            var that = this;
            var performances = _.filter(that.getPerformances(), function (performance) {
                var time = new Date(performance.get('showTime'));
                time.setMinutes(time.getMinutes() + (that.theater.get("performanceExpirationInMinutes") || 0));

                var now = new Date().toISOString();

                return (time.toISOString() >= now && performance.get('businessDate') === DtHelper.getBussinessDate(window.mdSelectedBusinessDate));
            });

            return _.sortBy(performances, function (p) {
                return new Date(p.get('showTime'));
            });
        },

        getPerformances: function () {
            var that = this;
            var performances = [];
            var retPerformances = [];

            if (!window.appConfig.GroupMovies) {
                require(['models/performanceModel'], function (performanceModel) {
                    _.each(that.theater.performances, function (p) {
                        retPerformances.push(new performanceModel(p));
                    });
                });

                return retPerformances;
            }
            _.each(that.movieGroup, function (movie) {
                    _.each(movie.group, function (group) {
                        if (that.theater.get('id') === group.theaterId) {
                            performances = _.union(performances, group.performances.models);
                        }
                    });
            });

            require(['models/performanceModel'], function (performanceModel) {
                _.each(performances, function (p) {
                    retPerformances.push(new performanceModel(p));
                });
            });

            return retPerformances;
        },

        events:
		{
		    'tap .select-showtime': 'selectShowtime',
		    'tap #previousLayout': 'handleNavigationArrows',
		    'tap #nextLayout': 'handleNavigationArrows',
		    "tap .movie-poster-ncr": "showLargePoster",
		    'tap .trailer-btn': 'trailerButtonClick',
		    'tap #facebook-share-show': 'facebookSharePanel',
		    'change #theater-select': 'selectTheater'
		},
        selectTheater: function(e) {
            var that = this;

            window.showPageLoadingMessage();
            if (that.theater.get('id') == $(e.currentTarget).val()) return;

            that.theater = _.find(that.theaters, function(th) {
                return th.get('id') == $(e.currentTarget).val();
            });

            that.refreshLayouts();
        },
        refreshLayouts: function (callback) {
            var that = this;

            that.layouts = null;
            that.overrides = null;

            that.performances = that.getSelectedDatePerformances();
            that.selectedPerformance = _.filter(that.performances, function (p) { return p.get('isReservedSeating'); })[0];

            if (that.performances.length > 0) {
                window.SeatPreviewHelper.getPerformancesLayouts(that.performances, that.theater, function(layouts) {
                    that.layouts = layouts;
                    if (that.overrides) {
                        that.render(callback);
                    }
                });

                window.SeatPreviewHelper.getSeatsPreview(that.performances, that.theater, function(overrides) {
                    that.overrides = overrides;
                    if (that.layouts) {
                        that.render(callback);
                    }
                });
            } else {
                that.render();
                window.hidePageLoadingMessage();
            }
        },

        refreshStatuses: function (seatInfo, pLayout) {
            selectSeatsHelper.refreshStatuses(this, { touchEvents: true, seatInfo: seatInfo, pLayout: pLayout });
        },

        drawSeats: function (opt) {
            selectSeatsHelper.drawSeats(this, { imgPath: 'css/images/' }, opt);
        },

        getBrandStylesheet: function () {
            var styleSheet = '';

            _.each(document.styleSheets, function (item) {
                if (item.href && item.href.indexOf('brand.css') > 0)
                    styleSheet = item.rules || item.cssRules;
            });

            return styleSheet;
        },

        getZoneStyle: function (classes, className, property) {
            for (var x = 0; x < classes.length; x++) {
                if (classes[x].selectorText == className) {
                    return classes[x].style[property];
                }
            }
        },

        setLabels: function () {
            var that = this;
            var feature = $('svg g g path')[0];

            //Seat labels
            if (feature && feature.getBoundingClientRect().width > 20) {
                $('svg text[font-size="0px"]').css({ 'font-size': '12px' });
                that.labelSize = '12px';
            } else if (feature && feature.getBoundingClientRect().width > 12) {
                $('svg text[font-size="0px"]').css({ 'font-size': '8px' });
                that.labelSize = '8px';
            } else {
                $('svg text[font-size="0px"]').css({ 'font-size': '6px' });
                that.labelSize = '6px';
            }
        },

        findMovieName: function (performance) {
            var that = this;
            var selectedName;

            _.each(that.movieGroup, function (movie) {
                _.each(movie.group, function (group) {
                        var selectedPerormance = _.find(group.performances.models, function(model) {
                            return model.get('number') === performance.get('number');
                        });
                        if (selectedPerormance) {
                            selectedName = group ? group.get('name'): null;
                       }                   
                });
            });
            return selectedName;
        },

        render: function (callback) {
            var that = this;
            var movie = window.appConfig.GroupMovies ? that.movieGroup[0]: that.movie;
            var isThereShowTime = _.filter(that.performances, function (p) { return p.get('isReservedSeating'); })[1];
            if (isThereShowTime) {
                that.nextShowTime = Globalize.format(DtHelper.convertDate(isThereShowTime.get('showTime')), "t", that.theater.get('culture'));
            }
            that.previousShowTime = null;
            that.$el.empty();
            that.$el.html(that.template({
                localize: that.localize,
                performances: _.filter(that.performances, function (p) { return p.get('isReservedSeating'); }),
                performance: that.selectedPerformance,
                movie: movie,
                culture: that.theater.get('culture'),
                theaters: that.theaters,
                selectedTheater: that.theater
            }));

            $(that.el).enhanceWithin();
            _.defer(function () {
                if (that.performances.length > 0) {
                    that.renderSeats(that.selectedPerformance);
                }
                if (callback) {
                    callback();
                }
            });
            return that.$el;
        },

        showLargePoster: function (e) {
            e.preventDefault();
            var that = this;
            var movie = window.appConfig.GroupMovies ? that.movieGroup[0] : that.movie;

            var obj = $(this.el);
            var imgSrc = movie.get('image').large;
            if (!imgSrc) return;

            var callback = function () {
                obj.find('#NCR-calendar').css({ 'pointer-events': '' });
            };
            obj.find('#NCR-calendar').css({ 'pointer-events': 'none' });

            UIHelper.showLargePoster(obj, imgSrc, callback);
        },

        selectShowtime: function () {
            var that = this;

            window.AppProperties.SelectedPerformance = this.selectedPerformance;

            var featureCode = '';
            if (!window.appConfig.GroupMovies) {
                featureCode = this.movie.get('code').trim();
            } else {
                _.each(that.movieGroup, function (movie) {
                    _.each(movie.group, function (group) {
                        var performance = _.find(group.performances.models, function (p) {
                            return p.get('number') == that.selectedPerformance.get('number');
                        });
                        if (performance) {
                            featureCode = group.get('code').trim();
                        }   
                    });                    
                });
            }
            window.AppMethods.ChangePage("#tickets/" + this.selectedPerformance.get('number') + "/" + featureCode + "/" + this.theater.get('id'));
        },

        renderSeats: function (performance) {
            var that = this;
            var l = _.find(that.layouts, function (l) {
                return performance.get('auditorium') == l.auditorium;
            });          
            var result = _.find(that.overrides, function (o) { return o.performanceNumber == performance.get('number'); }).result;

            selectSeatsHelper.renderPage(that, true, {
                pLayout: l.layout,
                selector: 'seats-layout',
                showZoneColors: false,
                showSeatNumber: false,
                preventEventBindings: true,
                enableZoom: false,
                seatInfo: result
            });

            _.defer(function() {
                window.hidePageLoadingMessage();
                that.positionNavigationArrows();
                $(that.el).find('.olMapViewport').css('z-index', '-5000');
            });
            var movie = window.appConfig.GroupMovies ? that.movieGroup[0] : that.movie;
            var movieRating = movie.get('rating') ?  ('<b>' + movie.get('rating') + '</b>') : '';
            var dDDFlag = performance && performance.get('dDDFlag') ? ('<b>' + that.localize.dDDFlag + '</b>') : '';
            var imaxFlag = performance && performance.get('imaxFlag') ? ('<b>' + that.localize.imaxFlag + '</b>') : '';
            $(this.el).find('#prevShowTime').text(that.previousShowTime && that.previousShowTime != '' ? (that.localize.prior + that.previousShowTime) : '');
            $(this.el).find('#nextShowTime').text(that.nextShowTime && that.nextShowTime != '' ? (that.localize.next + that.nextShowTime) : '');

            var movieName = that.findMovieName(that.selectedPerformance);
            $(this.el).find('.movie-name').text(movieName ? movieName : movie.get('name'));

            $(this.el).find('.movie-rating').html(movieRating + dDDFlag + imaxFlag);
        },
        positionNavigationArrows: function() {
            var previous = $(this.el).find('#previousLayout');
            var next = $(this.el).find('#nextLayout');
            var seatsLayoutHeight = $(this.el).find('#seats-layout').height();

            previous.css('top', ((seatsLayoutHeight - previous.height()) / 2) + 'px');
            next.css('top', ((seatsLayoutHeight - next.height()) / 2) + 'px');
        },
        handleNavigationArrows: function (e) {
            if (isJqmGhostClick(e)) return;

            e.preventDefault();

            var that = this;

            var currentTarget = $(e.currentTarget);
            if (currentTarget.hasClass('ui-disabled')) return;

            var arrowId = currentTarget.attr('id');

            window.showPageLoadingMessage();

            var rsPerformances = _.filter(that.performances, function (p) { return p.get('isReservedSeating'); });

            var nextPerformanceIndex = $(that.el).find('#selectedPerformanceTime').data('performance-index') + currentTarget.data('addition');

            $(that.el).find('#selectedPerformanceTime').data('performance-index', nextPerformanceIndex);

            if (arrowId == 'previousLayout') {
                $(that.el).find('#nextLayout').removeClass('ui-disabled');

                if (nextPerformanceIndex == 0) {
                    $(e.currentTarget).addClass('ui-disabled');
                    that.previousShowTime = null;
                    that.nextShowTime = Globalize.format(DtHelper.convertDate(rsPerformances[nextPerformanceIndex + 1].get('showTime')), "t", that.theater.get('culture'));
               
                } else {
                    $(e.currentTarget).removeClass('ui-disabled');
                   
                    that.previousShowTime = Globalize.format(DtHelper.convertDate(rsPerformances[nextPerformanceIndex - 1].get('showTime')), "t", that.theater.get('culture'));
                    that.nextShowTime = Globalize.format(DtHelper.convertDate(rsPerformances[nextPerformanceIndex + 1].get('showTime')), "t", that.theater.get('culture'));
                }
            } else {
                $(that.el).find('#previousLayout').removeClass('ui-disabled');
                if (nextPerformanceIndex + 1 == rsPerformances.length) {
                    $(e.currentTarget).addClass('ui-disabled');
                    that.nextShowTime = null;
                    that.previousShowTime = Globalize.format(DtHelper.convertDate(rsPerformances[nextPerformanceIndex - 1].get('showTime')), "t", that.theater.get('culture')); 
                } else {
                    $(e.currentTarget).removeClass('ui-disabled');
          
                    that.previousShowTime = Globalize.format(DtHelper.convertDate(rsPerformances[nextPerformanceIndex - 1].get('showTime')), "t", that.theater.get('culture'));
                    that.nextShowTime = Globalize.format(DtHelper.convertDate(rsPerformances[nextPerformanceIndex + 1].get('showTime')), "t", that.theater.get('culture'));
                }
            }
            
            that.selectedPerformance = rsPerformances[nextPerformanceIndex];

            _.delay(function () {
                that.renderSeats(that.selectedPerformance);

                $(that.el).find('#selectedPerformanceTime').text(DtHelper.formatDateForSeatPreview(DtHelper.convertDate(that.selectedPerformance.get('showTime')), that.theater.get('culture')));
            }, 300);

        },

        trailerButtonClick: function (e) {
            if (ConfigHelper.isDeviceOnline() == false) {
                window.showAlert(this.localize.errorUnableConnectToInternet);
                return false;
            }
            e.preventDefault();

            var that = this;
            var url = $(e.currentTarget).data('trailer-url');
            var ios10 = window.AppProperties.iOSVersion.indexOf('10.') === 0;
            var youtubeIndex = url.indexOf('youtube');
            var inAppWin;

            if (window.AppProperties.IsAndroid) {
                if (youtubeIndex < 0) {
                    inAppWin = window.open(url, '_system');
                } else {
                    inAppWin = window.open(url, '_blank');
                }
            }
            else {
                if (window.AppProperties.IsIOS && ios10 && youtubeIndex >= 0) {
                    inAppWin = window.open(that.addPlaysinlineTrailerURL(url), '_blank', 'allowInlineMediaPlayback=yes');
                }
                else {
                    inAppWin = window.open(url, '_blank', 'location=no');
                }
            }

            var emptyUrl = function () {
                inAppWin.executeScript({
                    code: "if(document.getElementById('player1')) {document.getElementById('player1').pauseVideo()}"
                });
            };

            var inAppWinClose = function () {
                if (window.AppProperties.IsAndroid) {
                    emptyUrl();
                }

                inAppWin.removeEventListener('exit', inAppWinClose);
                inAppWin.removeEventListener('loadstop');
                inAppWin.close();
                inAppWin = null;
            };

            inAppWin.addEventListener('exit', inAppWinClose);
        },

        facebookSharePanel: function (e) {
            var that = this;
            e.preventDefault();
            var movie = window.appConfig.GroupMovies ? that.movieGroup[0] : that.movie;

            var theaterUrl = appConfig.SiteURL + '#movie/' + movie.get('code').trim();

            var facebookParameters = {
                movieName: window.appConfig.GroupMovies ? movie.get('tMDBName') || movie.get('name') : movie.get('name'),
                theaterCulture: '',
                theaterName: '',
                theaterUrl: theaterUrl,
                img: movie.get('image') ? movie.get('image').medium || window.appConfig.DefaultMoviePosterImage : window.appConfig.DefaultMoviePosterImage
            };

            if (that.previousPage != 'movies') {
                if (movie.theaters.length == 1) {
                    var theater = that.theater;
                    facebookParameters.theaterUrl += '/' + theater.get('id');
                    facebookParameters.theaterName = theater.get('name');
                    facebookParameters.theaterCulture = theater.get('culture');
                }
            } else {
                facebookParameters.theaterUrl += "/" + _.pluck(movie.theaters, 'id').join(',');
            }

            var shareText = "{text} {movieDetails} {theatreDetails} {timeDetails}";
            var theatreName = facebookParameters.theaterName === '' ? '' : 'at ' + facebookParameters.theaterName;

            shareText = shareText.replace("{text}", "I'm going to see");
            shareText = shareText.replace("{timeDetails}", '');

            shareText = shareText.replace("{movieDetails}", facebookParameters.movieName);
            shareText = shareText.replace("{theatreDetails}", theatreName);


            var options = {
                method: "share",
                href: facebookParameters.theaterUrl,
                caption: facebookParameters.movieName,
                description: shareText,
                picture: facebookParameters.img,
                share_feedBrowser: true // iOS only

            };
            window.facebookConnectPlugin.showDialog(options, function (response) {
                if (response && response.postId) {
                    window.showAlert('Posted successfully');
                } else if (window.AppProperties.IsAndroid) {
                    window.appAvailability.check(
                        'com.facebook.katana', null, function () {
                            window.showAlert('Posted successfully');
                        });
                }
            });
        },

        getSeatSizes: function (pLayout) {
            var performanceLayout = pLayout || window.performanceLayout;

            var seatWidth = ($(window).width()) / (performanceLayout.totalColumnCount + 2);
            var seatHeight = seatWidth;

            var cinemaHeight = seatHeight * (performanceLayout.totalRowCount + 2);
            var cinemaWidth = $(window).width();

            return { seatWidth: seatWidth, seatHeight: seatHeight, cinemaHeight: cinemaHeight, cinemaWidth: cinemaWidth };
        },

        addPlaysinlineTrailerURL: function (url) {
            if (url) {
                if (url.indexOf("?") > 0)
                    url = url.concat("&");
                else
                    url = url.concat("?");

                url = url.concat("playsinline=1");

                return url;
            }
        }
    });

    return view;
});