define([
'text!templates/SelectRecipientView.html',
'text!templates/RecipientView.html',
'collections/ticketCollection',
'backbone',
'js/libs/jquery.sonar.min',
'facebook',
'helpers/uiHelper',
'sharedhelpers/utilities'
], function (template, templateFriendList) {
    var view = Backbone.View.extend({
        template: _.template(template),
        localize: window.ObjectCollections.Localization.result,

        initialize: function () {
            this.orderModel = window.ObjectModels.OrderModel;
            this.facebook = new FacebookHelper();
            this.friendId = null;
            this.token = '';
            this.onLogin();

            var that = this;
            document.addEventListener("offline", function () {
                that.onOffline();
            }, false);
        },

        render: function () {
            var that = this;
            $(this.el).html(this.template({
                localize: that.localize,
                id: this.options.id,
                confirmationNumber: this.options.confirmationNumber
            }));

            var searchInput = $('#searchFriends input');
            searchInput.live('focus', that.showKeyboardHandler);
            searchInput.live('blur', that.hideKeyboardHandler);

            $(document).on("showkeyboard", that.showKeyboardHandler);
            $(document).on("hidekeyboard", function () {
                searchInput.blur();
                that.hideKeyboardHandler();
            });

            var fixHeader = function () {
                setTimeout(function () {
                    $("#header").css("top", "0px!important").css("left", "0px!important");
                }, 100);
            };

            window.onkeyup = fixHeader;
            window.onscroll = fixHeader;

            UIHelper.dismissKeyboard($(this.el));

            return this;
        },
        events: {
            "tap #sendTickets": "sendTickets",
            "tap .ui-checkbox": "selectFriend",
            "tap #postToFb": "postToFb",
            "tap .search-field .ui-icon-delete": "resetFilterFriendsByName",
            "keypress div.ui-input-search": "filterFriendsByName",
            "pagebeforehide ": "onPageHide"
        },

        onPageHide: function () {
            $(document).off('hidekeyboard');
            $(document).off('showkeyboard');
        },

        showKeyboardHandler: function () {
            $('.hold-seats-footer-container').hide();
            $('form a').show();

            $(document).on('touchmove.ncr touchstart.ncr', function (event) {
                event.preventDefault();
            });
        },

        hideKeyboardHandler: function () {
            $(document).off("touchmove.ncr touchstart.ncr");
            $('.hold-seats-footer-container').show();
            $('form a').hide();

            var friendImages = $('.ui-listview').find('li:not(.ui-screen-hidden) .fb-friend-img');

            _.each(friendImages, function (user) {
                _.defer(function () {
                    user.src = user.dataset.src;
                });
            });
        },

        resetFilterFriendsByName: function (e) {
            if (isJqmGhostClick(e)) return;

            $(this.el).find('input').val('');
            $(this.el).find('.ui-listview li').removeClass('ui-screen-hidden');
        },

        filterFriendsByName: function (e) {
            var that = this;

            if (e.keyCode == 13) {
                e.preventDefault();

                var input = $(e.currentTarget).find('input');
                var searchValue = $.trim(input.val()).toLowerCase();
                var friends = $(that.el).find('.ui-listview li:not(.ui-li-divider)');
                var dividers = $(that.el).find('.ui-listview li.ui-li-divider');

                if (!searchValue) {
                    friends.removeClass('ui-screen-hidden');
                    dividers.removeClass('ui-screen-hidden');
                    input.blur();
                    return;
                }

                var noFriendsInList = true;
                _.each(friends, function (friend) {
                    var name = $.trim($(friend).find('.friend-name').html()).toLowerCase();

                    if (name.indexOf(searchValue) >= 0) {
                        $(friend).removeClass('ui-screen-hidden');
                        noFriendsInList = false;
                    } else {
                        $(friend).addClass('ui-screen-hidden');
                    }
                });

                _.each(dividers, function (divider) {
                    var dividerLetter = $(divider).text();
                    var visibleItem = $(divider).nextAll(':not(.ui-screen-hidden)').eq(0);

                    var nextElement = $.trim(visibleItem.find('.friend-name').html());

                    if (dividerLetter == nextElement.slice(0, 1)) {
                        $(divider).removeClass('ui-screen-hidden');
                    } else {
                        $(divider).addClass('ui-screen-hidden');
                    }
                });

                if (noFriendsInList) {
                    $(this.el).find('#no-friends-msg').show();
                } else {
                    $(this.el).find('#no-friends-msg').hide();
                }

                input.blur();
                return;
            }
        },

        postToFb: function (e) {
            e.preventDefault();
            window.showPageLoadingMessage();
            var that = this;

            require(['sharedhelpers/storageProvider'], function () {
                var provider = new StorageProvider();
                provider.read('PurchaseHistory', function (item) {
                    var temp = $.grep(item, function (sItem) { return sItem.confirmationNumber === that.options.confirmationNumber; });

                    var giftTicketObj = {
                        firstName: that.friend.first_name,
                        lastName: that.friend.last_name,
                        phoneNumber: '',
                        facebookId: that.friend.id,
                        email: ''
                    };

                    var tickets = new TicketCollection();
                    tickets.giftTicket(giftTicketObj, that.options.id, temp[0].theaterId, function (confirmationNumber) {
                        if (confirmationNumber) {
                            var utilities = new Utilities();

                            var fbLink = appConfig.SiteURL +
                                '#pickup/' +
                                encodeURIComponent(utilities.encrypt($.trim(confirmationNumber))) + "/"
                                + encodeURIComponent(utilities.encrypt(that.friendId)) + "/"
                                + encodeURIComponent(utilities.encrypt(that.friend.first_name + " " + that.friend.last_name)) + "/"
                                + $.trim(temp[0].performance.number) + "/"
                                + $.trim(temp[0].movieCode) + "/"
                                + $.trim(temp[0].theaterId);

                            that.facebook.post(that.token,
                                {
                                    name: temp[0].movieName,
                                    caption: that.localize.giftTicketFbCaption,
                                    description: that.getFbMessage(temp[0].movieName, temp[0].theaterName, temp[0].performance.showTime, temp[0].theaterUrl, temp[0].theaterCulture),
                                    link: fbLink,
                                    picture: temp[0].movieImage ? temp[0].movieImage['medium'] || window.appConfig.DefaultMoviePosterImage : window.appConfig.DefaultMoviePosterImage,
                                    userList: that.friendId,
                                    actionType: 'gift',
                                    preventDelay: true
                                },
                                function (response) {
                                    if (response.success === true) {
                                        _.each(temp[0].selectedTickets, function (i) {
                                            if (!i.isGifted && i.serialNumber == that.options.id) {
                                                i.isGifted = true;
                                                i.friendId = that.friendId;
                                            }
                                        });

                                        provider.remove('PurchaseHistory');
                                        provider.store('PurchaseHistory', item);

                                        window.AppProperties.HistoryOrders = null;
                                    }

                                    window.history.back();
                                });
                        } else {
                            window.history.back();
                        }
                        window.hidePageLoadingMessage();
                    });
                });
            }); //storage provider
        },

        selectFriend: function (e) {
            var that = this;
            var useSelector = $(e.currentTarget);
            var checkboxLbl = useSelector.find('label');

            if (checkboxLbl.hasClass('ui-checkbox-off')) {
                checkboxLbl.removeClass('ui-checkbox-off').addClass('ui-checkbox-on');

                if (that.friendId) {
                    $(that.el).find('#' + that.friendId).find('label').removeClass('ui-checkbox-on').addClass('ui-checkbox-off');
                }

                that.friendId = useSelector.attr('id');
                that.friend = _.find(that.friends, function (friend) {
                    return friend.id == that.friendId;
                });
            } else {
                checkboxLbl.removeClass('ui-checkbox-on').addClass('ui-checkbox-off');
                that.friendId = null;
                that.friend = null;
            }
        },

        sendTickets: function (event) {
            event.preventDefault();
            var that = this;
            if (that.friendId != null) {
                that.postToFb(event);
            } else {
                window.showAlert(this.localize.errorSelectFriends);
            }

            return false;
        },
        getFbMessage: function (movieTitle, theatre, date, url, theaterCulture) {
            var showDate = DtHelper.convertDate(date);
            var dateValue = Globalize.format(showDate, "M", theaterCulture);
            var timeValue = Globalize.format(showDate, "t", theaterCulture);

            var fbMsg = $.sprintf(this.localize.fbMsgRecipientView,
                movieTitle,
                theatre,
                dateValue,
                timeValue);

            return fbMsg;
        },
        listFriends: function (friends) {
            var that = this;

            $(that.el).find('.hold-seats-status-container').show();
            $(that.el).find('.hold-seats-friends-container').show();
            $(that.el).find('.hold-seats-skip-button').html('Send Ticket');

            var filter = $(that.el).find('#searchFriends');
            filter.css('margin', 0);

            filter.appendTo('.search-field');

            that.friends = friends;

            var friendsList = $(that.el).find('.hold-seats-friends-container ul');
            friendsList.css('margin', 0);
            friendsList.css('margin-bottom', 55);

            that.sortFriends(that.friends);

            friendsList.append(_.template(templateFriendList, { friends: that.friends }));

            that.connectSonar();

            window.hidePageLoadingMessage();
        },

        sortFriends: function (friends) {
            friends.sort(function (a, b) {
                var nameA = a.name.toLowerCase();
                var nameB = b.name.toLowerCase();

                if (nameA < nameB) return -1;

                if (nameA > nameB) return 1;

                return 0;
            });
        },

        onOffline: function () {
            window.showAlert(this.localize.errorUnableConnectToInternet);
            window.history.back();
        },

        onLogin: function () {
            var that = this;
            window.showPageLoadingMessage();

            window.facebookConnectPlugin.getLoginStatus(function (response) {
                if (response.status === "connected") {
                    that.token = response.authResponse.accessToken;
                    window.facebookConnectPlugin.api('/me/friends', ["user_friends"], function (response) {
                        if (response.data) {
                            that.listFriends(response.data);
                        }
                    });

                } else {
                    window.facebookConnectPlugin.login(["public_profile"], function (result) {
                        that.token = result.authResponse.accessToken;
                        window.facebookConnectPlugin.api('/me/friends', ["user_friends"], function (response) {
                            if (response.data) {
                                that.listFriends(response.data);
                            }
                        });
                    });
                }
            });
        },

        connectSonar: function () {
            $(".fb-friend-img").bind("scrollin", function () {
                var myimg = this;
                var $myimg = $(myimg);
                $myimg.unbind("scrollin");
                myimg.src = $myimg.attr("data-src");
            });

            return false;
        },
    });

    return view;
});