define([
    'text!templates/SelectSeatsView.html',
    'views/pullDownStatusView',
    'collections/performanceSeatInfoCollection',
    'collections/ticketCollection',
    'sharedhelpers/configurationProvider',
    'sharedhelpers/selectSeatsHelper',
    'sharedlibs/openlayer/OpenLayers.mobile',
    'helpers/uiHelper'
], function (template, pullDownStatusView, performanceSeatInfoCollection, ticketCollection, configurationProvider, seatsHelper) {
    window.SelectSeatsView = Backbone.View.extend({
        localize: window.ObjectCollections.Localization.result,

        initialize: function () {
            window.tempHistoryCount++;

            this.pageRendered = false;
            this.firstLoad = null;
            this.touchEnd = null;

            this.showLoveSeat = null;
            this.showCompanion = null;
            this.showWheelChair = null;
            this.showRegular = null;

            this.pullDownStatusView = null;
            this.reselectFeature = true;

            var orderModel = window.ObjectModels.OrderModel;

            OpenLayers.Feature.Vector.style['default']['strokeWidth'] = '2';
            if (orderModel != null) {
                this.filterOtherItems(orderModel);
                this.tickets = orderModel.get('tickets');
                this.seatStatuses = orderModel.get('seatStatuses');
                this.saleId = orderModel.get('saleId');

                for (var i = 0; i < this.tickets.models.length; i++) {
                    var currTicketType = this.tickets.models[i];
                    currTicketType.unset('seats', { silent: true });

                    var volume = currTicketType.get('volume') || 1;
                    if (currTicketType.get('isReservedSeating') || currTicketType.get('isOnHoldTicketType')) {
                        currTicketType.set('remaining', (currTicketType.get('quantity') * volume), { silent: true });
                    }
                }
            }
        },

        filterOtherItems: function (orderModel) {
            var totalPrice = orderModel.get('totalPrice');

            var otherItems = _.reject(orderModel.get('otherItems'), function (it) {
                if (it.canBeRemoved) totalPrice -= it.total;

                return it.canBeRemoved;
            });

            otherItems = _.sortBy(otherItems, function (it) {
                return it.sortOrder;
            });

            orderModel.set('otherItems', otherItems);
            orderModel.set('totalPrice', totalPrice);
        },

        refreshStatusMessage: function () {
            var lockedSeats = this.seatStatuses.where({ status: 'Locked' });
            var seatsString = "";

            for (var i = 0; i < lockedSeats.length; i++) {
                var currLockedSeat = lockedSeats[i].get('seat');

                if (!currLockedSeat) continue;

                seatsString += currLockedSeat.get('name') + ', ';
            }

            seatsString = seatsString.slice(0, -2);

            $(this.el).find('.selected-seats-container').html('<span class="bodyText">' + this.localize.selectedSeats + ' </span> ' + seatsString);

            $(this.el).find('#seats-layout').css('margin-top', $(this.el).find('.screen-container').height());

            if (lockedSeats.length >= this.tickets.getAllTicketQuantity()) {
                $(this.el).find('#selectSeatsButton').css('color', '#FFFFFF');
                $(this.el).find('#selectSeatsButton').html(this.localize.continueButton);
            } else {
                var seatsSelectedMsg = $.sprintf(this.localize.numOfSeatsSelected,
                    lockedSeats.length,
                    this.tickets.getAllTicketQuantity()
                    );
                $(this.el).find('#selectSeatsButton').css('color', '#666666');
                $(this.el).find('#selectSeatsButton').html(seatsSelectedMsg);
            }
        },

        refreshStatuses: function () {
            seatsHelper.refreshStatuses(this, { touchEvents: true });
        },

        drawSeats: function () {
            seatsHelper.drawSeats(this, { imgPath: 'css/images/' });
        },

        getBrandStylesheet: function () {
            var styleSheet = '';

            _.each(document.styleSheets, function (item) {
                if (item.href && item.href.indexOf('brand.css') > 0)
                    styleSheet = item.rules || item.cssRules;
            });

            return styleSheet;
        },

        getZoneStyle: function (classes, className, property) {
            for (var x = 0; x < classes.length; x++) {
                if (classes[x].selectorText == className) {
                    return classes[x].style[property];
                }
            }
        },

        render: function () {
            var that = this;

            $(this.el).html(_.template(template, {
                localize: that.localize,
                seatCollection: window.performanceLayout
            }));

            that.firstLoad = true;
            that.touchEnd = false;

            that.pullDownStatusView = new pullDownStatusView();
            that.assign(that.pullDownStatusView, '.pullDownStatusHolder');

            $(this.el).find('#mapKeyDialog').on({
                popupafterclose: function () {
                    $(that.el).find('#mapKeyDialog').empty();
                },
                popupbeforeposition: function () {
                    $('.ui-popup-screen').off();
                }
            });

            return this;
        },
        assign: function (view, selector) {
            this.$(selector).empty();
            view.setElement(this.$(selector)).render();
            this.$(selector).enhanceWithin();
        },

        events:
		{
		    "pagehide": "pageHide",
		    "pageshow": "pageshow",
		    "tap .key-button": "showMapKeyLegend",
		    "tap #selectSeatsButton": "onContinue",
		    "touchend #seats-layout": "featureTouchEnd"
		},

        featureTouchEnd: function () {
            var that = this;
            that.touchEnd = true;
            that.firstLoad = false;
            this.reselectFeature = true;

            setTimeout(function () {
                that.touchEnd = false;
            }, 320);
        },

        onContinue: function (e) {
            if (isJqmGhostClickPropagated(e) || isJqmGhostClick(e)) return;

            var that = this;

            if (ConfigHelper.isDeviceOnline() == false) {
                window.showAlert(this.localize.errorUnableConnectToInternet);

                return false;
            }

            var lockedSeats = this.seatStatuses.where({ status: 'Locked' });

            if (lockedSeats.length < this.tickets.getAllTicketQuantity())
                return;

            var toContinue = "checkoutSummary";

            var theater = window.ObjectModels.OrderModel.get('theater');


            if (configurationProvider.useConssesion(theater)) {
                toContinue = "concessions/" + theater.get('id') + '/null';
            }

            var callback = function (success) {
                if (success) {
                    //Remove concession prices from total price if concessions are previously locked
                    if (window.ObjectModels.OrderModel.get('apiLockedConcessions')) {
                        var totalPrice = window.ObjectModels.OrderModel.get('totalPrice');
                        _.each(window.ObjectModels.OrderModel.get('apiLockedConcessions').models, function (c) {
                            totalPrice -= c.get('price');
                        });

                        window.ObjectModels.OrderModel.set('totalPrice', totalPrice);
                    }

                    //Get current URL.
                    window.AppProperties.SelectSeatsFragment = Backbone.history.fragment;
                    window.AppMethods.ChangePage(toContinue);
                } else {
                    that.firstLoad = true;
                    that.filterOtherItems(window.ObjectModels.OrderModel);

                    var sentSeats = window.ObjectModels.OrderModel.get('sentSeats');

                    _.each(sentSeats, function (sentSeat) {
                        var selectSeat = that.vectors.getFeatureByFid(sentSeat.get('row') + '-' + sentSeat.get('column'));
                        selectSeat.attributes.status = sentSeat.get('status');
                        that.firstLoad = true;
                        that.selectCtrl.unselect(selectSeat);
                    });

                    that.renderPage();

                    _.defer(function () {
                        that.pullDownStatusView = new pullDownStatusView();
                    });
                }
            };

            if (window.ObjectModels.TempOrder && window.ObjectModels.TempOrder.concessionsChanged) {
                new ConcessionCollection().lockConcessions(function () {
                    window.ObjectModels.TempOrder = { concessions: null };
                    that.tickets.lockSeats(that.saleId, callback);
                }, true);
            } else {
                window.ObjectModels.TempOrder = { concessions: null };
                that.tickets.lockSeats(that.saleId, callback);
            }
        },

        pageshow: function () {
            if (!this.pageRendered) {
                this.renderPage();
                this.pageRendered = true;
            }
        },

        renderPage: function () {
            seatsHelper.renderPage(this, false);
        },

        setLabels: function () {
            var that = this;
            var feature = $('svg g g path')[0];

            //Row labels
            if (feature && feature.getBoundingClientRect().width < 8) {
                $('svg text[font-size="16px"]').css({ 'font-size': '8px' });
            } else if (feature && feature.getBoundingClientRect().width < 12) {
                $('svg text[font-size="16px"]').css({ 'font-size': '10px' });
            } else {
                $('svg text[font-size="16px"]').css({ 'font-size': '16px' });
            }

            //Seat labels
            if (feature && feature.getBoundingClientRect().width > 20) {
                $('svg text[font-size="0px"]').css({ 'font-size': '12px' });
                that.labelSize = '12px';
            } else if (feature && feature.getBoundingClientRect().width > 12) {
                $('svg text[font-size="0px"]').css({ 'font-size': '8px' });
                that.labelSize = '8px';
            } else {
                $('svg text[font-size="0px"]').css({ 'font-size': '0px' });
                that.labelSize = '0px';
            }
        },

        showMapKeyLegend: function () {
            $('#selectSeatsButton').css({ 'pointer-events': 'none' });
            $('.pullDownStatusHolder').css({ 'pointer-events': 'none' });
            var that = this;
            var mapKeyView = 'selectSeatsMapKey';
            var seatCollection = window.performanceLayout;

            if (seatCollection.zones.models.length) {
                mapKeyView = 'selectSeatsZoneMapKey';
            }

            require(['views/' + mapKeyView], function (seatsKeyView) {
                var dimensions = that.getSeatSizes();

                var seatWidth = dimensions.seatWidth;
                var seatHeight = dimensions.seatHeight;

                var dialog = new seatsKeyView({
                    seatWidth: seatWidth, seatHeight: seatHeight, zones: seatCollection.zones.models, theater: window.ObjectModels.OrderModel.get('theater'), showLoveSeat: that.showLoveSeat, showCompanion: that.showCompanion, showWheelChair: that.showWheelChair, showRegular: that.showRegular, callback: function () {
                        $('#mapKeyDialog').popup("close");
                        $('.pullDownStatusHolder').css({ 'pointer-events': '' });
                        $('#selectSeatsButton').css({ 'pointer-events': '' });
                    }
                });

                $("#mapKeyDialog").html(dialog.render());
                $('#mapKeyDialog').popup("open");
                $("#mapKeyDialog").enhanceWithin();
            });
        },

        pageHide: function () {
            var uiContent = $(this.el).find('.ui-content');
            uiContent.css('padding', '15px').hide();
        },

        orginalSelectStyle: function (feature) {
            this.selectCtrl.selectStyle = { fillColor: feature.attributes.originalColor, fillOpacity: feature.attributes.originalOpacity, strokeColor: feature.attributes.originalColor, strokeOpacity: '0', strokeWidth: feature.attributes.strokeWidth, strokeLinejoin: 'round', label: feature.attributes.label, fontSize: this.labelSize || '0px', fontColor: '#9C9595' };
        },

        setSelectStyle: function (feature) {
            this.selectCtrl.selectStyle = { fillColor: feature.attributes.originalColor, fillOpacity: feature.attributes.selectedOpacity, strokeColor: feature.attributes.originalColor, strokeOpacity: feature.attributes.selectedOpacity, strokeWidth: feature.attributes.strokeWidth, strokeLinejoin: 'round', label: feature.attributes.label, fontSize: this.labelSize || '0px', fontColor: '#9C9595' };
        },

        changeStyle: function (feature) {
            var that = this;

            that.orginalSelectStyle(feature);

            if (feature.attributes.status == 'Reserved' || feature.attributes.status == 'Sold' || feature.attributes.type == 'GeneralAdmission' || feature.attributes.type == 'Blocked') {
                this.selectCtrl.selectStyle = { fillColor: feature.attributes.unavailableColor, fillOpacity: feature.attributes.originalOpacity, strokeOpacity: '0' };
                return;
            }

            if (that.touchEnd == false && that.firstLoad == false)
                return;

            that.setSelectStyle(feature);
        },

        onClickSeat: function (feature, selected) {
            var that = this;

            if (this.maxWarning) {
                that.setLabels();
                this.maxWarning = null;
                return;
            }

            if (that.touchEnd == false && that.firstLoad == false) {
                if (that.reselectFeature && selected)
                    setTimeout(function () { that.selectCtrl.select(feature); }, 190);
                else if (that.reselectFeature)
                    setTimeout(function () { that.selectCtrl.unselect(feature); }, 190);
                else if (selected)
                    that.selectCtrl.unselect(feature);
                that.reselectFeature = false;
                return;
            }

            var handleUnavailable = function () {
                that.maxWarning = true;
                that.selectCtrl.unselect(feature);
            };

            //Handle unavailable seats
            if (feature.attributes.status == 'Reserved') {
                handleUnavailable();
                window.showAlert(this.localize.alertSeatReserved);

                return;
            } else if (feature.attributes.status == 'Sold') {
                handleUnavailable();
                window.showAlert(this.localize.alertSeatSold);

                return;
            } else if (feature.attributes.type == 'GeneralAdmission') {
                handleUnavailable();
                window.showAlert(this.localize.alertSeatGeneralAdmission);

                return;
            } else if (feature.attributes.type == 'Blocked') {
                handleUnavailable();
                window.showAlert(this.localize.alertSeatBlocked);

                return;
            }

            var performanceLayout = window.performanceLayout;

            if (selected && !feature.attributes.refresh && (feature.attributes.type == 'Wheelchair' || feature.attributes.type == 'Companion')) {
                var seatMessage = '';
                _.each(performanceLayout.seateMessages.models, function (message) {
                    if (feature.attributes.type == message.get('type')) {
                        seatMessage = message.get('message');
                        return;
                    }
                });
                window.showAlert(seatMessage);
            }

            if (feature.attributes.type == 'NotSeat') return;
            var remainingTypes = _.filter(this.tickets.models, function (model) {
                return model.get('remaining') > 0;
            });

            var ticketType = null;
            var isOnHold = false;

            if (remainingTypes.length)
                ticketType = remainingTypes[0];

            var row = feature.attributes.row;
            var column = feature.attributes.col;

            var seat = performanceLayout.where({ row: row, column: column })[0];

            var lockedSeats = $.grep(this.seatStatuses.models, function (s) {
                return s.get('status') == 'Locked' || s.get('status') == 'Hold';
            });

            if (remainingTypes.length && feature.attributes.refresh && seat.get('status').get('status') == 'Hold') {
                ticketType = _.filter(this.tickets.models, function (model) {
                    return model.get('isOnHoldTicketType');
                })[0];
            }

            // Unselect first seat and select last one
            if (selected && lockedSeats.length >= this.tickets.getAllTicketQuantity() && feature.attributes.status != 'Locked' && feature.attributes.status != 'Hold' && !feature.attributes.refresh) {

                var seatCollection = window.performanceLayout;

                var currentTime = null;
                var firstSeat = null;

                for (var i = 0; i < seatCollection.models.length; i++) {
                    if (seatCollection.models[i].has('lastUpdate')) {
                        if (!currentTime || currentTime > seatCollection.models[i].get('lastUpdate')) {
                            currentTime = seatCollection.models[i].get('lastUpdate');
                            firstSeat = seatCollection.models[i];
                        }
                    }
                }

                if (firstSeat) {
                    firstSeat.unset('lastUpdate');

                    var seatElement = this.vectors.getFeatureByFid(firstSeat.get('row') + '-' + firstSeat.get('column'));

                    ticketType = firstSeat.get('ticket');

                    var seatZone = window.performanceLayout.zones.get(firstSeat.get('zoneId'));

                    if (seatElement.attributes.status != 'Hold')
                        that.updateSeatZones(seatZone, true);

                    seatElement.attributes.status = '';
                    $('[id="' + seatElement.geometry.id + '"]').css({ 'fill-opacity': feature.attributes.originalOpacity });

                    if (firstSeat.has('ticket')) {
                        var ticket = firstSeat.get('ticket');

                        ticket.set('remaining', ticket.get('remaining') + 1, { silent: true });

                        if (ticket.get('seats').length > 1) {
                            ticket.get('seats').remove(firstSeat, { silent: true });
                        } else {
                            ticket.unset('seats');
                        }

                        firstSeat.unset('ticket', { silent: true });

                        firstSeat.get('status').set('status', '');
                    }

                    seatElement.style.fillOpacity = feature.attributes.originalOpacity;
                    seatElement.style.strokeOpacity = '0';

                    this.maxWarning = true;
                    this.selectCtrl.unselect(seatElement);
                }
            }

            var zone = window.performanceLayout.zones.get(seat.get('zoneId'));

            //Unselect a seat
            if (feature.attributes.status == 'Locked' || feature.attributes.status == 'Hold') {
                if (feature.attributes.status != 'Hold')
                    that.updateSeatZones(zone, true);

                feature.attributes.status = '';
                $('[id="' + feature.geometry.id + '"]').css({ 'fill-opacity': feature.attributes.originalOpacity });

                if (seat.has('ticket')) {
                    var ticket = seat.get('ticket');
                    isOnHold = ticket.get('isOnHoldTicketType');

                    ticket.set('remaining', ticket.get('remaining') + 1, { silent: true });

                    if (ticket.get('seats').length > 1) {
                        ticket.get('seats').remove(seat, { silent: true });
                    } else {
                        ticket.unset('seats');
                    }

                    seat.unset('ticket', { silent: true });

                    seat.get('status').set('status', '');

                    seat.unset('lastUpdate');
                }

                //deselect
                feature.style.fillOpacity = feature.attributes.originalOpacity;
                feature.style.strokeOpacity = '0';

                that.maxWarning = true;
                that.selectCtrl.unselect(feature);

                that.refreshSeatNotification(seat, zone, isOnHold, true);

                return;
            }

            if (!selected) return;

            var confirmSeatSelection = function (selectedType) {
                if (selectedType == 'Hold') {
                    feature.attributes.status = 'Hold';
                    $('[id="' + feature.geometry.id + '"]').css({ 'fill-opacity': feature.attributes.originalOpacity });
                }
                else {
                    feature.attributes.status = 'Locked';
                    $('[id="' + feature.geometry.id + '"]').css({ 'fill-opacity': feature.attributes.selectedOpacity });
                }

                var status = 'Locked';

                //on zoom for selected
                if (feature.attributes.status == 'Hold') {
                    feature.style.fillOpacity = feature.attributes.originalOpacity;
                    feature.style.strokeOpacity = feature.attributes.selectedOpacity;
                } else {
                    feature.style.fillOpacity = feature.attributes.selectedOpacity;
                    feature.style.strokeOpacity = feature.attributes.selectedOpacity;
                }

                var seatStatus = $.grep(that.seatStatuses.models, function (s) {
                    return (s.get('status') == 'Locked' || s.get('status') == 'Hold') && s.get('row') == row && s.get('column') == column;
                });

                if (seatStatus.length == 0) {
                    var stat = new PerformanceSeatInfoModel({ row: row, column: column, status: status });
                    stat.set('seat', seat, { silent: true });
                    seat.set('status', stat, { silent: true });

                    that.seatStatuses.add(stat);
                } else {
                    seatStatus[0].set('status', status);
                }

                seat.set('lastUpdate', new Date().getTime(), { silent: true });
                isOnHold = seat.get('ticket').get('isOnHoldTicketType');

                feature.attributes.refresh = false;

                if (feature.attributes.status != 'Hold')
                    that.updateSeatZones(zone, false);

                that.refreshSeatNotification(seat, zone, isOnHold, feature.attributes.refresh);
            };

            if (ticketType) {
                if (!ticketType.has('seats')) {
                    ticketType.set('seats', new SeatCollection(), { silent: true });
                }

                ticketType.get('seats').push(seat, { silent: true });

                seat.set('ticket', ticketType, { silent: true });

                ticketType.set('remaining', ticketType.get('remaining') - 1, { silent: true });

                confirmSeatSelection(ticketType.get('name'));
            }
        },

        updateSeatZones: function (zone, clear) {
            if (!zone) return;

            var otherItems = window.ObjectModels.OrderModel.get('otherItems');
            var totalPrice = window.ObjectModels.OrderModel.get('totalPrice');
            var surchargeAmount = zone.get('surchargeAmount');

            var zones = $.grep(otherItems, function (i) { return i.name == zone.get('name'); });

            if (!clear && zones.length == 0) {
                otherItems.push({
                    name: zone.get('name'),
                    price: surchargeAmount,
                    quantity: 1,
                    sortOrder: otherItems.length + 1,
                    tax: 0,
                    total: surchargeAmount
                });

                totalPrice += surchargeAmount;
            } else {
                var item = zones[0];

                if (item) {
                    if (clear) {
                        item.quantity--;

                        if (item.quantity == 0) {
                            otherItems.splice(otherItems.indexOf(item), 1);
                        } else {
                            item.price -= surchargeAmount;
                            item.total -= surchargeAmount;
                        }

                        totalPrice -= surchargeAmount;
                    } else {
                        item.quantity++;
                        item.price += surchargeAmount;
                        item.total += surchargeAmount;

                        totalPrice += surchargeAmount;
                    }
                }
            }

            window.ObjectModels.OrderModel.set('totalPrice', totalPrice);
        },

        refreshSeatNotification: function (seat, zone, isOnHold, clear) {
            var that = this;

            var pullDownStatusHolder = $(that.el).find('.pullDownStatusHolder');

            pullDownStatusHolder.css({
                'opacity': '0'
            });

            var container = $('.notification-container').find('.notification-top-bar');
            var seatMessage = container.find('.seat-message');

            if (!isOnHold && zone) {
                seatMessage.html(that.localize.seatInMessage + ' ' + Globalize.format(zone.get('surchargeAmount'), 'c', window.ObjectModels.OrderModel.get('theater').get('culture')));
            } else {
                seatMessage.html('');
            }

            var seatMessageHtml = seatMessage[0].outerHTML;

            container[0].className = $('.notification-top-bar')[0].className.replace(/note-.*/g, '');
            if (clear) {
                container.addClass('note-danger');
            } else {
                container.addClass('note-success');
            }

            var htmlText = that.localize.seatInMessage + ' ' + seat.get('name') + ' ' + (clear ? that.localize.seatRemoved : that.localize.seatAdded) + seatMessageHtml;


            var noteCallback = function () {
                pullDownStatusHolder.css({ 'opacity': '100' });
            };

            require(['helpers/uiHelper'], function () {
                UIHelper.notificationBar(htmlText, noteCallback);
            });

            that.assign(that.pullDownStatusView, '.pullDownStatusHolder');

            that.refreshStatusMessage();
        },

        getSeatSizes: function () {
            var seatWidth = ($(window).width()) / (window.performanceLayout.totalColumnCount + 2);
            var seatHeight = seatWidth;

            var cinemaHeight = seatHeight * window.performanceLayout.totalRowCount;
            var cinemaWidth = $(window).width();

            return { seatWidth: seatWidth, seatHeight: seatHeight, cinemaHeight: cinemaHeight, cinemaWidth: cinemaWidth };
        }
    });

    return window.SelectSeatsView;
});