define([
  'text!templates/TheaterDetailsView.html',
  'text!templates/HomeScreenLoyaltyDetailsView.html',
  'sharedhelpers/configurationProvider',
  'helpers/uiHelper',
  'helpers/theatersHelper',
  'sharedhelpers/storageProvider'
], function (template, homeScreenLoyaltyDetailsTemplate, configurationProvider) {
    var view = Backbone.View.extend({
        template: _.template(template),
        homeScreenLoyaltyDetailsTemplate: _.template(homeScreenLoyaltyDetailsTemplate),
        localize: window.ObjectCollections.Localization.result,
        theaterMovies: undefined,

        initialize: function () {
            var that = this;

            that.today = DtHelper.getCurrentBusinessDate();
            that.today.setHours(0, 0, 0, 0);
        },

        likeButton: function (selector) {
            var that = this;

            var storage = new StorageProvider();
            storage.read('fbAccount', function (value) {
                if (!value) return;

                var token = value.accessToken;
                var theaterFacebookUrl = that.options.theater.get('facebookURL');

                if (token && theaterFacebookUrl) {
                    selector.find('#fbLike').html('<iframe id="fbLikeFrame" scrolling="no" frameborder="0" style="border:none; overflow:hidden; width:52px; height:28px;" allowtransparency="true" src="http://www.facebook.com/plugins/like.php?href=' + theaterFacebookUrl + '&send=false&layout=standard&width=50&show_faces=false&font&colorscheme=light&action=like&height=26&appId=' + appConfig.FacebookAPIKey + '"></iframe>');
                }
            });
        },

        events:
		{
		    'tap #theatreDrivingDirections': 'drivingDirections',
		    'tap .theater-details-movie-ticket': 'selectMovieTickets',
		    'tap .theater-details-movie-div, .feature': 'selectMovie',
		    'tap #previousDay': 'previousDateClick',
		    'tap #nextDay': 'nextDateClick',
		    'tap #currentDay': 'currentDay',
		    'change #performances-calendar-select': 'selectDate',
		    'tap #buyConcessions': 'navigateConcessions',
		    'tap .policy': 'warningInfo',
		    'tap .favicon': 'addTheaterAsFavorite',
		    'pagebeforeshow': 'beforeShow',
		    'tap .homescreen-loyalty-container': 'handleLoyaltyTap'
		},

        handleLoyaltyTap: function (e) {
            e.preventDefault();
            if (isJqmGhostClick(e) || isChangePageFired(e)) return;

            window.showPageLoadingMessage();
            var loyaltyContainer = $(e.currentTarget);

            if (loyaltyContainer.find(".giftCardBalanceLoader").length > 0 || !configurationProvider.useLoyalty()) {
                return;
            }
            window.AppMethods.ChangePage("loyaltycard");
        },

        loyaltyDataReceived: function (loyaltyData) {
            $(this.el).find('.homescreen-loyalty-container').html(this.homeScreenLoyaltyDetailsTemplate({
                loyalty: loyaltyData.data,
                signInLoyalty: false,
                loyaltyError: loyaltyData.error,
                localize: this.localize
            }));
        },

        beforeShow: function () {
            if (!this.options.theater.get('heroImage')) {
                $(this.el).find('.logo-image').hide();
            }
        },

        drivingDirections: function (e) {
            e.preventDefault();
            e.stopPropagation();

            var theaterId = $(e.currentTarget).attr('data-theaterid');

            require(['helpers/utilities'], function () {
                var utils = new Utils();
                utils.showDirections(theaterId);
            });
        },

        warningInfo: function (e) {
            e.preventDefault();

            if (isJqmGhostClick(e)) return;

            var that = this;
            var theater = that.options.theater;

            if (!theater.get('warningLong')) return;

            require(['views/infoDialogView'], function (infoView) {
                var dialog = new infoView({
                    title: theater.get('warningTitle'),
                    content: theater.get('warningLong'),
                    confirmButton: {
                        text: that.localize.ok,
                        callback: function () {
                            $('#popupDialog').popup("close");
                        }
                    }
                });

                $("#popupDialog").html(dialog.render());
                $('#popupDialog').popup("open");
                $("#popupDialog").enhanceWithin();
            });
        },

        navigateConcessions: function (e) {
            e.preventDefault();
            if (isJqmGhostClick(e) || isChangePageFired(e)) return;

            _.defer(function () {
                window.tempHistoryCount = 0;
                window.ObjectModels.TempOrder = { concessions: null };
                var theaterId = $(e.currentTarget).attr('data-theaterid');

                window.AppMethods.ChangePage('concessions/' + theaterId + '/null');
            });
        },

        selectMovie: function (e) {
            e.stopPropagation();
            e.preventDefault();
            if (isJqmGhostClick(e) || isChangePageFired(e)) return;

            // Prevents loading of outdated data
            window.mdSelectedBusinessDate = window.tdCurrentDate;
            var target = $(e.currentTarget);

            var theaterId = target.attr("data-theater");
            var featureCode = target.attr("data-featurecode");

            this.updateTheaterCollection(AppProperties.CurrentPosition.Latitude, AppProperties.CurrentPosition.Longitude, theaterId, featureCode);
            return false;
        },

        updateTheaterCollection: function (latitude, longitude, theaterId, featureCode) {
            require(['collections/theaterCollection', 'views/theaters'],
	        	function (theaterCollection) {
	        	    var theaters = new theaterCollection();
	        	    theaters.latitude = latitude;
	        	    theaters.longitude = longitude;

	        	    // Always true
	        	    if (window.ObjectCollections.TheaterCollection.models != undefined) {
	        	        var theatersHelper = new TheatersHelper();
	        	        window.AppProperties.SelectedTheater = null;
	        	        window.AppProperties.SelectedTheater = window.ObjectCollections.TheaterCollection.get(theaterId);
	        	        window.AppProperties.SelectedTheaterByZip = window.AppProperties.SelectedTheater;
	        	        theatersHelper.saveTheaterDefaultId(theaterId);
	        	    }

	        	    window.AppMethods.ChangePage("#movie/" + featureCode + "/" + theaterId);
	        	}
            );
        },

        selectMovieTickets: function (e) {
            e.stopPropagation();
            if (e.currentTarget.className.indexOf('ui-disabled') != -1 || isChangePageFired(e) || isJqmGhostClick(e)) return;

            // Allow us to fine tune theater caching option
            window.tdTicketSelected = true;
            window.tdTicketSelectedMovies = this.theaterMovies;

            var target = $(e.currentTarget);

            var performance = target.attr("data-performance");
            var theater = this.options.theater.get('id');
            var featureCode = target.attr("data-featurecode");

            var perfObj = null;

            _.each(this.theaterMovies.models, function (m) {
                _.each(m.performances.models, function (p) {
                    if (p.get('number') == performance && perfObj == null) {
                        perfObj = p;
                    }
                });
            });

            if (!perfObj) {
                window.showAlert(this.localize.theaterUpdatedSchedule);
                window.AppProperties.PageRefresh = true;
                window.AppMethods.ChangePage("#theater/" + theater, true);

                return;
            }

            // clone it to add ticket types
            var pNew = new PerformanceModel(perfObj);
            window.AppProperties.SelectedPerformance = pNew;

            window.AppMethods.ChangePage("#tickets/" + performance + "/" + featureCode + "/" + theater);

        },

        render: function (date, movies) {
            var that = this;

            var theatersHelper = new TheatersHelper();
            if (configurationProvider.chainType() != 'single') {
                theatersHelper.getFavoriteTheaters(function (value) {
                    that.favorite = value.indexOf(that.options.theater.get('id')) >= 0;
                });
            }

            // tdCurrentDate should be a business date, not selected performance's date
            window.tdCurrentDate = date || window.tdSelectedBusinessDate || DtHelper.getCurrentBusinessDate();

            var performancesDays = [];
            var defaultCulture = that.options.theater.get('culture');

            //Current day will be always shown.
            performancesDays.push({ displayDate: dateFormat(that.today.getTime(), 'ddd') + ' ' + that.localize.todayInCalendar, formatedDate: that.today });

            _.each(that.options.theater.get("showDates"), function (showDt) {
                var showDate = DtHelper.parseDate(showDt);

                var ddate = Globalize.format(showDate, 'd', defaultCulture);
                ddate = ddate.substring(0, ddate.length - 5);

                if (showDate > that.today) {
                    performancesDays.push({ displayDate: dateFormat(showDate, 'ddd') + ' (' + ddate + ')', formatedDate: showDate });
                }
            });

            if (window.tdTicketSelected) {
                that.theaterMovies = window.tdTicketSelectedMovies;
                delete window.tdTicketSelected;
                delete window.tdTicketSelectedMovies;
            }

            that.theaterMovies = movies || that.options.movies;

            var newMovieAdded = false;
            //In case the schedule call returned movies that weren't returned with the find theaters call
            _.each(that.theaterMovies.models, function (movie) {
                var theaterMovie = that.options.theater.movies.get(movie.get('id'));

                if (!theaterMovie) {
                    newMovieAdded = true;

                    movie.theaterCulture = that.options.theater.get('culture');
                    movie.theaterId = that.options.theater.get('id');
                    movie.theaterName = that.options.theater.get('name');

                    that.options.theater.movies.add(movie);
                    window.ObjectCollections.MovieCollection.add(movie);
                }
            });

            if (window.appConfig.GroupMovies && newMovieAdded) {
                window.ObjectCollections.MovieCollection.groupedMovies = window.ObjectCollections.MovieCollection.groupMultiTypeMovies();
            }

            var showDistance = (configurationProvider.chainType() != 'single') && window.AppProperties.ShowTheaterDistance;
            var chainType = configurationProvider.chainType() != 'single';
            
            var groupingCustomFlags = configurationProvider.getGroupableCustomFlags();
            var attributeMovieGroups = that.theaterMovies.groupMoviesByAttribute(groupingCustomFlags);

            var templateOptions = {
                logoImage: null,
                logoText: "TEXT LOGO",
                theater: that.options.theater,
                groupingCustomFlags: groupingCustomFlags,
                localize: that.localize,
                day: window.AppProperties.LastSelectedBusinessDateValue,
                showDay: window.tdCurrentDate,
                performancesDays: performancesDays,
                showDistance: showDistance,
                chainType: chainType,
                attributeMovieGroups: attributeMovieGroups,
                favorite: that.favorite,
                homeScreen: that.options.homeScreen,
                hideBackButton: that.options.hideBackButton
            };
            if (window.appConfig.GroupMovies) {
                for (var prop in attributeMovieGroups) {
                    attributeMovieGroups[prop] = window.ObjectCollections.MovieCollection.groupMultiTypeMovies(new MovieCollection(attributeMovieGroups[prop]));
                }
                templateOptions['attributeMovieGroups'] = attributeMovieGroups;
            }
            $(that.el).html(that.template(templateOptions));

            var currentDate = window.tdCurrentDate;
            currentDate.setHours(0, 0, 0, 0);
            $(that.el).find("#performances-calendar-select").val(currentDate.toString()).attr('selected', true).siblings('option').removeAttr('selected');

            if (performancesDays.length > 0 && performancesDays[0].formatedDate.getTime() == currentDate.getTime())
                $(that.el).find('#previousDay').addClass('ui-disabled');
            if (performancesDays.length > 0 && performancesDays[performancesDays.length - 1].formatedDate.getTime() == currentDate.getTime())
                $(that.el).find('#nextDay').addClass('ui-disabled');

            that.likeButton($(that.el));

            if (!configurationProvider.useConssesion(that.options.theater) || performancesDays[0].formatedDate.getTime() != currentDate.getTime()) {
                $(that.el).find('#btnBuyConcessions').hide();
            }

            window.BannerHelper.addBanner();
            if (!that.options.homeScreen) {
                _.defer(function () {
                    if ($(that.el).find('.banner-container').html().length > 0 && !that.options.theater.get('heroImage')) {
                        $(that.el).find('.logo-image').hide();
                    }
                });
            } else {
                window.HomeScreenHelper.loyaltyInfoStatus(function (status) {
                    switch (status) {
                        case 'LoyaltyDisabled':
                            $(that.el).find('.homescreen-loyalty-container').remove();
                            break;
                        case 'LoyaltyInfoPresent':
                            that.$el.on('loyaltyDataReceived', function (event, loyaltyData) {
                                that.loyaltyDataReceived(loyaltyData);
                            });
                            break;
                        case 'SignInLoyalty':
                            $(that.el).find('.homescreen-loyalty-container').html(that.homeScreenLoyaltyDetailsTemplate({
                                signInLoyalty: true,
                                localize: that.localize,
                                loyaltyError: false
                            }));
                            break;
                    }
                });
            }

            if (window.AppProperties.IsIOS && $(that.el).find('.policy').length > 0) {
                UIHelper.iOSNotificationPolicyFix(that);
            }

            return that;
        },

        previousDateClick: function (e) {
            e.preventDefault();

            if (isJqmGhostClick(e)) return;
            window.AppProperties.SelectedPerformance = null;

            var pager = this.calendarPager({ action: 'previous' });

            if (isNaN(pager.date)) {
                window.showAlert(this.localize.alertPastShowtimes);
                return;
            }

            var previousDate = DtHelper.getCurrentBusinessDate();
            var isPreviousBizDay = false;
            var timeDelta = (this.today.getTime() - pager.date.getTime()) / (3600 * 1000);
            if (previousDate.format('yyyymmdd') != pager.date.format('yyyymmdd') && (timeDelta > 0 && timeDelta < 6)) {
                isPreviousBizDay = true;
            }

            if (isNaN(pager.date.getTime())) {
                window.showAlert(this.localize.alertNoShowtimesTheater);
                return;
            }

            if (pager.date.getTime() == pager.selectedDate && isPreviousBizDay == false) {
                window.showAlert(this.localize.alertPastShowtimes);
                return;
            }

            var that = this;
            window.showPageLoadingMessage();

            setTimeout(function () {
                that.options.theater.getTheaterScheduleMobile(Globalize.format(pager.date, 'yyyyMMdd'), function (movieCollection) {
                    that.renderShowTimes(movieCollection, pager.date);
                });
            }, 10);
        },

        nextDateClick: function (e) {
            e.preventDefault();

            if (isJqmGhostClick(e)) return;
            window.AppProperties.SelectedPerformance = null;

            var pager = this.calendarPager({ action: 'next' });

            if (isNaN(pager.date.getTime())) {
                window.showAlert(this.localize.alertNoShowtimesTheater);
                return;
            }

            if (pager.date.getTime() == pager.selectedDate) {
                window.showAlert(this.localize.alertNoMoreShowtimesTheater);
                return;
            }

            var that = this;
            window.showPageLoadingMessage();

            setTimeout(function () {
                that.options.theater.getTheaterScheduleMobile(Globalize.format(pager.date, 'yyyyMMdd'), function (movieCollection) {
                    that.renderShowTimes(movieCollection, pager.date);
                });
            }, 10);
        },

        currentDay: function (e) {
            if (isJqmGhostClick(e)) return;
        },

        selectDate: function (e) {
            e.preventDefault();
            if (isChangePageFired(e) || isJqmGhostClick(e)) return;

            var that = this;
            window.showPageLoadingMessage();

            window.AppProperties.SelectedPerformance = null;
            $("#NCR-calendar").focus();

            var pager = this.calendarPager({ action: 'select', value: $(e.currentTarget).val() });
            setTimeout(function () {
                that.options.theater.getTheaterScheduleMobile(Globalize.format(pager.date, 'yyyyMMdd'), function (movieCollection) {
                    that.renderShowTimes(movieCollection, pager.date);
                });
            }, 300);
        },

        renderShowTimes: function (movieCollection, pagerDate) {
            var that = this;
            window.tdSelectedBusinessDate = pagerDate;
            that.render(pagerDate, movieCollection);

            if (!that.options.theater.get('heroImage')) {
                $(that.el).find('.logo-image').hide();
            }

            $(that.el).enhanceWithin();
            window.hidePageLoadingMessage();
            window.HomeScreenHelper.fetchLoyaltyInfo();
        },

        calendarPager: function (option) {
            var that = this;

            var selectedDateTime = new Date(tdCurrentDate).getTime();
            var showDateTimes = [];

            var previousDate = DtHelper.getCurrentBusinessDate();

            var isSameDay = tdCurrentDate.format('yyyymmdd') == that.today.format('yyyymmdd');
            var allowPreviousDay = previousDate.format('yyyymmdd') != that.today.format('yyyymmdd');

            //Add currenty day
            if (option.action == 'previous') {
                showDateTimes.push(that.today);
            }

            if (option.action == 'select') {
                return { date: new Date(option.value), selectedDate: selectedDateTime };
            }

            _.each(this.options.theater.get("showDates"), function (showDt) {
                var showDateTime = DtHelper.parseDate(showDt).getTime();
                if (showDateTime >= that.today.getTime()) {
                    showDateTimes.push(showDateTime);
                }
            });

            var selectedDateIndex = jQuery.inArray(selectedDateTime, showDateTimes);
            var index = option.action == 'previous' ?
                            selectedDateIndex > 0 ? selectedDateIndex - 1 : selectedDateIndex :
                            selectedDateIndex < showDateTimes.length - 1 ?
                            selectedDateIndex + 1 : selectedDateIndex;

            var date = new Date(showDateTimes[index]);

            if (isSameDay && allowPreviousDay && option.action == "previous") {
                date = previousDate;
                showDateTimes.push(previousDate);
            }

            return { date: date, selectedDate: selectedDateTime };
        },

        addTheaterAsFavorite: function (e) {
            e.preventDefault();
            var target = $(e.currentTarget);
            var id = target.attr('data-id');
            if (!id) return;

            target.toggleClass('yellow');
            var theatersHelper = new TheatersHelper();
            theatersHelper.markTheaterAsFavorite(id, target.hasClass('yellow'));

            //update Passbook
            if (window.AppProperties.PassbookAvailable.loyaltyPass || window.AppProperties.PassbookAvailable.dualCardPass) {
                require(['models/loyaltyAccountModel', 'sharedhelpers/storageProvider', 'helpers/passHelper'], function (loyaltyAccountModel) {

                    var storage = new StorageProvider();
                    storage.read('LoyaltyCard', function (savedCard) {
                        if (savedCard) {
                            var passHelper = new PassHelper();
                            var loyaltyCard = new loyaltyAccountModel(savedCard);
                            passHelper.getLoyaltyCardPassURL(loyaltyCard, true);
                        }
                    });
                });
            }
            if (window.AppProperties.PassbookAvailable.virtualGiftCardPass) {
                require(['helpers/passHelper', 'sharedhelpers/storageProvider'], function () {
                    var storage = new StorageProvider();
                    var passHelper = new PassHelper();

                    storage.read('PaymentMethod', function (paymentMethods) {
                        _.each(paymentMethods, function (paymentMethod) {
                            if (paymentMethod.Type == 'VirtualGiftCard') {
                                passHelper.getVirtualGiftCardPassURL(paymentMethod, true);
                            }
                        });
                    });
                    return false;
                });
            }
        }
    });
    return view;
});