define([
  'text!templates/TheatersView.html',
  'text!templates/HomeScreenLoyaltyDetailsView.html',
  'helpers/geolocationHelper',
  'sharedhelpers/configurationProvider',
  'sharedhelpers/cachingProvider',
  'helpers/uiHelper',
  'helpers/theatersHelper',
  'sharedhelpers/storageProvider',
  'js/libs/infobubble',
  'idangerousSwiper'
], function (template, homeScreenLoyaltyDetailsTemplate, geolocationHelper, configurationProvider) {
    var view = Backbone.View.extend({
        template: _.template(template),
        homeScreenLoyaltyDetailsTemplate: _.template(homeScreenLoyaltyDetailsTemplate),
        localize: window.ObjectCollections.Localization.result,

        initialize: function () {
            this.searchUnit = window.AppProperties.SearchUnit;
            this.resizeMap = true;
            this.favorites = this.options.favorites;
            this.isAndroid4x = window.AppProperties.IsAndroid4x;
        },

        render: function () {
            var that = this;
            that.radius = configurationProvider.searchRadius();

            that.searchRadius = window.AppProperties.searchRadius ? window.AppProperties.searchRadius : window.ObjectCollections.TheaterCollection.distance;

            var selectedTheaterViewType = window.ObjectCollections.TheaterCollection.theaterViewType;
            if (selectedTheaterViewType !== undefined) {
                that.options.theaterViewType = selectedTheaterViewType;
            }

            that.showLocationInfo = that.options.theaterViewType != 'favorites' && configurationProvider.chainType() != 'small';

            var isLocationAvailable = that.showLocationInfo && window.AppProperties.ShowTheaterDistance;
            var searchValue = window.AppProperties.CityOrZip || that.localize.yourLocation;

            var templateOptions = {
                collection: that.options.collection,
                sort: that.options.sort,
                localize: that.localize,
                showLocationInfo: that.showLocationInfo,
                isLocationAvailable: isLocationAvailable,
                searchValue: searchValue,
                searchRadius: that.searchRadius,
                searchUnit: that.searchUnit,
                favorites: that.favorites,
                viewType: that.options.theaterViewType,
                hideBackButton: that.options.hideBackButton,
                homeScreen: that.options.homeScreen
            };

            if (window.appConfig.GroupMovies) {
                var moviesToShow = new MovieCollection();

                _.each(window.ObjectCollections.MovieCollection.groupedMovies, function (group) {
                    var mov = _.filter(group, function (m) {
                        return m.get('image').medium != undefined;
                    });

                    mov = _.uniq(mov, false, function (m) {
                        return m.theaterId;
                    });

                    moviesToShow.add(mov);
                });

                templateOptions['movies'] = moviesToShow;
            }

            $(that.el).html(that.template(templateOptions));
            if (that.options.theaterViewType !== undefined) {
                that.toggleMap();
            }

            setTimeout(function () {
                $(that.el).find('.movies-carousel').each(function () {
                    $(this).swiper({
                        scrollContainer: true,
                        freeModeFluid: true,
                        slidesPerView: 'auto'
                    });
                });
            }, 5);

            window.BannerHelper.addBanner();

            if (that.options.homeScreen) {
                window.HomeScreenHelper.loyaltyInfoStatus(function (status) {
                    switch (status) {
                        case 'LoyaltyDisabled':
                            $(that.el).find('.homescreen-loyalty-container').remove();
                            break;
                        case 'LoyaltyInfoPresent':
                            that.$el.on('loyaltyDataReceived', function (event, loyaltyData) {
                                that.loyaltyDataReceived(loyaltyData);
                            });
                            break;
                        case 'SignInLoyalty':
                            $(that.el).find('.homescreen-loyalty-container').html(that.homeScreenLoyaltyDetailsTemplate({
                                signInLoyalty: true,
                                localize: that.localize,
                                loyaltyError: false
                            }));
                            break;
                    }
                });
            }
        },

        loyaltyDataReceived: function (loyaltyData) {
            $(this.el).find('.homescreen-loyalty-container').html(this.homeScreenLoyaltyDetailsTemplate({
                loyalty: loyaltyData.data,
                signInLoyalty: false,
                loyaltyError: loyaltyData.error,
                localize: this.localize
            }));
        },

        events: {
            "pageshow": "pageShow",
            "pagehide": "pageHide",
            "tap #locationSettings": "navigateLocationSettings",
            "tap li[for='tab-value']": "onTap",
            "tap .theater-link": "showTheaterDetails",
            "tap .theater-list-movie": "movieThumbClicked",
            "tap .favicon": "addTheaterAsFavorite",
            'tap .homescreen-loyalty-container': 'handleLoyaltyTap'
        },

        handleLoyaltyTap: function (e) {
            e.preventDefault();
            if (isJqmGhostClick(e) || isChangePageFired(e)) return;

            window.showPageLoadingMessage();
            var loyaltyContainer = $(e.currentTarget);

            if (loyaltyContainer.find(".giftCardBalanceLoader").length > 0 || !configurationProvider.useLoyalty()) {
                return;
            }
            window.AppMethods.ChangePage("loyaltycard");
        },

        reset: function () {
            this.render();

            $(this.el).enhanceWithin();
        },

        pageShow: function () {
            if (this.options.collection.length < 1 && this.options.theaterViewType != 'favorites') {
                var msg = $.sprintf(this.localize.noTheatersWithinRadius, this.searchRadius, this.searchUnit);
                window.showAlert(msg);
            }
        },

        pageHide: function () {
            $(this.el).find('div#map_canvas').empty();
        },

        navigateLocationSettings: function (e) {
            e.preventDefault();
            if (isJqmGhostClick(e) || isChangePageFired(e)) return;

            window.showPageLoadingMessage();

            window.AppMethods.ChangePage('#locationSettings', false, true);
        },

        showTheaterDetails: function (e) {
            e.preventDefault();

            var theaterId = $(e.currentTarget).attr('data-theaterid');
            window.AppMethods.ChangePage("#theater/" + theaterId);
        },

        showMap: function () {
            var that = this;

            var currentLocation = null;
            if (AppProperties.CurrentPosition) {
                currentLocation = new window.google.maps.LatLng(AppProperties.CurrentPosition.Latitude, AppProperties.CurrentPosition.Longitude);
            }

            if (currentLocation && currentLocation.lat() == 0 && currentLocation.lng() == 0)
                currentLocation = null;

            var markers = [];

            var mapOptions = {
                center: currentLocation,
                mapTypeControl: true,
                mapTypeId: window.google.maps.MapTypeId.ROADMAP,
                streetViewControl: false,
                disableDefaultUI: true,
                zoomControl: true,
                zoomControlOptions: {
                    'position': window.google.maps.ControlPosition.TOP_LEFT,
                    'style': window.google.maps.ZoomControlStyle.LARGE
                }
            };

            var canvas = $(that.el)[0];
            var map = new window.google.maps.Map($(canvas).find('div#map_canvas')[0], mapOptions);

            var bounds = new window.google.maps.LatLngBounds();

            if (currentLocation) {
                bounds.extend(currentLocation);
            }

            var theaterLocations = this.options.collection.getAllTheaterLocations();
            _.each(theaterLocations, function (location) {
                var locContent = '<div id="content" style="float: right; padding-top: 5%; "><a href="#theater/' + location.theaterId + '" class="info-balloon-img"></a></div><div class="info-balloon-name" style="float: none;">' + location.name + '</div>';
                var latlng = new window.google.maps.LatLng(location.coords.split(',')[0], location.coords.split(',')[1]);
                _addMarker(latlng, location.name, null, locContent);
                bounds.extend(latlng);
            });

            if (currentLocation) {
                var content = '<div id="content" style="color:#fff; fpnt-weight:bold">You are here.</div>';
                _addMarker(currentLocation, this.localize.youAreHere, '_green.png', content);
            }

            function _addMarker(latlng, myTitle, icon, locContent) {
                var marker = new window.google.maps.Marker({
                    position: latlng,
                    map: map,
                    title: myTitle,
                    icon: "http://maps.google.com/mapfiles/marker" + (icon == null ? String.fromCharCode(markers.length + 65) + ".png" : icon)
                });

                _bindInfoBubble(marker, map, locContent);
                markers.push(marker);
            }

            var infoBubbleClass = (function () {
                var instance = null;
                return {
                    getInstance: function () {
                        if (instance === null) {
                            instance = new InfoBubble
                            ({
                                borderWidth: 1,
                                borderColor: 'rgba(67,69,64,0.9)',
                                borderRadius: 5,
                                backgroundColor: 'rgba(67,69,64,0.9)',
                                minHeight: 'auto',
                                maxHeight: 'auto',
                                hideCloseButton: true
                            });
                        }
                        return instance;
                    }
                };
            })();

            function _bindInfoBubble(marker, map, locContent) {
                window.google.maps.event.addListener(marker, "click", function () {
                    infoBubbleClass.getInstance().setContent(locContent);
                    infoBubbleClass.getInstance().open(map, marker);
                });
            }

            function _radiusToZoom(miles, radius) {
                if (!miles) {
                    //Converting kilometers to miles
                    radius = radius * 0.621;
                }

                return Math.round(14 - Math.log(radius) / Math.LN2) + 2;
            }

            window.google.maps.event.addDomListener(infoBubbleClass.getInstance().bubble_, 'click', function (e) {
                if (e.target.className == 'info-balloon-img')
                    return;

                infoBubbleClass.getInstance().close();
            });

            if (theaterLocations.length === 0 && currentLocation) {
                bounds.extend(currentLocation);
            }

            if ((window.AppProperties.ShowTheaterDistance && that.showLocationInfo) || theaterLocations.length === 1) {
                if (currentLocation) {
                    map.setCenter(markers[markers.length - 1].getPosition());
                } else {
                    map.setCenter(bounds.getCenter());
                }
                map.setZoom(_radiusToZoom(that.searchUnit == that.localize.miles, that.radius));
            } else {
                map.fitBounds(bounds);
            }
        },

        getFavoriteTheaters: function (callback) {
            var that = this;

            var theaterIds = _.map(window.ObjectCollections.TheaterCollection.models, function (t) {
                return t.id;
            });

            var extraFavorites = _.difference(that.favorites, theaterIds);
            if (extraFavorites.length == 0) {
                callback();
                return;
            }

            window.showPageLoadingMessage();

            var addToModel = function (arr) {
                _.each(arr.getMovies().models, function (a) {
                    window.ObjectCollections.MovieCollection.add(a);
                });
                _.each(arr.models, function (a) {
                    window.ObjectCollections.TheaterCollection.add(a);

                    if (window.appConfig.GroupMovies) {
                        window.ObjectCollections.MovieCollection.groupedMovies = window.ObjectCollections.MovieCollection.groupMultiTypeMovies();
                    }

                    that.options.collection = window.ObjectCollections.TheaterCollection;
                });

                window.hidePageLoadingMessage();
                callback();
            };

            var added = new TheaterCollection();
            _.each(extraFavorites, function (e) {
                new TheaterModel({ id: e }).getTheater(function (res) {
                    added.add(res);
                    if (added.length == extraFavorites.length) {
                        addToModel(added);
                    }
                }, function () {
                    addToModel(added);
                });
            });
        },

        movieThumbClicked: function (e) {
            e.stopPropagation();

            var theaterId = $(e.currentTarget).attr('data-theater');
            var featureCode = $(e.currentTarget).attr('data-featurecode');

            this.updateTheaterCollection(AppProperties.CurrentPosition.Latitude, AppProperties.CurrentPosition.Longitude, theaterId, featureCode);
            return false;
        },

        updateTheaterCollection: function (latitude, longitude, theaterId, featureCode) {
            require(['collections/theaterCollection'],
              function (theaterCollection) {
                  var theaters = new theaterCollection();
                  theaters.latitude = latitude;
                  theaters.longitude = longitude;

                  // Prevent date collisions
                  window.mdSelectedBusinessDate = undefined;

                  // Always true
                  if (window.ObjectCollections.TheaterCollection.models != undefined) {
                      var theatersHelper = new TheatersHelper();
                      window.AppProperties.SelectedTheater = null;
                      window.AppProperties.SelectedTheater = window.ObjectCollections.TheaterCollection.get(theaterId);
                      window.AppProperties.SelectedTheaterByZip = window.AppProperties.SelectedTheater;
                      theatersHelper.saveTheaterDefaultId(theaterId);
                  }

                  window.AppMethods.ChangePage('#movie/' + featureCode + '/' + theaterId);
              });
        },

        onTap: function (e) {
            var that = this;
            that.options.theaterViewType = e.currentTarget.attributes['value'].value;
            window.ObjectCollections.TheaterCollection.theaterViewType = e.currentTarget.attributes['value'].value;

            if (that.options.theaterViewType == 'favorites') {
                that.getFavoriteTheaters(function () {
                    that.reset();
                });
            } else {
                that.reset();
            }
        },

        toggleMap: function () {
            if (ConfigHelper.isDeviceOnline() == false) {
                window.showAlert(this.localize.errorUnableConnectToInternet);
                return;
            }

            var that = this;

            if (that.options.theaterViewType == 'map') {
                $(that.el).find('#theaterViewListType').hide();
                $(that.el).find('#theaterViewMapType').show();

                setTimeout(function () {
                    require(['async!http://maps.googleapis.com/maps/api/js?sensor=false'], function () {
                        if (that.resizeMap) {
                            that.setHeightOfMap();
                        }

                        that.showMap();
                        that.resizeMap = true;
                    });
                }, 0);
            } else {
                $(that.el).find('#theaterViewMapType').hide();
                $(that.el).find('#theaterViewListType').show();
            }
        },

        setHeightOfMap: function () {
            var mapHeight = document.documentElement.clientHeight - $('#locationDetails').height() - 52;

            $('div#map_canvas').css('height', mapHeight);
        },

        addTheaterAsFavorite: function (e) {
            e.preventDefault();
            var target = $(e.currentTarget);

            var id = target.attr('data-id');
            if (!id) return;

            target.toggleClass('yellow');
            var index = this.favorites.indexOf(id);

            if (target.hasClass('yellow')) {
                if (index < 0) {
                    this.favorites.push(id);
                }
            } else {
                if (index >= 0) {
                    this.favorites.splice(index, 1);
                }
            }

            var theatersHelper = new TheatersHelper();
            theatersHelper.markTheaterAsFavorite(id, target.hasClass('yellow'));

            //update Passbook
            if (window.AppProperties.PassbookAvailable.loyaltyPass || window.AppProperties.PassbookAvailable.dualCardPass) {
                require(['models/loyaltyAccountModel', 'sharedhelpers/storageProvider', 'helpers/passHelper'], function (loyaltyAccountModel) {
                    var storage = new StorageProvider();
                    storage.read('LoyaltyCard', function (savedCard) {
                        if (savedCard) {
                            var passHelper = new PassHelper();
                            var loyaltyCard = new loyaltyAccountModel(savedCard);
                            passHelper.getLoyaltyCardPassURL(loyaltyCard, true);
                        }
                    });
                });
            }

            if (window.AppProperties.PassbookAvailable.virtualGiftCardPass) {
                require(['helpers/passHelper', 'sharedhelpers/storageProvider'], function () {
                    var storage = new StorageProvider();
                    var passHelper = new PassHelper();

                    storage.read('PaymentMethod', function (paymentMethods) {
                        _.each(paymentMethods, function (paymentMethod) {
                            if (paymentMethod.Type == 'VirtualGiftCard') {
                                passHelper.getVirtualGiftCardPassURL(paymentMethod, true);
                            }
                        });
                    });
                    return false;
                });
            }
        }
    });
    return view;
});