define([
  'text!templates/TicketsView.html',
  'models/orderModel',
  'collections/seatCollection',
  'collections/performanceSeatInfoCollection',
  'sharedhelpers/configurationProvider',
  'helpers/saleHelper',
  'libs/jquery.validate.min',
  'sharedhelpers/htmlHelper',
  'sharedhelpers/storageProvider'
], function (template, mainOrderModel, seatCollection, performanceSeatInfoCollection, configurationProvider) {
    var view = Backbone.View.extend({
        localize: window.ObjectCollections.Localization.result,

        initialize: function () {
            window.tempHistoryCount = 0;
            this.availableCompanionTickets = 0;
            this.seatsError = '';
        },

        render: function () {
            var that = this;

            var savedOrderModel = window.ObjectModels.OrderModel;

            var orderModel = new mainOrderModel();
            orderModel.set('tickets', that.options.tickets);
            orderModel.set('performance', that.options.performance);
            orderModel.set('movie', that.options.movie);
            orderModel.set('theater', that.options.theater);

            if (savedOrderModel && (that.options.back || savedOrderModel.get('saleId'))) {
                for (var i = 0; i < savedOrderModel.get('tickets').models.length; i++) {
                    var item = savedOrderModel.get('tickets').models[i];

                    var t = orderModel.get('tickets').where({
                        'name': item.get('name')
                    })[0];

                    if (t && !t.get('isOnHoldTicketType')) {
                        t.set('quantity', t.get('quantity') + item.get('quantity'));
                    }

                    if (t && item.get('subscriptionCardNumbers')) {
                        t.set('subscriptionCardNumbers', item.get('subscriptionCardNumbers'));
                    }

                    if (t && item.get('ctmCards')) {
                        t.set('ctmCards', item.get('ctmCards'));
                    }
                };

                orderModel.set('invalidSubscriptions', savedOrderModel.get('invalidSubscriptions'));
                orderModel.set('invalidCtmCards', savedOrderModel.get('invalidCtmCards'));
                orderModel.set('loyaltyCardNumber', savedOrderModel.get('loyaltyCardNumber'));
                orderModel.set('totalPrice', savedOrderModel.get('tickets').getTicketTotal());

                savedOrderModel.cancelSale(function () {
                    window.ObjectModels.OrderModel = orderModel;
                });
            } else {
                window.ObjectModels.OrderModel = orderModel;
            }

            var performanceToday = '';
            var showDate = new Date(orderModel.get('performance').get('showTime'));
            var today = new Date();

            var enableHold = configurationProvider.useHoldTickets(that.options.theater);
            if (enableHold) {
                var holdTime = (configurationProvider.holdTimeInMinutesBeforeShowTime(that.options.theater) + window.appConfig.TimeoutInMinutes * 2) * 60000;
                enableHold = showDate.getTime() - holdTime >= today.getTime();
            }

            showDate.setHours(0, 0, 0, 0);
            today.setHours(0, 0, 0, 0);

            if (showDate.getTime() == today.getTime()) {
                performanceToday = that.localize.todayInCalendar;
            }

            orderModel.set('performanceToday', performanceToday);

            $(that.el).html(_.template(template, {
                tickets: orderModel.get('tickets'),
                movie: orderModel.get('movie'),
                performance: orderModel.get('performance'),
                theater: orderModel.get('theater'),
                loyalty: that.options.loyaltyHolder,
                loyaltyNumber: orderModel.get('loyaltyCardNumber'),
                total: orderModel.get('tickets').getTicketTotal(),
                performanceToday: orderModel.get('performanceToday'),
                enableHold: enableHold,
                onHoldConfirmationNumber: that.options.onHoldConfirmationNumber,
                warning: orderModel.get('theater').get('warningShort'),
                localize: that.localize,
                configurationProvider: configurationProvider
            }));

            if (orderModel.get('tickets').where({ 'isReservedSeating': false, 'isOnHoldTicketType': false }).length > 0 && orderModel.get('tickets').getReservedTicketQuantity() < 1) {
                $(that.el).find('#selectSeatsButton').hide();
                $(that.el).find('#continueToCheckout').show();
            } else {
                $(that.el).find('#selectSeatsButton').show();
                $(that.el).find('#continueToCheckout').hide();
            }

            orderModel.get('tickets').bind("change:quantity", that.ticketQuantityChanged, { that: that, ticket: orderModel.get('tickets') });

            if (orderModel.get('performance').get('isReservedSeating')) {
                var seats = new seatCollection();
                seats.getSeats(orderModel.get('theater').get('id'), orderModel.get('performance').get('number'), function (result) {
                    if (result == undefined) {
                        that.seatsError = that.localize.errorLoadingSeats;
                        window.showAlert(that.seatsError);
                        return;
                    }

                    window.performanceLayout = result;
                });
            }

            var retView = configurationProvider.arangeFlow(that);

            $(that.el).find('#popupDialog').on({
                popupafterclose: function () {
                    $(that.el).find('#popupDialog').empty();
                },
                popupbeforeposition: function () {
                    $('.ui-popup-screen').off();
                }
            });

            return retView;
        },

        events:
		{
		    "tap .tickets-add-ticket": "addTicket",
		    "tap .tickets-remove-ticket": "removeTicket",
		    "tap #selectSeatsButton": "continueToSelectSeats",
		    "tap #continueToCheckout": "continueToCheckout",
		    "pageshow": "pageShow",
		    "pagebeforeshow": "pageBeforeShow",
		    "change #holdSeatsButton": "toggleHoldSeats",
		    "tap .add-loyalty-card": "addLoyaltyCard",
		    "tap .seat-fees": "seatFeesInfo",
		    "tap .policy": "warningInfo",
		    'tap #facebook-share-show': 'facebookSharePanel',
		    "tap .movie-poster-ncr": "showLargePoster"
		},
        showLargePoster: function (e) {
            e.preventDefault();
            var obj = $(this.el);
            var imgSrc = this.options.movie.get('image').large;
            if (!imgSrc) return;

            UIHelper.showLargePoster(obj, imgSrc);
        },
        addLoyaltyCard: function (e) {
            e.preventDefault();
            var that = this;

            require(['views/loyaltyCardDialogView'], function (loyaltyCardDialogView) {
                var dialog = new loyaltyCardDialogView({
                    callback: function (loyaltyNumber) {
                        if (loyaltyNumber) {
                            window.ObjectModels.OrderModel.set('loyaltyCardNumber', loyaltyNumber);
                            loyaltyNumber = loyaltyNumber.substring(loyaltyNumber.length - 4, loyaltyNumber);

                            $('.add-loyalty-card').hide();
                            $('#loyaltyNumber').html(that.localize.loyaltyCard + ' ...' + loyaltyNumber).show();
                        }
                        $('#popupDialog').popup('close');
                    }
                });

                $('#popupDialog').html(dialog.render());
                $('#popupDialog').popup('open');
                $('#popupDialog').enhanceWithin();
            });

            return false;
        },

        toggleHoldSeats: function (e) {
            var checked = $(e.currentTarget).val() == 'yes';

            var seatButton = $(this.el).find('#selectSeatsButton');
            var checkoutButton = $(this.el).find('#continueToCheckout');

            if (checked) {
                this.continueTo = 'holdSeats';

                seatButton.html(this.localize.selectFriends);
                checkoutButton.html(this.localize.selectFriends);
            } else {
                seatButton.html(this.localize.selectSeats);
                checkoutButton.html(this.localize.continueToCheckout);

                this.continueTo = 'selectSeats';
            }

            var orderModel = window.ObjectModels.OrderModel;

            orderModel.get('tickets').holdSeats = checked;
        },

        pageBeforeShow: function () {
            this.handleCTMTicketState();
        },

        pageShow: function () {
            var that = this;

            var showHoldButton = function () {
                var orderModel = window.ObjectModels.OrderModel;

                var r = orderModel.get('tickets').where({
                    'isOnHoldTicketType': true
                });

                if (r.length) {
                    if (orderModel.get('tickets').holdSeats) {
                        var button = $(that.el).find('#selectSeatsButton');

                        var holdSeatsButton = $(that.el).find('#holdSeatsButton');

                        holdSeatsButton.val(that.localize.yes);
                        holdSeatsButton.slider('refresh');

                        button.attr('href', '#holdSeats');
                        button.html(that.localize.selectFriends);
                    }
                }
            };

            var updateOrderModel = function () {
                if (that.options.onHoldConfirmationNumber && that.options.onHoldConfirmationNumber != 'share' && window.AppProperties.FBValid) {
                    window.ObjectModels.OrderModel.set('onHoldConfirmationNumber', that.options.onHoldConfirmationNumber);

                    $(that.el).find('.holdSeats').hide();
                } else {
                    showHoldButton();
                }

                window.hidePageLoadingMessage();
            };

            if (that.options.facebookId && window.AppProperties.FBValid === null) {
                window.showPageLoadingMessage();

                require(['helpers/deepLinkingHelper'], function () {
                    var deepLinking = new DeepLinkingHelper();
                    var fbValidationError = window.ObjectCollections.Localization.result.holdPickupFbValidationError;
                    var notLoggedOnFbError = window.ObjectCollections.Localization.result.holdPickupNotLoggedOnFbError;

                    deepLinking.validateFacebookAccount(that.options.facebookId, fbValidationError, notLoggedOnFbError, updateOrderModel);
                });
            } else {
                updateOrderModel();
            }

            if (window.ObjectModels.OrderModel.get('invalidSubscriptions') || window.ObjectModels.OrderModel.get('invalidCtmCards')) {

                var subscriptionTickets = _.filter(window.ObjectModels.OrderModel.get('tickets').models, function (ticket) {
                    return ticket.get('ticketValidationRuleId') != 99 && ticket.get('quantity') > 0 && (ticket.get('discountFlag') || ticket.get('validate'));
                });

                var ctmCards = _.filter(window.ObjectModels.OrderModel.get('tickets').models, function (ticket) {
                    return ticket.get('quantity') > 0 && ticket.get('pricedTicketRequired');
                });

                if (subscriptionTickets || ctmCards) {
                    $('#selectSeatsButton:visible, #continueToCheckout:visible').trigger('tap');
                }
            }
            that.handleCTMTicketState();
        },

        handleSubscriptionTickets: function (callback) {
            var that = this;
            var orderModel = window.ObjectModels.OrderModel;
            var subscriptionTickets = _.filter(orderModel.get('tickets').models, function (ticket) {
                return ticket.get('ticketValidationRuleId') != 99 && ticket.get('quantity') > 0 && (ticket.get('discountFlag') || ticket.get('validate'));
            });

            if (subscriptionTickets.length == 0) {
                that.handleCTMTickets(callback);
                return;
            }

            require(['views/infoDialogView'], function (infoView) {
                var form = $('<form id="frmSubscription" class="validate">');
                form.append('<span class="bodyText">' + that.localize.enterSubscriptionCards + '</span>');

                _.each(subscriptionTickets, function (ticket) {
                    //When going back to select tickets screen, pre-fill the popup with subscription card numbers
                    var subscriptionCards = ticket.get('subscriptionCardNumbers') || [];

                    for (var i = 0; i < ticket.get('quantity') ; i++) {
                        var cardNumber = subscriptionCards[i] ? subscriptionCards[i].number : '';
                        var input = $('<input type="tel" value="' + cardNumber + '" class="required input-dialog subscriptioncard cardNumber">');

                        input.attr({
                            'data-ticketid': ticket.get('id'),
                            'placeholder': ticket.get('name'),
                            'name': 'subscriptioncard' + i
                        });

                        form.append(input);

                        //Add error messages returned from the lock ticket response below the appropriate input field
                        _.each(orderModel.get('invalidSubscriptions'), function (invalidSubscription) {
                            if (invalidSubscription.cardNumber == cardNumber) {
                                input.addClass('error');

                                var error = $('<label for="subscriptioncard' + i + '"  generated="false" class="error invalid">' + invalidSubscription.message + '</label>');
                                form.append(error);
                            }
                        });
                    }
                });

                var emailValue = '';
                var storage = new StorageProvider();

                storage.read('LoyaltyCard', function (card) {
                    if (card && card.emailAddress) {
                        emailValue = card.emailAddress;
                    }

                    var validateInput = $('<span class="bodyText">' + that.localize.email + '</span><br/>' +
                                          '<input type="email" value="' + emailValue + '" class="email keyValue" required  maxlength="50" placeholder="' + that.localize.email + '"/>');

                    form.append(validateInput);
                });

                //Remove invalid subscription cards return from the lock ticket response
                orderModel.unset('invalidSubscriptions');

                var saveSubscriptionCards = function () {
                    //when going back to the select tickets,  empty the subscription numbers and start saving cards from the begining
                    _.each(subscriptionTickets, function (ticket) {
                        ticket.unset('subscriptionCardNumbers');
                    });

                    var frm = $('#frmSubscription');
                    if (frm.valid()) {
                        var cards = frm.find('.cardNumber');

                        _.each(cards, function (card) {
                            var id = $(card).attr('data-ticketid');
                            var cardNumber = $(card).val();
                            var keyValue = frm.find('.keyValue').val();

                            orderModel.get('tickets').get(id).addSubscriptionCard(cardNumber, keyValue);
                        });

                        $('#popupDialog').popup("close");
                        that.handleCTMTickets(callback);
                    }
                };

                var dialog = new infoView({
                    title: that.localize.subscriptionCards,
                    content: form[0].outerHTML,
                    confirmButton: {
                        text: that.localize.ok,
                        callback: saveSubscriptionCards
                    },
                    cancelButton: {
                        text: that.localize.cancel,
                        callback: function () {
                            $('#popupDialog').popup("close");
                        }
                    }
                });

                var popup = $("#popupDialog");
                popup.html(dialog.render());
                popup.popup("open");
                popup.enhanceWithin();

                $.validator.addMethod('subscriptioncard', function (value) {
                    var regExp = new RegExp('^[0-9]+$');
                    return regExp.test(value);
                }, that.localize.errorEnterValidSubscription);

                $('#frmSubscription').validate({
                    onkeyup: false,
                    focusCleanup: true
                });
            });
        },

        handleCTMTickets: function (callback) {
            var that = this;
            var labelNumber = 0;
            var orderModel = window.ObjectModels.OrderModel;

            var ctmTickets = _.filter(orderModel.get('tickets').models, function (ticket) {
                return ticket.get('pricedTicketRequired') && ticket.get('quantity') > 0;
            });

            if (ctmTickets.length == 0) {
                callback();
                return;
            }

            require(['views/infoDialogView'], function (infoView) {
                var form = $('<form id="frmCtmTickets" class="validate">');
                form.append('<span class="bodyText">' + that.localize.enterCTMCodes + '</span>');


                _.each(ctmTickets, function (ticket) {
                    //When going back to select tickets screen, pre-fill the popup with code numbers
                    var ctmCards = ticket.get('ctmCards') || [];

                    for (var i = 0; i < ticket.get('quantity') ; i++) {

                        var codeNumber = ctmCards[i] || '';
                        var input = $('<input type="tel" value="' + codeNumber + '" class="required input-dialog code-number">');

                        input.attr({
                            'data-ticketid': ticket.get('id'),
                            'placeholder': ticket.get('name'),
                            'name': 'code-number' + labelNumber++

                        });

                        form.append(input);

                        //Add error messages returned from the lock ticket response below the appropriate input field
                        _.each(orderModel.get('invalidCtmCards'), function (invalidCode) {
                            if (invalidCode.cardNumber == codeNumber) {
                                if (invalidCode.message != '') {
                                    input.addClass('error');

                                    var error = $('<label for="code-number' + i + '"  generated="false" class="error invalid">' + invalidCode.message + '</label>');
                                    form.append(error);
                                }
                            }
                        });
                    }
                });

                //Remove invalid subscription cards return from the lock ticket response
                orderModel.unset('invalidCtmCards');

                var saveCTMCodes = function () {
                    var frm = $('#frmCtmTickets');
                    frm.find('input').removeClass('error');

                    if (frm.valid()) {
                        //when going back to the select tickets,  empty the code numbers and start saving cards from the begining
                        _.each(ctmTickets, function (ticket) {
                            ticket.unset('ctmCards');
                        });

                        var cards = frm.find('.code-number');

                        _.each(cards, function (card) {
                            var id = $(card).attr('data-ticketid');
                            var codeNumber = $(card).val();

                            orderModel.get('tickets').get(id).addCTMCode(codeNumber);
                        });

                        $('#popupDialog').popup("close");
                        callback();
                    }
                };

                var dialog = new infoView({
                    title: that.localize.ctmCodes,
                    content: form[0].outerHTML,
                    confirmButton: {
                        text: that.localize.ok,
                        callback: saveCTMCodes
                    },
                    cancelButton: {
                        text: that.localize.cancel,
                        callback: function () {
                            $('#popupDialog').popup("close");
                        }
                    }
                });

                var popup = $("#popupDialog");
                popup.html(dialog.render());
                popup.popup("open");
                popup.enhanceWithin();
            });
        },

        continueToSelectSeats: function (e) {
            var that = this;
            e.preventDefault();
            if (isJqmGhostClick(e)) return false;

            if (ConfigHelper.isDeviceOnline() == false) {
                window.showAlert(this.localize.errorUnableConnectToInternet);
                return false;
            }

            if (this.seatsError) {
                window.showAlert(this.seatsError);
                return false;
            }

            var orderModel = window.ObjectModels.OrderModel;

            if (orderModel.get('tickets').getTicketTotal() == 0) {
                window.showAlert(this.localize.errorAddTicketToContinue);
            } else {
                var continueFlow = function () {
                    window.ObjectModels.TempOrder = { concessions: null };

                    if (orderModel.get('tickets').holdSeats) {
                        orderModel.set('holdTickets', orderModel.get('tickets').getOnHoldTickets());
                        window.AppMethods.ChangePage('#holdSeats/' + orderModel.get('theater').get('id') + '/' + orderModel.get('performance').get('number'));
                    } else {
                        var navigateToPage = 'selectSeats/' + orderModel.get('theater').get('id') + '/' + orderModel.get('performance').get('number');
                        that.lockTickets(navigateToPage);
                    }
                };

                this.handleSubscriptionTickets(continueFlow);
            }

            return false;
        },

        continueToCheckout: function (e) {
            var that = this;
            e.preventDefault();
            if (isJqmGhostClick(e)) return false;

            if (ConfigHelper.isDeviceOnline() == false) {
                window.showAlert(this.localize.errorUnableConnectToInternet);
                return false;
            }

            var orderModel = window.ObjectModels.OrderModel;

            if (orderModel.get('tickets').getTicketTotal() == 0) {
                window.showAlert(this.localize.errorAddTicketToContinue);
            } else {
                var continueFlow = function () {
                    var continueTo = 'checkoutSummary';
                    window.ObjectModels.TempOrder = { concessions: null };

                    if (configurationProvider.useConssesion(orderModel.get('theater'))) {
                        continueTo = 'concessions/' + orderModel.get('theater').get('id') + '/null';
                    }

                    if (orderModel.get('tickets').holdSeats) {
                        orderModel.set('holdTickets', orderModel.get('tickets').getOnHoldTickets());
                        window.AppMethods.ChangePage('#holdSeats/' + orderModel.get('theater').get('id') + '/' + orderModel.get('performance').get('number'));
                    } else {
                        that.lockTickets(continueTo);
                    }
                };

                this.handleSubscriptionTickets(continueFlow);
            }
            return false;
        },

        lockTickets: function (page) {
            var that = this;
            var orderModel = window.ObjectModels.OrderModel;

            var selectedGroupTickets = _.filter(orderModel.get('tickets').models, function (model) {
                return model.get('quantity') > 0 && model.get('volume') > 1;
            });

            var getTicketQuantity = function (tickets) {
                var quantity = 0;
                _.each(tickets, function (t) {
                    quantity += t.get('quantity');
                });
                return quantity;
            };

            var selectedGroupTicketsQuantity = getTicketQuantity(selectedGroupTickets);

            window.SaleHelper.lockTickets(function (success) {
                if (success) {
                    if (selectedGroupTickets.length != orderModel.get('lockedGroupTickets').length || selectedGroupTicketsQuantity != getTicketQuantity(orderModel.get('lockedGroupTickets'))) {
                        window.showAlert(that.localize.groupedTicketsNotification, {
                            postBack: function () {
                                Backbone.history.navigate(page, true);
                            }
                        });
                    } else {
                        Backbone.history.navigate(page, true);
                    }
                } else if (window.ObjectModels.OrderModel.get('invalidSubscriptions') || window.ObjectModels.OrderModel.get('invalidCtmCards')) {
                    $('#selectSeatsButton:visible, #continueToCheckout:visible').trigger('tap');
                }
            });
        },

        findTicket: function (e) {
            var orderModel = window.ObjectModels.OrderModel;

            var type = $(e.currentTarget).attr("data-ticket-type");
            return orderModel.get('tickets').get(type);
        },

        addTicket: function (e) {
            e.preventDefault();
            e.stopPropagation();

            var that = this;
            var ticket = that.findTicket(e);

            ticket.addTicket(true);

            if (ticket.get('pricedTicketRequired')) {
                that.availableCompanionTickets -= ticket.get('pricedTicketQty');
            } else if (!ticket.get('validate')) {
                if (that.options.theater.get('maxTickets') < that.availableCompanionTickets) {
                    that.availableCompanionTickets++;
                }
            }

            that.handleCTMTicketState();
            return false;
        },

        removeTicket: function (e) {
            e.preventDefault();
            e.stopPropagation();

            var that = this;
            var ticket = that.findTicket(e);
            ticket.removeTicket();

            if (ticket.get('subscriptionCardNumbers')) {
                ticket.removeSubscriptionCard();
            }

            if (ticket.get('ctmCards') && ticket.get('ctmCards').length > ticket.get('quantity')) {
                ticket.removeCtmCode();
            }

            if (ticket.get('pricedTicketRequired')) {
                that.availableCompanionTickets += ticket.get('pricedTicketQty');
            } else if (!ticket.get('validate')) {
                that.availableCompanionTickets--;

                if (that.availableCompanionTickets < 0) {
                    var orderModel = window.ObjectModels.OrderModel;

                    var ctmTickets = _.filter(orderModel.get('tickets').models, function (ticket) {
                        return ticket.get('pricedTicketRequired') && ticket.get('quantity') > 0;
                    });

                    var ctm = ctmTickets[ctmTickets.length - 1];
                    ctm.removeTicket();
                }
            }

            that.handleCTMTicketState();
            return false;
        },

        handleCTMTicketState: function () {
            var that = this;
            var ctmQuantity = 0;
            var totalCompanion = 0;

            var orderModel = window.ObjectModels.OrderModel;

            var ctmTickets = _.filter(orderModel.get('tickets').models, function (ticket) {
                return ticket.get('pricedTicketRequired');
            });

            _.each(orderModel.get('tickets').models, function (t) {

                if (t.get('pricedTicketRequired')) {
                    ctmQuantity += (t.get('quantity') * t.get('pricedTicketQty'));
                }
                else if (!t.get('validate')) {
                    totalCompanion += t.get('quantity');
                }
            });

            that.availableCompanionTickets = (totalCompanion - ctmQuantity);

            _.each(ctmTickets, function (ctm) {
                if (ctm.get('pricedTicketQty') > that.availableCompanionTickets) {
                    $('#' + ctm.get('id')).find('.tickets-add-ticket').addClass('ui-disabled');
                } else {
                    $('#' + ctm.get('id')).find('.tickets-add-ticket').removeClass('ui-disabled');
                }
            });

        },

        ticketQuantityChanged: function (e, num) {
            var orderModel = window.ObjectModels.OrderModel;
            var removeButton = $(this.that.el).find("#ticket-count-" + e.get('id'));
            removeButton.html(num);

            if (num == 0) {
                removeButton.prev().hide();
            } else {
                removeButton.prev().show();
            }

            if (configurationProvider.serviceChargeAmount())
                $(this.that.el).find("#service-charge").html(this.ticket.getTicketServiceChargeDisplay(orderModel.get('theater').get('culture')));

            orderModel.set('totalPrice', this.ticket.getTicketTotal());

            $(this.that.el).find("#ticket-total").html(Globalize.format(this.ticket.getTicketTotal(), "c", orderModel.get('theater').get('culture')));
            $(this.that.el).find("#ticket-tax").html(Globalize.format(orderModel.get('tickets').getTotalTax(), "c", orderModel.get('theater').get('culture')));
            $(this.that.el).find("#ticket-subtotal").html(Globalize.format((this.ticket.getTicketTotal() - orderModel.get('tickets').getTotalTax()), "c", orderModel.get('theater').get('culture')));

            if (orderModel.get('tickets').where({ 'isReservedSeating': false, 'isOnHoldTicketType': false }).length > 0 && orderModel.get('tickets').getReservedTicketQuantity() < 1) {
                $(this.that.el).find('#selectSeatsButton').hide();
                $(this.that.el).find('#continueToCheckout').show();
            } else {
                $(this.that.el).find('#selectSeatsButton').show();
                $(this.that.el).find('#continueToCheckout').hide();
            }
        },

        seatFeesInfo: function (e) {
            e.preventDefault();
            if (!window.performanceLayout) return;
            var mapKeyView = 'seatFees';

            require(['views/' + mapKeyView, 'collections/zoneCollection'], function (keyView) {
                var zoneCollection = window.performanceLayout.zones;

                var dialog = new keyView({
                    zones: zoneCollection.models, culture: window.ObjectModels.OrderModel.get('theater').get('culture'), callback: function () {
                        $('#popupDialog').popup("close");
                    }
                });

                $("#popupDialog").html(dialog.render());
                $('#popupDialog').popup("open");
                $("#popupDialog").enhanceWithin();
            });
        },
        warningInfo: function (e) {
            e.preventDefault();
            if (isJqmGhostClick(e)) return;

            var theater = window.ObjectModels.OrderModel.get('theater');
            var that = this;

            if (!theater.get('warningLong')) return;

            require(['views/infoDialogView'], function (infoView) {
                var dialog = new infoView({
                    title: theater.get('warningTitle'),
                    content: theater.get('warningLong'),
                    confirmButton: {
                        text: that.localize.ok,
                        callback: function () {
                            $('#popupDialog').popup("close");
                        }
                    }
                });

                $("#popupDialog").html(dialog.render());
                $('#popupDialog').popup("open");
                $("#popupDialog").enhanceWithin();
            });
        },
        facebookSharePanel: function () {

            var orderModel = window.ObjectModels.OrderModel;
            var movie = orderModel.get('movie');
            var theater = orderModel.get('theater');
            var performance = orderModel.get('performance');

            var theaterUrl = appConfig.SiteURL + '#tickets/' + performance.get('number') + '/' + movie.get('code').trim() + '/' + theater.get('id') + "/share";

            var img = movie.get('image') ? movie.get('image').medium || window.appConfig.DefaultMoviePosterImage : window.appConfig.DefaultMoviePosterImage;

            var movieName = movie.get('name');
            var theaterCulture = theater.get('culture');
            var theatreName = theater.get('name') === '' ? '' : 'at ' + theater.get('name');

            var showTime = performance.get('showTime');
            var shareText = "{text} {movieDetails} {theatreDetails} {timeDetails}";

            if (showTime) {
                var date = new Date();
                var month = date.getMonth() + 1;
                var dateTime = date.getFullYear() + '-' + month + '-' + date.getDate() + 'T' + date.getHours() + ':' + date.getMinutes() + ':' + date.getSeconds();
                var showDate = DtHelper.convertDate(showTime);
                var diff = DtHelper.dateDiff(DtHelper.convertDate(dateTime), showDate);

                if (diff < 0) {
                    shareText = shareText.replace("{text}", "I'm going to see");
                    shareText = shareText.replace("{timeDetails}", 'on ' + Globalize.format(showDate, "M", theaterCulture) + ' at ' + Globalize.format(showDate, 't', theaterCulture));
                } else {
                    shareText = shareText.replace("{text}", "I just saw");
                    shareText = shareText.replace("{timeDetails}", '');
                }

                if (shareText.indexOf('{text}') > -1) {
                    shareText = shareText.replace("{text}", "I'm going to see");
                }
                if (shareText.indexOf('{timeDetails}') > -1) {
                    shareText = shareText.replace("{timeDetails}", '');
                }
            } else {
                shareText = shareText.replace("{text}", "I'm going to see");
                shareText = shareText.replace("{timeDetails}", '');
            }

            shareText = shareText.replace("{movieDetails}", movieName);
            shareText = shareText.replace("{theatreDetails}", theatreName);

            var options = {
                method: "share",
                href: theaterUrl,
                caption: movieName,
                description: shareText,
                picture: img,
                share_feedBrowser: true // iOS only
            };

            window.facebookConnectPlugin.showDialog(options, function (response) {  
                if (response && response.postId) {
                    window.showAlert('Posted successfully');
                } else if (window.AppProperties.IsAndroid) {
                    window.appAvailability.check(
                        'com.facebook.katana', null, function () {
                            window.showAlert('Posted successfully');
                        });
                }
            });
        }
    });

    return view;
});