﻿define([
  'jquery',
  'underscore',
  'backbone',
  'text!templates/VirtualGiftCardsView.html',
  'sharedhelpers/configurationProvider',
  'helpers/uiHelper',
  'helpers/configHelper',
  'helpers/virtualGiftCardHelper',
  'sharedhelpers/giftCardHelper',
  'sharedhelpers/storageProvider',
  'sharedhelpers/utilities',
  'sharedlibs/jquery/jquery.qrcode.min',
  'models/orderModel',
  'idangerousSwiper',
  'models/orderModel'
], function ($, _, Backbone,template, configurationProvider) {
    var view = Backbone.View.extend({
        template: _.template(template),
        localize: window.ObjectCollections.Localization.result,

        initialize: function () {
            this.cards = this.options.cards || [];
            this.cardId = this.options.cardId;
            this.initialSwiperIndex = 0;
        },

        events: {
            'pageshow': 'onPageShow',
            'pagebeforehide': 'onPageHide',
            'tap li[for="tab-value"]': 'switchTabs',
            'tap #manage-btn': 'manageCard',
            'tap #vgc-add-new': 'addNewCard',
            'tap #vgc-add-existing': 'addExistingCard',
            'tap .vgc-reload a': 'reloadCard',
            'tap .set-primary': 'togglePrimaryCard',
            'tap .add-to-passbook': 'addToPassbook',
            'tap .change-design': 'changeDesign',
            'tap .delete-card': 'deleteCard'
        },

        render: function () {
            var that = this;

            _.each(that.cards, function (card, index) {
                if (card.Id == that.cardId) {
                    //Get the initial slider position based on the cardId sent from the router
                    that.initialSwiperIndex = index;

                    //If redirected from the finished payment we need to update the balance of the card with the amount that was added to the card
                    if (that.options.status && window.ObjectModels.OrderModel && window.ObjectModels.OrderModel.get('saleId')) {
                        var amountLoaded = window.ObjectModels.OrderModel.get('totalPrice');
                        card.Balance = card.Balance ? window.Globalize.parseFloatCurrency(card.Balance) + amountLoaded : amountLoaded;
                        card.Balance = window.Globalize.format(card.Balance, 'c', window.AppConfig.AppCulture);

                        //Save cached Balance
                        var storage = new window.StorageProvider();
                        storage.read('PaymentMethod', function (paymentMethods) {
                            _.each(paymentMethods, function (paymentMethod) {
                                if (paymentMethod.Id == card.Id) {
                                    paymentMethod.Balance = card.Balance;
                                }
                            });

                            storage.store('PaymentMethod', paymentMethods);
                        });
                    }
                }

                //Set primary card as first card in case there is no cardId sent from the router
                if (!that.cardId && card.Primary) {
                    that.initialSwiperIndex = index;
                }

                card.CardNumber = new window.Utilities().decrypt(card.CardNumber);
                let balanceTobeHidden=window.giftCardHelper.checkIfBalanceNeedsToBeHidden(card.CardNumber);
                if(balanceTobeHidden){
                  card.hideBalance=true;
                }
            });

            var showCardsTab = that.cards.length > 0;
            var passbookIcon = typeof window.device != 'undefined' ? window.device.version.slice(0, window.device.version.indexOf('.')) : 8;
            var showPassbookIcon = window.AppProperties.PassbookAvailable.virtualGiftCardPass;
            var vgcBackgrounds = configurationProvider.getGiftCardBackgrounds();
            var showChangeDesignButton = vgcBackgrounds.length > 0;

            that.$el.html(this.template({
                localize: that.localize,
                cards: that.cards,
                reloadValues: configurationProvider.getVirtualGiftCardReloadValues(),
                showCardsTab: showCardsTab,
                passbookIcon: passbookIcon,
                showPassbookIcon: showPassbookIcon,
                showAddExistingCardButton: configurationProvider.allowVirtualGiftCardsAddExistingCard(),
                hideBuyNewCardButton: configurationProvider.hideBuyNewCardButton(),
                hideBuyNewCardAndAddValueButtons: configurationProvider.hideBuyNewCardAndAddValueButtons(),
                showChangeDesignButton: showChangeDesignButton,
                hideBackButton: that.options.hideBackButton
            }));

            that.initializeSwiper();
            that.renderQrCodes();

            var backButton = $(that.el).find('#header [data-rel="back"]');
            backButton.on('tap', function () {
                window.AppMethods.ChangePage('#home');
                return false;
            });

            return that;
        },

        onPageShow: function () {
            var that = this;

            if (!window.giftCardBalanceCleared) {
                window.giftCardHelper.invalidateBalance(function () {
                    that.getCardBalance();
                });

                window.giftCardBalanceCleared = true;
            } else {
                var fetchBalance = false;
                _.each(that.cards, function (card) {
                    if (!card.Balance || card.UpdateBalance) {
                        fetchBalance = true;
                    }
                });

                if (fetchBalance) {
                    that.getCardBalance();
                }
            }

            this.handleNotification();
        },

        onPageHide: function () {
            window.giftCardHelper.abortBalanceCall(true);
        },

        handleNotification: function () {
            var that = this;
            var showNotificationAlert = function (callback) {
                var orderModel = window.ObjectModels.OrderModel;

                if (!orderModel || !orderModel.get('virtualGiftCard')) {
                    callback.call(that);
                    return;
                }

                var theater = orderModel.get('theater');
                var theaterName = configurationProvider.useVirtualGiftCardsDefaultSite() && (theater.get('id') != configurationProvider.virtualGiftCardsDefaultSite()) ?
                                  configurationProvider.virtualGiftCardsDefaultSiteName() : theater ? theater.get('name') : '';
                var amount = window.Globalize.format(orderModel.get('totalPrice'), 'c', window.AppConfig.AppCulture);
                var paymentType = orderModel.get('paymentType');

                var creditCards = _.filter(window.ObjectModels.OrderModel.get('paymentCards'), function (card) {
                    return card.Type == 'CreditCard';
                });

                if (creditCards.length > 0) {
                    amount = window.Globalize.format(creditCards[creditCards.length - 1].amount, 'c', window.AppConfig.AppCulture);
                    paymentType = 'credit';
                }

                var msg = $.sprintf(that.localize.virtualGiftCardNotification, amount, theaterName, paymentType);

                window.showAlert(msg, {
                    postBack: function () {
                        if (callback)
                            callback.call(that);
                    }
                });

                window.ObjectModels.OrderModel = new OrderModel();
            };

            if (that.options.status == 'new') {
                showNotificationAlert(function () {
                    window.UIHelper.notificationBar(that.localize.vgcActivateNotification);
                    that.passbookPrompt();
                });
            } else if (that.options.status == 'update') {
                showNotificationAlert(function () {
                    window.UIHelper.notificationBar(that.localize.vgcUpdateNotification);
                });
            } else if (that.options.status == 'existingCard') {
                showNotificationAlert(function () {
                    window.UIHelper.notificationBar(that.localize.vgcExistingCardNotification);
                });
            }

            if (window.ObjectModels.OrderModel && window.ObjectModels.OrderModel.get('saleId')) {
                window.ObjectModels.OrderModel.cancelSale();
                window.hidePageLoadingMessage();
            }
        },

        switchTabs: function (e) {
            e.preventDefault();
            if (window.isJqmGhostClick(e)) return;

            var currentTab = e.currentTarget.attributes['value'].value;

            if (currentTab == 'my-cards') {
                $(this.el).find('#my-cards').show();
                $(this.el).find('#new-card').hide();
                this.swiper.reInit();
            } else {
                $(this.el).find('#new-card').show();
                $(this.el).find('#my-cards').hide();
            }
        },

        manageCard: function (e) {
            if (e) {
                e.preventDefault();
                if (window.isJqmGhostClick(e)) return;
            }

            //Flip the card on active slide
            var activeVgcSlide = $(this.swiper.activeSlide()).find('.vgc-card');
            activeVgcSlide.toggleClass('rotate');

            if (activeVgcSlide.hasClass('rotate')) {
                $(this.el).find('#manage-btn').html(this.localize.doneEditing);
            } else {
                $(this.el).find('#manage-btn').html(this.localize.manage);
            }
        },

        reloadCard: function (e) {
            e.preventDefault();
            if (window.isJqmGhostClick(e)) return;

            if (window.ConfigHelper.isDeviceOnline() == false) {
                window.showAlert(this.localize.errorUnableConnectToInternet);
                return;
            }

            var cardNumber = $(this.swiper.activeSlide()).find('.vgc-card').attr('id');
            var cardName = $(this.swiper.activeSlide()).find('.vgc-card').attr('data-cardname');
            var amount = $(e.currentTarget).attr('data-amount');

            if (amount) {
                window.virtualGiftCardHelper.lockCard(amount, cardNumber, cardName);
            } else {
                this.lockCard(cardNumber, cardName);
            }
        },

        addNewCard: function (e) {
            e.preventDefault();

            this.lockCard();
        },

        lockCard: function (cardNumber, cardName) {
            if (window.ConfigHelper.isDeviceOnline() == false) {
                window.showAlert(this.localize.errorUnableConnectToInternet);
                return false;
            }

            require(['views/virtualGiftCardDialogView'], function (dialogView) {
                var dialog = new dialogView({
                    cardName: cardName,
                    callback: function (amount, name) {
                        if (amount) {
                            window.virtualGiftCardHelper.lockCard(amount, cardNumber, name);
                        }

                        $('#popupDialog').popup('close');
                    }
                });

                $('#popupDialog').html(dialog.render());
                $('#popupDialog').popup('open');
                $('#popupDialog').enhanceWithin();
            });

            return false;
        },

        addExistingCard: function (e) {
            e.preventDefault();
            if (window.isJqmGhostClick(e)) return;

            var that = this;
            var utilities = new window.Utilities();

            require(['views/infoDialogView', 'libs/jquery.validate.min'], function (infoView) {
                var form = $('<form id="frmVgc" class="validate">');
                var name = $('<input type="text" name="card-name" style="text-transform: capitalize;" class="input-dialog" maxlength="25" placeholder="' + that.localize.cardName + '">');
                var cardNumber = $('<input type="tel" name="card-number" class="required card-number input-dialog" placeholder="' + that.localize.cardNumber + '">');

                form.append('<span class="bodyText">' + that.localize.enterVirtualCardNumber + '</span>');
                form.append(name, cardNumber);

                var saveVgc = function () {
                    var frm = $('#frmVgc');
                    if (frm.valid()) {
                        //hiding IOS keyboard
                        frm.find('input').blur();

                        var card = {
                            name: utilities.capitalize($.trim(frm.find('input[name="card-name"]').val())),
                            number: frm.find('input[name="card-number"]').val()
                        };

                        var handlers = {
                            onSuccess: function (balance) {
                                card.balance = balance;
                                card.existingCard = true;
                                //Prevent card from being updated
                                window.virtualGiftCardHelper.saveCard(card, false, function (success) {
                                    if (success) {
                                        $('#popupDialog').popup('close');
                                    }
                                });
                            },
                            onError: function (model) {
                                if (model.get('invalidCard') == true || model.get('invalidCard') == undefined) {
                                    window.showAlert(that.localize.errorGiftCardDetails);
                                    $('#popupDialog').popup('close');
                                } else {
                                    this.onSuccess();
                                }
                            },
                            onComplete: function () {
                                window.hidePageLoadingMessage();
                            }
                        };

                        window.showPageLoadingMessage();

                          window.giftCardHelper.getBalance(card.number, handlers);
                    }
                };

                var dialog = new infoView({
                    title: that.localize.addCard,
                    content: form[0].outerHTML,
                    confirmButton: {
                        text: that.localize.ok,
                        callback: saveVgc
                    },
                    cancelButton: {
                        text: that.localize.cancel,
                        callback: function () {
                            $('#popupDialog').popup('close');
                        }
                    }
                });

                var popup = $('#popupDialog');
                popup.html(dialog.render());
                popup.popup('open');
                popup.enhanceWithin();

                if (window.AppProperties.IsIOS) {
                    $('#popupDialog').popup({
                        afteropen: function () {
                            $('#frmVgc input[name="card-name"]').focus();
                        }
                    });
                }

                $.validator.addMethod('card-number', function (value) {
                    var regExp = new RegExp('^[0-9]+$');
                    return regExp.test(value);
                }, that.localize.errorVirtualGiftCardNumber);

                $('#frmVgc').validate({
                    onkeyup: false,
                    focusCleanup: true
                });
            });
        },

        passbookPrompt: function () {
            var that = this;
            var onConfirm = function (button) {
                //OK = 1; Cancel = 2
                if (button == 1) {
                    that.getVgcPassUrl(that.cardId);
                }
            };

            if (window.AppProperties.PassbookAvailable.virtualGiftCardPass) {
                if (window.isMobileDevice()) {
                    navigator.notification.confirm(that.localize.vgcPassbookPrompt, onConfirm, that.localize.alertTitle, that.localize.ok + ', ' + that.localize.cancel);
                } else if (confirm(that.localize.vgcPassbookPrompt)) {
                    onConfirm(1);
                }
            }
        },

        addToPassbook: function (e) {
            e.preventDefault();
            if (window.isJqmGhostClick(e)) return;

            var cardId = $(e.currentTarget).closest('.vgc-card').attr('data-cardid');
            this.getVgcPassUrl(cardId);
        },

        getVgcPassUrl: function (cardId) {
            window.showPageLoadingMessage();

            require(['helpers/passHelper'], function () {
                var storage = new window.StorageProvider();

                storage.read('PaymentMethod', function (cards) {
                    var card = _.find(cards, function (c) {
                        return c.Id == cardId;
                    });

                    var passHelper = new window.PassHelper();

                    passHelper.getVirtualGiftCardPassURL(card);
                });
            });
        },

        togglePrimaryCard: function (e) {
            e.preventDefault();
            if (window.isJqmGhostClick(e)) return;

            var cardElement = $(e.currentTarget).closest('.vgc-card');
            var favIcon = $(e.currentTarget).find('.favorite-icon');
            var cardId = cardElement.attr('data-cardid');

            if (favIcon.hasClass('checked')) {
                favIcon.removeClass('checked');
                cardElement.find('.favorite-icon-small').removeClass('checked');
                window.virtualGiftCardHelper.unsetPrimary(cardId);
                return;
            }

            $(this.el).find('.virtual-gift-card-back .favorite-icon').removeClass('checked');
            $(this.el).find('.virtual-gift-card-front .favorite-icon-small').removeClass('checked');

            cardElement.find('.favorite-icon-small').addClass('checked');
            favIcon.addClass('checked');

            window.virtualGiftCardHelper.setAsPrimary(cardId);
        },

        changeDesign: function (e) {
            e.preventDefault();
            if (window.isJqmGhostClick(e)) return;

            var that = this;
            var cardElement = $(e.currentTarget).closest('.vgc-card');
            var cardId = cardElement.attr('data-cardid');
            var cardBackground = cardElement.attr('data-cardbackground');

            var callback = function (backgroundImage) {
                window.virtualGiftCardHelper.updateDesign(cardId, backgroundImage);

                cardElement.attr('data-cardbackground', backgroundImage);
                cardElement.find('.virtual-gift-card-front').css('background-image', 'url(' + backgroundImage + ')');

                //Flip the card back when the background is changed
                that.manageCard();
            };

            window.virtualGiftCardHelper.openBackgroundPicker(cardBackground, callback);
        },

        deleteCard: function (e) {
            e.preventDefault();
            if (window.isJqmGhostClick(e)) return;

            var that = this;

            var onConfirm = function (button) {
                if (button != 1) {
                    return;
                }

                require(['helpers/paymentMethodHelper'], function () {
                    var pmh = new window.PaymentMethodsHelper();
                    var cardId = $(e.currentTarget).closest('.vgc-card').attr('data-cardid');

                    pmh.remove(cardId, function (response) {
                        if (response.success) {
                            that.swiper.removeSlide(that.swiper.activeIndex);
                            that.swiper.reInit();

                            if (that.swiper.slides.length == that.swiper.activeIndex + 1) {
                                that.swiper.swipePrev();
                            }

                            if (!that.swiper.slides.length) {
                                $(that.el).find('.vgc-controls').hide();
                                $(that.el).find('.info-no-card').show();
                            }
                            $(that.el).find('#manage-btn').html(that.localize.manage);
                        }
                    });
                });
            };

            if (window.isMobileDevice()) {
                navigator.notification.confirm(that.localize.removeVirtualGiftCardFromApplication, onConfirm, that.localize.alertTitle, that.localize.deleteCard + ', ' + that.localize.cancel);
            } else if (confirm(that.localize.removeVirtualGiftCardFromApplication)) {
                onConfirm(1);
            }
        },

        getCardBalance: function () {
            var that = this;
            var paymentCards;
            var vgCards = [];
            var storage = new window.StorageProvider();
            var util = new window.Utilities();

            //Getting the updated cards data from the storage as previously it could have been invalidated
            storage.read('PaymentMethod', function (value) { paymentCards = value; });

            _.each(paymentCards, function (paymentObject) {
                var condition = paymentObject.Type === 'VirtualGiftCard' && (!paymentObject.Balance || paymentObject.UpdateBalance);

                if (condition) {
                    paymentObject.CardNumber = util.decrypt(paymentObject.CardNumber);
                    vgCards.push(paymentObject);
                }
            });

            _.each(vgCards, function (paymentObject) {
                var paymentCardNumber = paymentObject.CardNumber;
                var handlers = {
                    onSuccess: function (balance) {
                        var updateCard = _.find(vgCards, function (card) {
                            if (paymentCardNumber == card.CardNumber) {
                                card.Balance = balance;
                                $(that.el).find('#' + card.CardNumber + ' .vgc-balance').html(card.Balance);
                                return true;
                            }
                        });

                        if (updateCard.Type == 'VirtualGiftCard') {
                            window.virtualGiftCardHelper.updateBalance(updateCard);
                        }
                    },
                    onError: function () {
                        //TODO: If balance call fails it shouldn't display n/a per Jeff's latest ticket
                        //$(that.el).find('#' + paymentCardNumber + ' .vgc-balance').html("(" + window.ObjectCollections.Localization.result.na + ")");
                    },
                    onComplete: function () {
                        window.giftCardHelper.hideBalanceLoader.call(that, paymentCardNumber);
                    },
                    onProgress: function () {
                        window.giftCardHelper.showBalanceLoader.call(that, paymentCardNumber);
                    }
                };
                    window.giftCardHelper.getBalance(paymentCardNumber, handlers);
            });
        },

        renderQrCodes: function () {
            var cardContainers = $(this.el).find('.vgc-card');

            _.each(cardContainers, function (cardContainer) {
                var cardNumber = $(cardContainer).attr('id');
                $(cardContainer).find('.vgc-bar-code-container').qrcode({ text: cardNumber, width: 90, height: 90 });
            });
        },

        initializeSwiper: function () {
            var that = this;
            var createPagination = that.cards.length > 1;

            _.defer(function () {
                that.swiper = $(that.el).find('.swiper-container').swiper({
                    mode: 'horizontal',
                    pagination: createPagination ? $(that.el).find('.pagination').get(0) : false,
                    initialSlide: that.initialSwiperIndex,
                    onSlideChangeStart: function () {
                        var activeVgcSlide = $(that.swiper.activeSlide()).find('.vgc-card');

                        if (activeVgcSlide.hasClass('rotate')) {
                            $(that.el).find('#manage-btn').html(that.localize.doneEditing);
                        } else {
                            $(that.el).find('#manage-btn').html(that.localize.manage);
                        }
                    }
                });
            });
        }
    });

    return view;
});
