﻿define([
    'collections/seatCollection'
], function (seatCollection) {
    window.SeatPreviewHelper = (function () {
        var auditoriumLayoutCache = {};
        var getLayoutQueue = [];
        var refreshInterval = 10; // Refresh interval in minutes
        function _addPerformancesToQueue(performances, theater) {
            _.each(performances, function (p) {
                if (_addToQueue(p, theater)) {
                    getLayoutQueue.push({
                        auditorium: p.get('auditorium'),
                        performance: p
                    });
                }
            });
        }

        function _addToQueue(performance, theater) {
            var notInQueueAndReservedSeating = !_.find(getLayoutQueue, function (queueItem) { return queueItem.auditorium == performance.get('auditorium'); }) &&
                                               performance.get('isReservedSeating');
   
            var refreshCache = true;

            var theaterLayouts = auditoriumLayoutCache[theater.get('id')];

            if (theaterLayouts) {
                var cahceItem = _.find(theaterLayouts, function (tl) { return tl.auditorium == performance.get('auditorium'); })
                refreshCache = !cahceItem || Math.abs(new Date().getTime() - cahceItem.lastRefreshTime.getTime()) / 60000 > refreshInterval;
            }

            return notInQueueAndReservedSeating && refreshCache;
        }

        function _handleQueue(theater, callback) {
            var requests = [];
            if (getLayoutQueue.length == 0) {
                if (callback) {
                    callback();
                }
            } else {
                _.each(getLayoutQueue, function (queuItem) {
                    requests.push(new seatCollection({ performanceId: (queuItem.performance.get('number') || queuItem.performance.number), theaterId: theater.get('id') }).fetch({
                        success: function (seatsLayout) {
                            var cacheObject = {
                                auditorium: queuItem.auditorium,
                                layout: seatsLayout,
                                lastRefreshTime: new Date()
                            };
                            if (auditoriumLayoutCache[theater.get('id')]) {
                                var existingCacheItem = _.find(auditoriumLayoutCache[theater.get('id')], function (ci) {
                                    return ci.auditorium == cacheObject.auditorium;
                                });
                                if (existingCacheItem) {
                                    existingCacheItem.layout = cacheObject.layout;
                                    existingCacheItem.lastRefreshTime = new Date();
                                } else {
                                    auditoriumLayoutCache[theater.get('id')].push(cacheObject);
                                }
                            } else {
                                auditoriumLayoutCache[theater.get('id')] = [cacheObject];
                            }
                        }
                    }));
                });
                $.when.apply($, requests).then(function () {
                    getLayoutQueue = [];
                    if (callback) {
                        callback();
                    }
                });
            }
        }

        function _getPerformancesLayouts(performances, theater, callback) {
            performances = performances.models || performances;
            _addPerformancesToQueue(performances, theater);
            _handleQueue(theater, function () {
                callback(auditoriumLayoutCache[theater.get('id')]);
            });
        }

        function _getSeatsPreview(performances, theater, callback) {
            require(['collections/ticketCollection'], function (ticketCollection) {
                var t = new ticketCollection();                
                t.getSeatsPreview(theater.get('id'), _.filter(performances, function (p) { return p.get('isReservedSeating'); }), function (results, error) {
                    callback(results, error);
                });
            });
        }

        return {
            getPerformancesLayouts: _getPerformancesLayouts,
            getSeatsPreview: _getSeatsPreview
        };

    })();
})